SystemOrganization addCategory: #'Mondrian-Tests'!

TestCase subclass: #MOBehavior
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Tests'!

!MOBehavior commentStamp: 'AlexandreBergel 12/27/2011 10:12' prior: 0!
Testing the extension of the class Behavior!

!MOBehavior methodsFor: 'utils' stamp: ''!
refOfClasses
	^ {Set . Object . Array}! !

!MOBehavior methodsFor: 'tests' stamp: ''!
testNumberOfMethods

	self assert: MOEvent numberOfAttributes = MOEvent instVarNames size.! !

!MOBehavior methodsFor: 'tests' stamp: ''!
testReferencedClasses

	self assert: (self class referencedClasses includesAllOf: self refOfClasses).
	
! !

TestCase subclass: #MOElementShapePropertiesTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Tests'!

!MOElementShapePropertiesTest methodsFor: 'tests' stamp: ''!
testEquality
	| p1 p2 p3 p4 shape |
	p1 := MOElementShapeProperties shape: (shape := MORectangleShape new) key: #cache1.
	p2 := MOElementShapeProperties shape: shape key: #cache1.
	p3 := MOElementShapeProperties shape: MORectangleShape new key: #cache1.
	p4 := MOElementShapeProperties shape: shape key: #cache2.
	
	self assert: (p1 = p1).

	self assert: (p1 = p2).
	self assert: (p1 ~~ p2).
	self assert: (p1 hash = p2 hash).
	
	self assert: (p1 ~= p3).
	self assert: (p1 hash ~= p3 hash).

	self assert: (p1 ~= p4).
	self assert: (p1 hash ~= p4 hash).
	
	self assert: ({p1 .p2 .p3 .p4} allSatisfy: [ :p | p hash isNumber ])  ! !

TestCase subclass: #MOElementTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Tests'!

!MOElementTest methodsFor: 'tests' stamp: ''!
testSelectingEdge
	"Make sure edge cannot be selected and deselected. "

	| view |
	view := MOViewRenderer new.
	view nodes: (1 to: 4).
	view edges: {1 -> 2} from: #key to: #value.

	view root nodesDo: #select.
	self assert: (view root selections size = 4).
	
	view root deselectAll.
	self assert: (view root selections isEmpty).
	
	self shouldnt: [ view root edges first select ] raise: Error.
	self shouldnt: [ view root edges first deselect ] raise: Error.	! !

!MOElementTest methodsFor: 'tests' stamp: ''!
testSelection
	| root |
	root := MORoot new.
	root addNode: (MONode on: $a).
	root nodes first select.
	self assert: root selections notEmpty.
	root nodes first deselect.	
	self assert: root selections isEmpty! !

!MOElementTest methodsFor: 'tests' stamp: ''!
testSelectionOfForwarders
	| root parent child |
	root := MORoot new.
	parent := MONode on: 'parent'.
	child := MONode on: 'child'.
	child interaction forward.
	root addNode: parent.
	parent addNode: child.
	root nodes first select.
	self assert: root selections size = 1.
	self assert: root selections first = parent.
	self assert: child isSelected not.
	root nodes first deselect.	
	self assert: root selections isEmpty! !

TestCase subclass: #MONColorLinearNormalizerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Tests'!

!MONColorLinearNormalizerTest methodsFor: 'tests' stamp: ''!
testOneElementInContext

	| normalizer |
	normalizer := MONColorLinearNormalizer inContext: #(1) withCommand: #yourself.
	self assert: (normalizer moValue: 1) = Color gray! !

TestCase subclass: #MONormalizerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Tests'!

!MONormalizerTest methodsFor: 'testing' stamp: ''!
testBlockContext
	| normalizer |

	normalizer := MONColorLinearNormalizer inContext: [:entity | #(1 2 3 4 5 6 7 8)] withCommand: [:entity | entity].
	self assert: (normalizer maximumValue: nil) = 8.
	self assert: (normalizer moValue: 1) = ( Color white).
	self assert: (normalizer moValue: 8) = Color black.! !

!MONormalizerTest methodsFor: 'testing' stamp: ''!
testExplicitIdentity
	| normalizer colorOne colorTwo |
	
	normalizer := MONExplicitIdentityNormalizer 
						withCommand: #last 
						withColors: {Color blue . Color white . Color red} 
						withDefaultColor: Color pink.
						
	colorOne := normalizer moValue: 'alex'.
	colorTwo := normalizer moValue: 'suffix'.
	self assert: (colorOne = colorTwo)

! !

!MONormalizerTest methodsFor: 'testing' stamp: ''!
testExplicitIdentity2
	| normalizer colorOne colorTwo |
	
	normalizer := MONExplicitIdentityNormalizer
					withCommand: #yourself
					withColors: {Color blue . Color white . Color red} 
					withDefaultColor: Color pink.						

	self assert: (normalizer moValue: 1) = Color blue.
	self assert: (normalizer moValue: 2) = Color white.
	self assert: (normalizer moValue: 3) = Color red.
	self assert: (normalizer moValue: -1) = Color pink.
	self assert: (normalizer moValue: 1000) = Color pink.! !

!MONormalizerTest methodsFor: 'testing' stamp: ''!
testGradient
	| normalizer |

	normalizer := MONColorLinearNormalizer inContext: #(0 1 2 3 4 5 6 7 8 9 10) lowColor: Color yellow highColor: Color red.
	self assert: (normalizer maximumValue: nil) = 10.
	self assert: (normalizer moValue: 10) = Color red. 
	self assert: (normalizer moValue: 1) = (Color r: 1.0 g: 0.900012 b: 0.0).
	self assert: (normalizer moValue: 0) = (Color yellow).! !

!MONormalizerTest methodsFor: 'testing' stamp: ''!
testIdentity
	| colorOne colorTwo normalizer |
	
	normalizer := MONIdentityNormalizer new.
	colorOne := normalizer moValue: 1.
	colorTwo := normalizer moValue: 2.
	self deny: colorOne = colorTwo.

	colorOne := normalizer moValue: 12.
	colorTwo := normalizer moValue: 12.
	self assert: colorOne = colorTwo.! !

!MONormalizerTest methodsFor: 'testing' stamp: ''!
testIdentityModulo
	"This checks that there is no exception if there are more identities than colors"

	1 to: 150 do: [:index | MONIdentityNormalizer new moValue: index]! !

!MONormalizerTest methodsFor: 'testing' stamp: ''!
testIdentityWithCommand
	| colorOne colorTwo normalizer |
	
	normalizer := MONIdentityNormalizer withCommand: #last.
	colorOne := normalizer moValue: 'bob'.
	colorTwo := normalizer moValue: 'tom'.
	self deny: colorOne = colorTwo.

	colorOne := normalizer moValue: 'anna'.
	colorTwo := normalizer moValue: 'maria'.
	self assert: colorOne = colorTwo.! !

!MONormalizerTest methodsFor: 'testing' stamp: ''!
testLinearNormalizer
	| normalizer |

	normalizer := MONColorLinearNormalizer inContext: #(#Zuerich #Bern) withCommand: [:entity | entity size].
	self assert: (normalizer maximumValue: nil) = 7.
	self assert: (normalizer moValue: #Zuerich) = Color black. 
	self assert: (normalizer moValue: #Bern) = ( Color white).
	
	normalizer := MONLinearNormalizer inContext: #('Luzern' 'Basel') withCommand: [:entity | entity size].
	self assert: (normalizer maximumValue: nil) = 6.

	normalizer := MONLinearNormalizer inContext: #('Luzern' 'Basel') withCommand: 99.
	self assert: (normalizer maximumValue: nil) = 99.! !

!MONormalizerTest methodsFor: 'testing' stamp: ''!
testMultipleLinearNormalizer
	| normalizer |

	normalizer := MOMultipleColorLinearNormalizer 
					valueRange: {5 . 10 . 15} colorRange: {Color red . Color white . Color blue}.
	
	self assert: (normalizer moValue: 5) =  ( Color red). 
	self assert: (normalizer moValue: 6) =  ( (Color r: 1.0 g: 0.2 b: 0.2)). 
	self assert: (normalizer moValue: 15) = Color blue. 
! !

!MONormalizerTest methodsFor: 'testing' stamp: ''!
testNumberLinearNormalizer
	| normalizer |
	normalizer := MONumberLinearNormalizer inContext: (1 to: 10) withCommand: [:entity | entity * 1000 ].
	self assert: (normalizer moValue: 100) = 10.
	
	normalizer := MONumberLinearNormalizer inContext: (1 to: 100) withCommand: [:entity | entity * 1000 ].
	self assert: (normalizer moValue: 100) = 1.
	
	normalizer := MONumberLinearNormalizer inContext: (1 to: 100) withCommand: [:entity | entity ].
	self assert: (normalizer moValue: 100) = 1.
! !

!MONormalizerTest methodsFor: 'testing' stamp: ''!
testThreshold
	| normalizer skyBlue |
	
	skyBlue := (Color r:  0.529361 g: 0.807838 b: 0.921621).
	normalizer := MONColorLinearNormalizer inContext: #(0 1 2 3 4 5 6 7 8 9 10) lowColor: skyBlue lowThreshold: 0.3 highColor: Color brown highThreshold: 0.7.
	self assert: (normalizer maximumValue: nil) = 10.
	self assert: (normalizer moValue: 10) = ( (Color r: 0.579 g: 0.382 b: 0.277)). 
	self assert: (normalizer moValue: 1) = ( (Color r: 0.553 g: 0.601 b: 0.608)).
	self assert: (normalizer moValue: 0) = ( (Color r: 0.551 g: 0.626 b: 0.645)).
	
"The adapted VW version. Differences with Squeak propably lay in different variable initialization. For example, brown in Squeak is different from VW
	skyBlue := (Color r:  0.529361 g: 0.807838 b: 0.921621).
	normalizer := MONColorLinearNormalizer inContext: #(0 1 2 3 4 5 6 7 8 9 10) lowColor: skyBlue lowThreshold: 0.3 highColor: Color brown highThreshold: 0.7.
	self assert: (normalizer maximumValue: nil) = 10.
	self assert: (normalizer moValue: 10) = (Color r: 0.611769 g: 0.357588 b: 0.391771). 
	self assert: (normalizer moValue: 1) = (Color r: 0.569405 g: 0.589183 b: 0.664266).
	self assert: (normalizer moValue: 0) = (Color r: 0.564644 g: 0.614943 b: 0.694543)."! !

TestCase subclass: #MOQuadTreeTest
	instanceVariableNames: 'leafQuad compositeQuad nodes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Tests'!

!MOQuadTreeTest methodsFor: 'running' stamp: ''!
setUp

	| model |
	model := MOTestModel new.
	nodes := MOViewRenderer new nodes: model allNodes using: model shapeForNode.
	leafQuad := MOQuadTree with: (1@1 corner: 10@10).
	compositeQuad := MOQuadTree with: (0@0 corner: 40@40).
	nodes do: [:n | compositeQuad addElement: n]! !

!MOQuadTreeTest methodsFor: 'tests' stamp: ''!
testCompositeQuadtree

	| a1 b1 b2 c3 c4 d3 |
	
	a1 := nodes first.
	b1 := nodes second.
	b2 := nodes third.
"	c1 := nodes at: 4.
	c2 := nodes at: 5."
	c3 := nodes at: 6.
	c4 := nodes at: 7.
"	d1 := nodes at: 8.
	d2 := nodes at: 9."
	d3 := nodes at: 10.
	
	self assert: compositeQuad isComposite.
	self assert: compositeQuad count = 10.
	
	self assert: ((compositeQuad findElementsAt: 11@11) includesAll: {c3. c4. d3}).
	self assert: ((compositeQuad findElementsAt: 7@1) includes: a1).
	self deny: ((compositeQuad findElementsAt: 7@1) includes: b1).
	
	self assert: ((compositeQuad findElementsIntersecting: (-1@ -1 corner: 2@3)) includesAll: {a1. b1}).
	self assert: ((compositeQuad findElementsIntersecting: (7@1 corner: 8@2)) includes: a1).
	self deny: ((compositeQuad findElementsIntersecting: (7@1 corner: 8@2)) includes: b1).
	self assert: (compositeQuad findElementsIntersecting: (20@20 corner: 25@25)) isEmpty.
	
	compositeQuad removeElement: b1.
	compositeQuad removeElement: b2.
	self deny: compositeQuad isComposite.
	self assert: compositeQuad count = 8.
	! !

!MOQuadTreeTest methodsFor: 'tests' stamp: ''!
testLeafQuadtree

	| a1 b1 |
	self deny: leafQuad isComposite.
	self assert: leafQuad count = 0.
	
	a1 := nodes first.
	b1 := nodes second.
	
	leafQuad addElement: a1.
	leafQuad addElement: b1.
	self assert: leafQuad count = 2.
	
	self assert: ((leafQuad findElementsAt: 1@1) includesAll: (Array with: a1 with: b1)).
	self assert: ((leafQuad findElementsAt: 7@1) includes: a1).
	self deny: ((leafQuad findElementsAt: 7@1) includes: b1).
	
	self assert: ((leafQuad findElementsIntersecting: (-1@ -1 corner: 2@3)) includesAll: (Array with: a1 with: b1)).
	self assert: ((leafQuad findElementsIntersecting: (7@1 corner: 8@2)) includes: a1).
	self deny: ((leafQuad findElementsIntersecting: (7@1 corner: 8@2)) includes: b1).
	self assert: (leafQuad findElementsIntersecting: (-2@ -2 corner: -1@ -1)) isEmpty.! !

!MOQuadTreeTest methodsFor: 'tests' stamp: ''!
testMutation

	| a1 b1 q |
	q := MOQuadTree with: (0@0 corner: 30@30).
	self deny: q isComposite.
	
	a1 := nodes first.
	b1 := nodes second.
	q addElement: a1.
	q addElement: b1.
	q addElement: b1.
	q addElement: b1.
	q addElement: b1.
	q addElement: b1.
	q addElement: b1.
	q addElement: b1.
	self deny: q isComposite.

	q addElement: b1.
	self assert: q isComposite.

	q removeElement: a1.
	self deny: q isComposite.
! !

!MOQuadTreeTest methodsFor: 'tests' stamp: ''!
testQuadtreeGeometry

	self assert: (leafQuad contains: 1@1).
	self assert: (leafQuad contains: 4@7).
	self assert: (leafQuad contains: 9@9).
	self deny: (leafQuad contains: 0@0).
	self deny: (leafQuad contains: 11@10).
	
	self assert: (leafQuad intersects: (1@2 corner: 4@5)).
	self assert: (leafQuad intersects: (8@9 corner: 15@20)).
	self deny: (leafQuad intersects: (8@10 corner: 15@20)).
	self deny: (leafQuad intersects: (-1@ -1 corner: 1@1)).! !

TestCase subclass: #MondrianExporterSVGTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Tests'!

!MondrianExporterSVGTest methodsFor: 'tests' stamp: ''!
testBasic

	| view stream visitor |
	view := MOViewRenderer new.
	view shape rectangle fillColor: Color blue; size: 50.
	view nodes: (1 to: 4).
	view edges:  {1 -> 2 . 1 -> 3 . 1 -> 4} from: #key to: #value.
	view gridLayout gapSize: 40.
	view root applyLayout.
	
	stream := WriteStream on: String new.
	"visitor := MESVGVisitor stream: stream."
	visitor := MOSVGVisitor stream: stream.
	view root acceptVisitor: visitor.
	self assert: stream contents = '<svg  xmlns="http://www.w3.org/2000/svg"
	xmlns:xlink="http://www.w3.org/1999/xlink">
<rect x="40" y="40" height="50" width="50" style="stroke:rgb(0,0,0);fill:rgb(0,0,256)"/>
<line x1="90" y1="65" x2="170" y2="65" style="stroke:rgb(160,160,160);stroke-width:1"/>
<line x1="65" y1="90" x2="65" y2="170" style="stroke:rgb(160,160,160);stroke-width:1"/>
<line x1="65" y1="90" x2="195" y2="170" style="stroke:rgb(160,160,160);stroke-width:1"/>
<rect x="170" y="40" height="50" width="50" style="stroke:rgb(0,0,0);fill:rgb(0,0,256)"/>
<rect x="40" y="170" height="50" width="50" style="stroke:rgb(0,0,0);fill:rgb(0,0,256)"/>
<rect x="170" y="170" height="50" width="50" style="stroke:rgb(0,0,0);fill:rgb(0,0,256)"/>
</svg>'! !

!MondrianExporterSVGTest methodsFor: 'tests' stamp: ''!
testBasic2

	| view stream visitor |
	view := MOViewRenderer new.
	view shape rectangle fillColor: Color blue; size: [ :v | v * 10 ].
	view nodes: (1 to: 4).
	view edges:  {1 -> 2 . 1 -> 3 . 1 -> 4} from: #key to: #value.
	view gridLayout gapSize: 40.
	view root applyLayout.
	
	stream := WriteStream on: String new.
	"visitor := MESVGVisitor stream: stream."
	visitor := MOSVGVisitor stream: stream.
	view root acceptVisitor: visitor.
	self assert: stream contents = '<svg  xmlns="http://www.w3.org/2000/svg"
	xmlns:xlink="http://www.w3.org/1999/xlink">
<rect x="40" y="40" height="10" width="10" style="stroke:rgb(0,0,0);fill:rgb(0,0,256)"/>
<line x1="50" y1="45" x2="130" y2="50" style="stroke:rgb(160,160,160);stroke-width:1"/>
<line x1="45" y1="50" x2="55" y2="140" style="stroke:rgb(160,160,160);stroke-width:1"/>
<line x1="45" y1="50" x2="170" y2="140" style="stroke:rgb(160,160,160);stroke-width:1"/>
<rect x="130" y="40" height="20" width="20" style="stroke:rgb(0,0,0);fill:rgb(0,0,256)"/>
<rect x="40" y="140" height="30" width="30" style="stroke:rgb(0,0,0);fill:rgb(0,0,256)"/>
<rect x="150" y="140" height="40" width="40" style="stroke:rgb(0,0,0);fill:rgb(0,0,256)"/>
</svg>'! !

!MondrianExporterSVGTest methodsFor: 'easel' stamp: ''!
testEaselCommand
	| commands |
	self assert: (MOEaselCommand allCommandClasses includes: MOEaselCommandSVGExporter).
	
	self assert: (MOEasel new allCommandClasses select: [ :c | c == MOEaselCommandSVGExporter]) size = 1.


	commands := MOEaselCommand commandsForEasel: MOEasel new.
	self assert: (commands select: [ :c | c class == MOEaselCommandSVGExporter]) size = 1! !

!MondrianExporterSVGTest methodsFor: 'easel' stamp: ''!
testEaselCommandIsNotAbstract
	self deny: (MOEaselCommandSVGExporter isAbstract)! !

!MondrianExporterSVGTest methodsFor: 'tests' stamp: ''!
testSubview

	| view stream visitor |
	view := MOViewRenderer new.

	view nodes: (1 to: 5) forEach: [:each |
		view nodes: (1 to: each).
		view gridLayout
	].

	view node: 'outter node' forIt: [
		view shape label.
		view node: 'inner node'
	].
	view root applyLayout.
		
	stream := WriteStream on: String new.
	"visitor := MESVGVisitor stream: stream."
	visitor := MOSVGVisitor stream: stream.
	view root acceptVisitor: visitor.
	self assert: stream contents = '<svg  xmlns="http://www.w3.org/2000/svg"
	xmlns:xlink="http://www.w3.org/1999/xlink">
<rect x="5" y="5" height="15" width="15" style="stroke:rgb(0,0,0);fill:rgb(256,256,256)"/>
<rect x="10" y="10" height="5" width="5" style="stroke:rgb(0,0,0);fill:rgb(256,256,256)"/>
<rect x="30" y="5" height="15" width="30" style="stroke:rgb(0,0,0);fill:rgb(256,256,256)"/>
<rect x="35" y="10" height="5" width="5" style="stroke:rgb(0,0,0);fill:rgb(256,256,256)"/>
<rect x="50" y="10" height="5" width="5" style="stroke:rgb(0,0,0);fill:rgb(256,256,256)"/>
<rect x="70" y="5" height="30" width="30" style="stroke:rgb(0,0,0);fill:rgb(256,256,256)"/>
<rect x="75" y="10" height="5" width="5" style="stroke:rgb(0,0,0);fill:rgb(256,256,256)"/>
<rect x="90" y="10" height="5" width="5" style="stroke:rgb(0,0,0);fill:rgb(256,256,256)"/>
<rect x="75" y="25" height="5" width="5" style="stroke:rgb(0,0,0);fill:rgb(256,256,256)"/>
<rect x="110" y="5" height="30" width="30" style="stroke:rgb(0,0,0);fill:rgb(256,256,256)"/>
<rect x="115" y="10" height="5" width="5" style="stroke:rgb(0,0,0);fill:rgb(256,256,256)"/>
<rect x="130" y="10" height="5" width="5" style="stroke:rgb(0,0,0);fill:rgb(256,256,256)"/>
<rect x="115" y="25" height="5" width="5" style="stroke:rgb(0,0,0);fill:rgb(256,256,256)"/>
<rect x="130" y="25" height="5" width="5" style="stroke:rgb(0,0,0);fill:rgb(256,256,256)"/>
<rect x="150" y="5" height="30" width="45" style="stroke:rgb(0,0,0);fill:rgb(256,256,256)"/>
<rect x="155" y="10" height="5" width="5" style="stroke:rgb(0,0,0);fill:rgb(256,256,256)"/>
<rect x="170" y="10" height="5" width="5" style="stroke:rgb(0,0,0);fill:rgb(256,256,256)"/>
<rect x="185" y="10" height="5" width="5" style="stroke:rgb(0,0,0);fill:rgb(256,256,256)"/>
<rect x="155" y="25" height="5" width="5" style="stroke:rgb(0,0,0);fill:rgb(256,256,256)"/>
<rect x="170" y="25" height="5" width="5" style="stroke:rgb(0,0,0);fill:rgb(256,256,256)"/>
<rect x="205" y="5" height="23" width="67" style="stroke:rgb(0,0,0);fill:rgb(256,256,256)"/>
<rect x="210" y="10" height="13" width="57" style="stroke:rgb(256,256,256);fill:rgb(256,256,256)"/>
<text x="238" y="16" text-anchor="middle" dominant-baseline="mathematical" style="font-family:''Bitmap DejaVu Sans'';font-size:11.0">inner node</text>
</svg>'! !

!MondrianExporterSVGTest methodsFor: 'tests' stamp: ''!
testSubview2

	| view stream visitor |
	view := MOViewRenderer new.

	view node: 1 forIt: [
		view node: 1 forIt: [
			view node: 1 forIt: []]].

	view root applyLayout.
		
	stream := WriteStream on: String new.
	"visitor := MESVGVisitor stream: stream."
	visitor := MOSVGVisitor stream: stream.
	view root acceptVisitor: visitor.
	self assert: stream contents =  '<svg  xmlns="http://www.w3.org/2000/svg"
	xmlns:xlink="http://www.w3.org/1999/xlink">
<rect x="5" y="5" height="25" width="25" style="stroke:rgb(0,0,0);fill:rgb(256,256,256)"/>
<rect x="10" y="10" height="15" width="15" style="stroke:rgb(0,0,0);fill:rgb(256,256,256)"/>
<rect x="15" y="15" height="5" width="5" style="stroke:rgb(0,0,0);fill:rgb(256,256,256)"/>
</svg>'! !

!MondrianExporterSVGTest methodsFor: 'tests' stamp: ''!
testSubview3

	| view stream visitor |
	view := MOViewRenderer new.

	view node: 'hello' forIt: [
		view shape rectangle fillColor: Color blue; size: 50.
		view nodes: (1 to: 4).
		view edges:  {1 -> 2 . 1 -> 3 . 1 -> 4} from: #key to: #value.
		view gridLayout gapSize: 40.
	].

	view root applyLayout.
		
	stream := WriteStream on: String new.
	"visitor := MESVGVisitor stream: stream."
	visitor := MOSVGVisitor stream: stream.
	view root acceptVisitor: visitor.
	self assert: stream contents = '<svg  xmlns="http://www.w3.org/2000/svg"
	xmlns:xlink="http://www.w3.org/1999/xlink">
<rect x="5" y="5" height="260" width="260" style="stroke:rgb(0,0,0);fill:rgb(256,256,256)"/>
<rect x="45" y="45" height="50" width="50" style="stroke:rgb(0,0,0);fill:rgb(0,0,256)"/>
<line x1="95" y1="70" x2="175" y2="70" style="stroke:rgb(160,160,160);stroke-width:1"/>
<line x1="70" y1="95" x2="70" y2="175" style="stroke:rgb(160,160,160);stroke-width:1"/>
<line x1="70" y1="95" x2="200" y2="175" style="stroke:rgb(160,160,160);stroke-width:1"/>
<rect x="175" y="45" height="50" width="50" style="stroke:rgb(0,0,0);fill:rgb(0,0,256)"/>
<rect x="45" y="175" height="50" width="50" style="stroke:rgb(0,0,0);fill:rgb(0,0,256)"/>
<rect x="175" y="175" height="50" width="50" style="stroke:rgb(0,0,0);fill:rgb(0,0,256)"/>
</svg>'! !

TestCase subclass: #MondrianExporterTiKZTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Tests'!

!MondrianExporterTiKZTest methodsFor: 'tests' stamp: ''!
testBasic2TiKZ

	| view stream visitor |
	view := MOViewRenderer new.
	view shape rectangle fillColor: Color blue; size: [ :v | v * 10 ].
	view nodes: (1 to: 4).
	view edges:  {1 -> 2 . 1 -> 3 . 1 -> 4} from: #key to: #value.
	view gridLayout gapSize: 40.
	view root applyLayout.
	
	stream := WriteStream on: String new.
	view canvas exportAsTikzOn: stream.
	self assert: stream contents = '\begin{tikzpicture}[
	yscale=-1,
	node/.style={},
	print/.style={},
	edge/.style={}]
	\draw [draw={rgb,265:red,165;green,165;blue,165}, line width=0.02, edge] (2.6, 1.0) -- (1.0, 0.9);
	\draw [draw={rgb,265:red,165;green,165;blue,165}, line width=0.02, edge] (1.1, 2.8000000000000003) -- (0.9, 1.0);
	\draw [draw={rgb,265:red,165;green,165;blue,165}, line width=0.02, edge] (3.4, 2.8000000000000003) -- (0.9, 1.0);
	\path [fill={rgb,265:red,0;green,0;blue,265}, draw={rgb,265:red,0;green,0;blue,0}, line width=0.02, node] (0.8, 0.8) rectangle (1.0, 1.0);
	\path [fill={rgb,265:red,0;green,0;blue,265}, draw={rgb,265:red,0;green,0;blue,0}, line width=0.02, node] (2.6, 0.8) rectangle (3.0, 1.2);
	\path [fill={rgb,265:red,0;green,0;blue,265}, draw={rgb,265:red,0;green,0;blue,0}, line width=0.02, node] (0.8, 2.8000000000000003) rectangle (1.4000000000000001, 3.4);
	\path [fill={rgb,265:red,0;green,0;blue,265}, draw={rgb,265:red,0;green,0;blue,0}, line width=0.02, node] (3.0, 2.8000000000000003) rectangle (3.8000000000000003, 3.6);
\end{tikzpicture}
'! !

!MondrianExporterTiKZTest methodsFor: 'tests' stamp: ''!
testBasicTiKZ

	| view stream visitor |
	view := MOViewRenderer new.
	view shape rectangle fillColor: Color blue; size: 50.
	view nodes: (1 to: 4).
	view edges:  {1 -> 2 . 1 -> 3 . 1 -> 4} from: #key to: #value.
	view gridLayout gapSize: 40.
	view root applyLayout.

	stream := WriteStream on: String new.
	view canvas exportAsTikzOn: stream.
	self assert: stream contents = '\begin{tikzpicture}[
	yscale=-1,
	node/.style={},
	print/.style={},
	edge/.style={}]
	\draw [draw={rgb,265:red,165;green,165;blue,165}, line width=0.02, edge] (3.4, 1.3) -- (1.8, 1.3);
	\draw [draw={rgb,265:red,165;green,165;blue,165}, line width=0.02, edge] (1.3, 3.4) -- (1.3, 1.8);
	\draw [draw={rgb,265:red,165;green,165;blue,165}, line width=0.02, edge] (3.9, 3.4) -- (1.3, 1.8);
	\path [fill={rgb,265:red,0;green,0;blue,265}, draw={rgb,265:red,0;green,0;blue,0}, line width=0.02, node] (0.8, 0.8) rectangle (1.8, 1.8);
	\path [fill={rgb,265:red,0;green,0;blue,265}, draw={rgb,265:red,0;green,0;blue,0}, line width=0.02, node] (3.4, 0.8) rectangle (4.4, 1.8);
	\path [fill={rgb,265:red,0;green,0;blue,265}, draw={rgb,265:red,0;green,0;blue,0}, line width=0.02, node] (0.8, 3.4) rectangle (1.8, 4.4);
	\path [fill={rgb,265:red,0;green,0;blue,265}, draw={rgb,265:red,0;green,0;blue,0}, line width=0.02, node] (3.4, 3.4) rectangle (4.4, 4.4);
\end{tikzpicture}
'! !

!MondrianExporterTiKZTest methodsFor: 'tests' stamp: ''!
testSubviewTiKZ

	| view stream visitor |
	view := MOViewRenderer new.

	view nodes: (1 to: 5) forEach: [:each |
		view nodes: (1 to: each).
		view gridLayout
	].

	view node: 'outter node' forIt: [
		view shape label.
		view node: 'inner node'
	].
	view root applyLayout.
		
	stream := WriteStream on: String new.
	view canvas exportAsTikzOn: stream.
	self assert: stream contents =  '\begin{tikzpicture}[
	yscale=-1,
	node/.style={},
	print/.style={},
	edge/.style={}]
	\path [fill={rgb,265:red,265;green,265;blue,265}, draw={rgb,265:red,0;green,0;blue,0}, line width=0.02, node] (0.1, 0.1) rectangle (0.4, 0.4);
	\path [fill={rgb,265:red,265;green,265;blue,265}, draw={rgb,265:red,0;green,0;blue,0}, line width=0.02, node] (0.2, 0.2) rectangle (0.3, 0.3);
	\path [fill={rgb,265:red,265;green,265;blue,265}, draw={rgb,265:red,0;green,0;blue,0}, line width=0.02, node] (0.6, 0.1) rectangle (1.2, 0.4);
	\path [fill={rgb,265:red,265;green,265;blue,265}, draw={rgb,265:red,0;green,0;blue,0}, line width=0.02, node] (0.7000000000000001, 0.2) rectangle (0.8, 0.3);
	\path [fill={rgb,265:red,265;green,265;blue,265}, draw={rgb,265:red,0;green,0;blue,0}, line width=0.02, node] (1.0, 0.2) rectangle (1.1, 0.3);
	\path [fill={rgb,265:red,265;green,265;blue,265}, draw={rgb,265:red,0;green,0;blue,0}, line width=0.02, node] (1.4000000000000001, 0.1) rectangle (2.0, 0.7000000000000001);
	\path [fill={rgb,265:red,265;green,265;blue,265}, draw={rgb,265:red,0;green,0;blue,0}, line width=0.02, node] (1.5, 0.2) rectangle (1.6, 0.3);
	\path [fill={rgb,265:red,265;green,265;blue,265}, draw={rgb,265:red,0;green,0;blue,0}, line width=0.02, node] (1.8, 0.2) rectangle (1.9000000000000001, 0.3);
	\path [fill={rgb,265:red,265;green,265;blue,265}, draw={rgb,265:red,0;green,0;blue,0}, line width=0.02, node] (1.5, 0.5) rectangle (1.6, 0.6);
	\path [fill={rgb,265:red,265;green,265;blue,265}, draw={rgb,265:red,0;green,0;blue,0}, line width=0.02, node] (2.2, 0.1) rectangle (2.8000000000000003, 0.7000000000000001);
	\path [fill={rgb,265:red,265;green,265;blue,265}, draw={rgb,265:red,0;green,0;blue,0}, line width=0.02, node] (2.3000000000000003, 0.2) rectangle (2.4, 0.3);
	\path [fill={rgb,265:red,265;green,265;blue,265}, draw={rgb,265:red,0;green,0;blue,0}, line width=0.02, node] (2.6, 0.2) rectangle (2.7, 0.3);
	\path [fill={rgb,265:red,265;green,265;blue,265}, draw={rgb,265:red,0;green,0;blue,0}, line width=0.02, node] (2.3000000000000003, 0.5) rectangle (2.4, 0.6);
	\path [fill={rgb,265:red,265;green,265;blue,265}, draw={rgb,265:red,0;green,0;blue,0}, line width=0.02, node] (2.6, 0.5) rectangle (2.7, 0.6);
	\path [fill={rgb,265:red,265;green,265;blue,265}, draw={rgb,265:red,0;green,0;blue,0}, line width=0.02, node] (3.0, 0.1) rectangle (3.9, 0.7000000000000001);
	\path [fill={rgb,265:red,265;green,265;blue,265}, draw={rgb,265:red,0;green,0;blue,0}, line width=0.02, node] (3.1, 0.2) rectangle (3.2, 0.3);
	\path [fill={rgb,265:red,265;green,265;blue,265}, draw={rgb,265:red,0;green,0;blue,0}, line width=0.02, node] (3.4, 0.2) rectangle (3.5, 0.3);
	\path [fill={rgb,265:red,265;green,265;blue,265}, draw={rgb,265:red,0;green,0;blue,0}, line width=0.02, node] (3.7, 0.2) rectangle (3.8000000000000003, 0.3);
	\path [fill={rgb,265:red,265;green,265;blue,265}, draw={rgb,265:red,0;green,0;blue,0}, line width=0.02, node] (3.1, 0.5) rectangle (3.2, 0.6);
	\path [fill={rgb,265:red,265;green,265;blue,265}, draw={rgb,265:red,0;green,0;blue,0}, line width=0.02, node] (3.4, 0.5) rectangle (3.5, 0.6);
	\path [fill={rgb,265:red,265;green,265;blue,265}, draw={rgb,265:red,0;green,0;blue,0}, line width=0.02, node] (4.1, 0.1) rectangle (5.44, 0.56);
	\path [fill={rgb,265:red,265;green,265;blue,265}, node] (4.2, 0.2) rectangle (5.34, 0.46);
	\node [anchor=north west, align=center, text={rgb,265:red,0;green,0;blue,0}, text width=1.1400000000000001, text height=0.26, print] at (4.2, 0.2) {inner node};
\end{tikzpicture}
'! !

TestCase subclass: #MondrianTest
	instanceVariableNames: 'window'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Tests'!

MondrianTest subclass: #MOEventHandlerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Tests'!

!MOEventHandlerTest methodsFor: 'announcer' stamp: ''!
testAnnouncer
	| announcer block |
	announcer := MOAnnouncer new.
	self deny: (announcer hasHandlerFor: MOMouseEnter).
	
	announcer on: MOMouseEnter do: (block := [:ann | 'hello' ]).
	self assert: (announcer lookFor: MOMouseEnter) == block.
	self assert: (announcer hasHandlerFor: MOMouseEnter).

	"nothing should change with the unsubscribe: since MOMouseEnter is not a subcriber"
	announcer unsubscribe: MOMouseEnter.
	self assert: (announcer lookFor: MOMouseEnter) == block.
	self assert: (announcer hasHandlerFor: MOMouseEnter).

	announcer unsubscribeForEvent: MOMouseEnter.
	self deny: (announcer hasHandlerFor: MOMouseEnter).
	self assert: (announcer lookFor: MOMouseEnter) isNil! !

!MOEventHandlerTest methodsFor: 'edges' stamp: ''!
testEdgeOnClick
	
	| boolean view |
	boolean := false.
	view := MOViewRenderer new.

	view nodes: #(1 2).

	view interaction subscribe: MOMouseDown do: [ :announcement | boolean := true].
	view edges: {1 -> 2} from: #key to: #value.
	
	view announce: (MOMouseDown event: nil on: view root edges first).
	
	self assert: boolean.
! !

!MOEventHandlerTest methodsFor: 'nodes' stamp: ''!
testEventToTheRoot
	"The subscription made between a 'nodes:' and the 'open', are for the root. Currently, the announcer (event handler) is installed on the root only when open is sent."
	
	| view sum ui |
	view := MOViewRenderer new.
	sum := 0.
	view interaction on: MOMouseEnter do: [:ann | sum := sum + 10].
	view nodes: (1 to: 10).
	view root interaction on: MOMouseEnter do: [:ann | sum := sum + 1].
	ui := view open.

	view announce: (MOMouseEnter event: nil on: view root).
	self assert: (sum = 1).
	
	ui delete! !

!MOEventHandlerTest methodsFor: 'nodes' stamp: ''!
testEventToTheRoot2
	"The subscription made between a 'nodes:' and the 'open', are for the root. Currently, the announcer (event handler) is installed on the root only when open is sent."
	
	| view sum ui |
	view := MOViewRenderer new.
	sum := 0.
	view interaction on: MOMouseEnter do: [:ann | sum := sum + 10].
	view nodes: (1 to: 10).
	view resetInteraction.
	view root interaction on: MOMouseEnter do: [:ann | sum := sum + 1].
	ui := view open.
	
	view announce: (MOMouseEnter event: nil on: view root).
	self assert: (sum = 1).
	
	ui delete! !

!MOEventHandlerTest methodsFor: 'forwarder' stamp: ''!
testForward

	| view node1 node2 |
	view := MOViewRenderer new.
	node1 := view node: 10 forIt: [
		view shape rectangle size: 30.
		view interaction forward: MOElementSelection.
		node2 := view node: 20
	].
	
	self deny: (node1 isSelected).
	self deny: (node2 isSelected).
	
	node2 announce: (MOElementSelection new).
	self assert: (node1 isSelected).
	self deny: (node2 isSelected).
	! !

!MOEventHandlerTest methodsFor: 'forwarder' stamp: ''!
testForwarder
	| value view bigNode innerNode |
	value := 0.
	view := MOViewRenderer new.

	"No forwarder"
	view interaction subscribe: MOMouseDown do: [ :announcement | value := value + 10].
	bigNode := view node: #abc using: MORectangleShape new forIt: [
		view interaction subscribe: MOMouseDown do: [ :announcement | value := value + 1].
		innerNode := view node: #foo].	
	self assert: (bigNode model = #abc).
	self assert: (innerNode model = #foo).
	view announce: (MOMouseDown event: nil on: innerNode).
	self assert: (value = 1).
	value := 0.
	
	"With forwarder"
	view interaction subscribe: MOMouseDown do: [ :announcement | value := value + 10].
	bigNode := view node: #abc using: MORectangleShape new forIt: [
		view interaction forward.
		view interaction subscribe: MOMouseDown do: [ :announcement | value := value + 1].
		innerNode := view node: #foo].	
	self assert: (bigNode model = #abc).
	self assert: (innerNode model = #foo).
	view announce: (MOMouseDown event: nil on: innerNode).
	self assert: (value = 10).! !

!MOEventHandlerTest methodsFor: 'forwarder' stamp: ''!
testForwarderAndDraggable
	| view outerNode innerNode |
	view := MOViewRenderer new.

	view shape rectangle size: 90.
	outerNode := view node: #outer forIt: [
		view interaction forward.
		view shape rectangle size: 45.
		innerNode := view node: #inner].
	
	window := view open.
	self assert: (outerNode model = #outer).
	self assert: (innerNode model = #inner).
	
	self assert: (outerNode bounds topLeft = (5@5)).
	view announce: ((MOMouseDrag event: nil on: outerNode) step: 2 @ 1).
	self assert: (outerNode bounds topLeft = (7@6)).
	self assert: (innerNode bounds topLeft = (5@5)).
	
	view announce: ((MOMouseDrag event: nil on: innerNode) step: 2 @ 1).
	self assert: (outerNode bounds topLeft = (9@7)).
	self assert: (innerNode bounds topLeft = (5@5)).
! !

!MOEventHandlerTest methodsFor: 'forwarder' stamp: ''!
testForwarderAndElementSelection
	| view outerNode innerNode |
	view := MOViewRenderer new.

	view shape rectangle size: 90.
	outerNode := view node: #outer forIt: [
		view interaction forward.
		view shape rectangle size: 45.
		innerNode := view node: #inner].
	
	window := view open.
	
	self assert: (outerNode bounds topLeft = (5@5)).
	self deny: (outerNode isSelected).
	self deny: (innerNode isSelected).

	view announce: (MOElementSelection event: nil on: innerNode).
	self assert: (outerNode isSelected).
	self deny: innerNode isSelected! !

!MOEventHandlerTest methodsFor: 'announcer' stamp: ''!
testHasHandlerFor
	| announcer t |

	announcer := MOAnnouncer new.
	self deny: (announcer hasHandlerFor: MOMouseEnter).

	t := OrderedCollection new.	
	announcer on: MOMouseEnter do: [ :ann | t add: ann ].

	self assert: t isEmpty.
	self assert: (announcer hasHandlerFor: MOMouseEnter).
	
	announcer announce: MOMouseEnter new.
	self assert: t size = 1.
	self assert: (announcer hasHandlerFor: MOMouseEnter).

	announcer announce: MOMouseEnter new.
	self assert: t size = 2.

	announcer unsubscribeForEvent: MOMouseEnter.
	announcer announce: MOMouseEnter new.
	self assert: t size = 2.
	
	announcer on: MOMouseEnter do: [ :ann | t add: ann ].
	announcer announce: MOMouseEnter new.
	self assert: t size = 3.
! !

!MOEventHandlerTest methodsFor: 'nodes' stamp: ''!
testMultipleAnnouncer
	
	| view sum nodeFive nodeTen |
	sum := 0.
	view := MOViewRenderer new.
	view interaction subscribe: MOMouseDown do: [ :ann | sum := sum + ann element model].
	view nodes: (1 to: 5).

	view interaction subscribe: MOMouseDown do: [ :ann | sum := sum + (2 * ann element model)].
	view nodes: (6 to: 10).

	window := view open.
	
	nodeFive := view root nodes detect: [:monode | monode model = 5].
	nodeTen := view root nodes detect: [:monode | monode model = 10].
	
	self assert: (sum = 0).
	view announce: (MOMouseDown event: nil on: nodeFive).
	self assert: (sum = 5).
	view announce: (MOMouseDown event: nil on: nodeTen).
	self assert: (sum = 25).


! !

!MOEventHandlerTest methodsFor: 'nodes' stamp: ''!
testMultipleAnnouncer2
	
	| view ui sum nodeFive nodeTen |
	sum := 0.
	view := MOViewRenderer new.
	view interaction subscribe: MOMouseDown do: [ :ann | sum := sum + ann element model].
	view nodes: (1 to: 5).

	view resetInteraction.
	view interaction subscribe: MOMouseDown do: [ :ann | sum := sum + (2 * ann element model)].
	view nodes: (6 to: 10).

	ui := view open.
	
	nodeFive := view root nodes detect: [:monode | monode model = 5].
	nodeTen := view root nodes detect: [:monode | monode model = 10].
	
	self assert: (sum = 0).
	view announce: (MOMouseDown event: nil on: nodeFive).
	self assert: (sum = 5).
	view announce: (MOMouseDown event: nil on: nodeTen).
	self assert: (sum = 25).

	ui delete
! !

!MOEventHandlerTest methodsFor: 'nodes' stamp: ''!
testOnClick
	
	| boolean view |
	boolean := false.
	view := MOViewRenderer new.

	view interaction subscribe: MOMouseDown do: [ :announcement | boolean := true].
"	view interaction: (MOEventHandler onClick: [ :anEntity | boolean := true])."

	view node: #abc using: MORectangleShape new.
	"figure redButtonPressedEvent."
	
	view announce: (MOMouseDown event: nil on: view root nodes first).
	
	self assert: boolean.! !

!MOEventHandlerTest methodsFor: 'nodes' stamp: ''!
testOnDoubleClick
	
	| boolean view figure |
	boolean := false.
	view := MOViewRenderer new.
	view announcer on: MOMouseDouble do: [ :ann | boolean := true].
	figure := view node: #abc using: MORectangleShape new.
	view announce: (MOMouseDouble event: nil on: figure).
	self assert: boolean.! !

!MOEventHandlerTest methodsFor: 'nodes' stamp: ''!
testOnSelect
	
	| boolean view figure ui |
	boolean := false.
	view := MOViewRenderer new.
	view interaction subscribe: MOMouseDown do: [ :ann | boolean := true].
	figure := view node: #abc using: MORectangleShape new.
	ui := view open.
	view announce: (MOMouseDown event: nil on: figure).
	self assert: boolean.

	"We reset"
	boolean := false.

	view gridLayout.	
	view announce: (MOMouseDown event: nil on: figure).
	self assert: boolean.
	
	ui delete
! !

!MOEventHandlerTest methodsFor: 'TODO' stamp: ''!
testOnUnselect
	"unselecting does not seems supported currently. Is that really used in practice ?"
"	
	| boolean view figure ui |
	boolean := false.
	view := MOViewRenderer new.
	view interaction: (MOEventHandler onUnselect: [ :anEntity | boolean := true]).
	figure := view node: #abc using: MORectangleShape new.
	ui := view open.
	figure select.
	figure deselect.
	self assert: boolean.
	ui delete."! !

!MOEventHandlerTest methodsFor: 'TODO' stamp: ''!
testSubview
	
	| view int nodes |
	view := MOViewRenderer new.
	view shape: (MORectangleShape new width: 36; height: 10).
	view interaction popupText: [:es | 'first level'].
	view node: (1) forIt: [
		view interaction popupText: [:e | '2nd level'].
		view nodes: (1 to: 5).
		view gridLayout gapSize:0].

	view shape: (MORectangleShape new width: 36; height: 10).
	view node: (2).

	window := view open.

	nodes := view root nodes.
		
	"The root has a different event handler than its nodes"
	self assert: (view interaction ~~ nodes first interaction).
	
	"There are two calls to node:, therefore these two nodes must have different event handlers"
	self assert: (view root nodes first interaction ~~ nodes second interaction).
	
	"All subnodes must have the same interaction"
	int := view root nodes first nodes first interaction.
	self assert: (nodes first nodes allSatisfy: [:each | each interaction == int]).
	
	"the subnodes and the second node cannot have the same event handler"
	self assert: (nodes second hasInteraction)! !

!MOEventHandlerTest methodsFor: 'TODO' stamp: ''!
testSubview2
	
	| view int nodes |
	view := MOViewRenderer new.
	view shape: (MORectangleShape new width: 36; height: 10).
	view interaction popupText: [:es | 'first level'].
	view node: (1) forIt: [
		view nodes: (1 to: 5).
		view gridLayout gapSize:0].

	view shape: (MORectangleShape new width: 36; height: 10).
	view node: (2).

	window := view open.
	
	"The root has a different event handler than its nodes"
	self assert: (view interaction ~~ view root nodes first interaction).
	
	"There are two calls to node:, therefore these two nodes must have different event handlers"
	self assert: (view root nodes first interaction ~~ view root nodes second interaction).
	
	"All subnodes must have the same interaction"
	int := view root nodes first nodes first interaction.
	self assert: (view root nodes first nodes allSatisfy: [:each | each interaction == int]).
	
	"the subnodes and the second node cannot have the same event handler"
	nodes := view root nodes.
	self assert: (nodes second interaction ~~ int)! !

!MOEventHandlerTest methodsFor: 'nodes' stamp: ''!
testSubviews
	| t view nodes |
	t := 0.
	
	view := MOViewRenderer new.
	view nodes: (1 to: 2) forEach: [:each |
		view interaction on: MOMouseDouble do: [:ann | t := t + 1].
		nodes := view nodes: (1 to: 100).
	view gridLayout].

	nodes third announce: (MOMouseDouble event: nil on: nodes third).
	self assert: (t = 1).! !

!MOEventHandlerTest methodsFor: 'TODO' stamp: ''!
testUseOnceEdges
	"
	| view handler figure |
	
	handler := EventHandler onClick: [:entity | 'Test'].

	view := MOViewRenderer new.
	view nodes: #(1 2) using: MORectangleShape withBorder.
	self assert: view interaction hasNoFunctionality.
	view interaction: handler.
	self assert: view interaction = handler.
	figure := view edge: #a from: 1 to: 2 using: MOLineShape new.
	self assert: view interaction hasNoFunctionality.
	view evaluateLazyEdges.
	self assert: figure eventHandler = handler."! !

!MOEventHandlerTest methodsFor: 'TODO' stamp: ''!
testUseOnceForEach
	"
	| view handlerA handlerB figureA figureB |
	
	handlerA := EventHandler onClick: [:entity | 'Test'].
	handlerB := EventHandler onClick: [:entity | 'Test'].

	view := MOViewRenderer new.
	self assert: view interaction hasNoFunctionality.
	view interaction: handlerA.
	self assert: view interaction == handlerA.
	figureA := view node: #abc using: MORectangleShape withBorder forIt: [
		view interaction: handlerB.
		figureB := view node: #abcd using: MORectangleShape withBorder.
	].
	self assert: figureA eventHandler = handlerA.
	self assert: figureB eventHandler = handlerB.
	self assert: view interaction hasNoFunctionality."! !

!MOEventHandlerTest methodsFor: 'TODO' stamp: ''!
testUseOnceForEachReloaded
	
"	| view handlerA figureA figureB |
	
	handlerA := MOEventHandler onClick: [:entity | 'Test'].

	view := MOViewRenderer new.
	self assert: view interaction hasNoFunctionality.
	view interaction: handlerA.
	self assert: view interaction == handlerA.
	figureA := view node: #abc using: MORectangleShape new forIt: [
		figureB := view node: #abcd using: MORectangleShape new.
	].
	self assert: figureA eventHandler == handlerA.
	self assert: figureB eventHandler hasNoFunctionality.
	self assert: view interaction hasNoFunctionality."! !

!MOEventHandlerTest methodsFor: 'TODO' stamp: ''!
testUseOnceNodes
	"Does this still makes sense with announcement?"
"	| view handler figure |
	
	handler := MOEventHandler onClick: [:entity | 'Test'].
	self deny: handler hasNoFunctionality.

	view := MOViewRenderer new.
	self assert: view interaction hasNoFunctionality.
	view interaction: handler.
	self assert: view interaction == handler.
	figure := view node: #abc using: MORectangleShape new.
	self assert: figure eventHandler == handler.
	self assert: view interaction hasNoFunctionality."! !

MondrianTest subclass: #MOExporterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Tests'!

!MOExporterTest methodsFor: 'tests' stamp: ''!
testExporter
	| view canvas filename |
	filename := 'hellotestExporter.png' .
	view := MOViewRenderer  new.
	view nodes: (1 to: 5).
	window := view open.
	canvas := view canvas.
	
	self assert: (canvas isKindOf: MOCanvas).
	self deny: (FileDirectory default fileExists: filename).
		
	canvas exportWith: PNGReadWriter fullname: filename.
	self assert: (FileDirectory default fileExists: filename).
	
	FileDirectory default deleteFileNamed: filename
	! !

MondrianTest subclass: #MOFormsBuilderTest
	instanceVariableNames: 'view'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Tests'!

!MOFormsBuilderTest methodsFor: 'testing' stamp: ''!
expectedFailures
	^  #(#testPxSize #testColumnRightResize #testRowCenterResize #testColumnLeftResize #testRowBottomResize #testPxReSize #testRowTopResize #testColumnCenterResize #testError)! !

!MOFormsBuilderTest methodsFor: 'running' stamp: ''!
setUp
	view := MOViewRenderer new! !

!MOFormsBuilderTest methodsFor: 'accessing shapes - tests' stamp: ''!
testAccessingShapes
	| builder s1 s2 s3 s4 |
	builder := MOFormsBuilder new.
	builder column; column; row; row.
	builder x: 1 y: 1 add: (s1 := MORectangleShape new).
	builder x: 1 y: 2 add: (s2 := MORectangleShape new).
	builder x: 2 y: 1 add: (s3 := MORectangleShape new).
	builder x: 2 y: 2 add: (s4 := MOChildrenShape new).

	self assert: (builder positionOf: s1) = (1@1).
	self assert: (builder positionOf: s2) =( 1@2).
	self assert: (builder positionOf: s3) = (2@1).
	self assert: (builder positionOf: s4) = (2@2).! !

!MOFormsBuilderTest methodsFor: 'advanced - api' stamp: ''!
testAdvancedApi
	
	| builder shape figure orangeRectangle |

	orangeRectangle := MORectangleShape width: 10 height: 15 fillColor: Color orange.
	builder := MOFormsBuilder new.

	builder 
		column; pref;
		column; fill; pref; grow;
		column; pref;
		column; fill; pref; grow;
		column; fill; pref; grow.
	builder 
		row; fill; pref; grow;
		row; fill; pref; grow;
		row; pref;
		row; pref.
	builder x: 1 y: 2 add: orangeRectangle.
	builder x: 2	 y: 1 h: 2 add: (MORectangleShape width: 30 height: 30 fillColor: Color red).
	builder x: 2 y: 3 w: 3 add: (MORectangleShape new fillColor: Color blue).
	builder x: 3	 y: 1 add: ((MORectangleShape new withoutBorder; withText) text: [:entity | 'Hello']).
	builder x: 4 y: 2 add: ((MORectangleShape new withoutBorder; withText) text: [:entity | 'World']).
	builder x: 4	 y: 4 add: (MORectangleShape new width: 24;	height: 24; fillColor: Color green).
	builder x: 5 y: 1 add: (MORectangleShape new width: 10; height: 15; fillColor: Color brown).
	shape := builder shape.
	
	figure := view node: 1 using: shape.

	window := view open.
	self assert: (shape boundsFor: figure ) origin = (5 @ 5).
! !

!MOFormsBuilderTest methodsFor: 'children shape - tests' stamp: ''!
testCenteredChildren
	| outterNodes innerNode  builder childrenShape firstOutterNode veryFirstInnerNode |

	view := MOViewRenderer new.
	view shape form: [:b |
		b column; pref; center;
				row; fill;
				row; fill.
		b x:1 y:1 add: (MORectangleShape new width: #yourself).
		b x:1 y:2 add: (childrenShape := MOChildrenShape new) ].
	outterNodes := view nodes: #(300 20) forEach: [ :each |
		view nodes: #(1 2 3 4) ].

	view root applyLayout.

	firstOutterNode := outterNodes first.	
	builder :=firstOutterNode shape builder.

	self assert: (childrenShape absoluteOriginForChildrenElement: firstOutterNode) = (125@10).
	self assert: (builder boundsOf: childrenShape for: firstOutterNode) = ( 120.0@5 corner: 180.0@20).

	veryFirstInnerNode := firstOutterNode nodes first.
	self assert: (builder boundsOf: childrenShape for: veryFirstInnerNode) = ( 0.0@5 corner: 5.0@10).

	self assert: (veryFirstInnerNode shape positionOfChildrenForOwner: veryFirstInnerNode) =  ( 125.0@10)..
	self assert: veryFirstInnerNode bounds =  ( 5@5 corner: 10@10).

	self assert: (veryFirstInnerNode computeChildrenOrigin) = ( 120.0@5).
	self assert: (veryFirstInnerNode shape absoluteBoundsFor: veryFirstInnerNode) = ( 130.0@15 corner: 135.0@20).
	self assert: veryFirstInnerNode absoluteBounds = ( 130.0@15 corner: 135.0@20).
	! !

!MOFormsBuilderTest methodsFor: 'children shape - tests' stamp: ''!
testChildrenAndBorderWidth
	| outterNode innerNode  builder childrenShape formWithBug formExpected shape1 |
	view := MOViewRenderer new.
	view shape form: [ :b |
		b column; pref; fill; row; pref; fill; row; pref; fill.
		b x:1 y:1  add: (shape1 := MORectangleShape new fillColor: Color green; height: 20; text: '').
		b x:1 y:2 add: MOChildrenShape new.
	].
	outterNode := view node: 1 forIt: [
		view shape rectangle fillColor: Color red; borderColor: Color blue; borderWidth: 10; size:  90.
		innerNode := view node: 100].

	self deny: (innerNode hasCacheNamed:  #cacheborderWidthFor: of:  innerNode shape) = -1..
	view root applyLayout.
	self deny: (innerNode hasCacheNamed:  #cacheborderWidthFor: of:  innerNode shape) = -1..


	formWithBug := view getRenderedForm.
	
	self assert: innerNode shape width = 90.
	self assert: innerNode shape borderWidth = 10.
	self assert: (innerNode shape computeBorderWidthFor: innerNode) = 10.

	self assert: (innerNode cacheNamed: #cacheborderWidthFor:  of: innerNode shape ifAbsentInitializeWith: [ -1 ]) = 10..
	self assert: (innerNode cacheNamed: #cacheborderWidthFor:  of: shape1 ifAbsentInitializeWith: [ -1 ]) = -1.

	self assert: (innerNode shape borderWidthFor: innerNode) = 10.

	"****************"
	"We compare the bitmap"
	view := MOViewRenderer new.
	view shape form: [ :b |
		b column; pref; fill; row; pref; fill; row; pref; fill.
		b x:1 y:1  add: (MORectangleShape new fillColor: Color green; height: 20; text: '').
		b x:1 y:2 add: MOChildrenShape new.
	].
	outterNode := view node: 1 forIt: [
		view shape rectangle fillColor: Color red; borderColor: Color blue; borderWidth: 10; size:  90.
		innerNode := view node: 100].
	view root applyLayout.
	formExpected := view getRenderedForm.
	
	self assert: (formExpected bits = formWithBug bits).
	
	"
	formWithBug asMorph openInWindow
	formExpected asMorph openInWindow
	"! !

!MOFormsBuilderTest methodsFor: 'children shape - tests' stamp: ''!
testChildrenShape
	| outterNode innerNode builderOutter builderInner outterShape1 outterShape2 outterChildrenShape innerShape1 innerShape2 |
	view shape 
		form: [:b | b column; fill; row; fill; row; fill; row; fill. 
						b x: 1 y: 1 add: (outterShape1 := MORectangleShape new width: 80; height: 50).
						b x: 1 y: 2 add: (outterShape2 :=MORectangleShape new width: 80; height: 100).
						b x: 1 y: 2 add: (outterChildrenShape := MOChildrenShape new) ].

	outterNode := view node: 'outter node' forIt: [
		view shape 
			form: [:b | b column; fill; row; fill; row; fill; row; fill. 
							b x:1 y:1 add: (innerShape1 := MORectangleShape new width: 20; height: 10).
							b x:1 y:2 add: (innerShape2 :=MORectangleShape new width: 20; height: 20) ].
		innerNode := view node: 'hello world' ].

	view root applyLayout.

	builderOutter := outterNode shape builder.
	builderInner := innerNode shape builder.
	self assert: builderOutter notNil.
	self assert: builderInner notNil.
	self assert: builderOutter ~~ builderInner.

	self assert: (builderOutter boundsOf: outterShape1 for: outterNode ) = ( 0@0 corner: 80@50). 
	self assert: (builderOutter boundsOf: outterShape2 for: outterNode ) = ( 0@50 corner: 80@150). 

	self assert: (builderInner boundsOf: innerShape1 for: innerNode ) = ( 0@0 corner: 20@10). 
	self assert: (builderInner boundsOf: innerShape2 for: innerNode ) = ( 0@10 corner: 20@30). 

	self assert: (outterChildrenShape absoluteOriginForChildrenElement: outterNode) = ( 5@ 55).

	"
	view getRenderedForm.
	"
	"self assert: (outterChildrenShape originForChildrenElement: innerNode) = 0."


	self assert: (innerNode shape positionOfChildrenForOwner: innerNode) =  ( 5@ 55)..
	self assert: innerNode bounds =  ( 5@5 corner: 25@35).
	self assert: (innerNode shape absoluteBoundsFor: innerNode) = ( 10@60 corner: 30@90).
	self assert: innerNode absoluteBounds = ( 10@60 corner: 30@90).
	! !

!MOFormsBuilderTest methodsFor: 'children shape - tests' stamp: ''!
testChildrenShape2
	| outterNode innerNode builderOutter builderInner outterShape1 outterShape2 outterChildrenShape innerShape1 innerShape2 |
	view shape 
		form: [:b | b column; fill; row; fill; row; fill; row; fill. 
						b x: 1 y: 1 add: (outterShape1 := MORectangleShape new width: 80; height: 50).
						b x: 1 y: 2 add: (outterShape2 :=MORectangleShape new width: 80; height: 100).
						b x: 1 y: 1 add: (outterChildrenShape := MOChildrenShape new) ].

	outterNode := view node: 'outter node' forIt: [
		view shape 
			form: [:b | b column; fill; row; fill; row; fill; row; fill. 
							b x:1 y:1 add: (innerShape1 := MORectangleShape new width: 20; height: 10).
							b x:1 y:2 add: (innerShape2 :=MORectangleShape new width: 20; height: 20) ].
		innerNode := view node: 'hello world' ].

	view root applyLayout.

	builderOutter := outterNode shape builder.
	builderInner := innerNode shape builder.
	self assert: builderOutter notNil.
	self assert: builderInner notNil.
	self assert: builderOutter ~~ builderInner.

	self assert: (builderOutter boundsOf: outterShape1 for: outterNode ) = ( 0@0 corner: 80@50). 
	self assert: (builderOutter boundsOf: outterShape2 for: outterNode ) = ( 0@50 corner: 80@150). 

	self assert: (builderInner boundsOf: innerShape1 for: innerNode ) = ( 0@0 corner: 20@10). 
	self assert: (builderInner boundsOf: innerShape2 for: innerNode ) = ( 0@10 corner: 20@30). 

	self assert: (outterChildrenShape absoluteOriginForChildrenElement: outterNode) = ( 5@ 5).

	"
	view getRenderedForm.
	"
	"self assert: (outterChildrenShape originForChildrenElement: innerNode) = 0."


	self assert: (innerNode shape positionOfChildrenForOwner: innerNode) =  ( 5@ 5)..
	self assert: innerNode bounds =  ( 5@5 corner: 25@35).
	self assert: (innerNode shape absoluteBoundsFor: innerNode) = ( 10@10 corner: 30@40).
	self assert: innerNode absoluteBounds = ( 10@10 corner: 30@40).
	! !

!MOFormsBuilderTest methodsFor: 'alignment - tests' stamp: ''!
testColumnCenter
	
	| builder completeShape figure centeredRectangle |
	centeredRectangle := MORectangleShape width: 10 height: 10 fillColor: Color blue.
	builder := MOFormsBuilder new.
	builder column; center; pref.
	builder 
		row; pref; 
		row; pref.
	builder x: 1 y: 1 add: centeredRectangle.
	builder x: 1 y: 2 add: (MORectangleShape
							width: 30
							height: 10
							fillColor: Color red).
	completeShape := builder shape.
	figure := view node: #node using: completeShape.

	"I do not understand what this line is here for"
"	builder asLayout doExecute: figure."

	window := view open.
	self assert: (completeShape boundsFor: figure) = (Rectangle
						origin: 5 @ 5
						width: 30
						height: 20).
	self assert: (centeredRectangle boundsFor: figure) = (Rectangle
						origin: 15 @ 5
						width: 10
						height: 10).
! !

!MOFormsBuilderTest methodsFor: 'alignment - tests' stamp: ''!
testColumnCenterResize
	
	| builder shape element rectangle |
	rectangle := MORectangleShape
		width: 10
		height: 10
		fillColor: Color orange.
	builder := MOFormsBuilder new.
	builder column; center; pref; grow.
	builder 
		row; pref;
		row; pref.
	builder x: 1 y: 1 add: rectangle.
	builder
		x: 1
		y: 2
		add:
			(MORectangleShape
				width: 30
				height: 10
				fillColor: Color red).
	shape := builder shape.
	element := view node: #node using: shape.
	window := view open.
	element
		setBoundsTo:
			(Rectangle
				origin: 5 @ 5
				width: 50
				height: 20).
	self
		assert:
			(element shapeBoundsAt: shape)
				=
					(Rectangle
						origin: 5 @ 5
						width: 50
						height: 20).
	self
		assert:
			(element shapeBoundsAt: rectangle)
				=
					(Rectangle
						origin: 25 @ 5
						width: 10
						height: 10).
! !

!MOFormsBuilderTest methodsFor: 'alignment - tests' stamp: ''!
testColumnFill
	
	| builder ui shape figure rectangle |
	rectangle := MORectangleShape
		width: 10
		height: 10
		fillColor: Color orange.
	builder := MOFormsBuilder new.
	builder column; fill; pref.
	builder 
		row; pref;
	 	row; pref.
	builder x: 1 y: 1 add: rectangle.
	builder
		x: 1
		y: 2
		add:
			(MORectangleShape
				width: 30
				height: 10
				fillColor: Color red).
	shape := builder shape.
	figure := view node: #node using: shape.
	ui := view open.
	self
		assert:
			(shape boundsFor: figure )
				=
					(Rectangle
						origin: 5 @ 5
						width: 30
						height: 20).
	self
		assert:
			(rectangle boundsFor: figure )
				=
					(Rectangle
						origin: 5 @ 5
						width: 30
						height: 10).
	ui delete! !

!MOFormsBuilderTest methodsFor: 'alignment - tests' stamp: ''!
testColumnFillResize
	
	| builder shape figure rectangle |
	rectangle := MORectangleShape
		width: 10
		height: 10
		fillColor: Color orange.
	builder := MOFormsBuilder new.
	builder column; fill; pref; grow.
	builder 
		row; pref;
		row; pref.
	builder x: 1 y: 1 add: rectangle.
	builder
		x: 1
		y: 2
		add:
			(MORectangleShape
				width: 30
				height: 10
				fillColor: Color red).
	shape := builder shape.
	figure := view node: #node using: shape.
	window := view open.
	figure
		setBoundsTo:
			(Rectangle
				origin: 5 @ 5
				width: 50
				height: 20).
	self
		assert:
			(shape boundsFor: figure )
				=
					(Rectangle
						origin: 5 @ 5
						width: 50
						height: 20).
	self
		assert:
			(rectangle boundsFor: figure )
				=
					(Rectangle
						origin: 5 @ 5
						width: 50
						height: 10).! !

!MOFormsBuilderTest methodsFor: 'alignment - tests' stamp: ''!
testColumnLeft
	| builder shape figure smallRectangle largeRectangle |
	smallRectangle := MORectangleShape width: 10 height: 10 fillColor: Color orange.
	largeRectangle := (MORectangleShape width: 30 height: 10 fillColor: Color red).
	builder := MOFormsBuilder new.
	builder
		column; left; pref.
	builder row; pref; row; pref.
	builder x: 1 y: 1 add: smallRectangle.
	builder x: 1 y: 2 add: largeRectangle.
	shape := builder shape.
	figure := view node: #node using: shape.
	window := view open.
	self assert: (shape boundsFor: figure) = (Rectangle 
				origin: 5 @ 5
				extent: 30 @ 20).
	self assert: (smallRectangle boundsFor: figure) = (Rectangle 
				origin: 5 @ 5
				extent: 10 @ 10).
	self assert: (largeRectangle boundsFor: figure) = (Rectangle 
				origin: 5 @ 15
				extent: 30 @ 10).
! !

!MOFormsBuilderTest methodsFor: 'alignment - tests' stamp: ''!
testColumnLeftResize
	
	| builder shape figure rectangle |
	rectangle := MORectangleShape
		width: 10
		height: 10
		fillColor: Color orange.
	builder := MOFormsBuilder new.
	builder 
		column; 
			left; 
			pref; 
			grow.
		builder 
		row; pref;
		row; pref.
	builder x: 1 y: 1 add: rectangle.
	builder
		x: 1
		y: 2
		add:
			(MORectangleShape
				width: 30
				height: 10
				fillColor: Color red).
	shape := builder shape.
	figure := view node: #node using: shape.
	window := view open.
	figure
		setBoundsTo:
			(Rectangle
				origin: 5 @ 5
				width: 50
				height: 20).
	self
		assert:
			(shape boundsFor: figure )
				=
					(Rectangle
						origin: 5 @ 5
						width: 50
						height: 20).
	self
		assert:
			(rectangle boundsFor: figure )
				=
					(Rectangle
						origin: 5 @ 5
						width: 10
						height: 10).
! !

!MOFormsBuilderTest methodsFor: 'alignment - tests' stamp: ''!
testColumnRight
	
	| builder ui shape figure rectangle |
	rectangle := MORectangleShape width: 10 height: 10 fillColor: Color orange.
	builder := MOFormsBuilder new.
	builder column; right; pref.
	builder row; pref; row; pref.
	builder x: 1 y: 1 add: rectangle.
	builder x: 1 y: 2 add: (MORectangleShape width: 30 height: 10 fillColor: Color red).
	shape := builder shape.

	figure := view node: #node using: shape.
	ui := view open.
	self assert: (shape boundsFor: figure) = (5 @ 5 extent: 30@20).
	self assert: (rectangle boundsFor: figure) = (25 @ 5 extent: 10 @ 10).
	ui delete! !

!MOFormsBuilderTest methodsFor: 'alignment - tests' stamp: ''!
testColumnRightResize
	
	| builder shape figure rectangle |
	rectangle := MORectangleShape
		width: 10
		height: 10
		fillColor: Color orange.
	builder := MOFormsBuilder new.
	builder column; right; pref.
	builder row; pref; row; pref.
	builder x: 1 y: 1 add: rectangle.
	builder x: 1	y: 2 add: (MORectangleShape width: 30 height: 10 fillColor: Color red).
	shape := builder shape.

	figure := view node: #node using: shape.
	window := view open.
	figure
		setBoundsTo:
			(Rectangle
				origin: 5 @ 5
				width: 50
				height: 20).
	self
		assert:
			(shape boundsFor: figure)
				=
					(Rectangle
						origin: 5 @ 5
						width: 50
						height: 20).
	self
		assert:
			(rectangle boundsFor: figure)
				=
					(Rectangle
						origin: 45 @ 5
						width: 10
						height: 10).
! !

!MOFormsBuilderTest methodsFor: 'children shape - tests' stamp: ''!
testDisplaying
	"We first compare the canvasTranslation methods, and then we compare the bitmaps"

	| shape1 builder1 builder2 shape2 outterNode innerNode innerInnerNode oRecShape1 oRecShape2 oChildShape form1 form2 iRecShape iChildShape |
	builder1 := MOFormsBuilder new.
	builder1 column; fill; row; fill; row; fill.
	builder1 x:1 y:1 add: (oRecShape1 := MORectangleShape new).
	builder1 x:1 y:2 add: (oRecShape2 := MORectangleShape new).
	builder1 x:1 y:2 add: (oChildShape := MOChildrenShape new) .
	shape1 := builder1 asShape.
	view nodeShape: shape1.

	outterNode := view node: 'outter node' forIt: [
		builder2 := MOFormsBuilder new.
		builder2 column; fill; row; fill; row; fill.
		builder2 x:1 y:1 add: (iRecShape := MORectangleShape new).
		builder2 x:1 y:1 add: (iChildShape := MOChildrenShape new) .
		shape2 := builder2 asShape.
		view nodeShape: shape2.
		innerNode := view node: 'inner node' forIt: [
			innerInnerNode := view node: 'inner inner node' ].
		view verticalLineLayout].
	
	view root applyLayout.
	self assert: (view root computeElementsToDisplay asArray =  {outterNode . innerNode .  innerInnerNode}).
	self assert: (view root elementsToDisplay asArray =  {outterNode . innerNode .  innerInnerNode}).

	form1 := view getRenderedForm.
	"Since the inner nodes are cached"
	self assert: (view root elementsToDisplay asArray =  {outterNode}).


	"***************************"
	"We compare the canvas translation step"
	"outterNode shapeBounds."
	"canvasTranslationOf:For: is called in MOFormsShape>>displayOn:"
	self assert: (outterNode shapeBoundsAt: outterNode shape) = ( 5@5 corner: 30@35).  
	self assert: (outterNode shapeBoundsAt: oRecShape1) = ( 5@5 corner: 30@10).  
	self assert: (outterNode shapeBoundsAt: oRecShape2) = ( 5@10 corner: 30@35).  
	self assert: (outterNode shapeBoundsAt: oChildShape) = ( 5@10 corner: 30@35).  

	self assert: (outterNode shape canvasTranslationOf: outterNode shape For: outterNode) = (0@0).
	self assert: (outterNode shape canvasTranslationOf: oRecShape1 For: outterNode) = (0@0).
	self assert: (outterNode shape canvasTranslationOf: oRecShape2 For: outterNode) = (0@5).
	self assert: (outterNode shape canvasTranslationOf: oChildShape For: outterNode) = (0@5).

	self assert: (innerNode shapeBoundsAt: innerNode shape) = ( 5@5 corner: 20@20).  
	self assert: (innerNode shapeBoundsAt: iRecShape) = ( 5@5 corner: 20@20).  
	self assert: (innerNode shapeBoundsAt: iChildShape) = ( 5@5 corner: 20@20).  

	self assert: (innerNode shape canvasTranslationOf: innerNode shape For: innerNode) = (0@0).
	self assert: (innerNode shape canvasTranslationOf: iRecShape For: innerNode) = (0@0).
	self assert: (innerNode shape canvasTranslationOf: iChildShape For: innerNode) = (0@0).


	"***************************"
	"We compare the form"
	view := MOViewRenderer new.
	builder1 := MOFormsBuilder new.
	builder1 column; fill; row; fill; row; fill.
	builder1 x:1 y:1 add: (oRecShape1 := MORectangleShape new).
	builder1 x:1 y:2 add: (oRecShape2 := MORectangleShape new).
	builder1 x:1 y:2 add: (oChildShape := MOChildrenShape new) .
	shape1 := builder1 asShape.
	view nodeShape: shape1.

	view node: 'outter node' forIt: [
		 view node: 'inner node' forIt: [
			view node: 'inner inner node' ] ].
	view root applyLayout.
	form2 := view getRenderedForm.
	
	self assert: form1 bits = form2 bits
	"
	form1 asMorph openInWindow
	form2 asMorph openInWindow

	"! !

!MOFormsBuilderTest methodsFor: 'drag and drop' stamp: ''!
testDragAndDrop
	| outterNode innerNode absoluteBoundsNode absoluteBoundsInnerNode |
	outterNode := view node: $a forIt: [
		view shape 
			form: [ :b | b column; fill; row; fill. 
					b x:1 y:1 add: (MORectangleShape new fillColor: Color lightGreen). ].
		innerNode := view node: 'hello world' ].

	view root applyLayout.

	"****************************"
	"Drag and dropping the outter node"
	self assert: outterNode bounds = ( 5@5 corner: 20@20).
	self assert: innerNode bounds = ( 5@5 corner: 10@10).
	self assert: (innerNode shapeBoundsAt: innerNode shape  ) =  innerNode bounds.
	self assert: outterNode absoluteBounds = ( 5@5 corner: 20@20).
	self assert: innerNode absoluteBounds = ( 10@10 corner: 15@15).

	
	absoluteBoundsNode := outterNode absoluteBounds.
	absoluteBoundsInnerNode := innerNode absoluteBounds.
	outterNode translateBy: 15@10.
	
	self assert: (absoluteBoundsNode translateBy: (15@10)) = outterNode absoluteBounds.
	self assert: (innerNode shapeBoundsAt: innerNode shape  ) =  innerNode bounds.
	self assert: (absoluteBoundsInnerNode translateBy: (15@10)) = innerNode absoluteBounds.


	self assert: outterNode bounds = (( 5@5 corner: 20@20) translateBy: (15@10)).
	self assert: innerNode bounds = ( 5@5 corner: 10@10).

	"****************************"
	"Drag and dropping the inner node"
	absoluteBoundsNode := outterNode absoluteBounds.
	absoluteBoundsInnerNode := innerNode absoluteBounds.

	self assert: (innerNode getBoundedTranslationStep: 2@1) = (2@1).
	innerNode translateBy: 2@1.

	self assert: absoluteBoundsNode = outterNode absoluteBounds.
	self assert: (absoluteBoundsInnerNode translateBy: (2@1)) = innerNode absoluteBounds.
	self assert: ((innerNode shapeBoundsAt: innerNode shape) origin + outterNode bounds origin) =  innerNode absoluteBounds origin.

	"****************************"
	"Drag and dropping the outter node using an event"
	absoluteBoundsNode := outterNode absoluteBounds.
	absoluteBoundsInnerNode := innerNode absoluteBounds.

	outterNode announce: (MOMouseDrag step: 2@1).

	self assert: (absoluteBoundsNode  translateBy: (2@1)  )= outterNode absoluteBounds.
	self assert: (absoluteBoundsInnerNode translateBy: (2@1)) = innerNode absoluteBounds.
	self assert: ((innerNode shapeBoundsAt: innerNode shape) origin + outterNode bounds origin) =  innerNode absoluteBounds origin.
! !

!MOFormsBuilderTest methodsFor: 'drag and drop' stamp: ''!
testDragAndDrop2
	"Bug 574"
			
	| shape1 builder1 builder2 shape2 outterNode innerNode innerInnerNode formWithBuilder view2 formWithoutBuilder |
	builder1 := MOFormsBuilder new.
	builder1 column; fill; row; fill.
	builder1 x:1 y:1 add: (MORectangleShape new).
	builder1 x:1 y:1 add: (MOChildrenShape new) .
	shape1 := builder1 asShape.
	view nodeShape: shape1.

	outterNode := view node: 'outter' forIt: [
		builder2 := MOFormsBuilder new.
		builder2 column; fill; row; fill.
		builder2 x:1 y:1 add: (MORectangleShape new).
		builder2 x:1 y:1 add: (MOChildrenShape new) .
		shape2 := builder2 asShape.
		view nodeShape: shape2.
		innerNode := view node: 'inner' forIt: [
			innerInnerNode := view node: 'innerinnernode' ] ].
	view root applyLayout.

	formWithBuilder := view getRenderedForm.

	self assert: innerInnerNode bounds = ( 5@5 corner: 10@10).
	self assert: innerInnerNode absoluteBounds = ( 15@15 corner: 20@20).
	self assert: innerInnerNode absoluteBounds = (innerInnerNode bounds translateBy:  innerNode absoluteBounds origin ).

	self assert: outterNode bounds = ( 5@5 corner: 30@30).
	self assert: outterNode absoluteBounds = ( 5@5 corner: 30@30).
	
	self assert: innerNode bounds = ( 5@5 corner: 20@20).
	self assert: innerNode absoluteBounds = ( 10@10 corner: 25@25).
	
	
	"************************"
	"Building what the visualization should be"
	view2 := MOViewRenderer new.
	view2 node: $a forIt: [ view2 node: $b forIt: [ view2 node: #c ] ].
	view2 root applyLayout.
	formWithoutBuilder := view2 getRenderedForm.
	self assert: (formWithBuilder bits = formWithoutBuilder bits)
	
	"
	formWithBuilder asMorph openInWindow
	formWithoutBuilder asMorph openInWindow

	"! !

!MOFormsBuilderTest methodsFor: 'children shape - tests' stamp: ''!
testDragAndDropLimit
	| nodes formShape builder shapes |
	
	formShape := view shape
		form: [ :b | b column; fill; row; fill; row; fill; row; fill. 
				 b x: 1 y: 1 add: (MORectangleShape new).
				 b x: 1 y: 2 add: (MORectangleShape new).
				 b x: 1 y: 2 add: (MOChildrenShape new).
				 ].
	nodes := view nodes: (1 to: 5) forEach: [ :each |
		view shape rectangle fillColor: Color red.
		view node: 'inner node'
	].

	view root applyLayout.
	
	builder := formShape builder.
	self assert: (nodes allSatisfy: [ :n | n shape == formShape ]  ).

	shapes := builder shapes.
	self assert: (shapes allSatisfy: [ :c | c class == MOShapeAdaptor ]  ).
! !

!MOFormsBuilderTest methodsFor: 'children shape - tests' stamp: ''!
testElementAt

	| shape1 builder1 builder2 shape2 outterNode innerNode innerInnerNode form1 form2 |
	builder1 := MOFormsBuilder new.
	builder1 column; fill; row; fill; row; fill.
	builder1 x:1 y:1 add: (MORectangleShape new).
	builder1 x:1 y:2 add: (MORectangleShape new).
	builder1 x:1 y:2 add: (MOChildrenShape new) .
	shape1 := builder1 asShape.
	view nodeShape: shape1.

	outterNode := view node: 'outter node' forIt: [
		builder2 := MOFormsBuilder new.
		builder2 column; fill; row; fill; row; fill.
		builder2 x:1 y:1 add: (MORectangleShape new).
		builder2 x:1 y:1 add: (MOChildrenShape new) .
		shape2 := builder2 asShape.
		view nodeShape: shape2.
		innerNode := view node: 'inner node' forIt: [
			innerInnerNode := view node: 'inner inner node' ].
		view verticalLineLayout].
	
	view root applyLayout.
	form1 := view getRenderedForm.
	
	self assert: outterNode absoluteBounds = ( 5@5 corner: 30@35).
	self assert: innerNode absoluteBounds = ( 10@15 corner: 25@30).
	self assert: innerNode bounds = ( 5@5 corner: 20@20).
	self assert: innerInnerNode  bounds = ( 5@5 corner: 10@10).

	self assert: (innerInnerNode shape positionOfChildrenForOwner: innerInnerNode ) = (10@15).
	self assert: (innerInnerNode shape absoluteBoundsFor: innerInnerNode) origin = (15@20).
	self assert: innerInnerNode absoluteBounds origin = ( innerInnerNode bounds origin + 
													innerNode bounds origin +
													(0@5) +
													outterNode bounds origin).
	self assert: innerInnerNode absoluteBounds = ( 15@20 corner: 20@25).
	self assert: (view root elementAt: 15@20) == innerInnerNode.

! !

!MOFormsBuilderTest methodsFor: 'element size - tests' stamp: ''!
testElementSizeHorizontally
	| builder node nodeBounds nodeShape shape1 shape2 childrenShape |
	builder := MOFormsBuilder new.
	builder column; fill; pref; grow.
	builder row; fill; row; fill; pref; grow.
	builder x: 1 y: 1 add: (shape1 := MORectangleShape new ).
	builder x: 1 y: 2 add: (shape2 := MORectangleShape new).
	builder x: 1 y: 2 add: (childrenShape := MOChildrenShape new).
	view shape: builder shape.

	node := view node: 'hello' forIt: [
		view nodes: (1 to: 5).
		view verticalLineLayout
	].

	"********************************"
	"Before doing the applyLayout"
	nodeShape := node shape.
	self assert: (nodeShape builder == builder).
	
	self assert: (builder heightsFor: node) =  #(5 5.0).
	self assert: (builder widthsFor: node) =  #(5.0).
	nodeShape computeBoundsFor: node.
		
	nodeBounds := node bounds.
	self assert: nodeBounds width = 5.
	self assert: nodeBounds height = 10.
	"********************************"
	
	"********************************"
	"After doing the applyLayout"
	view root applyLayout.

"	self assert: (shape1 computeHeightFor: node).
	self assert: (shape1 computeWidthFor: node).
"
	nodeShape := node shape.
	self assert: (nodeShape builder == builder).	

	self assert: (builder boundsOf:  shape1 for: node) = (0@0 corner: 15@5).
	self assert: (builder boundsOf:  childrenShape for: node) = ( 0@5 corner: 15@80).
	self assert: (builder boundsOf:  shape2 for: node) = ( 0@5 corner: 15@80).
	
	self assert: (builder shapes collect: [ :s | s heightFor: node ]) asArray = #(5 5 75).

	self assert: (builder heightsFor: node) =  #(5 75.0).
	self assert: (builder widthsFor: node) =   #(15.0).

	nodeBounds := node bounds.
	self assert: nodeBounds width = 15.
	self assert: nodeBounds height = 80.

	self assert: (childrenShape absoluteOriginForChildrenElement: node) = ( 5@10)
	"********************************"! !

!MOFormsBuilderTest methodsFor: 'element size - tests' stamp: ''!
testElementSizeVertically
	| builder node nodeBounds nodeShape shape1 shape2 childrenShape |
	builder := MOFormsBuilder new.
	builder column; fill; column; fill; pref; grow.
	builder row; fill; pref; grow.
	builder x: 1 y: 1 add: (shape1 := MORectangleShape new ).
	builder x: 2 y: 1 add: (shape2 := MORectangleShape new).
	builder x: 2 y: 1 add: (childrenShape := MOChildrenShape new).
	view shape: builder shape.

	node := view node: 'hello' forIt: [
		view nodes: (1 to: 5).
		view verticalLineLayout
	].

	"********************************"
	"Before doing the applyLayout"
	nodeShape := node shape.
	self assert: (nodeShape builder == builder).
	
	self assert: (builder heightsFor: node) =  #(5.0).
	self assert: (builder widthsFor: node) =   #(5 5.0).
		
	nodeBounds := node bounds.
	self assert: nodeBounds width = 10.
	self assert: nodeBounds height = 5.
	"********************************"
	
	"********************************"
	"After doing the applyLayout"
	view root applyLayout.

"	self assert: (shape1 computeHeightFor: node).
	self assert: (shape1 computeWidthFor: node).
"
	nodeShape := node shape.
	self assert: (nodeShape builder == builder).	

	self assert: (builder boundsOf:  shape1 for: node) = ( 0@0 corner: 5@75).
	self assert: (builder boundsOf:  childrenShape for: node) = ( 5@0 corner: 20@75).
	self assert: (builder boundsOf:  shape2 for: node) = ( 5@0 corner: 20@75).
	
	self assert: (builder shapes collect: [ :s | s widthFor: node]) asArray = #(5 5 15).

	self assert: (builder heightsFor: node) =  #(75.0).
	self assert: (builder widthsFor: node) =   #(5 15.0).

	nodeBounds := node bounds.
	self assert: nodeBounds width = 20.
	self assert: nodeBounds height = 75.
	
	self assert: (childrenShape absoluteOriginForChildrenElement: node) = ( 10@5)

	"********************************"! !

!MOFormsBuilderTest methodsFor: 'errors - test' stamp: ''!
testError 
	| shape1 shape2 |
	shape1 := view shape 
		form: [:b | b column; fill; row; fill; row; fill; row; fill. 
						b x:1 y:1 add: (MORectangleShape new width: 80; height: 50).
						b x:1 y:2 add: (MORectangleShape new width: 80; height: 100) ].
	self should: [ view node: 'outter node' forIt: [ ] ] raise: Error.
	self deny: shape1 canHandleChildren. 
	
	shape2 := view shape 
		form: [:b | b column; fill; row; fill; row; fill; row; fill. 
						b x:1 y:1 add: (MORectangleShape new width: 80; height: 50).
						b x:1 y:2 add: (MORectangleShape new width: 80; height: 100).
						b x: 1y: 2 add: MOChildrenShape new ].
	self shouldnt: [ view node: 'outter node' forIt: [ ] ] raise: Error.
	self assert: shape2 canHandleChildren. 
! !

!MOFormsBuilderTest methodsFor: 'shape tests' stamp: ''!
testForm
	view shape
		form: [:b | 
						b column; fill; row; fill; row; fill; row; fill. 
						b x:1 y:1 add: (MORectangleShape new fillColor: Color lightGreen).
						b x:1 y:2 add: (MORectangleShape new withoutBorder; withText).
						b x:1 y:3 add: (MORectangleShape new fillColor: Color lightBlue).].
	view node: 'hello world'.

	window := view open.
	
	self assert: (view root nodes size = 1).
	self assert: (view root nodes first model = 'hello world').
	self assert: (view root nodes first shape isKindOf: MOFormsShape).
	self assert: (view root nodes first shape builder numberOfRows = 3).
	self assert: (view root nodes first shape builder numberOfColumns = 1).! !

!MOFormsBuilderTest methodsFor: 'nesting' stamp: ''!
testNesting
	"Bug identified in http://code.google.com/p/moose-technology/issues/detail?id=574 , comment 13"
	"Nesting 3 nodes leads to a strange situation. the inner nodes are not correction located in the scene."
	
	"Need to check if the shape is fisically the same in the inner nodes"
	
	| shape builder outterNodes innerNode innerInnerNodes innerInnerNodes1 innerInnerNodes2 childrenShape |
	builder := MOFormsBuilder new.
	builder column; fill; row; fill; row; fill; row; fill. 
	builder x:1 y:1 add: (MORectangleShape new).
	builder x:1 y:2 add: (MORectangleShape new).
	builder x:1 y:2 add: (childrenShape := MOChildrenShape new) .
	shape := builder asShape.
	view nodeShape: shape.
	outterNodes := view nodes: (1 to: 2) forEach: [:x |
		view nodeShape: builder asShape.
		view nodes: {10} forEach: [:y |
			view nodeShape: shape.
			view nodes: {11}.
			view verticalLineLayout ] .
		view verticalLineLayout ].
	
	view root applyLayout.
	
	self assert: (outterNodes collect: #absoluteBounds) =  {(5@5 corner: 30@45). (40@5 corner: 65@45)}.
	
	innerNode := outterNodes first nodes first.
	self assert: (innerNode absoluteBounds) = ( 10@15 corner: 25@40).

	innerInnerNodes1 := outterNodes first nodes first nodes first.
	innerInnerNodes2 := outterNodes second nodes first nodes first.
	self assert: (innerInnerNodes1 bounds = innerInnerNodes2 bounds).

	self assert: (childrenShape absoluteOriginForChildrenElement: innerInnerNodes1 owner) ~= (childrenShape absoluteOriginForChildrenElement: innerInnerNodes2 owner).

	self assert: ((innerInnerNodes1 shape absoluteBoundsFor:  innerInnerNodes1) 
					~= (innerInnerNodes2 shape absoluteBoundsFor:  innerInnerNodes2)).	
	self assert: (innerInnerNodes1 absoluteBounds ~= innerInnerNodes2 absoluteBounds).	! !

!MOFormsBuilderTest methodsFor: 'advanced - api' stamp: ''!
testNestingOfFormBuilder

	|  builder rectangleShape1 rectangleShape2 bigNode subnodes adaptor1 adaptor2 formShape |

	builder := MOFormsBuilder new.
	builder column; fill.
	builder row; fill; row; fill.
	builder x: 1 y: 1 add: (rectangleShape1 := MORectangleShape new width: 30; height: 25; fillColor: Color lightBlue).
	builder x: 1 y: 2 add: (rectangleShape2 := MORectangleShape new width: 30; height: 25).

	bigNode := view node: 10 forIt: [
		view shape: builder shape.
		subnodes := view nodes: #(1 2).
	].

	window := view open.

	self assert: (subnodes size = 2).
	self assert: (subnodes first bounds = ( 5@5 corner: 35@55)).
	self assert: (subnodes second bounds = ( 45@5 corner: 75@55)).
	
	"Some hygienic checks"
	self assert: (subnodes first shape isKindOf: MOFormsShape).
	self assert: (subnodes first shape builder == builder).

	self assert: (subnodes second shape isKindOf: MOFormsShape).
	self assert: (subnodes second shape builder == builder).

	self assert: (subnodes second shape builder shapes isCollection).
	self assert: (subnodes second shape builder shapes size = 2).
	adaptor1 := subnodes second shape builder shapes first.
	adaptor2 := subnodes second shape builder shapes second.
	
	self assert: (adaptor1 isKindOf: MOShapeAdaptor).
	self assert: (adaptor2 isKindOf: MOShapeAdaptor).
	self assert: (adaptor1 shape == rectangleShape1).
	self assert: (adaptor2 shape == rectangleShape2).
	self assert: ((adaptor1 shape absoluteBoundsFor: subnodes first) = ( 10@10 corner: 40@35)).
	self assert: ((adaptor2 shape absoluteBoundsFor: subnodes first) = ( 10@35 corner: 40@60)).

	self assert: (bigNode absoluteBounds = ( 5@5 corner: 85@65)).
	
	self assert: (subnodes second owner == bigNode).
	formShape := subnodes second shape.
	self assert: (formShape isKindOf: MOFormsShape).
	self assert: ((formShape boundsFor: subnodes second) = (subnodes second bounds)).
	self assert: ((formShape absoluteBoundsFor: subnodes second) = ( 50@10 corner: 80@60)).
	
	self assert: (builder boundsOf: rectangleShape1 for: bigNode) = ( 0@0 corner: 30@25).
	self assert: (builder boundsOf: rectangleShape2 for: bigNode) = ( 0@25 corner: 30@50).

	self assert: ((adaptor1 shape absoluteBoundsFor: subnodes second) = ( 50@10 corner: 80@35)).

! !

!MOFormsBuilderTest methodsFor: 'oo - api - tests' stamp: ''!
testOoApi
	
	| builder shape figure rectangle |
	rectangle := MORectangleShape
		width: 10
		height: 10
		fillColor: Color orange.
	builder := MOFormsBuilder new.
	builder 	column; fill; pref; grow.
	builder row; pref.
	builder	row; pref.
	builder x: 1 y: 1 add: rectangle.
	builder x: 1	y: 2 add: (MORectangleShape width: 30 height: 10 fillColor: Color red).
	shape := builder shape.
	figure := view node: #node using: shape.
	window := view open.
	figure
		setBoundsTo:
			(Rectangle
				origin: 5 @ 5
				width: 50
				height: 20).
	self
		assert:
			(shape boundsFor: figure)
				=
					(Rectangle
						origin: 5 @ 5
						width: 50
						height: 20).
	self
		assert:
			(rectangle boundsFor: figure )
				=
					(Rectangle
						origin: 5 @ 5
						width: 50
						height: 10).! !

!MOFormsBuilderTest methodsFor: 'children shape - tests' stamp: ''!
testProper
	| nodes formShape builder shapes |
	
	formShape := view shape
		form: [ :b | b column; fill; row; fill; row; fill; row; fill. 
				 b x: 1 y: 1 add: (MORectangleShape new).
				 b x: 1 y: 2 add: (MORectangleShape new).
				 b x: 1 y: 2 add: (MOChildrenShape new).
				 ].
	nodes := view nodes: (1 to: 5) forEach: [ :each |
		view shape rectangle fillColor: Color red.
		view node: 'inner node'
	].

	view root applyLayout.
	
	builder := formShape builder.
	self assert: (nodes allSatisfy: [ :n | n shape == formShape ]  ).

	shapes := builder shapes.
	self assert: (shapes allSatisfy: [ :c | c class == MOShapeAdaptor ]  ).
! !

!MOFormsBuilderTest methodsFor: 'alignment - tests' stamp: ''!
testPxReSize
	
	| builder shape figure r1 r2 |
	r1 := MORectangleShape fillColor: Color orange.
	r2 := (MORectangleShape
		width: 30
		height: 10
		fillColor: Color red).
	builder := MOFormsBuilder new.
	builder columns: 'fill:20px'.
	builder rows: '50px, pref'.
	builder x: 1 y: 1 add: r1.
	builder x: 1 y: 2 add: r2.
	shape := builder asShape.
	figure := view node: #node using: shape.
	window := view open.
	figure
		setBoundsTo:
			(Rectangle
				origin: 5 @ 5
				width: 100
				height: 200).
	self assert: (shape boundsFor: figure ) = (5 @ 5 corner: 105 @ 205).
	self assert: (r1 boundsFor: figure ) = (5 @ 5 corner: 25.0 @ 10).
	self assert: (r2 boundsFor: figure ) = (6 @ 56 corner: 24.0 @ 66).
! !

!MOFormsBuilderTest methodsFor: 'alignment - tests' stamp: ''!
testPxSize
	
	| builder shape figure r1 r2 |
	r1 := MORectangleShape fillColor: Color orange.
	r2 := (MORectangleShape
		width: 30
		height: 10
		fillColor: Color red).
	builder := MOFormsBuilder new.

	builder columns: 'fill:20px'.
	builder rows: '50px, pref'.
	builder x: 1 y: 1 add: r1.
	builder x: 1 y: 2 add: r2.
	shape := builder asShape.
	figure := view node: #node using: shape.
	window := view open.
	self assert: (shape boundsFor: figure ) = (5 @ 5 corner: 25 @ 67).
	self assert: (r1 boundsFor: figure ) = (5 @ 5 corner: 25.0 @ 10).
	self assert: (r2 boundsFor: figure ) = (6 @ 56 corner: 24.0 @ 66).
! !

!MOFormsBuilderTest methodsFor: 'shape tests' stamp: ''!
testRecomputePreferredBounds
"	| builder view node |
	builder := MOFormsBuilder new.
	builder row; fill; pref; grow.
	builder column; fill; pref; grow.
	builder x: 1 y: 1 add: MORectangleShape withBorder.
	view := MOViewRenderer new.
	node := view
		node: #test
		using: builder asShape.
	window := view open.
	

	self assert: node bounds = (5 @ 5 extent: 7 @ 7).
	node setBoundsTo: (40 @ 35 extent: 20 @ 21).
	self assert: node bounds = (40 @ 35 extent: 20 @ 21).
	node recomputePreferredBounds.
	self assert: node bounds = (40 @ 35 extent: 7 @ 7).
"! !

!MOFormsBuilderTest methodsFor: 'alignment - tests' stamp: ''!
testRowBottom
	
	| builder ui shape figure rectangle |
	rectangle := MORectangleShape
		width: 10
		height: 10
		fillColor: Color orange.
	builder := MOFormsBuilder new.
	builder 
		column; pref; 
		column; pref.
	builder row; bottom; pref.
	builder x: 1 y: 1 add: rectangle.
	builder x: 2	y: 1	add: (MORectangleShape width: 20 height: 30 fillColor: Color red).
	shape := builder shape.
	figure := view node: #node using: shape.
	ui := view open.
	self	assert: (shape boundsFor: figure ) =	(Rectangle origin: 5 @ 5	width: 30 height: 30).
	self	assert: (rectangle boundsFor: figure ) = (Rectangle origin: 5 @ 25	width: 10 height: 10).
	ui delete! !

!MOFormsBuilderTest methodsFor: 'alignment - tests' stamp: ''!
testRowBottomResize
	
	| builder shape figure rectangle |
	rectangle := MORectangleShape
		width: 10
		height: 10
		fillColor: Color orange.
	builder := MOFormsBuilder new.
	builder 
		column; pref;
		column; pref.
	builder row; bottom; pref; grow.
	builder x: 1 y: 1 add: rectangle.
	builder x: 2	y: 1	 add: (MORectangleShape width: 20 height: 30 fillColor: Color red).
	shape := builder shape.
	figure := view node: #node using: shape.
	window := view open.
	figure setBoundsTo: 	(Rectangle origin: 5 @ 5	width: 30 height: 50).
	self	assert:(shape boundsFor: figure ) = (Rectangle origin: 5 @ 5 width: 30 height: 50).
	self	assert: (rectangle boundsFor: figure ) = (Rectangle origin: 5 @ 45 width: 10 height: 10).
! !

!MOFormsBuilderTest methodsFor: 'alignment - tests' stamp: ''!
testRowCenter
	
	| builder shape figure rectangle |
	rectangle := MORectangleShape
		width: 10
		height: 10
		fillColor: Color orange.
	builder := MOFormsBuilder new.
	builder 
		column; pref; 
		column; pref.
	builder row; center; pref.
	builder x: 1 y: 1 add: rectangle.
	builder x: 2 y: 1 add: (MORectangleShape width: 20 height: 30 fillColor: Color red).
	shape := builder shape.
	figure := view node: #node using: shape.
	window := view open.
	self assert:	(shape boundsFor: figure ) = (Rectangle origin: 5 @ 5 width: 30 height: 30).
	self assert:	(rectangle boundsFor: figure ) = (Rectangle origin: 5 @ 15 width: 10	height: 10).
! !

!MOFormsBuilderTest methodsFor: 'alignment - tests' stamp: ''!
testRowCenterResize
	
	| builder shape figure rectangle |
	rectangle := MORectangleShape
		width: 10
		height: 10
		fillColor: Color orange.
	builder := MOFormsBuilder new.
	builder 
		column; pref;
		column; pref.
	builder row; center; pref; grow.
	builder x: 1 y: 1 add: rectangle.
	builder x: 2 y: 1 add: (MORectangleShape width: 20 height: 30 fillColor: Color red).
	shape := builder shape.
	figure := view node: #node using: shape.
	window := view open.
	figure setBoundsTo:	(Rectangle origin: 5 @ 5 width: 30 height: 50).
	self assert: (shape boundsFor: figure ) = (Rectangle	origin: 5 @ 5 width: 30 height: 50).
	self assert: (rectangle boundsFor: figure ) = (Rectangle origin: 5 @ 25 width: 10 height: 10).
! !

!MOFormsBuilderTest methodsFor: 'alignment - tests' stamp: ''!
testRowFill
	
	| builder shape figure rectangle |
	rectangle := MORectangleShape
		width: 10
		height: 10
		fillColor: Color orange.
	builder := MOFormsBuilder new.
	builder 
		column; pref;
		column; pref.
	builder row; fill; pref.
	builder x: 1 y: 1 add: rectangle.
	builder x: 2 y: 1 add: (MORectangleShape width: 20 height: 30 fillColor: Color red).
	shape := builder shape.
	figure := view node: #node using: shape.
	window := view open.
	self assert: (shape boundsFor: figure ) =	(Rectangle origin: 5 @ 5 width: 30 height: 30).
	self	assert: (rectangle boundsFor: figure ) = (Rectangle origin: 5 @ 5 width: 10 height: 30).
	window delete! !

!MOFormsBuilderTest methodsFor: 'alignment - tests' stamp: ''!
testRowFillResize
	
	| builder shape figure rectangle |
	rectangle := MORectangleShape
		width: 10
		height: 10
		fillColor: Color orange.
	builder := MOFormsBuilder new.
	builder 
		column; pref; 
		column; pref.
	builder row; fill; pref; grow.
	builder x: 1 y: 1 add: rectangle.
	builder x: 2 y: 1 add:(MORectangleShape width: 20 height: 30 fillColor: Color red).
	shape := builder shape.
	figure := view node: #node using: shape.
	window := view open.
	figure setBoundsTo: (Rectangle origin: 5 @ 5 width: 30 height: 50).
	self assert: (shape boundsFor: figure ) = (Rectangle origin: 5 @ 5 width: 30 height: 50).
	self	assert: (rectangle boundsFor: figure ) = (Rectangle origin: 5 @ 5 width: 10 height: 50).
! !

!MOFormsBuilderTest methodsFor: 'alignment - tests' stamp: ''!
testRowTop
	
	| builder shape figure rectangle |
	rectangle := MORectangleShape
		width: 10
		height: 10
		fillColor: Color orange.
	builder := MOFormsBuilder new.
	builder 
		column; pref;
		column; pref.
	builder row; top; pref.
	builder x: 1 y: 1 add: rectangle.
	builder x: 2 y: 1 add: (MORectangleShape width: 20 height: 30 fillColor: Color red).
	shape := builder shape.
	figure := view node: #node using: shape.
	window := view open.
	self assert: (shape boundsFor: figure ) = (Rectangle origin: 5 @ 5 width: 30 height: 30).
	self assert: (rectangle boundsFor: figure ) = (Rectangle origin: 5 @ 5 width: 10 height: 10).
	window delete! !

!MOFormsBuilderTest methodsFor: 'alignment - tests' stamp: ''!
testRowTopResize
	
	| builder shape figure rectangle |
	rectangle := MORectangleShape
		width: 10
		height: 10
		fillColor: Color orange.
	builder := MOFormsBuilder new.
	builder 
		column; pref;
		column; pref.
	builder row; top; pref.
	builder x: 1 y: 1 add: rectangle.
	builder x: 2 y: 1 add: (MORectangleShape width: 20 height: 30 fillColor: Color red).
	shape := builder shape.
	figure := view node: #node using: shape.
	window := view open.
	figure setBoundsTo: (Rectangle origin: 5 @ 5 width: 30 height: 50).
	self assert: (shape boundsFor: figure ) = (Rectangle origin: 5 @ 5 width: 30 height: 50).
	self assert: (rectangle boundsFor: figure ) = (Rectangle origin: 5 @ 5 width: 10 height: 10).
! !

!MOFormsBuilderTest methodsFor: 'alignment - tests' stamp: ''!
testSimple
	
	| builder shape figure rectangle |
	rectangle := MORectangleShape width: 10 height: 15 fillColor: Color blue.
	builder := MOFormsBuilder new.
	builder column; center; pref.
	builder row; pref.
	builder x: 1 y: 1 add: rectangle.
	shape := builder shape.
	figure := view node: #node using: shape.

	window := view open.
	self assert: (shape boundsFor: figure) =
					(Rectangle
						origin: 5 @ 5
						width: 10
						height: 15).
	self assert: (shape collectAllShapes size = 2).
! !

!MOFormsBuilderTest methodsFor: 'children shape - tests' stamp: ''!
testchildrenOfADifferentType

	view := MOViewRenderer new.
	view shape form: [:builder |
		builder column; pref; center; row; fill; row; fill.
		builder x:1 y:1 add: (MORectangleShape new text: [:col | (col first + 100) asString ]).
		builder x:1 y:2 add: MOChildrenShape new].
	view node: #(10 20) forIt: [
		view nodes: #(a b) ].
	view root applyLayout.
	view getRenderedForm ! !

MondrianTest subclass: #MOLayoutTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Tests'!

!MOLayoutTest methodsFor: 'testing' stamp: ''!
expectedFailures
	^ #(#testContextTransfer #testScatterplotLayoutBoundsWithUpSideDown)! !

!MOLayoutTest methodsFor: 'line layout' stamp: ''!
horizontalLayoutStrech4TOBEFIXED
	| view nodes ui |
	view := MOViewRenderer new.

"	view shape rectangle width: 150; height: 130."
	view node: #foo forIt: [
		view shape rectangle width: #x; height: #y.
		view nodes: {80@30}.
		
		view node: #bar forIt: [
"			view shape rectangle width: #x; height: #y."
			view nodes: {40@30}.
			view horizontalLineLayout stretchHorizontally. 		
		].
		view verticalLineLayout stretchHorizontally. 
	].

	window := view open.

! !

!MOLayoutTest methodsFor: 'obsolete' stamp: ''!
obsoleteTestScatterplot
	
"	| view nodeA nodeC ui nodeB |
	view := MOViewRenderer new.
	nodeA := view
		node: #a
		forIt:
			[view graphElement
				x: [:entity | #centered]
				y: 10.
			nodeB := view node: #b.
			view graphElement x: 90 y: 30.
			nodeC := view node: #c.
			view layout: (MOScatterplotLayout new
				xOutGap: 0;
				yOutGap: 5)].
	ui := view open.
	self assert: nodeA bounds = (5.0 @ 5 corner: 104.0 @ 49).
	self assert: nodeB bounds = (51.0 @ 16 corner: 58.0 @ 23).
	self assert: nodeC bounds = (96.0 @ 36 corner: 103.0 @ 43).
	ui delete"! !

!MOLayoutTest methodsFor: 'test layout' stamp: ''!
testBoxFigureResize
	
	| model view box |
	model := MOTestModel new.
	view := MOViewRenderer new.
	box := view
		node: #box
		using: MORectangleShape withBorder
		forIt:
			[view
				nodes: model allNodes
				using: model shapeForNode.
			view
				localEdges: model allEdges
				from: [:entity | entity first]
				to: [:entity | entity last]
				using: model shapeForEdge.
			view layout: MOVerticalNarrowTreeLayout new.
			].
	window := view open.

	self assert: box origin = (5 @ 5).
	"box applyLayout."	"Reapply the layout"
	view root applyLayout.
	self assert: box origin = (5 @ 5).
! !

!MOLayoutTest methodsFor: 'testBulletTree' stamp: ''!
testBulletTree

	| view form |
	view := MOViewRenderer new.
	view nodes: Collection withAllSubclasses.
	view edgesFrom: #superclass.
	view bulletTreeLayout.
	view root applyLayout.
	form := view getRenderedForm.
	"form openAsMorph"
! !

!MOLayoutTest methodsFor: 'test layout' stamp: ''!
testChangingLayout
	
	| view node layout |
	layout := MOVerticalNarrowTreeLayout new.
	view := MOViewRenderer new.
	view
		nodes: #(#'1' #'2a' #'2b' #'3a' #'3b' #'3c').
		
	view
		edges:
			#(#(#'1' #'2a') #(#'1' #'2b') #(#'2a' #'3a') #(#'2a' #'3b') #(#'2b' #'3c'))
		from: [:entity | entity first]
		to: [:entity | entity last].

	view layout: layout.
	window := view open.
"	view evaluate."

	"node := view peek deepFindNodeWithEntity: #'2a'."	"Test that applying the layout worked"
	node := view nodeForDomainValue: #'2a'.

	self assert: (layout childrenFor: node) size = 2.
	self assert: (layout parentsFor: node) size = 1.	"Reapply the same layout"
	view root applyLayout.
	self assert: (layout childrenFor: node) size = 2.
	self assert: (layout parentsFor: node) size = 1.	"Use an other layout"
	layout := MOHorizontalTreeLayout new.
	view root layout: layout.
	view root applyLayout.
	self assert: (layout childrenFor: node) size = 2.
	self assert: (layout parentsFor: node) size = 1.

	layout := MOHorizontalNarrowTreeLayout new.
	view root layout: layout.
	view root applyLayout.
	self assert: (layout childrenFor: node) size = 2.
	self assert: (layout parentsFor: node) size = 1.
! !

!MOLayoutTest methodsFor: 'utils' stamp: ''!
testChildrenNodes
	
	| view layout nodes node |
	layout := MOTreeLayout new.
	view := MOViewRenderer new.
	node := view node: 1.
	nodes := view nodes: #(2 3).
	view
		edge: 1 -> 2
		from: #key
		to: #value.
	view
		edge: 1 -> 3
		from: #key
		to: #value.
	view layout: layout.
	window := view open.
"	view evaluate."
	self assert: (layout childrenFor: node) size = 2.
	self assert: ((layout childrenFor: node) includes: nodes first).
	self assert: ((layout childrenFor: node) includes: nodes second)! !

!MOLayoutTest methodsFor: 'test layout' stamp: ''!
testCircleLayout
	
	self assert: MOCircleLayout new scaleFactor = 11.	"Default  value"
	self assert: (MOCircleLayout scaleBy: 35) scaleFactor = 35! !

!MOLayoutTest methodsFor: 'test layout' stamp: ''!
testConnectionPositions
	
	| model view graphElement |
	model := MOTestModel new.
	view := MOViewRenderer new.
	view
		nodes: model allNodes
		using: model shapeForNode.
	view
		localEdges: model allEdges
		from: [:entity | entity first]
		to: [:entity | entity last]
		using: model shapeForEdge.
	view layout: MOVerticalNarrowTreeLayout new.

	view root applyLayout.
	graphElement := view root edges
		detect:
			[:aFigure | 
			aFigure source model == model b1
				& (aFigure target model == model c1)].
	self assert: graphElement fromPositions = #(#bottomCenter).
	self assert: graphElement toPositions = #(#topCenter)! !

!MOLayoutTest methodsFor: 'test layout' stamp: ''!
testContextTransfer
	"When we change the layout the nodes and edges information should be passed to the"
	
	| view edges model layout |
	model := MOTestModel new.
	view := MOViewRenderer new.
	view
		nodes: model allNodes
		using: model shapeForNode.
	edges := view
		localEdges: model allEdges
		from: [:entity | entity first]
		to: [:entity | entity last]
		using: model shapeForEdge.	"Add an other edge that should not be taken in to account."
	view
		edge:
			(Array
				with: model a1
				with: model d1)
		from: [:entity | entity first]
		to: [:entity | entity last]
		using: model shapeForEdge.
	layout := MOTreeLayout withEdges: edges.
	view layout: layout.
	window := view open.

	self
		assert:
			(layout childrenFor: (view root deepFindNodeWithEntity: model a1)) size = 2.
	self
		assert:
			(layout childrenFor: (view root deepFindNodeWithEntity: model d1)) isEmpty.	"Change to a totaly other layout"
	view layout: MOHorizontalLineLayout new.
	view root deepApplyLayout.	"Change back to a tree layout"
	view updateWindow.
	layout := MOVerticalNarrowTreeLayout new.
	view layout: layout.
	view root deepApplyLayout.
	self
		assert:
			(layout childrenFor: (view root deepFindNodeWithEntity: model a1)) size = 2.
	self
		assert:
			(layout childrenFor: (view root deepFindNodeWithEntity: model d1)) isEmpty! !

!MOLayoutTest methodsFor: 'dominance tree layout' stamp: ''!
testDominanceTreeLayout
	| view nodes node1 node2 node3 node4 x y |
	view := MOViewRenderer new.
	nodes := view nodes: (1 to: 4).
	view edges: {1 -> 2. 1-> 3. 1->4} from: #key to: #value.
	view dominanceTreeLayout.
	window := view open.
	
	node1 := nodes first.
	node2 := nodes second.
	node3 := nodes third.
	node4 := nodes fourth.
	
	x := nodes asSortedCollection: [:n1 :n2 | n1 bounds topLeft x < n2 bounds topLeft x].
	self assert: (x asArray = {node2 . node3. node1. node4}).

	y := nodes asSortedCollection: [:n1 :n2 | n1 bounds topLeft y < n2 bounds topLeft y].
	self assert: (y asArray = {node1 . node3. node2. node4})! !

!MOLayoutTest methodsFor: 'loops' stamp: ''!
testDoubleLoopWithNoRoot
	
	| view |
	view := MOViewRenderer new.
	view nodes: #(1 2 3 4 5 6).
	view
		edge: 1 -> 2
		from: #key
		to: #value.
	view
		edge: 2 -> 3
		from: #key
		to: #value.
	view
		edge: 3 -> 1
		from: #key
		to: #value.
	view
		edge: 4 -> 5
		from: #key
		to: #value.
	view
		edge: 5 -> 6
		from: #key
		to: #value.
	view
		edge: 6 -> 4
		from: #key
		to: #value.
	view layout: (MOVerticalNarrowTreeLayout new).
	window := view open! !

!MOLayoutTest methodsFor: 'flowInLayout' stamp: ''!
testFlowInLayout
	" self debug: #testFlowInLayout "
	| shape view nodes |

	view := MOViewRenderer new.
	shape := MORectangleShape new width: #yourself; height: #yourself.
	shape fillColor: Color blue.
	view shape: shape.
	nodes := view nodes: #(10 30 50 100 35 10 45).
	view layout: (MOFillInFlowLayout new).
	window := view open.
	
	self assert: nodes first bounds = (5@5 corner: 15@15).
	self assert: nodes third bounds = (65@5 corner: 115@55).
	self assert: nodes fourth bounds = (5@65 corner: 105@165).
! !

!MOLayoutTest methodsFor: 'force based Layout' stamp: ''!
testForceBasedLayout
	| view edges |
	view := MOViewRenderer new.
	view shape rectangle size: 40.
	view nodes: (1 to: 5).
	view shape arrowedLine width: 2.
	edges := view edges: {1 -> 2 . 1 -> 3 . 1 -> 5 . 2 -> 4 . 4 -> 1 } from: #key to: #value.
	view forceBasedLayout.
	
	self assert: (edges allSatisfy: [:edge | edge fromPositions isNil ]).
	self assert: (edges allSatisfy: [:edge | edge toPositions isNil ]).

	window := view open.
	
	self assert: (edges allSatisfy: [:edge | edge fromPositions = #() ]).
	self assert: (edges allSatisfy: [:edge | edge toPositions = #() ]).

	self assert: (edges collect: [:edge | | t |
		t := edge shape getAttachPointsFor: edge.
		{t first asIntegerPoint . t second asIntegerPoint } ] ) asArray = {{114@95. 146@110}. {74@95. 45@70}. {94@75. 114@45}. {146@110. 137@165}. {117@145. 94@115}}! !

!MOLayoutTest methodsFor: 'circle layout' stamp: ''!
testForceBasedLayout1
	| view nodes node1 node2 node3 node4 x y |
	view := MOViewRenderer new.
	nodes := view nodes: (1 to: 4).
	view edges: {1 -> 2. 1-> 3. 1->4} from: #key to: #value.
	view circleLayout.
	window := view open.
	
	node1 := nodes first.
	node2 := nodes second.
	node3 := nodes third.
	node4 := nodes fourth.
	
	x := nodes asSortedCollection: [:n1 :n2 | n1 bounds topLeft x < n2 bounds topLeft x].
	self assert: (x asArray = {node3 . node4. node2. node1}).

	y := nodes asSortedCollection: [:n1 :n2 | n1 bounds topLeft y < n2 bounds topLeft y].
	self assert: (y asArray = {node4 . node1. node3. node2})! !

!MOLayoutTest methodsFor: 'forms - tests' stamp: ''!
testFormsLayout
	
	| builder view nodeA nodeB nodeC |
	"Initialize the layout"
	builder := MOFormsBuilder new.
	builder columns: 'pref, 20px, pref, 50px, pref'.
	builder rows: 'pref'.	"Add nodes"
	view := MOViewRenderer new.

	view
		shape:
			(MORectangleShape
				width: 40
				height: 50
				fillColor: Color green).

	nodeA := view node: #a.
	view
		shape:
			(MORectangleShape
				width: 20
				height: 90
				fillColor: Color blue).

	nodeB := view node: #b.
	view
		shape:
			(MORectangleShape
				width: 80
				height: 20
				fillColor: Color yellow).

	nodeC := view node: #c.	"Position the components"
	builder x: 1 y: 1 add: nodeA.
	builder x: 3 y: 1 add: nodeB.
	builder x: 5 y: 1 add: nodeC.	"Apply layout"
	view layout: builder asLayout.

	window := view open.

	self assert: nodeA bounds = ( 0@0 corner: 40@50).
	self assert: nodeB bounds = ( 60@0 corner: 80@90).
	self assert: nodeC bounds = ( 130@0 corner: 210@20).
! !

!MOLayoutTest methodsFor: 'test layout' stamp: ''!
testFromPosition
	
	| view handle |
	view := MOViewRenderer new.
	view nodes: #(1 2 3).
	handle := view
		edge: 1 -> 2
		from: #key
		to: #value.
	view
		layout:
			(MOVerticalNarrowTreeLayout new fromPositions:
					#(#rightCenter #leftCenter #topCenter #bottomCenter)).
	window := view open.
	self
		assert:
			handle value fromPositions =
					#(#rightCenter #leftCenter #topCenter #bottomCenter)! !

!MOLayoutTest methodsFor: 'dominance tree layout' stamp: ''!
testHorizontalDominanceTreeLayout
	| view nodes node1 node2 node3 node4 x y |
	view := MOViewRenderer new.
	nodes := view nodes: (1 to: 4).
	view edges: {1 -> 2. 1-> 3. 1->4} from: #key to: #value.
	view horizontalDominanceTreeLayout.
	window := view open.
	
	node1 := nodes first.
	node2 := nodes second.
	node3 := nodes third.
	node4 := nodes fourth.
	
	x := nodes asSortedCollection: [:n1 :n2 | n1 bounds topLeft x < n2 bounds topLeft x].
	self assert: (x asArray = {node1 . node3. node2. node4}).

	y := nodes asSortedCollection: [:n1 :n2 | n1 bounds topLeft y < n2 bounds topLeft y].
	self assert: (y asArray = {node2 . node3. node1. node4})! !

!MOLayoutTest methodsFor: 'line layout' stamp: ''!
testHorizontalLayoutStrech
	| view nodes outterNode innerNodes form form2 canvas2 firstInnerNode |
	view := MOViewRenderer new.

	view shape rectangle width: 150; height: 130.
	outterNode := view node: #foo forIt: [
		view shape rectangle width: #x; height: #y.
		innerNodes := view nodes: {10@30 . 40 @ 20}.
		view horizontalLineLayout stretchVertically. 
	].

	view root applyLayout.
	form := view getRenderedForm. 
	self assert: (outterNode bounds = ( 5@5 corner: 155@135)).
	self assert: (innerNodes collect: #bounds )= {5@5 corner: 15@125. 25@5 corner: 65@125}.
	self assert: (innerNodes collect: #absoluteBounds )= {(10@10 corner: 20@130). (30@10 corner: 70@130)}.
	
	firstInnerNode := innerNodes first.
	self assert: (firstInnerNode shape extentFor: firstInnerNode) = firstInnerNode bounds extent.
	
	"Checking the bitmap"
	form2 := Form extent: view root bounds extent depth: Display depth.
	canvas2 := FormCanvas on: form2.
	canvas2 
			frameAndFillRectangle: ( 5@5 corner: 155@135)
				fillColor: Color white
				borderWidth: 1
				borderColor: Color black.
	canvas2 
				frameAndFillRectangle: (10@10 corner: 20@130)
					fillColor: Color white
					borderWidth: 1
					borderColor: Color black.
	canvas2 
				frameAndFillRectangle: (30@10 corner: 70@130)
					fillColor: Color white
					borderWidth: 1
					borderColor: Color black.

	self assert: form bits = form2 bits
! !

!MOLayoutTest methodsFor: 'line layout' stamp: ''!
testHorizontalLayoutStrech2
	| view nodes |
	view := MOViewRenderer new.

	view node: #foo forIt: [
		view shape rectangle width: #x; height: #y.
		nodes := view nodes: {10@30 . 40 @ 20}.
		view horizontalLineLayout stretchVertically. 
	].

	window := view open.

	self assert: (nodes collect: #bounds )=  {5@5 corner: 15@35. 25@5 corner: 65@35}! !

!MOLayoutTest methodsFor: 'line layout' stamp: ''!
testHorizontalLayoutStrech3
	| view nodes |
	view := MOViewRenderer new.

	view shape rectangle width: 150; height: 130.
	view node: #foo forIt: [
		view shape rectangle width: #x; height: #y.
		nodes := view nodes: {10@30 . 40 @ 20}.
		view horizontalLineLayout stretchHorizontally. 
	].

	window := view open.
	self assert: (nodes collect: #bounds) = {5@5 corner: 31@35. 41@5 corner: 145@25}.
! !

!MOLayoutTest methodsFor: 'line layout' stamp: ''!
testHorizontalLineLayout
	| shape view nodes |
	view := MOViewRenderer new.

	shape := MORectangleShape new width: #yourself; height: #yourself.
	shape fillColor: Color blue.
	view shape: shape.
	nodes := view nodes: #(10 30 50).
	view horizontalLineLayout alignment: #center.
	
	window := view open.

	self assert: nodes first bounds = (5@25 corner: 15@35).
	self assert: nodes second bounds = (25@15 corner: 55@45).	
	self assert: nodes third bounds = (65@5 corner: 115@55).
	
! !

!MOLayoutTest methodsFor: 'line layout' stamp: ''!
testHorizontalLineLayout2
	| shape view nodes |
	view := MOViewRenderer new.

	shape := MORectangleShape new width: #yourself; height: #yourself.
	shape fillColor: Color blue.
	view shape: shape.
	nodes := view nodes: #(10 30 50).
	view horizontalLineLayout alignment: #bottom.
	
	window := view open.

	self assert: nodes first bounds = (5@45 corner: 15@55).
	self assert: nodes second bounds = (25@25 corner: 55@55).	
	self assert: nodes third bounds = (65@5 corner: 115@55).
	! !

!MOLayoutTest methodsFor: 'line layout - strech' stamp: ''!
testHorizontalLineLayoutStretchHorizontally
	| view bigNode smallNodes |

	view := MOViewRenderer new.

	view shape rectangle width: 400.
	bigNode := view node: #foo forIt: [
		smallNodes := view nodes: (1 to: 5).
		view horizontalLineLayout stretchHorizontally
	].

	window := view open.

	self assert: (bigNode bounds) width = 400.

	self assert: (smallNodes collect: #absoluteBounds) = {10@10 corner: 80@15. 90@10 corner: 160@15. 170@10 corner: 240@15. 250@10 corner: 320@15. 330@10 corner: 400@15}! !

!MOLayoutTest methodsFor: 'line layout - strech' stamp: ''!
testHorizontalLineLayoutStretchHorizontallyNested


	| view bigNode innerNode innerInnerNodes |
	view := MOViewRenderer new.

bigNode := view node: 1 forIt: [
	view shape rectangle width: 100.
	view node: #foo.
	innerNode := view node: #bar forIt:[
		innerInnerNodes := view nodes: #(1 2).
		view horizontalLineLayout stretchHorizontally
	].
	view verticalLineLayout stretchHorizontally].



	window := view open.
	self assert: (bigNode bounds) width = 110.
	self assert: (innerNode bounds) width = 100.
	self assert: (innerInnerNodes collect: #width) = #(40 40).
	self assert: (innerInnerNodes collect: #bounds) =  {5@5 corner: 45@10. 55@5 corner: 95@10}.! !

!MOLayoutTest methodsFor: 'line layout - strech' stamp: ''!
testHorizontalLineLayoutStretchHorizontallyNestedNoHorizontalGap


	| view bigNode innerNode innerInnerNodes |
	view := MOViewRenderer new.

bigNode := view node: 1 forIt: [
	view shape rectangle width: 100.
	view node: #foo.
	innerNode := view node: #bar forIt:[
		innerInnerNodes := view nodes: #(1 2).
		view horizontalLineLayout horizontalGap: 0; stretchHorizontally
	].
	view verticalLineLayout stretchHorizontally].



	window := view open.
	self assert: (bigNode bounds) width = 110.
	self assert: (innerNode bounds) width = 100.
	self assert: (innerInnerNodes collect: #width) = #(45 45).
	self assert: (innerInnerNodes collect: #bounds) = {5@5 corner: 50@10. 50@5 corner: 95@10}! !

!MOLayoutTest methodsFor: 'line layout - strech' stamp: ''!
testHorizontalLineLayoutStretchHorizontallyNestedNoHorizontalOutGap


	| view bigNode innerNode innerInnerNodes |
	view := MOViewRenderer new.

	bigNode := view node: 1 forIt: [
		view shape rectangle width: 100.
		view node: #foo.
		innerNode := view node: #bar forIt:[
			innerInnerNodes := view nodes: #(1 2).
			view horizontalLineLayout horizontalOutGap: 0; stretchHorizontally
		].
	view verticalLineLayout stretchHorizontally].



	window := view open.
	self assert: (bigNode bounds) width = 110.
	self assert: (innerNode bounds) width = 100.
	self assert: (innerInnerNodes collect: #width) = #(45 45).
	self assert: (innerInnerNodes collect: #bounds) = {0@5 corner: 45@10. 55@5 corner: 100@10}! !

!MOLayoutTest methodsFor: 'line layout - strech' stamp: ''!
testHorizontalLineLayoutStretchHorizontallyNoHorizontalGap
	| view bigNode smallNodes |

	view := MOViewRenderer new.

	view shape rectangle width: 400.
	bigNode := view node: #foo forIt: [
		smallNodes := view nodes: (1 to: 5).
		view horizontalLineLayout horizontalOutGap: 0; stretchHorizontally
	].

	window := view open.
	self assert: (bigNode bounds) width = 400.

	self assert: (smallNodes collect: #absoluteBounds) = {5@10 corner: 77@15. 87@10 corner: 159@15. 169@10 corner: 241@15. 251@10 corner: 323@15. 333@10 corner: 405@15}! !

!MOLayoutTest methodsFor: 'line layout - strech' stamp: ''!
testHorizontalLineLayoutStretchHorizontallyNoHorizontalGap2


	| view bigNode smallNode |
	view := MOViewRenderer new.

view shape rectangle width: 400.
bigNode := view node: #foo forIt: [
	smallNode := view node: #bar.
	view horizontalLineLayout horizontalOutGap: 0; stretchHorizontally
].

	window := view open.
	self assert: (bigNode bounds) width = 400.
	self assert: (smallNode bounds) width = 400.! !

!MOLayoutTest methodsFor: 'line layout - strech' stamp: ''!
testHorizontalLineLayoutStretchVertically
	| view bigNode smallNode1 smallNode2 |

	view := MOViewRenderer new.

view shape rectangle width: 50; height: 30.
bigNode := view node: #abc forIt: [
	view shape rectangle width: 50; height: 30.
	smallNode1 := view node: #foo.
	view shape rectangle width: 20; height: 10.
	smallNode2 := view node: #bar.
	view horizontalLineLayout stretchVertically
].

	window := view open.

	self assert: ({bigNode . smallNode1 . smallNode2} collect: #bounds) = {5@5 corner: 95@45. 5@5 corner: 55@35. 65@5 corner: 85@35}.

	self assert: ({bigNode . smallNode1 . smallNode2} collect: #absoluteBounds) = {5@5 corner: 95@45. 10@10 corner: 60@40. 70@10 corner: 90@40}! !

!MOLayoutTest methodsFor: 'line layout - strech' stamp: ''!
testHorizontalLineLayoutStretchVerticallyNested
	| view bigNode smallNode1 smallNode2 |

	view := MOViewRenderer new.

view shape rectangle width: 50; height: 30.
bigNode := view node: #abc forIt: [
	smallNode1 := view node: #foo forIt: [
			view shape rectangle width: 50; height: 30.
			view node: #foofoo
	].
	view shape rectangle width: 20; height: 10.
	smallNode2 := view node: #bar.
	view horizontalLineLayout stretchVertically
].

	window := view open.

	self assert: ({bigNode . smallNode1 . smallNode2} collect: #bounds) = {5@5 corner: 105@55. 5@5 corner: 65@45. 75@5 corner: 95@45}.

	self assert: ({bigNode . smallNode1 . smallNode2} collect: #absoluteBounds) =  {5@5 corner: 105@55. 10@10 corner: 70@50. 80@10 corner: 100@50}! !

!MOLayoutTest methodsFor: 'line layout - strech' stamp: ''!
testHorizontalLineLayoutStretchVerticallyNestedNested
	| view bigNode smallNode1 smallNode2 smallNode3 |

	view := MOViewRenderer new.

view shape rectangle height: 60.
bigNode := view node: #foo forIt: [
	smallNode1 := view node: #a.
	smallNode2 := view node: #b forIt: [ 
		smallNode3 := view node: #c.
		view horizontalLineLayout stretchVertically
	].
	view horizontalLineLayout stretchVertically
].

	window := view open.

	self assert: ({bigNode . smallNode1 . smallNode2. smallNode3} collect: #bounds) =  {5@5 corner: 45@65. 5@5 corner: 10@55. 20@5 corner: 35@55. 5@5 corner: 10@45}.

	self assert: ({bigNode . smallNode1 . smallNode2 . smallNode3} collect: #absoluteBounds) = {5@5 corner: 45@65. 10@10 corner: 15@60. 25@10 corner: 40@60. 30@15 corner: 35@55}! !

!MOLayoutTest methodsFor: 'with timeline' stamp: ''!
testHorizontalTime2
	" 
	self debug: #testHorizontalTime2
	"

	| view |
	view := MOViewRenderer new.
	view nodes: (1 to: 6).
	view edgesFromAssociations: {1 -> 5. 2 -> 3 . 3 -> 4 . 1 -> 4 . 5 -> 6}.
	view layout: (MOHorizontalDominanceTreeWithTimelineLayout new timeBlock: [:v | v * 10]).
	
	self shouldnt: [ view root applyLayout ] raise: Error.
	
	self assert: (view root nodes collect: [: n | n bounds topLeft ]) asArray = {(15@5.0). (25@13.0). (60@13.0). (95@13.0). (80@5.0). (115@5.0)}! !

!MOLayoutTest methodsFor: 'horizontal tree layout' stamp: ''!
testHorizontalTreeLayout
	| view nodes node1 node2 node3 node4 x y |
	view := MOViewRenderer new.
	nodes := view nodes: (1 to: 4).
	view edges: {1 -> 2. 1-> 3. 1->4} from: #key to: #value.
	view horizontalTreeLayout.
	window := view open.
	
	node1 := nodes first.
	node2 := nodes second.
	node3 := nodes third.
	node4 := nodes fourth.
	
	x := nodes asSortedCollection: [:n1 :n2 | n1 bounds topLeft x < n2 bounds topLeft x].
	"self assert: (x asArray = {node1 . node3. node2. node4})."
	self assert: x asArray first == node1.

	"y := nodes asSortedCollection: [:n1 :n2 | n1 bounds topLeft y < n2 bounds topLeft y].
	self assert: (y asArray = {node2 . node3. node1. node4})"! !

!MOLayoutTest methodsFor: 'testBulletTree' stamp: ''!
testLayeredBulletTree

	| view form |
	view := MOViewRenderer new.
	view nodes: Collection withAllSubclasses.
	view edgesFrom: #superclass.
	view bulletTreeLayout layered.
	view root applyLayout.
	form := view getRenderedForm.
	"form openAsMorph"
! !

!MOLayoutTest methodsFor: 'horizontal tree layout' stamp: ''!
testLayoutingTreeLayout
	| view nodes node1 node2 node3 node4 |
	view := MOViewRenderer new.
	view shape rectangle height: [:x | x * 10].
	nodes := view nodes: (1 to: 4).
	view edges: {1 -> 2. 3->4} from: #key to: #value.
	view treeLayout layered; verticalGap: 10.
	view root applyLayout.
	
	node1 := nodes first.
	node2 := nodes second.
	node3 := nodes third.
	node4 := nodes fourth.
	
	self assert: node1 absoluteBounds top = 5.
	self assert: node2 absoluteBounds top = 45.
	self assert: node3 absoluteBounds top = 5.
	self assert: node4 absoluteBounds top = 45.! !

!MOLayoutTest methodsFor: 'loops' stamp: ''!
testLoopWithNoRoot
	
	| view |
	view := MOViewRenderer new.
	view nodes: #(1 2 3 4 5 6).
	view
		edge: 1 -> 2
		from: #key
		to: #value.
	view
		edge: 2 -> 3
		from: #key
		to: #value.
	view
		edge: 3 -> 1
		from: #key
		to: #value.
	view narrowTreeLayout.
	window := view open! !

!MOLayoutTest methodsFor: 'test layout' stamp: ''!
testMOTreeLayoutWithLocalEdges
	"The tree layout should only take local edges in to account"
	
	| view edges model layout |
	model := MOTestModel new.
	view := MOViewRenderer new.
	view
		nodes: model allNodes
		using: model shapeForNode.
	edges := view
		localEdges: model allEdges
		from: [:entity | entity first]
		to: [:entity | entity last]
		using: model shapeForEdge.	"Add an other edge that should not be taken in to account."
	view
		edge:
			(Array
				with: model a1
				with: model d1)
		from: [:entity | entity first]
		to: [:entity | entity last]
		using: model shapeForEdge.
	layout := MOTreeLayout withEdges: edges.
	view layout: layout.
	window := view open.
	self
		assert:
			(layout childrenFor: (view root deepFindNodeWithEntity: model a1)) size = 2.
	self
		assert:
			(layout childrenFor: (view root deepFindNodeWithEntity: model d1)) isEmpty! !

!MOLayoutTest methodsFor: 'line layout - strech' stamp: ''!
testMixOfLayout

	| view |
	view := MOViewRenderer new.

	view node: 'B' forIt:[
			view node: 'C'.
			view shape: (MORectangleShape new borderWidth: 2; borderColor: Color red).
			view node: 'D' forIt: [view nodes: #(1 2 3). view horizontalTreeLayout].
			view horizontalLineLayout gapSize:4;stretchVertically].


	self shouldnt: [window := view open] raise: Error.
	
! !

!MOLayoutTest methodsFor: 'line layout - strech' stamp: ''!
testMixOfLayout2

	| view |
	view := MOViewRenderer new.

	view node: 'B' forIt:[
			view node: 'C'.
			view shape: (MORectangleShape new borderWidth: 2; borderColor: Color red).
			view node: 'D' forIt: [view nodes: #(1 2 3). view horizontalTreeLayout].
			view horizontalLineLayout gapSize:4;stretchVertically].


	self shouldnt: [ window := view open ] raise: Error.
	
! !

!MOLayoutTest methodsFor: 'narrow tree' stamp: ''!
testNarrowHorizontalTreeFromToPositions
	| layout |
	layout := MOHorizontalNarrowTreeLayout new.
	self assert: layout fromPositions = #(rightCenter).
	self assert: layout toPositions = #(leftCenter).! !

!MOLayoutTest methodsFor: 'narrow tree' stamp: ''!
testNarrowVerticalTreeFromToPositions
	| layout |
	layout := MOVerticalNarrowTreeLayout new.
	self assert: layout fromPositions = #(bottomCenter).
	self assert: layout toPositions = #(topCenter).! !

!MOLayoutTest methodsFor: 'forms - tests' stamp: ''!
testNegativeGap
	
	| builder view nodeA nodeB nodeC |
	"Initialize the layout"
	builder := MOFormsBuilder new.
	builder columns: 'pref, -1px, pref, -1px, pref'.
	builder rows: 'pref'.	"Add nodes"
	view := MOViewRenderer new.
	view
		shape:
			(MORectangleShape
				width: 40
				height: 50
				fillColor: Color green).

	nodeA := view node: #a.
	view
		shape:
			(MORectangleShape
				width: 20
				height: 90
				fillColor: Color blue).

	nodeB := view node: #b.
	view
		shape:
			(MORectangleShape
				width: 80
				height: 20
				fillColor: Color yellow).

	nodeC := view node: #c.	"Position the components"
	builder x: 1 y: 1 add: nodeA.
	builder x: 3 y: 1 add: nodeB.
	builder x: 5 y: 1 add: nodeC.	"Apply layout"
	view layout: builder asLayout.
	
	window := view open.
	self assert: nodeA bounds = ( 0@0 corner: 40@50).
	self assert: nodeB bounds = ( 39@0 corner: 59@90).
	self assert: nodeC bounds = ( 58@0 corner: 138@20).
! !

!MOLayoutTest methodsFor: 'forms - tests' stamp: ''!
testNested
	
	| builder view node |
	"Initialize the layout"
	builder := MOFormsBuilder new.
	builder columns: 'pref'.
	builder rows: 'pref'.
	view := MOViewRenderer new.
	node := view
		node: #a
		using: MORectangleShape withBorder
		forIt:
			[view nodes: #(1 2 3 4 5).
			view layout: MOVerticalLineLayout new].
	builder x: 1 y: 1 add: node.
	view layout: builder asLayout.
	window := view open.

	self assert: node bounds = ( 0@0 corner: 15@75)! !

!MOLayoutTest methodsFor: 'abstractlayout' stamp: ''!
testNoLeafAbstractLayout
	self assert: (MOLayout allSubclasses 
					select: [:cls | cls subclasses isEmpty and: [ cls isAbstractClass ] ]) isEmpty
	! !

!MOLayoutTest methodsFor: 'test layout' stamp: ''!
testNodeFigureResize
	
	| model view box |
	model := MOTestModel new.
	view := MOViewRenderer new.
	view
		node: #aNode
		using: MORectangleShape new
		forIt:
			[view
				nodes: model allNodes
				using: MORectangleShape new.
			view
				localEdges: model allEdges
				from: [:entity | entity first]
				to: [:entity | entity last]
				using: model shapeForEdge.
			view layout: MOVerticalNarrowTreeLayout new.
			box := view root].
	window := view open.
	self assert: box origin = (5 @ 5).
	box applyLayout.	"Reapply the layout"
	self assert: box origin = (0@0).

	view root applyLayout.	"Reapply the layout"
	self assert: box origin = (5 @ 5).
! !

!MOLayoutTest methodsFor: 'test layout' stamp: ''!
testParentNodes
	
	| view layout nodeOne nodeTwo |
	layout := MOTreeLayout new.
	view := MOViewRenderer new.
	nodeOne := view node: 1.
	nodeTwo := view node: 2.
	view
		edge: 1 -> 2
		from: #key
		to: #value.
	view layout: layout.
	window := view open.
	self assert: (layout parentsFor: nodeOne) isEmpty.
	self assert: (layout parentsFor: nodeTwo) size = 1.
	self assert: (layout parentsFor: nodeTwo) first = nodeOne! !

!MOLayoutTest methodsFor: 'radial tree layout' stamp: ''!
testRadialTreeLayout
	| view nodes node1 node2 node3 node4 x y |
	view := MOViewRenderer new.
	nodes := view nodes: (1 to: 4).
	view edges: {1 -> 2. 1-> 3. 1->4} from: #key to: #value.
	view radialTreeLayout.
	window := view open.
	
	node1 := nodes first.
	node2 := nodes second.
	node3 := nodes third.
	node4 := nodes fourth.
	
	x := nodes asSortedCollection: [:n1 :n2 | n1 bounds topLeft x < n2 bounds topLeft x].
	self assert: (x asArray = {node3 . node1. node4. node2}).

	y := nodes asSortedCollection: [:n1 :n2 | n1 bounds topLeft y < n2 bounds topLeft y].
	self assert: (y asArray = {node4 . node1. node3. node2}).

	self assert: ((nodes collect: #absoluteBounds)  collect: #truncated) = {(30@30 corner: 35@35). (44@54 corner: 49@59). (3@30 corner: 8@35). (44@7 corner: 49@12)}! !

!MOLayoutTest methodsFor: 'utils' stamp: ''!
testRootNodes
	
	| view layout nodes |
	layout := MOTreeLayout new.
	view := MOViewRenderer new.
	nodes := view nodes: #(1 2 3 4).
	view
		edge: 1 -> 2
		from: #key
		to: #value.
	view
		edge: 2 -> 3
		from: #key
		to: #value.
	view layout: layout.
	window := view open.
	self assert: (layout rootNodesFor: nodes) size = 2.
	self assert: (layout rootNodesFor: nodes) first = nodes first.
	self assert: (layout rootNodesFor: nodes) last = nodes last! !

!MOLayoutTest methodsFor: 'forms - tests' stamp: ''!
testRunWithWidthsHeights

	| builder view nodes nodeA nodeB nodeC |
	builder := MOFormsBuilder new.
	builder columns: 'pref, -1px, pref, -1px, pref'.
	builder rows: 'pref'.	"Add nodes"
	view := MOViewRenderer new.
	view
		shape:
			(MORectangleShape
				width: 40
				height: 50
				fillColor: Color green).

	nodeA := view node: #a.
	view
		shape:
			(MORectangleShape
				width: 20
				height: 90
				fillColor: Color blue).

	nodeB := view node: #b.
	view
		shape:
			(MORectangleShape
				width: 80
				height: 20
				fillColor: Color yellow).

	nodeC := view node: #c.	"Position the components"
	builder x: 1 y: 1 add: nodeA.
	builder x: 3 y: 1 add: nodeB.
	builder x: 5 y: 1 add: nodeC.	"Apply layout"
	view layout: builder asLayout.
	view root applyLayout.
	
	builder runWithWidths: #(40 -1 20 -1 80) heights: #(90).
	self should: [builder runWithWidths: #(40 20 -1 80) heights: #(90)] raise: Error.
	self should: [builder runWithWidths: #(40 -1 20 -1 80) heights: #()] raise: Error.	
! !

!MOLayoutTest methodsFor: 'scatterplot' stamp: ''!
testScatterplotLayout
	
	| view nodes |
	view := MOViewRenderer new.
		
	nodes := view
		nodes: #(#(10 20) #(30 40))
		using: MORectangleShape withBorder.
	view layout: (MOScatterplotLayout new 
						x: [:entity | entity first ];
						y: [:entity | entity last ]).
	window := view open.
	self assert: nodes first bounds topLeft x = 10.
	self assert: nodes first bounds topLeft y = 20.

	self assert: nodes last bounds topLeft x = 30.
	self assert: nodes last bounds topLeft y = 40.
! !

!MOLayoutTest methodsFor: 'scatterplot' stamp: ''!
testScatterplotLayoutBounds
	"
	self debug: #testScatterplotLayoutBounds
	"
	| view nodes |
	view := MOViewRenderer new.
		
	view node: #foo forIt: [
		view layout: (MOScatterplotLayout new 
							x: #first ;
							y: #last).
		nodes := view nodes: #(#(10 20) #(30 40)).
	].

	window := view open.
	self assert: nodes first bounds topLeft x = 10.
	self assert: nodes first bounds topLeft y = 20.

	self assert: nodes last bounds topLeft x = 30.
	self assert: nodes last bounds topLeft y = 40.
! !

!MOLayoutTest methodsFor: 'scatterplot' stamp: ''!
testScatterplotLayoutBoundsWithUpSideDown
	"
	self debug: #testScatterplotLayoutBoundsWithUpSideDown
	"
	| view nodes |
	view := MOViewRenderer new.
		
	view node: #foo forIt: [
		view layout: (MOScatterplotLayout new 
							x: #first ;
							y: #last;
							upSideDown).
		nodes := view nodes: #(#(10 20) #(30 40)).
	].

	window := view open.
	self assert: nodes first bounds topLeft x = 10.
	self assert: nodes first bounds topLeft y = 20.

	self assert: nodes last bounds topLeft x = 30.
	self assert: nodes last bounds topLeft y = 40.

! !

!MOLayoutTest methodsFor: 'scatterplot' stamp: ''!
testScatterplotLayoutOffset
	
	| view nodes |
	view := MOViewRenderer new.
		
	nodes := view
		nodes: #(#(10 20) #(30 40))
		using: MORectangleShape withBorder.
	view layout: (MOScatterplotLayout new 
						offset: 2;
						x: [:entity | entity first ];
						y: [:entity | entity last ]).
	window := view open.
	self assert: nodes first bounds topLeft x = 12.
	self assert: nodes first bounds topLeft y = 22.

	self assert: nodes last bounds topLeft x = 32.
	self assert: nodes last bounds topLeft y = 42.
! !

!MOLayoutTest methodsFor: 'scatterplot' stamp: ''!
testScatterplotLayoutScalled

	| view nodes |
	view := MOViewRenderer new.
		
	view node: #foo forIt: [
		view layout: (MOScatterplotLayout new 
							x: #first ;
							y: #last
							;
							scaledToWidth: 20;
							scaledToHeight: 15 ).
		nodes := view nodes: #(#(10 20) #(30 40)).
	].

	window := view open.
	self assert: nodes first bounds topLeft x = 6.
	self assert: nodes first bounds topLeft y =  7.

	self assert: nodes last bounds topLeft x = 20.
	self assert: nodes last bounds topLeft y = 15.

! !

!MOLayoutTest methodsFor: 'loops' stamp: ''!
testSimpleLoop
	
	| view |
	view := MOViewRenderer new.
	view nodes: #(1 2 3 4).
	view
		edge: 1 -> 2
		from: #key
		to: #value.
	view
		edge: 2 -> 3
		from: #key
		to: #value.
	view
		edge: 3 -> 4
		from: #key
		to: #value.
	view
		edge: 4 -> 2
		from: #key
		to: #value.
	view treeLayout.
! !

!MOLayoutTest methodsFor: 'sort-block' stamp: ''!
testSortBlockA
	
	| a b list |
	a := #a -> (SortedCollection with: 1 with: 4 with: 7).
	b := #b -> (SortedCollection with: 3 with: 8).
	list := SortedCollection sortBlock: MOSortBlock new.
	list
		add: a;
		add: b.
	self assert: list first key = #b.
	list := SortedCollection sortBlock: MOSortBlock new.
	list
		add: b;
		add: a.
	self assert: list first key = #b! !

!MOLayoutTest methodsFor: 'sort-block' stamp: ''!
testSortBlockB
	
	| a b list |
	a := #a -> (SortedCollection with: 3 with: 4 with: 9).
	b := #b -> (SortedCollection with: 1 with: 5 with: 9).
	list := SortedCollection sortBlock: MOSortBlock new.
	list
		add: a;
		add: b.
	self assert: list first key = #b.
	list := SortedCollection sortBlock: MOSortBlock new.
	list
		add: b;
		add: a.
	self assert: list first key = #b! !

!MOLayoutTest methodsFor: 'sort-block' stamp: ''!
testSortBlockC
	
	| a b list |
	a := #a -> (SortedCollection with: 3 with: 3).
	b := #b -> (SortedCollection with: 3 with: 3).
	list := SortedCollection sortBlock: MOSortBlock new.
	list
		add: a;
		add: b.
	self assert: list first key = #a.
	list := SortedCollection sortBlock: MOSortBlock new.
	list
		add: b;
		add: a.
	self assert: list first key = #b! !

!MOLayoutTest methodsFor: 'sort-block' stamp: ''!
testSortBlockD
	
	| a b list |
	a := #a -> (SortedCollection with: 3 with: 3).
	b := #b -> (SortedCollection with: 3).
	list := SortedCollection sortBlock: MOSortBlock new.
	list
		add: a;
		add: b.
	self assert: list first key = #a.
	list := SortedCollection sortBlock: MOSortBlock new.
	list
		add: b;
		add: a.
	self assert: list first key = #a! !

!MOLayoutTest methodsFor: 'sort-block' stamp: ''!
testSortBlockE
	
	| a b list |
	a := #a
		->
			((SortedCollection new)
				add: 2;
				add: 3;
				add: 4;
				add: 7;
				yourself).
	b := #b
		->
			((SortedCollection new)
				add: 2;
				add: 3;
				add: 5;
				add: 7;
				yourself).
	list := SortedCollection sortBlock: MOSortBlock new.
	list
		add: a;
		add: b.
	self assert: list first key = #b.
	list := SortedCollection sortBlock: MOSortBlock new.
	list
		add: b;
		add: a.
	self assert: list first key = #b! !

!MOLayoutTest methodsFor: 'spring layout' stamp: ''!
testSpringLayout1
	| view nodes layout edge node1 node2 |
	view := MOViewRenderer new.
"
	view interaction 
		every: 10 do: [ :v | view layout doExecute: view root. view updateWindow ].
"
	view shape rectangle size: 15.
	nodes := view nodes: #(1 2).
	edge := view edge: nil from: 1 to: 2.
	layout := MOSpringLayout new.
	view layout: (layout).

	"view layout doExecute: view root"

	view root applyLayout.
	window := view open.

	node1 := nodes first.
	node2 := nodes second.	
	self assert: (layout hookeAttractionFor: node1 and: edge) = (layout hookeAttractionFor: node2 and: edge) negated.
	self assert: (layout coulombRepulsion: node1 and: node2) = (layout coulombRepulsion: node2 and: node1) negated.
! !

!MOLayoutTest methodsFor: 'spring layout' stamp: ''!
testSpringLayout2
	| view nodes edges layout edge node1 node2 |
	view := MOViewRenderer new.
"
	view interaction 
		every: 30 do: [ :v | view layout doExecute: view root. view updateWindow ].
"
	view shape rectangle size: 15.
	nodes := view nodes: (1 to: 5).
	edge := view edges: ((2 to: 5) collect: [:v | 1 -> v]) from: #key to: #value.
	layout := MOSpringLayout new.
	view layout: (layout).

	"view layout doExecute: view root"

	view root applyLayout.
	window := view open.

"
	10 timesRepeat: [ view layout doExecute: view root. view updateWindow ].
"

"	node1 := nodes first.
	node2 := nodes second.	
	self assert: (layout hookeAttractionFor: node1 and: edge) = ( 0.1@0.0).
	self assert: (layout hookeAttractionFor: node2 and: edge) = ( -0.1@0.0).
	self assert: (layout coulombRepulsion: node1 and: node2).
	self halt.
"! !

!MOLayoutTest methodsFor: 'utils' stamp: ''!
testSwapElement
	self assert: (#($a $b $c $d) copy swapElement: $a withElement: $d) = #($d $b $c $a)! !

!MOLayoutTest methodsFor: 'test layout' stamp: ''!
testToPosition
	
	| view handle |
	view := MOViewRenderer new.
	view nodes: #(1 2 3).
	handle := view
		edge: 1 -> 2
		from: #key
		to: #value.
	view
		layout:
			(MOVerticalNarrowTreeLayout new toPositions:
					#(#rightCenter #leftCenter #topCenter #bottomCenter)).
	window := view open.
	self
		assert:
			handle value toPositions =
					#(#rightCenter #leftCenter #topCenter #bottomCenter)! !

!MOLayoutTest methodsFor: 'test layout' stamp: ''!
testTransparentEdges
	"
	self debug: #testTransparentEdges

	"
	| view edges model form |
	model := MOTestModel new.
	view := MOViewRenderer new.
	view
		nodes: model allNodes
		using:
			(MORectangleShape new width: 60; height: 60 "fixedWidth: 60 fixedHeight: 60").
	edges := view
		localEdges: model allEdges
		from: [:entity | entity first]
		to: [:entity | entity last]
		using: model shapeForEdge.
	form := view getRenderedForm.
	self assert: edges value size = model allEdges size.
	view layout: MOTreeMapLayout new.
	view root deepApplyLayout.
	edges value do: [:edge | self deny: edge isVisible].
	view layout: MOTreeLayout new.
	view root deepApplyLayout.
	edges value do: [:edge | self assert: edge isVisible]! !

!MOLayoutTest methodsFor: 'tree' stamp: 'AlexandreBergel 5/13/2012 21:55'!
testTreeLayout
	| view  |
	view := MOViewRenderer new.
	
	view shape rectangle size: 10.
	view nodes: (1 to: 4).
	view edges: {1 -> 2 . 1 -> 3. 2 -> 4} from: #key to: #value.

	view treeLayout horizontalGap: 0.
	view root applyLayout.
	"the extent of each node should be rectange size"
		(view root nodes collect: #extent) do: [ :each | self assert: each = (10.0@10) ].
	"Nodes at same y-origin=35, x-origins are separated by two rectange sizes"	
		self assert: ({(5.0@65). (15.0@35). (10@5). (15.0@65). (5.0@35). (49.0@45)} includesAllOf:  (view root nodes collect: #origin)).  
	
	view treeLayout horizontalGap: 20.
	view root applyLayout.
	"the extent of each node should be rectange size"
		(view root nodes collect: #extent) do: [ :each | self assert: each = (10.0@10) ].
	"Nodes at same y-origin=35, separation increased by horiztonalGap"
		self assert: ( { (5.0@35). (5.0@65). (20@5). (35.0@35). (35.0@65) } includesAllOf: (view root nodes collect: #origin)  ).! !

!MOLayoutTest methodsFor: 'tree' stamp: 'AlexandreBergel 3/28/2012 18:30'!
testTreeLayoutGap
	| view bounds newbounds |
	view := MOViewRenderer new.
	view nodes: (1 to: 4).
	view edges: {1 -> 2 . 1 -> 3. 2 -> 4} from: #key to: #value.
	
	view treeLayout.
	view root applyLayout.

	bounds := view root nodes collect: [ :n | n bounds copy ].
	view treeLayout topGap: 10; leftGap: 8.
	view root applyLayout.
	newbounds := view root nodes collect: [ :n | n bounds copy ].
	
	self assert: (  {(9.0@5) corner: (14.0@10). (13.0@30) corner: (18.0@35). (13.0@55) corner: (18.0@60). (5.0@30) corner: (10.0@35). (5.0@55) corner: (10.0@60)} includesAllOf: bounds).
	
	self assert: ({(16.0@60) corner: (21.0@65). (8.0@60) corner: (13.0@65). (12.0@10) corner: (17.0@15). (16.0@35) corner: (21.0@40). (8.0@35) corner: (13.0@40)} includesAllOf: newbounds)! !

!MOLayoutTest methodsFor: 'treemap' stamp: ''!
testTreeMap

	| view nodes |
	view := MOViewRenderer new.
	nodes := view nodes: (1 to: 5).
	view edgesFromAssociations: {1->3. 1-> 5. 2->4}.
	view layout: MOTreeMapLayout new.
	view root applyLayout.

	self assert: (nodes collect: #bounds) = {(10@5 corner: 15@10). (25@5 corner: 30@10). (10@5 corner: 15@10). (25@5 corner: 30@10). (12@5 corner: 17@10)} ! !

!MOLayoutTest methodsFor: 'line layout' stamp: ''!
testVerticalLineLayout
	| shape view nodes |
	view := MOViewRenderer new.

	shape := MORectangleShape new width: #yourself; height: #yourself.
	shape fillColor: Color blue.
	view shape: shape.
	nodes := view nodes: #(10 30 50).
	view verticalLineLayout alignment: #center.
	
	window := view open.

	self assert: nodes first bounds = (25@5 corner: 35@15).
	self assert: nodes second bounds = (15@25 corner: 45@55).	
	self assert: nodes third bounds = (5@65 corner: 55@115).
	! !

!MOLayoutTest methodsFor: 'line layout' stamp: ''!
testVerticalLineLayout2
	| shape view nodes |
	view := MOViewRenderer new.

	shape := MORectangleShape new width: #yourself; height: #yourself.
	shape fillColor: Color blue.
	view shape: shape.
	nodes := view nodes: #(10 30 50).
	view verticalLineLayout alignment: #right.
	
	window := view open.

	self assert: nodes first bounds = (45@5 corner: 55@15).
	self assert: nodes second bounds = (25@25 corner: 55@55).	
	self assert: nodes third bounds = (5@65 corner: 55@115).
	
! !

!MOLayoutTest methodsFor: 'line layout - strech' stamp: ''!
testVerticalLineLayoutStretchHorizontally
	| view bigNode smallNode1 smallNode2 |

	view := MOViewRenderer new.

view shape rectangle width: 50; height: 30.
bigNode := view node: #abc forIt: [
	view shape rectangle width: 50; height: 30.
	smallNode1 := view node: #foo.
	view shape rectangle width: 20; height: 30.
	smallNode2 := view node: #bar.
	view verticalLineLayout stretchHorizontally
].

	window := view open.

	self assert: ({bigNode . smallNode1 . smallNode2} collect: #bounds) = {5@5 corner: 65@85. 5@5 corner: 55@35. 5@45 corner: 55@75}.

	self assert: ({bigNode . smallNode1 . smallNode2} collect: #absoluteBounds) =  {5@5 corner: 65@85. 10@10 corner: 60@40. 10@50 corner: 60@80}! !

!MOLayoutTest methodsFor: 'line layout - strech' stamp: ''!
testVerticalLineLayoutStretchHorizontallyNested
	| view bigNode smallNode1 smallNode2 |

	view := MOViewRenderer new.

view shape rectangle width: 50; height: 30.
bigNode := view node: #abc forIt: [
	smallNode1 := view node: #foo forIt: [
		view shape rectangle width: 50; height: 30.
		view node: #foofoo.
	].
	view shape rectangle width: 20; height: 30.
	smallNode2 := view node: #bar.
	view verticalLineLayout stretchHorizontally
].


	window := view open.

	self assert: ({bigNode . smallNode1 . smallNode2} collect: #bounds) =  {5@5 corner: 75@95. 5@5 corner: 65@45. 5@55 corner: 65@85}.

	self assert: ({bigNode . smallNode1 . smallNode2} collect: #absoluteBounds) =  {5@5 corner: 75@95. 10@10 corner: 70@50. 10@60 corner: 70@90}! !

!MOLayoutTest methodsFor: 'line layout - strech' stamp: ''!
testVerticalLineLayoutStretchHorizontallyNestedNested
	| view bigNode smallNode1 smallNode2 smallNode3 |

	view := MOViewRenderer new.

view shape rectangle width: 30.
bigNode := view node: #foo forIt: [
	smallNode1 := view node: #a.
	smallNode2 := view node: #b forIt: [ 
		smallNode3 := view node: #c.
		view verticalLineLayout stretchHorizontally
	].
	view verticalLineLayout stretchHorizontally
].

	window := view open.

	self assert: ({bigNode . smallNode1 . smallNode2} collect: #bounds) =   {5@5 corner: 35@45. 5@5 corner: 25@10. 5@20 corner: 25@35}.

	self assert: ({bigNode . smallNode1 . smallNode2} collect: #absoluteBounds) = {5@5 corner: 35@45. 10@10 corner: 30@15. 10@25 corner: 30@40}.
	
	self assert: (smallNode3 bounds = ( 5@5 corner: 15@10)).
	self assert: (smallNode3 absoluteBounds = ( 15@30 corner: 25@35))! !

!MOLayoutTest methodsFor: 'line layout - strech' stamp: ''!
testVerticalLineLayoutStretchHorizontallyNestedNoHorizontalGap

	| view smallNode smallInnerNodes |
	view := MOViewRenderer new.

view nodes: #('test test') forEach: [:nod |
	view shape: (MORectangleShape new text: [:meth | meth asString]; width: 200; height: 40; borderColor: Color gray).
	view node: nod.
	view shape:  (MORectangleShape new borderColor: Color green).
	smallNode := view node: nod forIt:[
			view shape rectangle withoutBorder.view node: nod.
			view shape: (MORectangleShape new borderWidth: 2; borderColor: Color red).
			view node: nod.
			view horizontalLineLayout gapSize:1;stretch].
	view verticalLineLayout 
		"gapSize:1"
		verticalGap: 0;
		horizontalGap: 0;
		verticalOutGap: 0;
		horizontalOutGap: 0;
		stretch].
view gridLayout.

	window := view open.

	self assert: (smallNode bounds = ( 0@40 corner: 200@47)).
	self assert: (smallNode absoluteBounds = ( 5@45 corner: 205@52)).
	smallInnerNodes := smallNode nodes.
	self assert: (smallInnerNodes size = 2).

	self assert: (smallInnerNodes collect: #bounds) asArray =  {(1@1 corner: 99@6). (101@1 corner: 199@6)}.
	
	self assert: (smallInnerNodes collect: #absoluteBounds) asArray =  {(6@46 corner: 104@51). (106@46 corner: 204@51)}! !

!MOLayoutTest methodsFor: 'line layout - strech' stamp: ''!
testVerticalLineLayoutStretchHorizontallyNoHorizontalGap

	| view smallNodes smallNode1 |
	view := MOViewRenderer new.

	view shape rectangle width: 400.
	view node: #foo forIt: [
		smallNodes := view nodes: (1 to: 5).
		view verticalLineLayout horizontalGap: 0; stretchHorizontally
	].

	window := view open.

	self assert: (smallNodes collect: #bounds) =  {5@5 corner: 395@10. 5@20 corner: 395@25. 5@35 corner: 395@40. 5@50 corner: 395@55. 5@65 corner: 395@70}.

	self assert: (smallNodes collect: #absoluteBounds) = {10@10 corner: 400@15. 10@25 corner: 400@30. 10@40 corner: 400@45. 10@55 corner: 400@60. 10@70 corner: 400@75}! !

!MOLayoutTest methodsFor: 'line layout - strech' stamp: ''!
testVerticalLineLayoutStretchHorizontallyNoHorizontalOutGap

	| view smallNodes smallNode1 |
	view := MOViewRenderer new.

	view shape rectangle width: 400.
	view node: #foo forIt: [
		smallNodes := view nodes: (1 to: 5).
		view verticalLineLayout horizontalOutGap: 0; stretchHorizontally
	].

	window := view open.

	self assert: (smallNodes collect: #bounds) = {0@5 corner: 400@10. 0@20 corner: 400@25. 0@35 corner: 400@40. 0@50 corner: 400@55. 0@65 corner: 400@70}.

	self assert: (smallNodes collect: #absoluteBounds) = {5@10 corner: 405@15. 5@25 corner: 405@30. 5@40 corner: 405@45. 5@55 corner: 405@60. 5@70 corner: 405@75}! !

!MOLayoutTest methodsFor: 'line layout - strech' stamp: ''!
testVerticalLineLayoutStretchVertically
	| view bigNode smallNodes |

	view := MOViewRenderer new.

	view shape rectangle height: 200.
	bigNode := view node: #foo forIt: [
		smallNodes := view nodes: (1 to: 5).
		view verticalLineLayout stretchVertically
	].

	window := view open.

	self assert: (bigNode bounds) height = 200.

	self assert: (smallNodes collect: #absoluteBounds) = {10@10 corner: 15@40. 10@50 corner: 15@80. 10@90 corner: 15@120. 10@130 corner: 15@160. 10@170 corner: 15@200}! !

!MOLayoutTest methodsFor: 'line layout - strech' stamp: ''!
testVerticalLineLayoutStretchVerticallyNoHorizontalGap
	| view bigNode smallNodes |

	view := MOViewRenderer new.

view shape rectangle height: 200.
bigNode := view node: #foo forIt: [
	smallNodes := view nodes: (1 to: 5).
	view verticalLineLayout horizontalGap: 0; stretchVertically
].

	window := view open.

	self assert: (bigNode bounds) height = 200.

	self assert: (smallNodes collect: #absoluteBounds) = {10@10 corner: 15@40. 10@50 corner: 15@80. 10@90 corner: 15@120. 10@130 corner: 15@160. 10@170 corner: 15@200}! !

!MOLayoutTest methodsFor: 'line layout - strech' stamp: ''!
testVerticalLineLayoutStretchVerticallyNoVerticalGap
	| view bigNode smallNodes |

	view := MOViewRenderer new.

view shape rectangle height: 200.
bigNode := view node: #foo forIt: [
	smallNodes := view nodes: (1 to: 5).
	view verticalLineLayout verticalGap: 0; stretchVertically
].

	window := view open.

	self assert: (bigNode bounds) height = 200.

	self assert: (smallNodes collect: #absoluteBounds) = {10@10 corner: 15@48. 10@48 corner: 15@86. 10@86 corner: 15@124. 10@124 corner: 15@162. 10@162 corner: 15@200}! !

!MOLayoutTest methodsFor: 'line layout - strech' stamp: ''!
testVerticalLineLayoutStretchVerticallyNoVerticalOutGap
	| view bigNode smallNodes |

	view := MOViewRenderer new.

	view shape rectangle height: 200.
	bigNode := view node: #foo forIt: [
		smallNodes := view nodes: (1 to: 5).
		view verticalLineLayout verticalOutGap: 0; stretchVertically
	].

	window := view open.

	self assert: (bigNode bounds) height = 200.

	self assert: (smallNodes collect: #absoluteBounds) = {10@5 corner: 15@37. 10@47 corner: 15@79. 10@89 corner: 15@121. 10@131 corner: 15@163. 10@173 corner: 15@205}! !

!MOLayoutTest methodsFor: 'with timeline' stamp: ''!
testVerticalTime
	" 
	self debug: #testVerticalTime
	"

	| view |
	view := MOViewRenderer new.
	view nodes: (1 to: 100 by: 10).
	view layout: (MOVerticalLineWithTimelineLayout new).
	
	self shouldnt: [ view root applyLayout ] raise: Error.
	
	self assert: (view root nodes collect: [: n | n bounds topLeft ]) asArray = {(0@5). (0@20). (0@35). (0@50). (0@65). (0@80). (0@95). (0@110). (0@125). (0@140)}! !

!MOLayoutTest methodsFor: 'with timeline' stamp: ''!
testVerticalTime2
	" 
	self debug: #testVerticalTime
	"

	| view |
	view := MOViewRenderer new.
	view nodes: (1 to: 100 by: 10).
	view layout: (MOVerticalLineWithTimelineLayout new timeBlock: #yourself).
	
	self shouldnt: [ view root applyLayout ] raise: Error.
	
	self assert: (view root nodes collect: [: n | n bounds topLeft ]) asArray =  {(1@5). (11@20). (21@35). (31@50). (41@65). (51@80). (61@95). (71@110). (81@125). (91@140)}! !

MondrianTest subclass: #MONewFormsBuilderTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Tests'!

!MONewFormsBuilderTest methodsFor: 'test - errors' stamp: ''!
testErrorBounds
	"We create a 'small' builder, and we add element at an outside location"

	"The size of the builder is 1x1"
	| builder shape |
	builder := MOFormsBuilder new.
	builder column; center; pref.
	builder row; pref.
	
	shape := MORectangleShape new.
	
	self shouldnt: [builder x: 1 y: 1 add: shape] raise: Error.
	self should: [builder x: 2 y: 1 add: shape] raise: Error.	
	self should: [builder x: 1 y: 2 add: shape] raise: Error.	
	self should: [builder x: 2 y: 2 add: shape] raise: Error.	! !

!MONewFormsBuilderTest methodsFor: 'test - errors' stamp: ''!
testErrorShape
	"Only shape must be added in a form builder"

	| builder shape |
	
	self deny: (Object new isShape).
	self assert: (MORectangleShape new isShape).
	
	builder := MOFormsBuilder new.
	builder column; center; pref.
	builder row; pref.
	
	shape := Object new.
	
	self shouldnt: [builder x: 1 y: 1 add: shape] raise: Error.
	self should: [builder x: 2 y: 1 add: shape] raise: Error.	
	self should: [builder x: 1 y: 2 add: shape] raise: Error.	
	self should: [builder x: 2 y: 2 add: shape] raise: Error.	! !

!MONewFormsBuilderTest methodsFor: 'test' stamp: ''!
testFormsShape
	
	| builder view shape rectangle firstAdaptor |
	rectangle := MORectangleShape width: [:each | 10] height: [:each | 15] fillColor: Color blue.
	builder := MOFormsBuilder new.
	builder column; center; pref.
	builder row; pref.
	builder x: 1 y: 1 add: rectangle.
	shape := builder shape.
	view := MOViewRenderer new.
	view node: #node using: shape.

	self assert: (shape class == MOFormsShape).
	self assert: (shape builder == builder).
	self assert: ((shape builder shapes collect: [:sh | sh class]) asArray = {MOShapeAdaptor }).
	
	"Adaptor for the rectangle shape"
	firstAdaptor := shape builder shapes first.
	self assert: (firstAdaptor component class == MORectangleShape ).
! !

!MONewFormsBuilderTest methodsFor: 'test' stamp: ''!
testLayout
	"This test ensure that applying layouts does not raise exception"
	| view layouts |
	layouts := {MOCircleLayout . MOGridLayout . MOBottomFlowLayout . MOTreeLayout . MORadialTreeLayout . MOForceBasedLayout}.
	
	view := MOViewRenderer new.
	view nodes: (1 to: 10).
	view edgesFrom: [:each | each * each].
	
	"we iterate over different layout"
	layouts do: [:l|
		view layout: l new.
		self shouldnt: [window := view open. window delete] raise: Error.
		]	! !

!MONewFormsBuilderTest methodsFor: 'test' stamp: ''!
testSimple
	
	| builder view shape figure rectangle |
	rectangle := MORectangleShape width: 10 height: 15 fillColor: Color blue.
	builder := MOFormsBuilder new.
	builder column; center; pref.
	builder row; pref.
	builder x: 1 y: 1 add: rectangle.
	shape := builder shape.
	view := MOViewRenderer new.
	figure := view node: #node using: shape.

	self assert: (figure bounds) =
					(Rectangle
						origin: 0 @ 0
						width: 10
						height: 15).

	window := view open.
	self assert: (figure bounds) =
					(Rectangle
						origin: 5 @ 5
						width: 10
						height: 15).
	self assert: (shape collectAllShapes size = 2).
! !

!MONewFormsBuilderTest methodsFor: 'test' stamp: ''!
testSimple2
	
	| builder view shape figure rectangle |
	rectangle := MORectangleShape width: [:each | 10] height: [:each | 15] fillColor: Color blue.
	builder := MOFormsBuilder new.
	builder column; center; pref.
	builder row; pref.
	builder x: 1 y: 1 add: rectangle.
	shape := builder shape.
	view := MOViewRenderer new.
	figure := view node: #node using: shape.

	self assert: (shape boundsFor: figure) =
					(Rectangle
						origin: 0 @ 0
						width: 10
						height: 15).

	window := view open.
	self assert: (shape boundsFor: figure) =
					( 5@5 extent: 10@15).
	
	"A form shape and a rectangle shape is contained"
	self assert: (shape collectAllShapes size = 2).
! !

!MONewFormsBuilderTest methodsFor: 'test' stamp: ''!
testSimple2FormsShape
	
	| builder view shape firstAdaptor secondAdaptor rectangleShape labelShape |
	builder := MOFormsBuilder new.
	builder column; center.
	builder row; pref; row; pref.
	builder x: 1 y: 1 add: (rectangleShape := MORectangleShape width: 10 height: 15 fillColor: Color blue).
	builder x: 1 y: 1 add: (labelShape := (MORectangleShape new withoutBorder; withText) text: [:entity | 'Hello']).
	shape := builder shape.
	view := MOViewRenderer new.
	view node: #bouba using: shape.

	self assert: (shape class == MOFormsShape).
	self assert: (shape builder == builder).
	self assert: ((shape builder shapes collect: [:sh | sh class]) asArray = {MOShapeAdaptor . MOShapeAdaptor}).
	
	"Adaptor for the rectangle shape"
	firstAdaptor := shape builder shapes first.
	self assert: (firstAdaptor component class == MORectangleShape ).
	
	"Adaptor for the children shape???? What the hell is that"
	secondAdaptor := shape builder shapes second.
	self assert: (secondAdaptor component class == MORectangleShape ).

	"Adaptor for the children shape"
"	thirdAdaptor := shape builder shapes third.
	self assert: (thirdAdaptor component class == MOChildrenShape ).
	self assert: (thirdAdaptor component parent isNil ).
	self assert: ((thirdAdaptor component instVarNamed: 'next') isNil ).
"	
	window := view open.

	"Adaptor for the children shape"
"	thirdAdaptor := shape builder shapes third.
	self assert: (thirdAdaptor component class == MOChildrenShape ).
	self assert: (thirdAdaptor component parent isNil ).
	self assert: ((thirdAdaptor component instVarNamed: 'next') isNil ).
"
! !

!MONewFormsBuilderTest methodsFor: 'test' stamp: ''!
testSimpleWithComposedRectangle
	
	| builder view shape figure rectangle |
	rectangle := MORectangleShape width: 10 height: 15 fillColor: Color blue.
	builder := MOFormsBuilder new.
	builder column; left; column; left.
	builder row; left; row; left.
	builder x: 1 y: 1 add: rectangle.
	builder x: 2 y: 1 add: (rectangle copy fillColor: Color green).
	builder x: 1 y: 2 add: (rectangle copy fillColor: Color red).
	shape := builder shape.
	view := MOViewRenderer new.
	figure := view node: #node using: shape.

	"The shape has a reference to itself"						
	self assert: (shape collectAllShapes size = 4).
	self assert: (shape collectAllShapes first  == shape).
	self assert: (shape collectAllShapes noneSatisfy: [:s | s isKindOf: MOShapeAdaptor]).

	"Before we open the window"
	self assert: (figure bounds) =
					(Rectangle
						origin: 0@0
						width: 20
						height: 30).


	window := view open.
	"After we have openend the window"
	self assert: (figure bounds) =
					(Rectangle
						origin: 5 @ 5
						width: 20
						height: 30).! !

!MONewFormsBuilderTest methodsFor: 'test' stamp: ''!
testSimpleWithLabel
	
	| builder view shape figure rectangleShape labelShape |
	builder := MOFormsBuilder new.
	builder column; center.
	builder row; pref; row; pref.
	builder x: 1 y: 1 add: (rectangleShape := MORectangleShape width: 10 height: 15 fillColor: Color blue).
	builder x: 1 y: 2 add: (labelShape := (MORectangleShape new withoutBorder; withText) text: [:entity | 'Hello']).
	shape := builder shape.
	view := MOViewRenderer new.
	figure := view node: #bouba using: shape.
			
	self assert: (shape collectAllShapes size = 3).
	self assert: (shape collectAllShapes first  == shape).
	self assert: (shape collectAllShapes noneSatisfy: [:s | s isKindOf: MOShapeAdaptor]).

	self assert: (shape collectAllShapes anySatisfy: [:s | s == labelShape]).
	self assert: (shape collectAllShapes anySatisfy: [:s | s == rectangleShape]).


	"Before we open the window"
	self assert: (figure bounds) = ( 0@0 corner: (labelShape widthFor: figure)@(15 + (labelShape heightFor: figure ))).

	"After we open the window"
	window := view open.
	self assert: (figure bounds) = ( 5@5 extent: (labelShape widthFor: figure)@(15 + (labelShape heightFor: figure ))).
! !

!MONewFormsBuilderTest methodsFor: 'test' stamp: ''!
testViewAndBuilder
	
	| builder view figure |
	view := MOViewRenderer new.
	view node: #node forIt: [
		builder := MOFormsBuilder new.
		builder column; center; pref.
		builder row; pref.
		builder x: 1 y: 1 add: (view node: #blah forIt: [view nodes: (1 to: 5)]).

		"view layout: builder asLayout"
	].

	window := view open.
	figure := view root nodes first.
	self assert: figure bounds = (5@5 corner: 90@30)! !

MondrianTest subclass: #MOSerializationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Tests'!

!MOSerializationTest methodsFor: 'test' stamp: ''!
testXML
	| view stream content |
	view := MOViewRenderer new.
	view shape rectangle 
		width: [:m | m asInteger - $a asInteger * 20 ];
		height: [:m | ((m asInteger - $a asInteger) * 20) + 10].
	view nodes: {$a . $b . $c}.
	view edges: {$a -> $b . $b -> $c . $c -> $a} from: #key to: #value.
	
	stream := WriteStream on: String new.
	self assert: (stream isEmpty).
	view canvas exportAsXMLOn: stream.
	self deny: (stream isEmpty).
	content := stream contents.

	self assert: (content = 
'<graph>
	<node id="1" name="$a width ="5" height ="10""/>
	<node id="2" name="$b width ="20" height ="30""/>
	<node id="3" name="$c width ="40" height ="50""/>
	<edge source="1" target= "2"/>
	<edge source="2" target= "3"/>
	<edge source="3" target= "1"/>
</graph>
')! !

MondrianTest subclass: #MOViewRendererTest
	instanceVariableNames: 'view'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Tests'!

!MOViewRendererTest commentStamp: 'AlexandreBergel 12/7/2010 11:39' prior: 0!
MOViewRendererTest is the main unit test of Mondrian. It tests all the basic features. Most of the test are self-contain short scenarios. This class can also be used as a good source of documentation!

!MOViewRendererTest methodsFor: 'bitmap cache' stamp: ''!
TODOtestResize
	| view nodes semaphore |
	view := MOViewRenderer new.
	nodes := view nodes: (1 to: 20) forEach: [:each  | view nodes: (1 to: 100). view gridLayout].

	self assert: (nodes allSatisfy: [:each | each hasCachedForm not]).  

	window := view open.
	
	semaphore := Semaphore new.
	[(Delay forSeconds: 2) wait. semaphore signal] fork.
	semaphore wait.
	"(Delay forSeconds: 2) wait."

	self assert: (nodes first hasCachedForm).  
	
	window extent: 300@500.
	self assert: (nodes allSatisfy: [:each | each hasCachedForm not]).  ! !

!MOViewRendererTest methodsFor: 'subviews' stamp: ''!
TOFIXtestUpdateNodeAndSubview
	| view nodes |
	view := MOViewRenderer new.
	nodes := view nodes: {1 . 10 . 20} forEach: [:each|
		view shape rectangle size: 20.
		view interaction 
			whenEnteringUpdateNode: [:node | 
				view nodes: (1 to: node).
				view edges: (1 to: node) from: #yourself to: [ :v | v \\ 3] fromGlobal: false toGlobal: false.
				view treeLayout.
				view updateWindow ] 
			whenLeavingUpdateNode:  [:node | ]  
			withLayoutUpdate: true.
		view node: each].

	window := view open.
	
	self assert: (nodes first bounds = ( 5@5 corner: 35@35)).
	nodes first announce: (MOMouseEnter new).
	self assert: (nodes first bounds ~= ( 5@5 corner: 35@35)).
! !

!MOViewRendererTest methodsFor: 'type covering' stamp: ''!
covers: oneArgBlock for: aSymbol
	| types |
	types := self perform: aSymbol.
	types do: [:c | 
		self shouldnt: [oneArgBlock value: (self createInstanceOf: c)] raise: Error ]
	! !

!MOViewRendererTest methodsFor: 'type covering' stamp: ''!
createInstanceOf: aClass
	aClass isMeta
		ifTrue: [ ^ aClass theNonMetaClass ].
		
	^ aClass initializedInstance
! !

!MOViewRendererTest methodsFor: 'running' stamp: 'AlexandreBergel 8/8/2012 08:49'!
expectedFailures
	^ #(#testInteraction7AddingAndRemovingInnerEdges #testInteraction11AddingAndRemovingEdges)! !

!MOViewRendererTest methodsFor: 'type covering' stamp: ''!
notExecutetestNoError
	| view |
	view := MOViewRenderer new.
	
	self covers: [:shape | view nodes: (1 to: 100) using: shape] for: #typeOfMOViewRendererNodesUsing.
	self covers: [:shape | view node: #foo using: shape] for: #typeOfMOViewRendererNodeUsing.
	self covers: [:shape | view node: #foo using: shape forIt: [view node: $y]] for: #typeOfNodeUsingForIt.	
	
	self 
		covers: 
			[:shape | view nodes: (1 to: 5) using: shape forEach: [:each | view node: $a]] 
		for: #typeOfMOViewRendererNodesUsingForEach.
	
	self covers: 
		[:shape | view edges: (1 to: 5) from: #yourself toAll: (1 to: 5) using: shape fromGlobal: true toGlobal: true ] for: #typeOfEdgesFromToAllUsingFromGlobalToGlobal.

	self covers: 
		[:shape | view edge: (1) from: #yourself to: (2) using: shape ] for: #typeOfEdgeFromToUsing.

	self covers: 
		[:shape | view edges: (1 to: 5) from: #yourself to: 1 using: shape ] for: #typeOfEdgesFromToUsing. 

	self covers: 
			[:shape | view edges: (1 to: 5) from: #yourself toAll: (1 to: 5) using: shape ] 
		for: #typeOfEdgesFromToAllUsing. 


	self covers: [ :announcement | view announce: announcement ] for: #typeOfMOViewRendererAnnounce.
	
	self covers: 
		[ :layout | 
			view layout: layout.
			window := view open.
			window delete ] for: #typeOfMOViewRendererlayout! !

!MOViewRendererTest methodsFor: 'running' stamp: ''!
setUp
	super setUp.
	view := MOViewRenderer new! !

!MOViewRendererTest methodsFor: 'nodes' stamp: ''!
testAbsoluteBoundsAndBounds
	| node1 node2 |
	
	node1 := view node: $a forIt: [ node2 := view node: $b ].
	view root applyLayout.
	
	self assert: node1 bounds = ( 5@5 corner: 20@20).
	self assert: node2 bounds = ( 5@5 corner: 10@10).
	
	self assert: node1 absoluteBounds = ( 5@5 corner: 20@20).
	self assert: node2 absoluteBounds = ( 10@10 corner: 15@15).
	
	self assert: (node1 shapeBoundsAt: node1 shape) = ( 5@5 corner: 20@20).
	self assert: (node2 shapeBoundsAt: node2 shape) = ( 5@5 corner: 10@10).
	
	node1 translateBy: 2@3.  
	
	self assert: node1 bounds = (( 5@5 corner: 20@20) translateBy: (2@3)  ).
	self assert: node2 bounds = (( 5@5 corner: 10@10)).
	
	self assert: node1 absoluteBounds = (( 5@5 corner: 20@20)translateBy: (2@3)).
	self assert: node2 absoluteBounds = (( 10@10 corner: 15@15)translateBy: (2@3)).
	
	self assert: (node1 shapeBoundsAt: node1 shape) = (( 5@5 corner: 20@20)translateBy: (2@3)).
	self assert: (node2 shapeBoundsAt: node2 shape) = (( 5@5 corner: 10@10)).
! !

!MOViewRendererTest methodsFor: 'edges' stamp: ''!
testAbsoluteBoundsForEdges
	"Check that an edge knows its absolute position, even when inner node and after a drag and drop"

	| absoluteBoundsBeforeLayout firstEdge secondNode thirdNode |
	view nodes: { 1 . 2 }.
	view edgesFromAssociations: { 1 ->  2 }.
	
	view node: 3 forIt: [
		view nodes: #(4 5).
		view edgesFromAssociations: {4 -> 5}. 
		].

	firstEdge := view root edges first.
	absoluteBoundsBeforeLayout := firstEdge absoluteBounds.
	self assert: (absoluteBoundsBeforeLayout = ( 0@0 corner: 5@5)).
	self assert: (firstEdge shape absoluteBoundsFor: firstEdge) = absoluteBoundsBeforeLayout.

	view root applyLayout.
	
	self assert: (view root edges first absoluteBounds = ( 5@5 corner: 20@10)).
	
	"We drag the second node"
	secondNode := view root nodes second.
	secondNode translateBy: 2@40.
	
	self assert: (secondNode incomingEdges asArray = {firstEdge}).

	self assert: (view root edges first absoluteBounds = ( 5@5 corner: 22@45)).
	self assert: (firstEdge shape absoluteBoundsFor: firstEdge) = view root edges first absoluteBounds.

	"-------"
	thirdNode := view root nodes third.
	self assert: (thirdNode edges first absoluteBounds = ( 40@10 corner: 55@15)).

	thirdNode nodes second translateBy: 1@2 bounded: true.
	self assert: (thirdNode edges first absoluteBounds = ( 40@10 corner: 56@15)).

	thirdNode nodes second translateBy: 100@200 bounded: true.
	self assert: (thirdNode edges first absoluteBounds = (  40@10 corner: 60@15)).
! !

!MOViewRendererTest methodsFor: 'commands' stamp: ''!
testAddingCommand
	| buttons |
	self assert: view userCommands isEmpty.
	
	view addUserCommand: {'command 1' . [ :v | v inspect ]}.
	self assert: view userCommands notEmpty.
	
	window := view open.
	buttons := window allMorphs select: [:m | m isKindOf: PluggableButtonMorph ].
	self assert: (buttons size = 3).
	window delete.
	
	view addUserCommand: {'command 2' . [ :v | v inspect ]}.
	view addUserCommand: {'command 3' . [ :v | v inspect ]}.
	window := view open.
	buttons := window allMorphs select: [:m | m isKindOf: PluggableButtonMorph ].

	self assert: (buttons size = 5).! !

!MOViewRendererTest methodsFor: 'edges' stamp: ''!
testAllEdgesConnectedToMe
	| nodes allEdges |
	nodes := view nodes: (1 to: 5).
	view edges: (2 to: 5) from: #yourself to: 1.

	allEdges := nodes first allEdgesConnectedToMe.
	self assert: (allEdges size = 4 ).
	
	allEdges first remove.
	self assert: (nodes first allEdgesConnectedToMe size = 3 ).! !

!MOViewRendererTest methodsFor: 'nodes' stamp: ''!
testAllNodeShapeProperlyInitialized
	MOShape allSubclasses do: [:cls | 
		cls isAbstractClass
			ifFalse: [ view shape: cls new.
					view node: 10 ] ].
			
	view root applyLayout.
	self shouldnt: [ view getRenderedForm ]  raise: Error! !

!MOViewRendererTest methodsFor: 'nodes' stamp: ''!
testAllNodesDo
	| sum |
	view nodes: #(1 10) forEach: [:each |
		view node: 2].
	
	sum := 0.
	view root allNodesDo: [:aNode | sum := sum + aNode model].
	self assert: (sum = 15)! !

!MOViewRendererTest methodsFor: 'shapes' stamp: ''!
testArrowedEdges
	view nodes: {1 . 2}.
	
	view shape: (MOStraightLineShape new).
	view edges: {1 . 2} from: 1 to: 2.	
	self shouldnt: [window := view open. window delete] raise: Error.
	
	view edgeShape: (MOArrowedLineShape new).
	view edges: {1 . 2} from: 1 to: 2.
	self shouldnt: [window := view open. window changed. window delete] raise: Error.
	window := view open.! !

!MOViewRendererTest methodsFor: 'nodes' stamp: ''!
testAsMorph
	| morph |
	view nodes: #(1 10) forEach: [:each |
		view node: 2].
	morph := view asMorph.
	self assert: (morph isMorph).
	self assert: (morph class == MOCanvas).! !

!MOViewRendererTest methodsFor: 'attributes' stamp: ''!
testAttributes
	| nodes firstNode |
	view interaction on: MOElementSelection do: [:ann | ann element attributeAt: #foo  put: 'zork' ].
	nodes := view nodes: (1 to: 50).
	
	firstNode := nodes first.
	self deny: (firstNode hasAttribute: #foo).
	self should: [ firstNode attributeAt: #foo ] raise: Error.

	firstNode attributeAt: #foo put: 50.
	self shouldnt: [ firstNode attributeAt: #foo ] raise: Error.
	self assert: (firstNode attributeAt: #foo) = 50.
	
	firstNode removeAttribute: #foo.
	self deny: (firstNode hasAttribute: #foo).
	
	firstNode announce: (MOElementSelection event: nil on: firstNode).
	self assert: (firstNode hasAttribute: #foo).
	self assert: (firstNode attributeAt: #foo) = 'zork'.
	

	self deny: (firstNode hasAttribute: #zork).
	self assert: (firstNode attributeAt: #zork ifAbsent: [30]) = 30.
	self assert: (firstNode attributeAt: #zork ifAbsentPut: [20]) = 20.

	self assert: (firstNode hasAttribute: #zork).
	self assert: (firstNode attributeAt: #zork) = 20.

	firstNode attributeAt: #zorzor put: true.
	firstNode attributeAt: #bouba put: true.
	self assert: (firstNode attributeKeysMatching: '*') asSortedCollection asArray
					=   #(#bouba #foo #zork #zorzor).
	self assert: (firstNode numberOfCacheKeys = 4).
	firstNode removeAttributesMatching: 'zor*'.

	self deny: (firstNode hasAttribute: #zork).
	self deny: (firstNode hasAttribute: #zorzor).
	self assert: (firstNode hasAttribute: #bouba).
	self assert: (firstNode numberOfCacheKeys = 2).! !

!MOViewRendererTest methodsFor: 'attributes' stamp: ''!
testAttributes2
	| t node |
	t := 0.

	node := MONode on: 10.
	self assert: (node attributeAt: #foo ifAbsent: [ -2] ) = -2.
	node attributeAt: #foo ifAbsentPut:  [ t := t + 1].
	self assert: t = 1..
	
	self assert: (node attributeAt: #foo) = 1.
	self assert: (node attributeAt: #foo ifAbsentPut:  [ t := t + 1]) = 1.
	self assert: (node attributeAt: #foo) = 1.
! !

!MOViewRendererTest methodsFor: 'bitmap cache' stamp: ''!
testBasic
	| node1 node2 form form2 canvas2 |
	node1 := view node: #foo forIt: [
		node2 := view node: #bar 
	].
	view root applyLayout.

	self assert: (node1 bounds = ( 5@5 corner: 20@20)).
	self assert: (node2 bounds = ( 5@5 corner: 10@10)).


	form := view getRenderedForm. 
	
	form2 := Form extent: view root bounds extent depth: Display depth.
	canvas2 := FormCanvas on: form2.
	canvas2 
			frameAndFillRectangle: ( 5@5 extent: 15@15)
				fillColor: Color white
				borderWidth: 1
				borderColor: Color black.
	canvas2 
				frameAndFillRectangle: (10@10 extent: 5@5)
					fillColor: Color white
					borderWidth: 1
					borderColor: Color black.

	self assert: form bits = form2 bits
"
form2 asMorph openInWindow
"! !

!MOViewRendererTest methodsFor: 'bitmap cache' stamp: ''!
testBasic2
	| node1 node2 form canvas form2 canvas2 node3 |
	node1 := view node: #foo forIt: [
		node2 := view node: #bar forIt: [
			node3 := view node: #zork.
		]
	].
	view root applyLayout.

	self assert: (node1 bounds = ( 5@5 corner: 30@30)).
	self assert: (node2 bounds = ( 5@5 corner: 20@20)).
	self assert: (node3 bounds = ( 5@5 corner: 10@10)).

	form := view getRenderedForm. 
		
	form2 := Form extent: view root bounds extent depth: Display depth.
	canvas2 := FormCanvas on: form2.
	canvas2 
				frameAndFillRectangle: ( 5@5 extent: 25@25)
					fillColor: Color white
					borderWidth: 1
					borderColor: Color black.
	canvas2 
				frameAndFillRectangle: (10@10 extent: 15@15)
					fillColor: Color white
					borderWidth: 1
					borderColor: Color black.

	canvas2 
				frameAndFillRectangle: (15@15 extent: 5@5)
					fillColor: Color white
					borderWidth: 1
					borderColor: Color black.

	self assert: form bits = form2 bits
	"
	form2 asMorph openInWindow
	"! !

!MOViewRendererTest methodsFor: 'bitmap cache' stamp: ''!
testBasic3

	| bounds form canvas form2 canvas2 node1 node2 node3 node4 |
	node1 := view node: 10 forIt: [
		node2 := view node: #foo forIt: [
			node3 := view node: #foo forIt: [
				node4 := view node: 1]].

		view verticalLineLayout ].
	view root applyLayout.

	self assert: (node1 bounds = ( 5@5 corner: 40@40)).
	self assert: (node2 bounds = ( 5@5 corner: 30@30)).
	self assert: (node3 bounds = ( 5@5 corner: 20@20)).
	self assert: (node4 bounds = ( 5@5 corner: 10@10)).
	
	form := view getRenderedForm. 
	
	form2 := Form extent: view root bounds extent depth: Display depth.
	canvas2 := FormCanvas on: form2.
	canvas2 
				frameAndFillRectangle: ( 5@5 extent: 35@35)
					fillColor: Color white
					borderWidth: 1
					borderColor: Color black.
	canvas2 
				frameAndFillRectangle: (10@10 extent: 25@25)
					fillColor: Color white
					borderWidth: 1
					borderColor: Color black.

	canvas2 
				frameAndFillRectangle: (15@15 extent: 15@15)
					fillColor: Color white
					borderWidth: 1
					borderColor: Color black.

	canvas2 
				frameAndFillRectangle: (20@20 extent: 5@5)
					fillColor: Color white
					borderWidth: 1
					borderColor: Color black.
	
	self assert: form bits = form2 bits
! !

!MOViewRendererTest methodsFor: 'block evaluation' stamp: ''!
testBlockEvaluation
	self assert: ([ 10 ] moValue: 5) = 10.
	self assert: ([ 10 ] moValue: #(1 2)) = 10.
	
	self assert: ([ :v | v + 10 ] moValue: 5) = 15.

	self assert: ([ :v | v first + 10 ] moValue: #(5)) = 15.
	
	self assert: ([ :v1 :v2 | v1 + v2 ] moValue: #(10 5)) = 15.
	
	self should: [ [ :v1 :v2 | v1 + v2 ] moValue: #(10 ) ] raise: Error.
	
	self shouldnt: [ [ :v1 | v1  ] moValue: #(10 20 ) ] raise: Error.! !

!MOViewRendererTest methodsFor: 'cache' stamp: ''!
testCache
	| node shape |
	shape := MORectangleShape new.
	node := MONode on: 'hello'.
	self assert: node numberOfCachedMetrics isZero.
	
	self deny: (node hasCache: (MOElementShapeProperties shape: shape key: #cacheName1)).
	self assert: (node cacheNamed: #cacheName1 of: shape ifAbsentInitializeWith: [ 10 ]) = 10.
	self assert: (node hasCache: (MOElementShapeProperties shape: shape key: #cacheName1)).
	self assert: (node cacheNamed: #cacheName1 of: shape) = 10 

! !

!MOViewRendererTest methodsFor: 'bitmap cache' stamp: ''!
testCache1

	| edge bounds form canvas |
	view nodes: (1 to: 2).
	edge := view edge: #(a) from: [:each | 1 ] to: [:each | 2].
	window := view open.
	
	bounds := view root bounds.	
	form := Form extent: bounds extent depth: Display depth.
	canvas := FormCanvas on: form.

	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: canvas).
	self assert: edge cacheFromPoint notNil.
	self assert: edge cacheToPoint notNil.
	
	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: canvas).
	
	self assert: edge cacheFromPoint notNil.
	self assert: edge cacheToPoint notNil.
	! !

!MOViewRendererTest methodsFor: 'bitmap cache' stamp: ''!
testCache2
	| size nodes bounds form canvas |
	
	size := 10.
	nodes := view nodes: (1 to: size) forEach: [:each | view nodes: (1 to: size)].
	view edges: (1 to: size) from: #yourself toAll: (1 to: size).

	"Before to do a display, we make sure that no bitmap was created"
	self assert: (nodes allSatisfy: [:node | node hasCachedForm not]).

	view root applyLayout.
	bounds := view root bounds.

	form := Form extent: bounds extent depth: Display depth.
	canvas := FormCanvas on: form.
	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: canvas).
	self assert: ((nodes select: [:n | n bounds origin x < 10000])
						anySatisfy: [:node | node hasCachedForm]).! !

!MOViewRendererTest methodsFor: 'attributes' stamp: ''!
testCacheInNodes
	| t view nullCanvas |
	t := 0.
	"With cache"
	view := MOViewRenderer new.
	view shape rectangle size: [:v | t := t + 1. v].
	view nodes: (1 to: 20).
	
	nullCanvas := CountingNullCanvas new.
	self assert: t = 0.

	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: nullCanvas).
	self assert: t = 40.

	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: nullCanvas).
	self assert: t = 40.
	
	"---------------"
	"Without cache"
	t := 0.
	view := MOViewRenderer new.
	view shape rectangle size: [:v | t := t + 1. v]; disableCache.
	view nodes: (1 to: 20).
	
	nullCanvas := CountingNullCanvas new.
	self assert: t = 0.
	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: nullCanvas).
	self assert: t = 120.
	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: nullCanvas).
	self assert: t = 120.
	! !

!MOViewRendererTest methodsFor: 'bitmap cache' stamp: 'TudorGirba 5/26/2012 23:21'!
testCacheIsEffective

	| edge bounds form canvas node nullCanvas |

	node := view node: #(b) forIt: [
		view nodes: (1 to: 2).
		edge := view edge: #(a) from: 1 to: 2 ].
	view root applyLayout.

	"three nodes and one edge are displayed "
	view root resetFormCacheRecursively.

	self assert: (node externalEdges isEmpty).
	self assert: (view root externalEdges isEmpty).
	
	"Check if the edge is among the elements to be displayed"
	self assert: (view root elementsToDisplay includes: edge).
	self assert: (view root elementsToLookup includes: edge).
	
	bounds := view root bounds.	
	form := Form extent: bounds extent depth: Display depth.
	canvas := FormCanvas on: form.
	nullCanvas := CountingNullCanvas new.
	self assert: nullCanvas numberOfRectangles = 0.
	self assert: nullCanvas numberOfLines = 0.
	
	self deny: (node hasCachedForm).

	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: canvas).
	self assert: (node hasCachedForm).

	"Check if the edge is among the elements to be displayed"
"	self assert: (view root elementsToDisplay includes: edge).
	self assert: (view root elementsToLookup includes: edge).
"
	
	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: nullCanvas).
	"Only one rectangle is displayed, the root, which is the white background"
	self assert: nullCanvas numberOfRectangles = 0.
	self assert: nullCanvas numberOfLines = 0.
	"Only one image is displayed, the cache of #b"
	self assert: nullCanvas numberOfImages = 1.
	
	"If we display once more, the cache should be rendered"
	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: nullCanvas).
	self assert: (node hasCachedForm).
	self assert: nullCanvas numberOfRectangles = 0.
	self assert: nullCanvas numberOfLines = 0.
	self assert: nullCanvas numberOfImages = 2.

	"a third time"
	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: nullCanvas).
	self assert: (node hasCachedForm).
	self assert: nullCanvas numberOfRectangles = 0.
	self assert: nullCanvas numberOfLines = 0.
	self assert: nullCanvas numberOfImages = 3.
! !

!MOViewRendererTest methodsFor: 'bitmap cache' stamp: 'AlexandreBergel 4/15/2012 21:17'!
testCacheIsEffective2

	| bounds form canvas node nullCanvas nodes |
	nodes := view nodes: {$a . $b . $c} forEach: [ :each|
		view node: #foo forIt: [
			view nodes: (1 to: 200).
			view gridLayout gapSize: 1 ]
	].
	node := nodes first.
	view root applyLayout.
	
	"three nodes and one edge are displayed "
	view root resetFormCacheRecursively.

	self assert: (node bounds = ( 5@5 corner: 134@99)).
	self assert: (node externalEdges isEmpty).
	self assert: (view root externalEdges isEmpty).
	
	self assert: (view root elementsToDisplay size =   606).
	self assert: (view root elementsToLookup size = 606).
	
	bounds := view root bounds.	
	form := Form extent: bounds extent depth: Display depth.
	canvas := FormCanvas on: form.
	nullCanvas := CountingNullCanvas new.
	self assert: nullCanvas numberOfRectangles = 0.
	self assert: nullCanvas numberOfLines = 0.
	
	self deny: (node hasCachedForm).
	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: canvas).

	self assert: (view root elementsToDisplay size =  3).
	self assert: (view root elementsToLookup size = 606).

	node resetFormCache.
	self assert: (view root elementsToDisplay size =  606).

	"We simulate the cache"
	nodes do: [ :n |  
		n cacheCanvas: canvas. 
		n updateOwner ]. 
	self assert: (node hasCachedForm).
	
	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: canvas).
	self assert: (view root elementsToDisplay size =  3).
	node select.
	self assert: (view root elementsToDisplay size =  606).
	
	node announce: (MOMouseDrag step: 1@2).

	self assert: (view root elementsToDisplay size =  606).
	"We simulate the display"
	nodes do: [ :n |  
		n cacheCanvas: canvas. 
		n updateOwner ]. 
	self assert: (view root elementsToDisplay size =  3).

	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: nullCanvas).
	self assert: (nodes allSatisfy: #hasCachedForm).
	self assert: nullCanvas numberOfRectangles = 0.
	self assert: nullCanvas numberOfImages = 3.
	
	"If we display once more, the cache should be rendered"
	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: nullCanvas).
	self assert: (node hasCachedForm).
	self assert: nullCanvas numberOfRectangles = 0.
	self assert: nullCanvas numberOfImages = 6.

	"a third time"
	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: nullCanvas).
	self assert: (node hasCachedForm).
	self assert: nullCanvas numberOfRectangles = 0.
	self assert: nullCanvas numberOfImages = 9.
! !

!MOViewRendererTest methodsFor: 'bitmap cache' stamp: ''!
testCacheIsEffective3

	| edge step bounds form canvas node nullCanvas |
	
	view node: 1 forIt: [view node: 1 forIt: [view node: 1 forIt: []]].
	view root applyLayout.

	nullCanvas := CountingNullCanvas new.
	self assert: nullCanvas numberOfRectangles = 0.
	self assert: nullCanvas numberOfLines = 0.
	

	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: nullCanvas).
	self assert: nullCanvas numberOfRectangles = 0.
	self assert: nullCanvas numberOfLines = 0.
	"Only one image is displayed, the cache of #b"
	self assert: nullCanvas numberOfImages = 1.
	
	"If we display once more, the cache should be rendered"
	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: nullCanvas).
	self assert: nullCanvas numberOfRectangles = 0.
	self assert: nullCanvas numberOfLines = 0.
	self assert: nullCanvas numberOfImages = 2.

	"a third time"
	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: nullCanvas).
	self assert: nullCanvas numberOfRectangles = 0.
	self assert: nullCanvas numberOfLines = 0.
	self assert: nullCanvas numberOfImages = 3.
! !

!MOViewRendererTest methodsFor: 'shapes' stamp: ''!
testCachedShape
	self assert: MONode new shape isCached.
	self assert: MOEdge new shape isCached.
	
	self assert: MORectangleShape new isCached.
	self assert: (MOShape defaultNodeShapeInstance isCached)! !

!MOViewRendererTest methodsFor: 'nodes' stamp: ''!
testCanvas
	self assert: (view canvas notNil).
	self assert: (view canvas isKindOf: MOCanvas).! !

!MOViewRendererTest methodsFor: 'centered and stretch' stamp: ''!
testCenter
	| nodes |
	view shape: (MORectangleShape new width: #yourself; height: #yourself).
	nodes := view nodes: #(10 100).
	view layout: (MOHorizontalLineLayout new center).
	window := view open.

	self assert: (nodes first bounds = (5@50 corner: 15@60)).
	self assert: (nodes second bounds = (25@5 corner: 125@105)).! !

!MOViewRendererTest methodsFor: 'centered and stretch' stamp: ''!
testCenter2
	| nodes |
	
	view shape: (MORectangleShape new width: #yourself; height: #yourself).
	nodes := view nodes: #(10 100).
	view layout: (MOVerticalLineLayout new center).
	window := view open.
	
	self assert: (nodes first bounds = ( 50@5 corner: 60@15)).
	self assert: (nodes second bounds = ( 5@25 corner: 105@125)).! !

!MOViewRendererTest methodsFor: 'children shape' stamp: ''!
testChildrenShape
	| builder formShape |
	self deny: MORectangleShape default canHandleChildren.
	self deny: MOFormsShape default canHandleChildren.
	
	builder := MOFormsBuilder new.
	builder column; row.
	builder x: 1 y: 1 add: MOChildrenShape new.
	formShape := builder asShape.
	self assert: formShape canHandleChildren.

! !

!MOViewRendererTest methodsFor: 'children shape' stamp: ''!
testChildrenShapeBounds

	| umlClassShape node1 node2 |
	

	umlClassShape := MOChildrenShape new.
	view nodeShape: umlClassShape.
	node1 := view node: 10 forIt: [ view nodes: (1 to: 5)].

	node2 := view node: 10 forIt: [ view nodes: (1 to: 5)].

	view root applyLayout.

	self assert: ((node2 shape computeExtentHavingChildrenFor: node2) = ( 75@15)).
	self assert: (node2 bounds = ( 90@5 corner: 165@20)).

	self assert: ((node1 shape computeBoundsFor: node1) = ( 0@0 corner: 75@15)).
	self assert: ((node1 shape computeExtentHavingChildrenFor: node1) = ( 75@15)).
	self assert: (node1 bounds = ( 5@5 corner: 80@20))! !

!MOViewRendererTest methodsFor: 'children shape' stamp: ''!
testChildrenShapeBounds2
	|  childrenShape node2 node1 builder formShape innerNodes |
	childrenShape := MOChildrenShape new.
	builder := MOFormsBuilder new.
	builder column; row.
	builder x: 1 y: 1 add: childrenShape.
	formShape := builder asShape.
	view shape: formShape.
	node1 := view node: 10 forIt: [ innerNodes := view nodes: (1 to: 5)].

	view root applyLayout.

"	self assert: (node1 shapeBounds size = 1).
	self assert: (node1 shapeBounds keys includesAllOf: {formShape}).
"
	self assert: (formShape builder widthsFor: node1) = #(75).
	self assert: (formShape builder heightsFor: node1) = #(15).

	self deny: (formShape builder shapes includes: childrenShape).
	self assert: ((node1 shape computeBoundsFor: node1) = ( 0@0 corner: 75@15)).
	self assert: ((node1 shape computeExtentFor: node1) = ( 75@15)).

	"We are now testing the dictionary that contains the shapes"
	self assert: (node1 shapeBounds size = 2).
	self assert: (node1 shapeBounds keys includesAllOf: {childrenShape . formShape}).

	self assert: ((node1 shape boundsFor: node1) = ( 5@5 corner: 80@20)).
	self assert: (node1 bounds = ( 5@5 corner: 80@20)).
	
	"---------"
	"We check inner nodes"
	self assert: (innerNodes size = 5).
	"The height of the node is 15. An inner node can therefore not move more than 15 pixels vertically"
	self assert: (innerNodes first getBoundedTranslationStep: 76@16) =  ( 65@5).! !

!MOViewRendererTest methodsFor: 'edges' stamp: ''!
testClickOnEdges
	| t edges |
	view nodes: {1 . 2}.
	
	t := nil.
	view interaction 
		on: MOElementSelection do: [ :ann | t := ann].
	edges := view edgesFromAssociations: { 1 -> 2}.
	  
	self assert: t isNil.
	edges first announce: MOMouseDown new.
	self assert: t isNil.
	
	edges first announce: MOElementSelection new.
	self assert: t notNil.
! !

!MOViewRendererTest methodsFor: 'clipping' stamp: ''!
testClipping
	| form canvasBounds edge |
	
	view nodes: #(1 2).
	view edge: 1 -> 2 from: #key to: #value.
	view root applyLayout.

	view root nodes first announce: (MOMouseDrag step: 50@30).
	
	canvasBounds := view canvas bounds.
	self assert: (canvasBounds = (0@0 corner: 60@40) ).
	self assert: (view root nodes allSatisfy: [ :node | node isVisibleInBounds: canvasBounds ] ).
	
	edge := view root edges first.
	self assert: (edge isVisibleInBounds: canvasBounds).
	
	self deny: (edge isVisibleInBounds: (55@0 corner: 60@20)).
! !

!MOViewRendererTest methodsFor: 'clipping' stamp: 'AlexandreBergel 4/15/2012 21:17'!
testClipping2
	| form canvasBounds edge |
	
	view nodes: #(1 2) forEach: [ :each | view nodes: #(3 4) ].
	view edge: 1 -> 2 from: #key to: #value.
	view root applyLayout.

	view root nodes first announce: (MOMouseDrag step: 50 @ 30).
	view root nodes first announce: (MOMouseDrag step: -50 @ -30).

	"Size of the canvas"	
	self assert: (view canvas bounds = ( 0@0 corner: 85@50)).

	"We display on the complete canvas"
	form := view getRenderedFormSized: ( 0@0 corner: 85@50).
	self assert: (view root allNodes select: #hasCachedForm) size = 2.
	view root resetFormCacheRecursively.

	"We display on a part of the canvas"
	form := view getRenderedFormSized: ( 80@0 corner: 85@40).
	self assert: (view root allNodes select: #hasCachedForm) isEmpty.
	view root resetFormCacheRecursively.
! !

!MOViewRendererTest methodsFor: 'subviews' stamp: ''!
testCloseContainment
	"A subnode cannot escape its enclosing object by dragging it"
	| bigNode smallNode |
	
	bigNode := view node: #abc forIt: [
		smallNode := view node: #bar ].
	
	window := view open.

	self assert: bigNode bounds = (5@5 corner: 20@20).
	self assert: smallNode bounds = (5@5 corner: 10@10).
	
	view announce: ((MOMouseDrag event: nil on: smallNode) step: 100 @ 40).
	self assert: (smallNode bounds  = (10@10 corner: 15@15))! !

!MOViewRendererTest methodsFor: 'subviews' stamp: ''!
testCloseContainment2
	"A subnode cannot escape its enclosing object by dragging it"
	| bigNode smallNode |
	
	bigNode := view node: #abc forIt: [
		smallNode := view node: #bar].
	
	window := view open.

	self assert: bigNode bounds = (5@5 corner: 20@20).
	self assert: smallNode bounds = (5@5 corner: 10@10).
	
	view announce: ((MOMouseDrag event: nil on: smallNode) step: -100 @ -40).
	self assert: (smallNode bounds = (0@0 corner: 5@5))! !

!MOViewRendererTest methodsFor: 'subviews' stamp: ''!
testCloseContainment3
	"A subnode cannot escape its enclosing object by dragging it"
	| bigNode smallNode |
	
	bigNode := view node: #abc forIt: [
		smallNode := view node: #bar].
	
	window := view open.

	self assert: bigNode bounds = (5@5 corner: 20@20).
	self assert: smallNode bounds = (5@5 corner: 10@10).
	
	view announce: ((MOMouseDrag event: nil on: smallNode) step: 5 @ 5).
	view announce: ((MOMouseDrag event: nil on: smallNode) step: 5 @ 5).
	view announce: ((MOMouseDrag event: nil on: smallNode) step: 5 @ 5).
	view announce: ((MOMouseDrag event: nil on: smallNode) step: 5 @ 5).
	view announce: ((MOMouseDrag event: nil on: smallNode) step: 5 @ 5).
	self assert: (smallNode bounds = (10@10 corner: 15@15))! !

!MOViewRendererTest methodsFor: 'shapes' stamp: ''!
testConditionalColor
	"
	self debug: #testConditionalColor
	"
	| nodes |
	
	view shape rectangle 
			if: #odd fillColor: Color red.
	
	nodes := view nodes: (1 to: 10).

	
	self assert: (nodes first shape fillColorFor: nodes first) == Color red.
	self assert: (nodes second shape fillColorFor: nodes second) == Color white.

	view shape rectangle 
			if: #odd fillColor: Color blue;
			if: [:el | el even ] fillColor: Color yellow.
	
	nodes := view nodes: (1 to: 10).
	self assert: (nodes first shape fillColorFor: nodes first) == Color blue.
	self assert: (nodes second shape fillColorFor: nodes second) == Color yellow.

! !

!MOViewRendererTest methodsFor: 'type covering' stamp: ''!
testCoverage

	self covers: [:shape | view nodes: (1 to: 100) using: shape] for: #typeOfMOViewRendererNodesUsing.
	self covers: [:shape | view node: #foo using: shape] for: #typeOfMOViewRendererNodeUsing.
	self covers: [:shape | view node: #foo using: shape forIt: [view node: $y]] for: #typeOfNodeUsingForIt.	
	
	self 
		covers: 
			[:shape | view nodes: (1 to: 5) using: shape forEach: [:each | view node: $a]] 
		for: #typeOfMOViewRendererNodesUsingForEach.
	
	self covers: 
		[:shape | view edges: (1 to: 5) from: #yourself toAll: (1 to: 5) using: shape fromGlobal: true toGlobal: true ] for: #typeOfEdgesFromToAllUsingFromGlobalToGlobal.

	self covers: 
		[:shape | view edge: (1) from: #yourself to: (2) using: shape ] for: #typeOfEdgeFromToUsing.

	self covers: 
		[:shape | view edges: (1 to: 5) from: #yourself to: 1 using: shape ] for: #typeOfEdgesFromToUsing. 

	self covers: 
			[:shape | view edges: (1 to: 5) from: #yourself toAll: (1 to: 5) using: shape ] 
		for: #typeOfEdgesFromToAllUsing. 


	self covers: [ :announcement | view announce: announcement ] for: #typeOfMOViewRendererAnnounce.
	
	window := view open! !

!MOViewRendererTest methodsFor: 'type covering' stamp: ''!
testCoverageLayout
	
	view nodes: (1 to: 100).
	view edges: (1 to: 100) from: #yourself to: [:x | x + 1].
	
	self covers: 
		[ :layout | 
			view layout: layout.
			window := view open.
			window delete ] for: #typeOfMOViewRendererlayout! !

!MOViewRendererTest methodsFor: 'shapes' stamp: ''!
testDashedLines

	| edge1 edge2 edge3 edge4 edge5 |
	view shape rectangle size: 15.
	view nodes: (1 to: 5).
	
	view shape arrowedDashedLine width: 3.
	edge1 := view edge: 1 -> 2 from: #key to: #value.

	view shape dashedLine.
	edge2 := view edge: 2 -> 3 from: #key to: #value.

	view shape dashedLine dashLengths: {5 . 2}; width: 4.
	edge3 := view edge: 2 -> 4 from: #key to: #value.

	view shape dashedLine dashLengths: {1 . 0}; width: 4.
	edge4 := view edge: 2 -> 5 from: #key to: #value.
	
	view shape dashedLine secondaryColor: Color red.
	edge5 := view edge: 4 -> 5 from: #key to: #value.

	view treeLayout.
	
	window := view open.
	self assert: edge1 shape isDashed.
	self assert: edge1 shape isArrowed.
	self assert: edge1 shape towardsTo.
	
	self assert: edge2 shape isDashed.
	self deny: edge2 shape isArrowed.	

	self assert: edge3 shape isDashed.
	self deny: edge3 shape isArrowed.

	"edge 4 is not dashed since its the space between dashes is 0"
	self deny: edge4 shape isDashed.
	self deny: edge4 shape isArrowed.
	
	self assert: (edge4 shape secondaryColor = Color white).
	self assert: (edge5 shape secondaryColor = Color red)
	! !

!MOViewRendererTest methodsFor: 'labels' stamp: ''!
testDefaultFontFamily

	| shape v |
	shape := MORectangleShape new.
	v := shape fontFamilyFor: MONode new.
	self assert: v notNil.
	self assert: v = shape class defaultFont familyName.! !

!MOViewRendererTest methodsFor: 'nodes' stamp: ''!
testDefaultLayout
	
	self assert: (view layout notNil).
	self assert: (view layout isKindOf: MOHorizontalLineLayout).! !

!MOViewRendererTest methodsFor: 'edges' stamp: ''!
testDefaultShapeForEdges

	self deny: (MOShape isDefaultLineShape).

	self assert: (MOEdge new shape notNil).
	self assert: (MOEdge new shape isKindOf: MOLineShape).
	self assert: (MOEdge new shape == MOShape defaultLineShapeInstance).! !

!MOViewRendererTest methodsFor: 'shapes' stamp: ''!
testDefaultShapeInstance
	| edge lineShape nodeShape |
	self assert: (view defaultNodeShapeInstance isNil).
	self assert: (view defaultLineShapeInstance isNil).
	
	"View is not aware of the default nodes"
	view node: 10.
	view node: 11.
	edge := view edges: {10 -> 11} from: #key to: #value.
	self assert: ((view nodeForDomainValue: 10) shape isKindOf: MOShape defaultNodeShapeClass).
	self assert: (edge first shape isKindOf: MOShape defaultLineShapeClass).
	
	view shape: nil.

	"View is aware of the default nodes"
	view defaultLineShapeInstance: (lineShape := MOStraightLineShape new width: 3; color: Color red).
	view defaultNodeShapeInstance: (nodeShape := MORectangleShape new fillColor: Color blue).
	self assert: (view nodeShape == nodeShape).
	self assert: (view edgeShape == lineShape).
	self assert: (view nodeShape == nodeShape).

	view node: 12.
	view node: 13.
	self assert: (view nodeShape == nodeShape).
	edge := view edges: {12 -> 13} from: #key to: #value.
	
	self assert: (view nodeShape == nodeShape).
	
	self assert: ((view nodeForDomainValue: 12) shape == nodeShape).
	self assert: ((view nodeForDomainValue: 13) shape == nodeShape).
	self assert: (edge first shape == lineShape).

! !

!MOViewRendererTest methodsFor: 'nodes' stamp: ''!
testDefaultShapeNode

	self deny: (MOShape isDefaultNodeShape).
	
	self assert: (MONode new shape notNil).
	self assert: (MONode new shape isKindOf: MONodeShape).
	self assert: (MONode new shape == MOShape defaultNodeShapeInstance).
	
	self assert: (MONode defaultNodeShapeClass == MORectangleShape).
	self assert: (MOEdge defaultLineShapeClass == MOStraightLineShape).
! !

!MOViewRendererTest methodsFor: 'shapes' stamp: ''!
testDiamondAndTriangleShapes
	| node1 node2 node3 diamondNodes triangleNodes |
	view shape diamond height: 30; width: 15; fillColor: Color lightBlue.
	diamondNodes := view nodes: (1 to: 10).

	view shape triangle height: 30; width: 15; fillColor: Color lightGreen.
	triangleNodes := view nodes: (1 to: 10).
		
	window := view open.

	self assert: (diamondNodes first bounds =  (5@5 corner: 20@35)).
	self assert: (triangleNodes first bounds = (255@5 corner: 270@35)).
! !

!MOViewRendererTest methodsFor: 'children shape' stamp: ''!
testDisplayingFormsShape

	| builder childrenShape titleShape form formShape node |
	
	childrenShape := MOChildrenShape new.
	titleShape := MORectangleShape new height: 15.
	builder := MOFormsBuilder new.
	builder column; center; fill; pref; grow.
	builder row; center; pref; fill; grow; row; center; fill; pref; grow.
	builder x: 1 y: 1 add: titleShape.
	builder x: 1 y: 2 add: childrenShape.
	formShape := builder asShape.

	view nodeShape: formShape.
	node := view node: 1 forIt: [
		view nodes: (1 to: 2) ].
	
	view root applyLayout.

	form := view getRenderedForm.

	self assert: (formShape builder widthsFor: node) =  #(30.0).
	self assert: (formShape builder heightsFor: node) = #(15.0 15.0).

	self assert: (formShape builder boundsOf: titleShape for: node) = ( 0@0 corner: 30@15).
	self assert: (formShape builder boundsOf: childrenShape for: node) = ( 0@15 corner: 30@30).
! !

!MOViewRendererTest methodsFor: 'subviews' stamp: ''!
testDragAndDrop
	 | nodes classes |

	view shape rectangle size: 20.
	view interaction action: #inspect.
	view interaction 
		whenEnteringUpdateNode: [:node | 
			view interaction forwarder.
			view nodes: (1 to: 20).
				view interaction forwarder.
			view edgesFrom: [:v | v \\ 3].
			view treeLayout
			] 
		whenLeavingUpdateNode:  [:node | ]  
		withLayoutUpdate: false.
	nodes := view nodes: #(a b c).
	
	window := view open.
	
	self assert: view root allNodes size = 3.
	
	self assert: nodes first bounds =  (5@5 corner: 25@25).
	nodes first announce: MOMouseEnter new.
	self assert: nodes first bounds =  ( 5@5 corner: 156.0@45).
	self assert: view root allNodes size > 3.
	
	nodes first announce: (MOMouseDrag new step: 80@40).
	self assert: nodes first bounds =  ( 85@45 corner: 236.0@85).

	self assert: nodes first isSelected.
	nodes first announce: (MOElementSelection new).
	self assert: nodes first isSelected.
		
	nodes first announce: (MOMouseLeave new).
	self assert: nodes first bounds =  (85@45 corner: 105@65).
	self assert: view root allNodes size = 3.! !

!MOViewRendererTest methodsFor: 'event' stamp: ''!
testDragAndDropNodes

	|  firstNode secondNode oldBounds |
	view nodes: (1 to: 2).

	firstNode := view root nodes first.
	secondNode := view root nodes second.

	firstNode select.
	self assert: firstNode isSelected.
	oldBounds := firstNode bounds.

	self deny: secondNode isSelected.
	secondNode announce: (MOMouseDrag step: 5@5).
	self assert: secondNode isSelected.
	
	self assert: (oldBounds = firstNode bounds)! !

!MOViewRendererTest methodsFor: 'event' stamp: ''!
testDraggableOnEdges
	"Dragging an edge does not do anything"
	| edgeBound oldBounds |

	
	view nodes: {1 . 2}.
	
	view interaction draggable.
	view edgesFromAssociations: {1 -> 2}.

	oldBounds := view root nodes collect: #bounds.
	edgeBound := view root edges first bounds.
	
	self 
		shouldnt: [view root edges first announce: (MOMouseDrag step: 10@4)]
	 	raise: Error.
		
	
	self assert: (view root nodes collect: #bounds) asArray = oldBounds asArray.
	self assert: view root edges first bounds = edgeBound! !

!MOViewRendererTest methodsFor: 'edges' stamp: ''!
testDragging

	| nodes edge p1 p2 step bounds form canvas |
	
	nodes := view nodes: (1 to: 2).
	edge := view edge: #(a) from: [:each | 1 ] to: [:each | 2].
	window := view open.
	
	bounds := view root bounds.	
	form := Form extent: bounds extent depth: Display depth.
	canvas := FormCanvas on: form.

	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: canvas).
	
	p1 := edge cacheFromPoint.
	p2 := edge cacheToPoint.
	step := 1@2.
	nodes do: [:node |
		view announce: ((MOMouseDrag event: nil on: node) step: step)].
	
	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: canvas).
	
	self assert: p1 + step = edge cacheFromPoint.
	self assert: p2 + step = edge cacheToPoint.

	! !

!MOViewRendererTest methodsFor: 'bitmap cache' stamp: 'AlexandreBergel 4/15/2012 21:17'!
testDragging2

	| edge step bounds form canvas node |
	
	node := view node: #(b) forIt: [
		view nodes: (1 to: 2).
		edge := view edge: #(a) from: [:each | 1 ] to: [:each | 2]].
	window := view open.
	view root resetFormCacheRecursively.
	
	bounds := view root bounds.	
	form := Form extent: bounds extent depth: Display depth.
	canvas := FormCanvas on: form.

	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: canvas).
	step := 1@2.
	view announce: ((MOMouseDrag event: nil on: node) step: step).

	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: canvas).
	
"	self assert: edge cacheFromPoint isNil.
	self assert: edge cacheToPoint isNil.
"	
	
	view root resetFormCacheRecursively.
	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: canvas).
	self assert: edge cacheFromPoint notNil.
	self assert: edge cacheToPoint notNil.
	! !

!MOViewRendererTest methodsFor: 'edges' stamp: 'AlexandreBergel 4/15/2012 21:17'!
testDragging3

	| edge p1 p2 step bounds form canvas node t |
	
	node := view node: #(b) forIt: [
		view node: #(c) forIt: [
			view nodes: (1 to: 2).
			edge := view edge: #(a) from: [:each | 1 ] to: [:each | 2]]].
	window := view open.
	
	bounds := view root bounds.	
	form := Form extent: bounds extent depth: Display depth.
	canvas := FormCanvas on: form.

	view root resetFormCacheRecursively.
	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: canvas).

	p1 := edge cacheFromPoint.
	p2 := edge cacheToPoint.
	step := 1@2.
	
	t := OrderedCollection new.
	node withAllNodesDo: [:n | t add: n absoluteBounds ].
	self assert: (t asArray = {5@5 corner: 45@30. 10@10 corner: 40@25. 15@15 corner: 20@20. 30@15 corner: 35@20}).
	
	view announce: ((MOMouseDrag event: nil on: node) step: step).

	t := OrderedCollection new.
	node withAllNodesDo: [:n | t add: n absoluteBounds ].
	self assert: (t asArray =  {6@7 corner: 46@32. 11@12 corner: 41@27. 16@17 corner: 21@22. 31@17 corner: 36@22}).

	view root resetFormCacheRecursively.
	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: canvas).
	self assert: p1 + step = edge cacheFromPoint.
	self assert: p2 + step = edge cacheToPoint.
	! !

!MOViewRendererTest methodsFor: 'subviews' stamp: ''!
testDraggingAndSubviews
	| subnodes edge |

	view shape rectangle width: 70; height: 60.
	view node: 10 forIt: [
		view shape rectangle size: 30.
		subnodes := view nodes: #('a' 'b').
		edge := view edge: 'a' -> 'b' from: 'a' to: 'b' ].
	window := view open.

	self assert: (subnodes collect: #bounds) =  {5@5 corner: 35@35. 45@5 corner: 75@35}.
	self assert: ((edge shape getAttachPointsFor: edge) =  {40@25. 50@25}).
	self assert: ((edge shape computeAttachPointsFor: edge source and: edge target) = 
						{40@25. 50@25}).
	self assert: (subnodes noneSatisfy: #isSelected).

	self assert: (subnodes collect: #absoluteBounds) =   {10@10 corner: 40@40. 50@10 corner: 80@40}.
	self assert: ((edge shape getAttachPointsFor: edge) = {40@25. 50@25}).
	
	"We trigger a drag and drop"
	subnodes second announce: ((MOMouseDrag event: nil on: subnodes second) step: 5@5).

	"So the cache of the attach point much be set of nil"
	self deny: edge isCacheLoaded.

	self assert: (subnodes collect: #absoluteBounds) =  {10@10 corner: 40@40. 55@15 corner: 85@45}.

	self assert: (subnodes collect: #bounds) =  {5@5 corner: 35@35. 50@10 corner: 80@40}.
	self assert: ((edge shape computeAttachPointsFor: edge source and: edge target) =  {40@25. 55@30}).
	self assert: ((edge shape getAttachPointsFor: edge) = {40@25. 55@30}).

	self assert: (subnodes anySatisfy:   #isSelected).
! !

!MOViewRendererTest methodsFor: 'subviews' stamp: ''!
testDraggingAndSubviews2
	| edge xNode edges wishedWindowBounds innerNodes outerNode bounds form canvas firstSubEdge |
	outerNode := view node: #outer forIt: [
		xNode := view node: #x forIt: [
			innerNodes := view nodes: (1 to: 32).
			edges := view edgesFrom: [:x | x // 10 ].
			view treeLayout ] ].

	self assert: (edges noneSatisfy: #isCacheLoaded).
	
	view root applyLayout.
	bounds := view root bounds.	
	form := Form extent: bounds extent depth: Display depth.
	canvas := FormCanvas on: form.
	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: canvas).
	
	self assert: (outerNode isCacheLoaded).
	self deny: (xNode isCacheLoaded).
	
"	window := view open."

	"We force a refresh to active the edge cache"
	"wishedWindowBounds :=  13@182 corner: 763@562.
	window doLayoutIn: wishedWindowBounds."


	edges do: [ :ed | ed shape getAttachPointsFor: ed ].
	self assert: (edges allSatisfy: #isCacheLoaded).

	
	self assert: (xNode absoluteBounds = ( 10@10 corner: 249.0@50)).
	self assert: (xNode allNodes 
						allSatisfy: [:n | xNode absoluteBounds containsRect: n absoluteBounds ]).

	self assert: (xNode allEdges 
						allSatisfy: [:n | xNode absoluteBounds containsRect: n absoluteBounds ]).


	self assert: (xNode getBoundedTranslationStep: 20@30) = ( 5.0@5).
	self assert: (outerNode absoluteBounds containsRect: xNode absoluteBounds).

	xNode announce: (MOMouseDrag new step: 20@30).

	self assert: (outerNode absoluteBounds containsRect: xNode absoluteBounds).

	self assert: (xNode absoluteBounds = ( 15.0@15 corner: 254.0@55)).
	
	self assert: (xNode allNodes 
						allSatisfy: [:n | xNode absoluteBounds containsRect: n absoluteBounds ]).


	firstSubEdge := xNode allEdges first.
"	self assert: firstSubEdge bounds = (22@25 corner: 58@45).
	self assert: (firstSubEdge absoluteBounds = ( 37@40 corner: 73@60)).
"	self assert: (firstSubEdge shape computeBoundsFor: firstSubEdge) = firstSubEdge bounds.

	self assert: (xNode bounds containsRect: firstSubEdge bounds).	
	self assert: (xNode bounds origin + outerNode bounds origin) = xNode absoluteBounds origin.
	firstSubEdge resetCache.
"	self assert: (firstSubEdge bounds origin + outerNode bounds origin) = firstSubEdge absoluteBounds origin."
	self assert: (xNode absoluteBounds containsRect: firstSubEdge absoluteBounds).	

	self assert: (xNode bounds = ( 10@10 corner: 249@50)).	
	self assert: (xNode absoluteBounds = ( 15@15 corner: 254@55)).
	
	self assert: (xNode allEdges 
						allSatisfy: [:n | outerNode absoluteBounds containsRect: n absoluteBounds ]).
						
	self assert: (xNode allEdges 
						allSatisfy: [:n | xNode absoluteBounds containsRect: n absoluteBounds ]).
! !

!MOViewRendererTest methodsFor: 'edges' stamp: ''!
testEdgeContainsPoint
	| nodes edges domainNodes assoc edge edgeBound root |
	
	nodes := view nodes: #(1 2).
	assoc := 1 -> 2.
	edges := view edges: {assoc} from: #key to: #value.
	window := view open. 
	
	edge := edges anyOne.
	edgeBound := edge shape computeBoundsFor: edge.

	self assert: (edgeBound = ( 10@7 corner: 20@7)).

	"--------"
	"MORoot>>elementAt:"
	root := nodes first root.
	self assert: (root isRoot).
	self assert: (root elementAt: (edge shape computeBoundsFor: edge) center) == edge.
	"--------"
		
	nodes first translateBy: 0@5.

	self assert: (edge shape lineSegmentsFor: edge) size = 1. 
	self assert: (edge shape lineSegmentsFor: edge) first class == LineSegment. 
	self assert: (edge shape lineSegmentsFor: edge) first valueAtStart = ( 10@12). 
	self assert: (edge shape lineSegmentsFor: edge) first valueAtEnd = ( 20@7). 

	edgeBound := edge shape computeBoundsFor: edge.
	self assert: (edgeBound = ( 10@7 corner: 20@12)).

	self assert: (edge shape containsPoint: 7@15 for: edge).
	self assert: (edge shape containsPoint: 7@14 for: edge).
	self assert: (edge shape containsPoint: 7@11 for: edge).
	self deny:   (edge shape containsPoint: 7@10 for: edge).

! !

!MOViewRendererTest methodsFor: 'edges' stamp: ''!
testEdgeContainsPointCurveFitter
	| nodes edges domainNodes assoc edge edgeBound root |
	
	nodes := view nodes: #(1 2).
	assoc := 1 -> 2.
	view shape curvedLine.
	edges := view edgesFromAssociations: {assoc}.
	window := view open. 
	
	edge := edges anyOne.
	edgeBound := edge shape computeBoundsFor: edge.

	self assert: (edge shape getAttachPointsFor:  edge) =   {10@7. 20@7}.

	"--------"
	"MORoot>>elementAt:"
	root := nodes first root.
	self assert: (root isRoot).
	self assert: (root elementAt: (edge shape computeBoundsFor: edge) center) == edge.
	"--------"
		
	nodes first translateBy: 0@5.

	self assert: (edge shape lineSegmentsFor: edge) size = 2. 
	self assert: ((edge shape lineSegmentsFor: edge) allSatisfy: [:t | t class == LineSegment ]).
	self assert: (((edge shape lineSegmentsFor: edge) collect: [:line | line valueAtStart -> line valueAtEnd]) =  {(10@12)->(10@11). (10@11)->(20@7)}).


	edgeBound := edge shape computeBoundsFor: edge.
	self assert: (edgeBound = ( 10@7 corner: 20@12)).

	self assert: (edge shape containsPoint: (edgeBound center) for: edge).
	self assert: (edge shape containsPoint: (edgeBound center + (0@1)) for: edge).
	self assert: (edge shape containsPoint: (edgeBound center + (0@2)) for: edge).
	self assert: (edge shape containsPoint: (edgeBound center + (0@3)) for: edge).
	self deny: (edge shape containsPoint: (edgeBound center + (0@4)) for: edge).
	self deny: (edge shape containsPoint: (edgeBound center + (0@5)) for: edge).
	self deny: (edge shape containsPoint: (edgeBound center - (0@5)) for: edge).

! !

!MOViewRendererTest methodsFor: 'edges' stamp: ''!
testEdgeContainsPointSplineLine
	| nodes edges domainNodes assoc edge edgeBound root |
	
	nodes := view nodes: #(1 2).
	assoc := 1 -> 2.
	view shape splineLine.
	edges := view edgesFromAssociations: {assoc}.
	window := view open. 
	
	edge := edges anyOne.
	edgeBound := edge shape computeBoundsFor: edge.

	self assert: (edgeBound = ( 10@7 corner: 20@7)).

	"--------"
	"MORoot>>elementAt:"
	root := nodes first root.
	self assert: (root isRoot).
	self assert: (root elementAt: (edge shape computeBoundsFor: edge) center) == edge.
	"--------"
		
	nodes first translateBy: 0@5.

	self assert: (edge shape lineSegmentsFor: edge) size = 3. 
	self assert: ((edge shape lineSegmentsFor: edge) allSatisfy: [:t | t class == LineSegment ]).
	self assert: (((edge shape lineSegmentsFor: edge) collect: [:line | line valueAtStart -> line valueAtEnd]) =  {(10@12)->(10@11).( 10@11)->(18@10). (18@10)->(20@7)}).


	edgeBound := edge shape computeBoundsFor: edge.
	self assert: (edgeBound = ( 10@7 corner: 20@12)).

	self assert: (edge shape containsPoint: (edgeBound center) for: edge).
	self assert: (edge shape containsPoint: (edgeBound center + (0@1)) for: edge).
	self assert: (edge shape containsPoint: (edgeBound center + (0@2)) for: edge).
	self assert: (edge shape containsPoint: (edgeBound center + (0@3)) for: edge).
	self assert: (edge shape containsPoint: (edgeBound center + (0@4)) for: edge).
	self deny: (edge shape containsPoint: (edgeBound center + (0@5)) for: edge).
	self deny: (edge shape containsPoint: (edgeBound center - (0@5)) for: edge).

! !

!MOViewRendererTest methodsFor: 'edges' stamp: ''!
testEdgeContainsPointVerticalOrthogonalLine
	| nodes edges assoc edge edgeBound root |
	
	nodes := view nodes: #(1 2).
	assoc := 1 -> 2.
	view shape verticalOrthogonalLine.
	edges := view edges: {assoc} from: #key to: #value.
	window := view open. 
	
	edge := edges anyOne.
	edgeBound := edge shape computeBoundsFor: edge.

	self assert: (edgeBound = ( 10@7 corner: 20@7)).

	"--------"
	"MORoot>>elementAt:"
	root := nodes first root.
	self assert: root isRoot.
	self assert: (nodes first isNotRoot).
	self assert: (root elementAt: (edge shape computeBoundsFor: edge) center) == edge.
	"--------"
		
	nodes first translateBy: 0@5.

	self assert: (edge shape lineSegmentsFor: edge) size = 3. 
	self assert: ((edge shape lineSegmentsFor: edge) allSatisfy: [:t | t class == LineSegment ]).
	self assert: (((edge shape lineSegmentsFor: edge) collect: [:line | line valueAtStart -> line valueAtEnd]) = {(7@15)->(7@10). (7@10)->(22@10). (22@10)->(22@5)}).


	edgeBound := edge shape computeBoundsFor: edge.
	self assert: (edgeBound = ( 10@7 corner: 20@12)).

	self assert: (edge shape containsPoint: (edgeBound center) for: edge).
	self assert: (edge shape containsPoint: (edgeBound center + (0@1)) for: edge).
	self assert: (edge shape containsPoint: (edgeBound center + (0@2)) for: edge).
	self assert: (edge shape containsPoint: (edgeBound center + (0@3)) for: edge).
	self assert: (edge shape containsPoint: (edgeBound center + (0@4)) for: edge).
	self deny: (edge shape containsPoint: (edgeBound center + (0@5)) for: edge).
	self deny: (edge shape containsPoint: (edgeBound center - (0@5)) for: edge).

! !

!MOViewRendererTest methodsFor: 'edges' stamp: ''!
testEdgeContainsPointWithEdges
	| nodes edges domainNodes assoc edge edgeBound root |
	
	nodes := view nodes: #(1 2).
	assoc := 1 -> 2.
	view shape line width: [:el | el key + el value].
	edges := view edges: {assoc} from: #key to: #value.
	window := view open. 
	
	edge := edges anyOne.
	edgeBound := edge shape computeBoundsFor: edge.

	self assert: (edgeBound = ( 10@7 corner: 20@7)).

	"--------"
	"MORoot>>elementAt:"
	root := nodes first root.
	self assert: (root isRoot).
	self assert: (root elementAt: (edge shape computeBoundsFor: edge) center) == edge.
	"--------"
		
	nodes first translateBy: 0@5.

	self assert: (edge shape lineSegmentsFor: edge) size = 1. 
	self assert: (edge shape lineSegmentsFor: edge) first class == LineSegment. 
	self assert: (edge shape lineSegmentsFor: edge) first valueAtStart = ( 10@12). 
	self assert: (edge shape lineSegmentsFor: edge) first valueAtEnd = ( 20@7). 

	edgeBound := edge shape computeBoundsFor: edge.
	self assert: (edgeBound = ( 10@7 corner: 20@12)).

	self assert: (edge shape containsPoint: 7@15 for: edge).
	self assert: (edge shape containsPoint: 7@14 for: edge).
	self assert: (edge shape containsPoint: 7@11 for: edge).
	self deny:   (edge shape containsPoint: 7@10 for: edge).

! !

!MOViewRendererTest methodsFor: 'edges' stamp: ''!
testEdgeFromTo
	| edges |
	
	view nodes: (0 to: 9).
	edges := view edgeFrom: #yourself to: [:v | v + 1 \\ 10].
	view circleLayout.
	window := view open.
	self assert: edges size = 10.! !

!MOViewRendererTest methodsFor: 'nodes' stamp: ''!
testEdges
	
	view nodes: #(1 2).
	view edge: 1 from: #yourself to: [:x | x + 1].
	self assert: (view root edges size = 1).
	self assert: (view root edges first source model = 1).
	self assert: (view root edges first target model = 2).
	
	self deny: (view root allEdges allSatisfy: #isNode).
	self assert: (view root allNodes allSatisfy: #isNode).
	self assert: (view root allEdges allSatisfy: #isEdge).
	self deny: (view root allNodes allSatisfy: #isEdge).! !

!MOViewRendererTest methodsFor: 'interaction' stamp: ''!
testEdgesAndInteractiveExpansion

	| innerNodesBounds firstNode elementsToDisplay |
	
	view shape rectangle size: 20.
	view interaction action: #inspect.
	view interaction 
		whenEnteringUpdateNode: [:node | 
			view interaction forwarder.
			view nodes: (Set allSubclasses).
				view interaction forwarder.
			view edgesFrom: #superclass..
			view treeLayout
			] 
		whenLeavingUpdateNode:  [:node | ]  
		withLayoutUpdate: true.
	view nodes: (1 to: 3).
	
	view root applyLayout.
	
	firstNode := view root nodes first.
	"---"
	elementsToDisplay := view root elementsToDisplay.
	view forNode: 1 do: [ 
			view nodes: {10 . 20} . 
			view edges: {10 -> 20} from: #key to: #value. 
			view root applyLayout ].
	innerNodesBounds := firstNode nodes collect: #bounds.
	self assert: (firstNode nodes size = 2).
	self assert: (firstNode edges size = 1).
	self assert: (elementsToDisplay ~~ view root elementsToDisplay).
	self assert: (firstNode nodeWith: 10 ifAbsent: [ 'not found']) == firstNode nodes first.

	view forNode: 1 do: [].
	self assert: (firstNode nodes isEmpty).
	self assert: (firstNode edges isEmpty).
	self assert: (firstNode nodeWith: 10 ifAbsent: [ 'not found']) = 'not found'.

	view forNode: 1 do: [ 
			view nodes: {10 . 20} . 
			view edges: {10 -> 20} from: #key to: #value. 
			view root applyLayout ].
	self assert: (firstNode nodeWith: 10) == firstNode nodes first.
	self assert: innerNodesBounds = (firstNode nodes collect: #bounds).
	"---"
	
	firstNode announce: MOMouseEnter new. 
	innerNodesBounds := firstNode nodes collect: #bounds.

	view root nodes first announce: MOMouseLeave new. 
	view root nodes first announce: MOMouseEnter new.
	self assert: innerNodesBounds = (firstNode nodes collect: #bounds).! !

!MOViewRendererTest methodsFor: 'edges' stamp: ''!
testEdgesFromAllTo
	| edges |
	
	view nodes: (1 to: 10).
	
	edges := view edges: (1 to: 10) fromAll: [:v | (1 to: v)] to: #yourself.
	view circleLayout.
	window := view open.

	self assert: edges size = 45! !

!MOViewRendererTest methodsFor: 'edges' stamp: ''!
testEdgesFromAllToAll
	| relations edges |
	relations := #((1 2 3 4) (2 3 7 4 5) (6 8 9 10)).
	
	view nodes: #(1 2 3 4 5 6 7 8 9 10).
	edges := view edges: relations fromAll: [:each | each first: 2] toAll: [:each | each last: 2].
	window := view open.
	
	self assert: edges size = 12.
! !

!MOViewRendererTest methodsFor: 'edges' stamp: ''!
testEdgesToAll
	| edges |
	
	view nodes: (1 to: 20).
	edges := view edgesToAll: [:v | {v + 1 . v + 2. v + 3}].
	view treeLayout.
	window := view open.
	
	self assert: (edges size =  54).
	self assert: (edges first source model = 1).
	self assert: (edges first target model = 2).
	self assert: (edges second source model = 1).
	self assert: (edges second target model = 3).
	self assert: (edges third source model = 1).
	self assert: (edges third target model = 4).
	
	self assert: (edges fourth source model = 2).! !

!MOViewRendererTest methodsFor: 'children shape' stamp: ''!
testElementAtAndChildrenShape

	| builder childrenShape titleShape form formShape node innerNodes innerNode form2 canvas2 |

	childrenShape := MOChildrenShape new.
	titleShape := MORectangleShape new width: 30 ; height: 15.
	builder := MOFormsBuilder new.
	builder column; center; fill; pref; grow.
	builder row; center; pref; fill; grow; row; center; fill; pref; grow.
	builder x: 1 y: 1 add: titleShape.
	builder x: 1 y: 2 add: childrenShape.
	formShape := builder asShape.

	view nodeShape: formShape.
	node := view node: 10 forIt: [
		innerNodes := view nodes: (1 to: 2) ].

	view root applyLayout.

	form := view getRenderedForm.

	self assert: (node bounds = ( 5@5 corner: 35@35)).
	"on the border"
	self assert: (view root elementAt: (5@5)) == node.

	"in the title"
	self assert: (view root elementAt: (7@7)) == node.

	innerNode := innerNodes first.
	self assert: (innerNode owner == node).

	self assert: (formShape builder boundsOf: titleShape for: node) = ( 0@0 corner: 30@15).	
	self assert: (formShape builder boundsOf: childrenShape for: node) = ( 0@15 corner: 30@30).
	
	self assert: (innerNode hasParentShape).
	self assert: (innerNode computeParentShape) == childrenShape.
	self assert: (innerNode getAssociatedBuilder) == builder.

"	self assert: (innerNode shape boundsFor: innerNode) = ( 5@5 corner: 10@10).	"
	self assert: (innerNode shape absoluteBoundsFor: innerNode) = ( 10@25 corner: 15@30).
	self assert: (innerNode absoluteBounds = ( 10@25 corner: 15@30)).
	self assert: (innerNodes second absoluteBounds = ( 25@25 corner: 30@30)).
	
	
	"We draw in a form"
	form2 := Form extent: view root bounds extent depth: Display depth.
	canvas2 := FormCanvas on: form2.
	canvas2 
			frameAndFillRectangle: (( 0@0 corner: 30@15) translateBy: 5@5)
			fillColor: Color white
			borderWidth: 1
			borderColor: Color black.
	canvas2 
			frameAndFillRectangle: ( 10@25 corner: 15@30)
			fillColor: Color white
			borderWidth: 1
			borderColor: Color black.
	canvas2 
			frameAndFillRectangle: ( 25@25 corner: 30@30)
			fillColor: Color white
			borderWidth: 1
			borderColor: Color black.

	self assert: form bits = form2 bits.
	"-----------"

	"Drag and drop of the node"
	node announce: (MOMouseDrag step: 10 @ 5).
	self assert: (innerNode absoluteBounds = ( 20@30 corner: 25@35)).
	self assert: (innerNodes second absoluteBounds = ( 35@30 corner: 40@35)).
! !

!MOViewRendererTest methodsFor: 'children shape' stamp: ''!
testElementAtAndChildrenShape2

	| viewA childrenShape builder formShape nodeA innerNodesA viewB nodeB innerNodesB |
	viewA := MOViewRenderer new.
	childrenShape := MOChildrenShape new.
	builder := MOFormsBuilder new.
	builder column; center; fill; pref; grow.
	builder row; center; pref; fill; grow.
	builder x: 1 y: 1 add: childrenShape.
	formShape := builder asShape.

	viewA nodeShape: formShape.
	nodeA := viewA node: 10 forIt: [
		innerNodesA := viewA nodes: (1 to: 2) ].

	viewA root applyLayout.
	
	"========"
	viewB := MOViewRenderer new.
	nodeB := viewB node: 10 forIt: [
		innerNodesB := viewB nodes: (1 to: 2) ].

	viewB root applyLayout.

	"========"
	self assert: (nodeA shape computeBoundsFor: nodeA) = ( 0@0 corner: 30@15).
	self assert: (nodeB shape computeBoundsFor: nodeB) = ( 0@0 corner: 30@15).
	
	self assert: (nodeA bounds = nodeB bounds).
	self assert: (innerNodesA first bounds = innerNodesB first bounds).
	self assert: (innerNodesA second bounds = innerNodesB second bounds).

	self assert: (innerNodesA first absoluteBounds = innerNodesB first absoluteBounds).
	self assert: (innerNodesA second absoluteBounds = innerNodesB second absoluteBounds).! !

!MOViewRendererTest methodsFor: 'nodes' stamp: ''!
testElementSelection

	| node1 node2 t numberOfSelectionEvents |
	
	view nodes: (1 to: 2).

	node1 := view nodeForDomainValue: 1.
	node2 := view nodeForDomainValue: 2.

	t := 0.
	numberOfSelectionEvents := 0.
	node1 announcer
		on: MOElementSelection do: [:ann | numberOfSelectionEvents := numberOfSelectionEvents + 1 ].
	node1 announcer 
		on: MOElementDeselection do: [:ann | t := t + 1 ].
	
	self assert: t isZero.
	self deny: (node1 isSelected).
	self assert: (view root allSelectedNodes isEmpty).
	self assert: (numberOfSelectionEvents = 0).

	node1 announce: (MOElementSelection new).
	self assert: (numberOfSelectionEvents = 1).
	self assert: (view root allSelectedNodes asArray = {node1}).
	self assert: (node1 isSelected).
	self deny: (node2 isSelected).
	self assert: t isZero.
	
	node2 announce: (MOElementSelection new).
	self deny: (node1 isSelected).
	self assert: (node2 isSelected).
	self assert: t = 1.
	
	node1 announce: (MOElementSelection new).
	self assert: (node1 isSelected).
	self deny: (node2 isSelected).
	self assert: t = 2.
	! !

!MOViewRendererTest methodsFor: 'zOrder' stamp: ''!
testElementsToDisplay

	view nodes: #(a b).
	view edges: {#a -> #b} from: #key to: #value.

	self assert: view root allEdges size = 1. 
	self assert: view root elementsToDisplay size = 3. 

	window := view open.

	view node: #c.
	view edges: {#b-> #c . #a -> #c} from: #key to: #value.

	self assert: view root allEdges size = 3. 
	self assert: view root elementsToDisplay size = 6. 
! !

!MOViewRendererTest methodsFor: 'shapes' stamp: ''!
testEllipseShape
	| node1 node2 node3 |
	

	view shape ellipse width: 100; height: 40.
	node1 := view node: #foo.
	node2 := view node: #bar forIt: [
		view shape ellipse width: 50; height: 20.
		node3 := view node: #zork.
	].
		
	window := view open.

	self assert: (node1 bounds width = 100).
	self assert: (node1 bounds height = 40).
! !

!MOViewRendererTest methodsFor: 'event' stamp: ''!
testEventAndViewRenderer
	| t |
	
	t := false.
	view interaction on: MOMouseDown do: [:ann | t := (ann viewRenderer == view)].
	view nodes: (1 to: 5).
	view root nodes first announce: (MOMouseDown new).
	self assert: t
	! !

!MOViewRendererTest methodsFor: 'event' stamp: ''!
testEventOnRoot
	"
	self debug: #testEventOnRoot
	"
	| t nodes morphicEvent |
	t := 0.
	
	nodes := view nodes: (1 to: 100).
	view gridLayout.
	view root interaction
		on: MOMouseDown do: [:ann | t := t + 1 ].
	window := view open.

	"We send an event in the middle of  the first two nodes"
	morphicEvent := MouseEvent new  
						setPosition: ((nodes first bounds center + nodes second bounds center) / 2) asIntegerPoint.
	view root announce: (MOMouseDown event:  morphicEvent on: view root).

	self assert: (t = 1)! !

!MOViewRendererTest methodsFor: 'nodes' stamp: ''!
testExpandingBoundsOfNodes
 	| node |
	
	node := MONode new.
	self assert: node bounds = (0@0 corner: 5@5).
	node expandToIncludePoint: 30@6.

	self assert: node bounds = (0@0 corner: 30@6).
	
	node expandToIncludePoint: 5@6.
	
	self assert: node bounds = (0@0 corner: 30@6).! !

!MOViewRendererTest methodsFor: 'nodes' stamp: ''!
testExpandingBoundsOfRoot
 	| root |
	
	root := MORoot new.
	self assert: root bounds = (0@0 corner: 5@5).
	root expandToIncludePoint: 30@6.

	self assert: root bounds = (0@0 corner: 30@6).
	
	root expandToIncludePoint: 5@6.
	
	self assert: root bounds = (0@0 corner: 30@6).! !

!MOViewRendererTest methodsFor: 'shapes' stamp: ''!
testFixedRectangle
	| form form2 canvas2 |
	
	view shape: (MOFixedRectangleShape new
					width: 25; height: 25;
					withBorder;
					borderColor: Color black).
	view node: 1.

	view shape: (MORectangleShape new
					width: 25; height: 25;
					withBorder;
					borderColor: Color black).
	view node: 2.
	
	view root applyLayout.
	self assert: (view root nodes collect: #bounds) asArray =  {(5@5 corner: 30@30). (40@5 corner: 65@30)}.
	view root increaseZoom.

	"We first check that bounds are okay"
	self assert: (view root nodes first bounds extent  = view root nodes second bounds extent). 
	self assert: (view root nodes collect: #bounds) asArray =  {(5.5@5.5 corner: 33.0@33.0). (44.0@5.5 corner: 71.5@33.0)}.


	"Then, we check if the nodes are displayed properly"
"We draw in a form"
	form := view getRenderedForm.
	form2 := Form extent: view root bounds extent depth: Display depth.
	canvas2 := FormCanvas on: form2.
	canvas2 
			frameAndFillRectangle: (5.5@5.5 corner: 33.0@33.0)
			fillColor: Color white
			borderWidth: 1
			borderColor: Color black.
	canvas2 
			frameAndFillRectangle: (44.0@5.5 corner: 71.5@33.0)
			fillColor: Color white
			borderWidth: 1
			borderColor: Color black.

	self assert: form bits = form2 bits.
! !

!MOViewRendererTest methodsFor: 'shapes' stamp: ''!
testFixedRectangle2
	| form form2 canvas2 |
	
	view shape: (MOFixedRectangleShape new
					width: 25; height: 25;
					withBorder;
					borderColor: Color black).
	view node: 1 forIt: [ view nodes: (1 to: 100)].
	
	view root applyLayout.
	self assert: (view root nodes collect: #bounds) asArray =  {(5@5 corner: 30@30)}.
	view root increaseZoom.

	self assert: (view root nodes collect: #bounds) asArray =   {(5.5@5.5 corner: 33.0@33.0)}.
! !

!MOViewRendererTest methodsFor: 'children shape' stamp: ''!
testFormsAndTranslateTo

	| viewA childrenShape builder formShape nodeA innerNodesA viewB nodeB innerNodesB |
	viewA := MOViewRenderer new.
	childrenShape := MOChildrenShape new.
	builder := MOFormsBuilder new.
	builder column; center; fill; pref; grow.
	builder row; center; pref; fill; grow.
	builder x: 1 y: 1 add: childrenShape.
	formShape := builder asShape.

	viewA nodeShape: formShape.
	nodeA := viewA node: 10 forIt: [
		innerNodesA := viewA nodes: (1 to: 2) ].
	
	viewA root applyLayout.

	"========"
	self assert: (nodeA bounds = ( 5@5 corner: 35@20)).
	self assert: (innerNodesA first absoluteBounds = ( 10@10 corner: 15@15)).
	self assert: (innerNodesA second absoluteBounds = ( 25@10 corner: 30@15)).
	
	nodeA translateTo: 23@14.
	
	self assert: (nodeA bounds = ( 23@14 corner: 53@29)).
	self assert: (innerNodesA first absoluteBounds = ( 28@19 corner: 33@24)).
	self assert: (innerNodesA second absoluteBounds = ( 43@19 corner: 48@24)).
! !

!MOViewRendererTest methodsFor: 'edges' stamp: ''!
testHandlerOnEdge
	| t edge1 edge2 |
	t := OrderedCollection new.
	view nodes: #(1 2 3).

	view interaction 
		on: MOMouseDown do: [:v | t add: v ].
	edge1 := view edge: 1 from: #yourself to: 2.

	view interaction 
		on: MOMouseDown do: [:v1 :v2 | t add: v1; add: v2 ].
	edge2 := view edge: 2 from: #yourself to: 3.

	self assert: t isEmpty.
	edge1 announce: MOMouseDown new.
	self assert: t size = 1.
"	self assert: t first = 1.

	edge2 announce: MOMouseDown new.
	self assert: t asArray = {1 . 2 . 3}"! !

!MOViewRendererTest methodsFor: 'shapes' stamp: ''!
testHasBorder 
	
	self assert: (MORectangleShape new hasBorder).
	self deny: (MORectangleShape new withoutBorder; hasBorder).
! !

!MOViewRendererTest methodsFor: 'shapes' stamp: ''!
testHorizontalOrthogonalLine
	
	view nodes: (1 to: 10).
	view shape horizontalOrthogonalLine.
	view edges: (1 to: 10) from:  1 to: # yourself.
	view horizontalTreeLayout.
	window := view open.

	self assert: ((view root nodes collect: #bounds) asArray includesAllOf: {5@37.0 corner: 10@42.0. 30@5.0 corner: 35@10.0. 30@13.0 corner: 35@18.0. 30@21.0 corner: 35@26.0. 30@29.0 corner: 35@34.0. 30@37.0 corner: 35@42.0. 30@45.0 corner: 35@50.0. 30@53.0 corner: 35@58.0. 30@61.0 corner: 35@66.0. 30@69.0 corner: 35@74.0})! !

!MOViewRendererTest methodsFor: 'shapes' stamp: ''!
testImageShape

	self assert: (MOImageShape new form == MondrianIcons mondrian).
	self assert: ((MOImageShape form: (MondrianIcons mondrianZoomIn)) form
					== MondrianIcons mondrianZoomIn).! !

!MOViewRendererTest methodsFor: 'shapes' stamp: ''!
testImageShape2
					
	| form form2 canvas2 |
	view shape image form: (MondrianIcons mondrianZoomOut).
	view nodes: {1 . 2}.
	view root applyLayout.
	form := view getRenderedForm.
	
	form2 :=  Form extent: view root bounds extent depth: Display depth.
	canvas2 := FormCanvas on: form2.
	{5@5 corner: 21@21 . 31@5 corner: 47@21} do: [:b |
		canvas2 paintImage: MondrianIcons mondrianZoomOut at: b origin ].
	
	self assert: (form2 bits = form bits)! !

!MOViewRendererTest methodsFor: 'edges' stamp: ''!
testIncomingAndOutgoingEdges
	| t |
	view node: 1.
	view node: 2 forIt: [
		view nodes: {20 . 21 . 22}.
		view edges: {20 -> 21 . 20 -> 22} from: #key to: #value.
	].
	view node: 3.
	view edges: {1 -> 2} from: #key to: #value.

	t := OrderedCollection new.
	view root nodes first incomingAndOutgoingEdgesDo: [ :edge | t add: edge ].
	self assert: t size = 1.
	self assert: (t includesAllOf: view root edges).
	
	t := OrderedCollection new.
	view root nodes third incomingAndOutgoingEdgesDo: [ :edge | t add: edge ].
	self assert: t isEmpty.! !

!MOViewRendererTest methodsFor: 'edges' stamp: ''!
testIncomingEdges
	| edge |
	view nodes: #(10 20).
	view edge: 10 from: #yourself to: 20.
	
	self assert: (view root edges size = 1).

	edge := view root edges first.
	self assert: (edge source model = 10).
	self assert: (edge target model = 20).
	
	self assert: (edge target incomingEdges asArray = {edge}).
	self deny: (edge source hasIncomingEdges).
	self assert: (edge target hasIncomingEdges).! !

!MOViewRendererTest methodsFor: 'edges' stamp: ''!
testIncomingEdges2
	| edges1 |
view node: #tr forIt: [
view node: 1 forIt: [ view node: #foo forIt: [view node: #zork1 ] ].
view node: 2 forIt: [ view node: #bar forIt: [view node: #zork2 ]].

].

view node: #tr2 forIt: [
view node: #zork1 .
view node: #zork2 .

edges1 := (view edges: {#zork1 -> #zork2 }  from: #key to: #value fromGlobal: true toGlobal: false).
].

	self assert: (edges1 allSatisfy: [:ed | ed target incomingEdges asArray = {ed} ]).
! !

!MOViewRendererTest methodsFor: 'edges' stamp: ''!
testIncomingEdgesAndSubviews
	| edge subnodes |
	
	view shape rectangle width: 70; height: 60.
	view node: 10 forIt: [
		view shape rectangle size: 30.
		subnodes := view nodes: #('a' 'b').
		edge := view edge: 'a' -> 'b' from: 'a' to: 'b'].
	
	window := view open.

	self assert: subnodes first outgoingEdges asArray = { edge }.
	self assert: subnodes first incomingEdges isEmpty.
	self assert: subnodes first allOutcomingEdges asArray = { edge }.
	self assert: subnodes first allIncomingEdges isEmpty.

	self assert: subnodes second outgoingEdges isEmpty.
	self assert: subnodes second incomingEdges asArray = { edge }.
	self assert: subnodes first allOutcomingEdges asArray = { edge }.
	self assert: subnodes first allIncomingEdges isEmpty.
! !

!MOViewRendererTest methodsFor: 'edges' stamp: 'AlexandreBergel 8/8/2012 08:46'!
testInnerEdgeContainsPoint
	| nodes domainNodes assoc edge edgeBound root |
	

	view shape rectangle size: 50.
	nodes :=view nodes: #(1 2) forEach:[:e |
		view nodes: #(3 4).
		view interaction
			item: 'inspect' action: #inspect.
		edge := view edge: 3->4 from: #key to: #value].

	window := view open. 
	
	edgeBound := edge shape computeBoundsFor: edge.

	self assert: (edgeBound = ( 30@12 corner: 70@12)).

	"--------"
	"Check how many elements"
	self assert: (view root allEdges size = 2).
	self assert: (view root allNodes size = 6).
	self assert: (view root elementsToDisplay size = 8).
	"--------"


	"--------"
	"MORoot>>elementAt:"
	root := nodes first root.
	self assert: (root isRoot).
	
	"Is it really what is expected? Not sure about it"
	self assert: (root elementAt: (edge shape computeBoundsFor: edge) center) == root nodes first edges first.
	"--------"

	self assert: (edge shape lineSegmentsFor: edge) size = 1. 
	self assert: (edge shape lineSegmentsFor: edge) first class == LineSegment. 
	self assert: (edge shape lineSegmentsFor: edge) first valueAtStart = ( 70@12). 
	self assert: (edge shape lineSegmentsFor: edge) first valueAtEnd = ( 30@12). 

	edgeBound := edge shape computeBoundsFor: edge.
	self assert: (edgeBound = ( 30@12 corner: 70@12)).

	self assert: (edgeBound center = (50@12)).

	self deny: (edge shape containsPoint: 50@16 for: edge).	
	self assert: (edge shape containsPoint: 50@15 for: edge).
	self assert: (edge shape containsPoint: 50@14 for: edge).
	self assert: (edge shape containsPoint: 50@13 for: edge).

	self assert: (edge shape containsPoint: 50@12 for: edge).
	self assert: (edge shape containsPoint: 50@11 for: edge).
	self assert: (edge shape containsPoint: 50@10 for: edge).
	self assert: (edge shape containsPoint: 50@9 for: edge).
	self deny: (edge shape containsPoint: 50@8 for: edge).
! !

!MOViewRendererTest methodsFor: 'interaction' stamp: ''!
testInteraction
	| nodes |
	
	view interaction highlightWhenOver.
	nodes := view nodes: (1 to: 5).
	
	self assert: (nodes first shape borderColor == Color black).
	nodes first announce: (MOMouseEnter new).
	self assert: (nodes first shape borderColor == Color red).
	nodes first announce: (MOMouseLeave new).
	self assert: (nodes first shape borderColor == Color black).! !

!MOViewRendererTest methodsFor: 'interaction' stamp: ''!
testInteraction10AddingSubNodesAndElementToDisplay
	| elementsToDisplay v bounds form canvas |
	
	view shape rectangle 
		width: 40;
		height: 30.
	view nodes: (1 to: 16) forEach: [:each | view nodes: (1 to: 5)].
	view gridLayout gapSize: 2.
	window := view open.
	
	self assert: (view root allNodes size = 96).
	elementsToDisplay := view root elementsToDisplay.
	self assert: elementsToDisplay size = 96.
	view forNode: 4 do: [ view nodes: (1 to: 20). view gridLayout ].
	
	bounds := view root bounds.	
	form := Form extent: bounds extent depth: Display depth.
	canvas := FormCanvas on: form.
	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: canvas).
	
	v := view root elementsToDisplay size.
	self assert:  v = view root nodes size.
	self assert: elementsToDisplay ~=  view root elementsToDisplay.
	self assert: (view root allNodes size = 111).
! !

!MOViewRendererTest methodsFor: 'interaction' stamp: 'AlexandreBergel 8/8/2012 08:47'!
testInteraction11AddingAndRemovingEdges
	| nodes |
	
	view shape rectangle size: 30.
	view interaction
	  whenEnteringUpdateNode: [:aValue |
		  view edges: ((1 to: 20) collect: [:v | aValue -> v]) from: #key to: #value ]
	  withLayoutUpdate: false.
	nodes := view nodes: (1 to: 20).
	view circleLayout.
	
	window := view open.

	self assert: (view root allEdges isEmpty).
	self assert: (nodes first allEdges isEmpty).
	
	nodes first announce: (MOMouseEnter new).
	self assert: (view root allEdges size = 19).
	self assert: (nodes first allEdges isEmpty).

	nodes first announce: (MOMouseLeave new).
	nodes first removeAllEdges.
	self assert: (nodes first allEdges isEmpty).
	self assert: (view root allEdges size = 19).
	
	"-------"
	nodes first announce: (MOMouseEnter new).
	self assert: (view root allEdges size = 19).
	self assert: (nodes first allEdges size = 19).

	nodes first announce: (MOMouseLeave new).
	self assert: (nodes first allEdges isEmpty).
	self assert: (view root allEdges isEmpty).

! !

!MOViewRendererTest methodsFor: 'interaction' stamp: ''!
testInteraction12ExpandingNodes

	| bounds5 node5 node10 |	
	view shape rectangle size: 50.
	view interaction 
		whenEnteringUpdateNode: [ :node | view node: #bar ].
	
	view node: 10 forIt: [
		view shape rectangle size: 20.
		view node: 5 ].
	
	window := view open.
	
	node5 := view root nodeForDomainValue: 5.
	node10 := view root nodeForDomainValue: 10.
	
	self assert: node5 notNil.
	bounds5 := node5 bounds.
	self assert: (view root elementAt: bounds5 center) == node5.
	
	self assert: (node5 elementsToLookup isEmpty).
	self assert: (node10 elementsToLookup notEmpty).
	self assert: (node10 elementsToLookup size = 1).

	node10 announce: MOMouseEnter new.
	self assert: (node10 nodes size = 1 ).
	self assert: (node10 nodes first model == #bar ).
	self assert: (node10 elementsToLookup size = 1 ).
	self assert: (node10 elementsToLookup first model = #bar ).
	
	self deny: (view root elementsToLookup includes: node5).

	node10 announce: MOMouseLeave new.
	self assert: (node10 nodes isEmpty ).
! !

!MOViewRendererTest methodsFor: 'interaction' stamp: ''!
testInteraction13NestedInteractions

	| node5 node10 innerNodes |
	
	
	view interaction on: MOMouseDown do: [:ann |
		view forNode: ann element model do: [
			view interaction on: MOMouseDown do: [:ann2|
				view forNode: ann2 element model do: [
					view shape triangle. view nodes: {6 . 7 . 8 . 9} ] ].
			view shape diamond size: 20. 
			view nodes: {3 . 4 . 5} ] ].
	view nodes: {1 . 2}.
	
	view root applyLayout.
"	window := view open."

	self deny: (view root nodes first isCacheLoaded).
	view getRenderedForm. 
	"No cache since there is no inner nodes"
	self deny: (view root nodes first isCacheLoaded).
	
	self assert: (view root allNodes size = 2).
	self assert: (view root nodes size = 2).
	self assert: ((view root nodes collect: #model) includesAllOf: {1 . 2}).
	
	"All the nodes are eligible to be looked up"
	self assert: (view root elementsToLookup size = 2).
	
	"We click on the first node"
	view root nodes first announce: MOMouseDown new.
	view getRenderedForm. 
	self assert: (view root nodes first isCacheLoaded).

	"All the nodes are eligible to be looked up"
	self assert: (view root elementsToLookup size = 5).
	
	"We still have the two nodes"
	self assert: (view root nodes size = 2).
	self assert: ((view root nodes collect: #model) includesAllOf: {1 . 2}).
	self assert: (view root allNodes size = 5).
	self assert: (view root nodes first nodes size = 3).
	self assert: ((view root nodes first nodes collect: #model) includesAllOf: {3 . 4 . 5}).	
	
	"We click on an inner node"
	innerNodes := view root nodes first nodes.
	self assert: innerNodes first nodes isEmpty.
	innerNodes first announce: MOMouseDown new.

	"We haven't redisplayed the node, so the cache should be invalidated'"
	self deny: (view root nodes first isCacheLoaded).
	view getRenderedForm. 
	"We redisplay, so it is loaded"
	self assert: (view root nodes first isCacheLoaded).

	self deny: innerNodes first nodes isEmpty.
	self assert: innerNodes first nodes size = 4.
	self assert: ((innerNodes first nodes collect: #model) includesAllOf: {6 . 7 . 8 . 9}).

	"All the nodes are eligible to be looked up"
	self assert: (view root elementsToLookup size = view root allNodes size).
! !

!MOViewRendererTest methodsFor: 'interaction' stamp: ''!
testInteraction14AddingSubnodeToIncreaseRootSize
	"We have a small node, click on it, we check the size of the root increased"

	| bounds5 node5 node10 innerNodes initialRootBounds updatedRootBounds |
	
	
	view interaction on: MOMouseDown do: [:ann |
		view forNode: ann element model do: [
			view shape rectangle size: 100.
			view node: 2.
		] ].
	view node: 1.

	view root applyLayout.
	initialRootBounds := view root bounds.

	"We click on the first node"
	view root nodes first announce: MOMouseDown new.

	self assert: (view root nodeForDomainValue: 2) bounds = (5@5 corner: 105@105).
	self assert: (view root nodeForDomainValue: 2) absoluteBounds = ( 10@10 corner: 110@110).

	updatedRootBounds := view root bounds.
	self assert: ((0@0 corner: 115@115)= updatedRootBounds)! !

!MOViewRendererTest methodsFor: 'interaction' stamp: ''!
testInteraction15HighlightingInnerNodes
	| nodes |
	
	view node: #foo forIt: [
		view interaction strongHighlightWhenOver: [:v | {v - 1 . v + 1. v + 4 . v - 4}].
		view shape rectangle 
			width: 40;
			height: 30;
			withText.
		nodes := view nodes: (1 to: 16).
		view gridLayout gapSize: 2 ].
	window := view open.
	
	self assert: (nodes first shape fillColor == Color white).
	nodes first announce: (MOMouseEnter new).
	self assert: ((nodes at: 2)  shape fillColor == Color red).
	self assert: ((nodes at: 5)  shape fillColor == Color red).
	nodes first announce: (MOMouseLeave new).
	self assert: (nodes first shape fillColor == Color white).! !

!MOViewRendererTest methodsFor: 'interaction' stamp: ''!
testInteraction16HighlightingInnerNode
	| outterNode innerNode form form2 canvas2 |
	
	outterNode := view node: 10 forIt: [
		view interaction highlightWhenOver.
		view shape rectangle 
			width: 40;
			height: 30.
		innerNode := view node: 20 ].
	
	view root applyLayout.
	view getRenderedForm.
	
	self assert: (innerNode shape fillColor == Color white).
	self assert: (outterNode absoluteBounds = ( 5@5 corner: 55@45)).
	self assert: (innerNode absoluteBounds = ( 10@10 corner: 50@40)).

	innerNode announce: (MOMouseEnter new).
	form := view getRenderedForm.
	
	self assert: (innerNode shape borderColor == Color red).
	
	self assert: (outterNode absoluteBounds = ( 5@5 corner: 55@45)).
	self assert: (innerNode absoluteBounds = ( 10@10 corner: 50@40)).
	
	"----"
	form2 := Form extent: view root bounds extent depth: Display depth.
	canvas2 := FormCanvas on: form2.
	canvas2 
			frameAndFillRectangle: (( 5@5 corner: 55@45))
			fillColor: Color white
			borderWidth: 1
			borderColor: Color black.
	canvas2 
			frameAndFillRectangle: (( 10@10 corner: 50@40) )
			fillColor: Color white
			borderWidth: 1
			borderColor: Color red.

	self assert: form bits = form2 bits.
	
	"----"

	innerNode announce: (MOMouseLeave new).
	self assert: (innerNode shape fillColor == Color white).! !

!MOViewRendererTest methodsFor: 'interaction' stamp: ''!
testInteraction2
	| nodes shape1 shape2 t |
	
	t := 0.
	view interaction highlightWhenOver: [:v | t := t + 1. {v - 1 . v + 1. v + 4 . v - 4}].
	view shape rectangle 
		width: 40;
		height: 30;
		withText.
	nodes := view nodes: (1 to: 16).
	view gridLayout gapSize: 2.
	
	shape1 := nodes first shape.
	self assert: (nodes first shape borderColor == Color black).
	nodes first announce: (MOMouseEnter new).
	shape2 := nodes first shape.
	self assert: (t = 1).
	self assert: (shape1 == shape2).
	self assert: ((nodes at: 2)  shape borderColor == Color red).
	self assert: ((nodes at: 5)  shape borderColor == Color red).
	nodes first announce: (MOMouseLeave new).
	self assert: (nodes first shape borderColor == Color black).! !

!MOViewRendererTest methodsFor: 'interaction' stamp: ''!
testInteraction3
	| nodes |
	
	view interaction strongHighlightWhenOver: [:v | {v - 1 . v + 1. v + 4 . v - 4}].
	view shape rectangle 
		width: 40;
		height: 30;
		withText.
	nodes := view nodes: (1 to: 16).
	view gridLayout gapSize: 2.
	window := view open.
	
	self assert: (nodes first shape fillColor == Color white).
	nodes first announce: (MOMouseEnter new).
	self assert: ((nodes at: 2)  shape fillColor == Color red).
	self assert: ((nodes at: 5)  shape fillColor == Color red).
	nodes first announce: (MOMouseLeave new).
	self assert: (nodes first shape fillColor == Color white).! !

!MOViewRendererTest methodsFor: 'interaction' stamp: ''!
testInteraction4AddingSubNodes
	| elementsToDisplay |
	
	view shape rectangle 
		width: 40;
		height: 30.
	view nodes: (1 to: 16).
	view gridLayout gapSize: 2.
	window := view open.
	
	self assert: (view root allNodes size = 16).
	elementsToDisplay := view root elementsToDisplay.
	self assert: elementsToDisplay size = 16.
	view forNode: 4 do: [ view nodes: (1 to: 20). view gridLayout ].
	self assert:  view root elementsToDisplay size = 36.
	self assert: elementsToDisplay ~=  view root elementsToDisplay.
	self assert: (view root allNodes size = 36).
	self assert: (view root allNodes collect: #bounds) asArray =  {2@2 corner: 42@32. 46@2 corner: 86@32. 90@2 corner: 130@32. 134@2 corner: 209@62. 5@5 corner: 10@10. 20@5 corner: 25@10. 35@5 corner: 40@10. 50@5 corner: 55@10. 65@5 corner: 70@10. 5@20 corner: 10@25. 20@20 corner: 25@25. 35@20 corner: 40@25. 50@20 corner: 55@25. 65@20 corner: 70@25. 5@35 corner: 10@40. 20@35 corner: 25@40. 35@35 corner: 40@40. 50@35 corner: 55@40. 65@35 corner: 70@40. 5@50 corner: 10@55. 20@50 corner: 25@55. 35@50 corner: 40@55. 50@50 corner: 55@55. 65@50 corner: 70@55. 2@36 corner: 42@66. 46@36 corner: 86@66. 90@36 corner: 130@66. 134@36 corner: 174@66. 2@70 corner: 42@100. 46@70 corner: 86@100. 90@70 corner: 130@100. 134@70 corner: 174@100. 2@104 corner: 42@134. 46@104 corner: 86@134. 90@104 corner: 130@134. 134@104 corner: 174@134}! !

!MOViewRendererTest methodsFor: 'interaction' stamp: ''!
testInteraction5AddingSubNodesUponMooseEvent
	| nodes tmp |
	
	view interaction
		forNode: #yourself
		whenEnteringUpdateNode: [:node | tmp := node.  view nodes: #(a b c d f). view gridLayout] 
		whenLeavingUpdateNode: [:node | ]
		withLayoutUpdate: true.
	view shape rectangle size: 10.
	nodes := view nodes: (1 to: 16).
	view gridLayout.
	window := view open.

	self assert: (view root allNodes size = 16).

	nodes first announce: (MOMouseEnter new).
	self assert: (tmp == nodes first model).
	self assert: (view root allNodes size = 21).

	nodes first announce: (MOMouseLeave new).
	self assert: (nodes first nodes isEmpty ).
	self assert: (view root allNodes size = 16).! !

!MOViewRendererTest methodsFor: 'interaction' stamp: ''!
testInteraction6AddingSubNodesUponMooseEventWithEdge
	| nodes edge |
	
	view interaction
		forNode: #yourself
		whenEnteringUpdateNode: [:node | view nodes: #(a b c d f). view gridLayout] 
		whenLeavingUpdateNode: [:node | ]
		withLayoutUpdate: true.
	view shape rectangle size: 10.
	nodes := view nodes: (1 to: 16).
	edge := view edge: 1 from: 1 to: 2.
	view gridLayout.
	window := view open.

	self assert: ((edge shape getAttachPointsFor: edge) =  {15@10. 25@10}).

	nodes second announce: (MOMouseEnter new).
	self assert: ((edge shape getAttachPointsFor: edge) =  {15@10. 25@20}).

	nodes second announce: (MOMouseLeave new).
	self assert: ((edge shape getAttachPointsFor: edge) =  {15@10. 25@10}).
! !

!MOViewRendererTest methodsFor: 'interaction' stamp: ''!
testInteraction7AddingAndRemovingInnerEdges
	| nodes edge |
	
	view interaction
		forNode: #yourself
		whenEnteringUpdateNode: [:node | view edges: {1 -> 2} from: #key to: #value ] 
		whenLeavingUpdateNode: [:node | ]
		withLayoutUpdate: true.
	view shape rectangle size: 10.
	nodes := view nodes: (1 to: 2).
	window := view open.
	
	self assert: (view root allEdges isEmpty).
	
	nodes first announce: (MOMouseEnter new).
	self assert: (view root allEdges size = 1).

	nodes first announce: (MOMouseLeave new).
	self assert: (view root allEdges isEmpty).

	nodes first announce: (MOMouseEnter new).
	self assert: (view root allEdges size = 1).
! !

!MOViewRendererTest methodsFor: 'interaction' stamp: ''!
testInteraction8AddingAndRemovingInnerEdges2
	| nodes edge |
	
	view interaction
		forNode: #yourself
		whenEnteringUpdateNode: [:node | view nodes: #(#a #b). view edges: {#a -> #b} from: #key to: #value ] 
		whenLeavingUpdateNode: [:node | ]
		withLayoutUpdate: true.
	view shape rectangle size: 50.
	nodes := view nodes: (1 to: 2).
	window := view open.

	self assert: (view root allEdges isEmpty).
	
	nodes first announce: (MOMouseEnter new).
	self assert: (view root allEdges size = 1).

	nodes first announce: (MOMouseLeave new).
	self assert: (view root allEdges isEmpty).

	nodes first announce: (MOMouseEnter new).
	self assert: (view root allEdges size = 1).
! !

!MOViewRendererTest methodsFor: 'interaction' stamp: ''!
testInteraction9AddingAndRemovingInnerEdges3
	| nodes |
	
	view shape rectangle size: 20.
	view interaction action: #inspect.
	view interaction 
		whenEnteringUpdateNode: [:node | 
			view interaction forwarder.
			view nodes: (MOShape allSubclasses).
			view interaction forwarder.
			view edgesFrom: #superclass.
			view treeLayout ] 
		whenLeavingUpdateNode:  [:node | ]  
		withLayoutUpdate: true.
		
	nodes := view nodes: (1 to: 3).
	window := view open.

	self assert: (view root allEdges isEmpty).
	
	nodes first announce: (MOMouseEnter new).
	self assert: (view root allEdges size > 16).

	nodes first announce: (MOMouseLeave new).
	self assert: (view root allEdges isEmpty).

	nodes first announce: (MOMouseEnter new).
	self assert: (view root allEdges size > 16).
! !

!MOViewRendererTest methodsFor: 'interaction' stamp: ''!
testInteractionWithColoredNodes
	| nodes |
	
	view interaction highlightWhenOver.
	view shape rectangle borderColor: Color green.
	nodes := view nodes: (1 to: 5).
	
	self assert: (nodes first shape borderColor == Color green).
	nodes first announce: (MOMouseEnter new).
	self assert: (nodes first shape borderColor == Color red).
	nodes first announce: (MOMouseLeave new).
	self assert: (nodes first shape borderColor == Color green).! !

!MOViewRendererTest methodsFor: 'shapes' stamp: ''!
testLabel
	"
	self debug: #testLabel
	"

	| node1 node2 string1 whiteSpaces string2 |
	view shape label.
	string1 := 'hello', String tab, 'world'.

	string2 := 'hello', MOBoundedShape whiteSpacesPerTabulation, 'world'. 
	view nodes: { string1 . string2 }.
	view root applyLayout.

	node1 := view root nodes first.
	node2 := view root nodes second.

	self assert: ( node1 bounds height = node2 bounds height).
	self assert: ( node1 bounds width = node2 bounds width).	! !

!MOViewRendererTest methodsFor: 'labels' stamp: ''!
testLabel1

	| nodes |
	
	view shape: (MORectangleShape new 
						width: 100; height: 100; text: 'hello').
"	view node: 10 forIt: [ view nodes: #(a b c) ]."
	view node: 10.
	
	view shape label 
		linearFontSize: #yourself within: (1 to: 40).
	nodes := view nodes: (1 to: 40).
	
	window := view open.
	
	self assert: (view root nodes first bounds = (5@5 extent: 100@100)).
	self assert: ((nodes at: 10) shape fontSizeFor: (nodes at: 10)) = 14.
	self assert: ((nodes at: 20) shape fontSizeFor: (nodes at: 20)) = 24.	
! !

!MOViewRendererTest methodsFor: 'labels' stamp: ''!
testLabel10TextHeight

	| node1 node2 multiLineSubnode singleLine |
	
	node1 := view node: #outternode forIt: [
		view shape label.
		view nodes: #('hello' 'world' 'bonjour').
		view verticalLineLayout verticalGap: 0.
	].

	node2 := view node: #outternode forIt: [
		view shape label.
		view node: 'hello', String cr, 'world', String cr, 'bonjour'.
		view verticalLineLayout verticalGap: 0.
	].

	view root applyLayout.
	multiLineSubnode := node2 nodes first.
	singleLine := node1 nodes first.
	self assert: (multiLineSubnode height ) = (singleLine height * 3).
	self assert: (multiLineSubnode shape computeTextHeightFor:  multiLineSubnode ) = (singleLine height * 3).

	self assert: node1 extent = node2 extent
	! !

!MOViewRendererTest methodsFor: 'labels' stamp: ''!
testLabel2

	| shape node |
	
	view shape: (MORectangleShape new 
						width: 100; height: 0; text: 'hello').
"	view node: 10 forIt: [ view nodes: #(a b c) ]."
	node := view node: 10.
	window := view open.
	shape := view root nodes first shape.
	self assert: (view root nodes first bounds = (5@5 extent: 100@(shape textHeightFor: node)))! !

!MOViewRendererTest methodsFor: 'labels' stamp: ''!
testLabel3

	| shape node |
	
	view shape: (MORectangleShape new 
						text: 'hello').
"	view node: 10 forIt: [ view nodes: #(a b c) ]."
	node := view node: 10.
	window := view open.
	shape := view root nodes first shape.
	self assert: (view root nodes first bounds = 
						(5@5 extent: (((shape fontFor: node) widthOfString: 'hello') + (2 * (shape computeBorderWidthFor: node)))@
						(shape textHeightFor: node)))! !

!MOViewRendererTest methodsFor: 'labels' stamp: ''!
testLabel4

	| shape |
	
	view shape: (MORectangleShape new 
						width: 100; height: 100;
						text: 'hello').
	view node: 10 forIt: [ view nodes: #(a b c) ].
	window := view open.
	shape := view root nodes first shape.
	self assert: (view root nodes first bounds = 
						(5@5 extent: 100@100))! !

!MOViewRendererTest methodsFor: 'labels' stamp: ''!
testLabel5Multiline
	| shape node font |
	
	view shape rectangle withText.
	node := view node: 'Hello' , (String with: Character cr) , 'World '.
	window := view open.
	shape := node shape.
	font := shape defaultFont.

	self assert: node bounds extent = ((shape textWidthFor: node) @ (shape textHeightFor: node))! !

!MOViewRendererTest methodsFor: 'labels' stamp: ''!
testLabel6Padding

	| node1 node2 shape1 innerNode1 |

	node1 := view node: #foo forIt: [
		shape1 := view shape label fontSize: 10.
		view interaction forwarder.
		innerNode1 := view node: 'hello world!!'.
	].

	view shape rectangle withText; textPadding: 5; fontSize: 10.
	node2 := view node: 'hello world!!'.

	window := view open.

	self assert: (innerNode1 bounds extent = ((shape1 textWidthFor: innerNode1) @ (shape1 textHeightFor: innerNode1))).
	self assert: (node1 bounds extent = (innerNode1 bounds extent + (10 @ 10))).
	
	self assert: (node2 bounds extent = (((node2 shape textWidthFor: node2) @ (node2 shape textHeightFor: node2)) )).! !

!MOViewRendererTest methodsFor: 'labels' stamp: ''!
testLabel7HorizontalPadding

	| node |
	view shape rectangle withText; textHorizontalPadding: 5; fontSize: 10.
	node := view node: 'hello world!!'.

	window := view open.


	self assert: (node bounds extent = (((node shape textWidthFor: node) @ (node shape textHeightFor: node)) )).! !

!MOViewRendererTest methodsFor: 'labels' stamp: ''!
testLabel8VerticalPadding

	| node |
	
	view shape rectangle withText; textVerticalPadding: 5.
	node := view node: 'hello world!!'.

	window := view open.

	self assert: (node bounds extent = (((node shape textWidthFor: node) @ (node shape textHeightFor: node)) )).
! !

!MOViewRendererTest methodsFor: 'labels' stamp: ''!
testLabel9TextColor
	| node1 node2 |
	
	view shape rectangle withoutBorder; withoutBackground.
	node1 := view node: 4 forIt: [
			view shape rectangle
				 borderColor:[ :each| Color blue ].
			view node: 10 forIt: [ 
				view node: 10 using: (view shape rectangle withoutBorder 
														withoutBackground;
														fontColor:[ :each| Color black ];
														text:[ :e| e asString ]).
				view layout gapSize: 0
			]. 
		] .
	view layout gapSize: 0.

	view getRenderedForm.
	self assert: node1 hasCachedForm.

	self assert: (node1 bounds = ( 0@0 corner: 14@13)).
	
	"-----------"
	view := MOViewRenderer new.
	view shape rectangle
			 borderColor:[ :each| Color blue ].
	node2 := view node: 10 forIt: [ 
		view node: 10 using: (view shape rectangle withoutBorder 
											withoutBackground;
											fontColor:[ :each| Color black ];
											text:[ :e| e asString ]).
		view layout gapSize: 0].
	
	view getRenderedForm.
	self assert: node2 hasCachedForm.

	self assert: (node2 bounds = ( 0@0 corner: 14@13)).

	"-----------"
	self deny: (node1 cacheForm bits = node2 cacheForm bits )! !

!MOViewRendererTest methodsFor: 'bitmap cache' stamp: ''!
testLargeNode
	"The node below the root is too big for being cached. Its size is above what is fixed in #shouldCache. Inner nodes are then cached."
	"This test is currently ignored because it consumes a lot of memory"
	| largeNode smallNode bounds form canvas form2 canvas2 verySmallNode |
	view shape rectangle 
		width: 2000;
		height: 2000.
	largeNode := view node: #foo forIt: [
		smallNode := view node: 5 forIt: [
			verySmallNode := view node: 6 ]].
	view root applyLayout.

	self assert: (largeNode bounds = ( 5@5 corner: 2005@2005)).
	self assert: (smallNode bounds = ( 5@5 corner: 20@20)).
	self assert: (verySmallNode bounds = ( 5@5 corner: 10@10)).

	self deny: largeNode shouldCache.
	self assert: smallNode shouldCache.	
		
	bounds := view root bounds.

	form := Form extent: bounds extent depth: Display depth.
	canvas := FormCanvas on: form.
	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: canvas).
	
	form2 := Form extent: bounds extent depth: Display depth.
	canvas2 := FormCanvas on: form2.
	canvas2 
				frameAndFillRectangle: ( 5@5 extent: 2000@2000)
					fillColor: Color white
					borderWidth: 1
					borderColor: Color black.
	canvas2 
				frameAndFillRectangle: (10@10 extent: 15@15)
					fillColor: Color white
					borderWidth: 1
					borderColor: Color black.
	canvas2 
				frameAndFillRectangle: (15@15 extent: 5@5)
					fillColor: Color white
					borderWidth: 1
					borderColor: Color black.
	self assert: form bits = form2 bits! !

!MOViewRendererTest methodsFor: 'layout' stamp: ''!
testLayout
	| layout |
	self assert: (view layout isKindOf: MOHorizontalLineLayout).
	
	layout := MOTreeLayout new.
	view layout: layout.
	self assert: (view layout == layout)! !

!MOViewRendererTest methodsFor: 'interaction' stamp: ''!
testMenu

	| root aWindow rootAnnouncer menuMorph |
	
	view root interaction action: #inspect.
	view node: #foo.
		
	"The root has an announcer"
	root := view root.
	rootAnnouncer := root announcer.
	self assert: rootAnnouncer notNil.
	
	"The following test should pass. But MOAnnouncer>>menuMorphFor: anElement check if menuMorphBlock is nil or not."
	self assert: (rootAnnouncer menuMorphFor: root ) hasItems.

	rootAnnouncer menuMorphBlock: [:v | MenuMorph new ].

	self assert: (rootAnnouncer menuMorphFor: root ) hasItems.
! !

!MOViewRendererTest methodsFor: 'edges' stamp: ''!
testModelAndEdges
	| nodes edges domainNodes assoc |
	
	nodes := view nodes: #(1 2).
	assoc := 1 -> 2.
	edges := view edges: {assoc} from: #key to: #value.
	
	self assert: (edges size = 1).
	self assert: (edges first model == assoc).! !

!MOViewRendererTest methodsFor: 'nodes' stamp: ''!
testMultipleElementDragAndDrop

	| dragAndDropEvent |
	view nodes: (1 to: 5).

	dragAndDropEvent := MOMouseDrag step: 10@30.
	view root nodes first announce: dragAndDropEvent.
	self assert: (dragAndDropEvent nodesToDrag asArray = {view root nodes first}).
	
	view root nodes first select.
	view root nodes second select.
	view root nodes third select.
	
	dragAndDropEvent := MOMouseDrag step: 10@30.
	view root nodes first announce: dragAndDropEvent.
	self assert: (dragAndDropEvent nodesToDrag 
					includesAllOf: {view root nodes first. view root nodes second. view root nodes third}).
! !

!MOViewRendererTest methodsFor: 'nodes' stamp: ''!
testMultipleElementSelectionEvent

	self deny: (MOElementSelection new) isMultiple.
	self assert: (MOElementSelection new) isUnique.
	self assert: (MOElementSelection new setAsMultiple) isMultiple.
	self deny: (MOElementSelection new setAsMultiple) isUnique.


	! !

!MOViewRendererTest methodsFor: 'nodes' stamp: ''!
testMultipleElementSelectionsAndDragAndDrop

	| node1 node2 t numberOfSelectionEvents dragAndDropEvent node1Bounds node2Bounds |
	
	view nodes: (1 to: 2).
	view root applyLayout.
	
	node1 := view nodeForDomainValue: 1.
	node2 := view nodeForDomainValue: 2.

	t := 0.
	numberOfSelectionEvents := 0.
	node1 announcer
		on: MOElementSelection do: [:ann | numberOfSelectionEvents := numberOfSelectionEvents + 1 ].
	node1 announcer 
		on: MOElementDeselection do: [:ann | t := t + 1 ].

	node1 announce: (MOElementSelection new).
	self assert: (numberOfSelectionEvents = 1).
	self assert: (view root allSelectedNodes asArray = {node1}).
	self assert: (node1 isSelected).
	self deny: (node2 isSelected).
	self assert: t isZero.
	
	node2 announce: (MOElementSelection new setAsMultiple).
	self assert: (node1 isSelected).
	self assert: (node2 isSelected).
	self assert: t = 0.
	
	node1 announce: (MOElementSelection new setAsMultiple).
	self assert: (node1 isSelected).
	self assert: (node2 isSelected).
	self assert: t = 0.
	
	dragAndDropEvent := MOMouseDrag step: 10@30.
	node1Bounds := node1 bounds.
	node2Bounds := node2 bounds.
	node1 announce: (dragAndDropEvent).
	self assert: (dragAndDropEvent nodesToDrag size = 2).
	self assert: (dragAndDropEvent nodesToDrag includesAllOf: { node1 . node2 }).
	self assert: (node1 bounds = (node1Bounds translateBy: (10@30))).
	self assert: (node2 bounds = (node2Bounds translateBy: (10@30))).
! !

!MOViewRendererTest methodsFor: 'multiple selection' stamp: ''!
testMultipleSelectionBox

	view nodes: (1 to: 10).
	view root applyLayout.
	view root nodes first translateBy: 0@40.
	
	self assert: view root bounds  =  (0@0 corner: 145@50).
	
	self deny: view root hasSelectionBox.

	"We click in the background"
	view root announce: (MOMouseDown 
							event: (MouseEvent new setPosition: 1@2) on: view root).
	self deny: view root hasSelectionBox.

	"We move the mouse"
	view root announce: (MOMouseDrag 
							event: (MouseEvent new setPosition: 50@67) on: view root).
	self assert: view root hasSelectionBox.	
	self assert: (view root selectBoxBounds = (1@2 corner: 50@67)).
	self assert: (view root allNodes noneSatisfy: #isSelected).

	"We click again to select the elements"
	view root announce: (MOMouseDrag
							event: (MouseEvent new setPosition: 50@67) on: view root).
	self assert: view root hasSelectionBox.
	self assert: ((view root allNodes select: #isSelected) includesAllOf: (view root nodes copyFrom: 1 to: 3)).
! !

!MOViewRendererTest methodsFor: 'multiple selection' stamp: ''!
testMultipleSelectionBoxNotNil

	self deny: view root hasSelectionBox.
	self should: [ view root selectBoxBounds ] raise: AssertionFailure.! !

!MOViewRendererTest methodsFor: 'multiple selection' stamp: ''!
testMultipleSelectionBoxWithInnerNodes

	view nodes: (1 to: 2) forEach: [ :each | view nodes: (10 to: 20). view gridLayout ].
	view root applyLayout.

	"We click in the background"
	view root announce: (MOMouseDown 
							event: (MouseEvent new setPosition: 1@2) on: view root).

	"No selection box since we haven't moved the mouse"
	self deny: view root hasSelectionBox.

	"We move the mouse"
	view root announce: (MOMouseMove 
							event: (MouseEvent new setPosition: 50@67) on: view root).
	self assert: (view root allNodes noneSatisfy: #isSelected).

	"We drag the root"
	view root announce: (MOMouseDrag 
							event: (MouseEvent new setPosition: 52@67) on: view root).

	"Since we have moved the mouse, it has a selection box"
	self assert: view root hasSelectionBox.
	self assert: view root selectionBoxCorner1 = (1@2).
	self assert: view root selectionBoxCorner2 = (52@67).

	"We click again to select the elements"
	view root announce: (MOMouseUp
							event: (MouseEvent new setPosition: 50@67) on: view root).
	self assert: ((view root nodes) includesAllOf: (view root allNodes select: #isSelected)).

	"After the mooseup, there is no selection box"
	self deny: view root hasSelectionBox.! !

!MOViewRendererTest methodsFor: 'edges' stamp: ''!
testNestedEdges

	| edge |
	view nodes: #(10 20) forEach: [:each |
		view nodes: (each to: each + 9).
	].
	edge := view edge: 11 from: [:each | each + 10] to: #yourself.
	self assert: (view root edges size = 1).
	self assert: (view root edges first source model = 21).
	self assert: (view root edges first target model = 11).

	self assert: (view root elementsToDisplay includes: edge).
! !

!MOViewRendererTest methodsFor: 'edges' stamp: 'TudorGirba 5/26/2012 23:06'!
testNestedEdges2

	| nodeA nodeB edge fromPoint toPoint attachPoints bounds form canvas |
	
	nodeA := 
		view node: #a forIt: [
			view node: #source].

	nodeB := view node: #b forIt: [
		view node: #target.
		view shape arrowedLine.
		edge := view edges: {#source -> #target} from: #key to: #value fromGlobal: true toGlobal: false].
	edge := edge first.
	window := view open.

	self assert: nodeB externalEdges size = 1.
	self assert: (nodeB externalEdges includes: edge).

	bounds := view root bounds.	
	form := Form extent: bounds extent depth: Display depth.
	canvas := FormCanvas on: form.
	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: canvas).

	self assert: (view root elementsToDisplay includes: edge).
	self assert: (view root computeElementsToDisplay includes: edge).

	self assert: (view root elementsToLookup includes: edge).
	
	self assert: nodeA edges isEmpty.
	self assert: nodeA nodes first edges isEmpty.
	self assert: nodeA nodes first outgoingEdges size = 1.
	self assert: nodeB edges isEmpty.
	self assert: nodeB outgoingEdges isEmpty.
	
	self assert: view root allEdges size = 1.
	self assert: view root edges size = 1.
	
	attachPoints := edge shape getAttachPointsFor: edge.
	self assert: attachPoints = {(15@12). (35@12)}.
	
	nodeA announce: (MOMouseDrag new step: 30@50).
	self assert: (edge shape getAttachPointsFor: edge) ~=  {(15@12). (35@12)}.
	! !

!MOViewRendererTest methodsFor: 'edges' stamp: ''!
testNestedEdges3

	| nodeA nodeB edge fromPoint toPoint attachPoints |
	
	nodeA := 
		view node: #a forIt: [
			view node: #source].

	nodeB := view node: #b forIt: [
		view node: #target ].
	view shape arrowedLine.
	edge := view edges: {#source -> #target} from: #key to: #value fromGlobal: true toGlobal: false.
	edge := edge first.
	window := view open.

	self assert: nodeA edges isEmpty.
	self assert: nodeA nodes first edges isEmpty.
	self assert: nodeA nodes first outgoingEdges size = 1.
	self assert: nodeB edges isEmpty.
	self assert: nodeB outgoingEdges isEmpty.
	self assert: nodeB allEdges isEmpty.
	
	self assert: view root allEdges size = 1.
	self assert: view root edges size = 1.
	self deny: view root edges isEmpty.
	
	attachPoints := edge shape getAttachPointsFor: edge.
	self assert: attachPoints = {(15@12). (35@12)}.
	
	nodeA announce: (MOMouseDrag new step: 30@50).
	self assert: (edge shape getAttachPointsFor: edge) ~=  {(15@12). (35@12)}.! !

!MOViewRendererTest methodsFor: 'edges' stamp: ''!
testNestedEdges4

	| nodeA edge bounds form canvas attachPoints pointToLookup |
	
	nodeA := 
		view node: #a forIt: [
			view node: #source].

	view node: #b forIt: [
		view node: #target.
		view shape arrowedLine.
		edge := view edges: {#source -> #target} from: #key to: #value fromGlobal: true toGlobal: false].
	edge := edge first.
	window := view open.

	bounds := view root bounds.	
	form := Form extent: bounds extent depth: Display depth.
	canvas := FormCanvas on: form.
	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: canvas).
	
	attachPoints := edge shape getAttachPointsFor: edge.
	self assert: attachPoints = {(15@12). (35@12)}.
	
	nodeA announce: (MOMouseDrag new step: 30@50).
	self assert: (edge shape getAttachPointsFor: edge) =  {(42@60). (37@15)}.

	pointToLookup := (((42@60) + (37@15)) / 2) asIntegerPoint.
	self assert: (edge shape lineSegmentsFor: edge) size = 1.
	self assert: (edge shape lineSegmentsFor: edge) first controlPoints = ( LineSegment from: 42@60 to: 37@15) controlPoints.	
	self assert: (edge shape containsPoint: pointToLookup for: edge).
	self assert: (view root elementAt: pointToLookup) == edge.
	

! !

!MOViewRendererTest methodsFor: 'edges' stamp: ''!
testNestedEdges5
	"Same than testNestedEdges4 but with the edge defined just below the root"
	| nodeA nodeB edge bounds form canvas attachPoints pointToLookup |
	
	nodeA := 
		view node: #a forIt: [
			view node: #source].

	nodeB := view node: #b forIt: [
		view node: #target ].

	view shape arrowedLine.
	edge := view edges: {#source -> #target} from: #key to: #value fromGlobal: true toGlobal: false.
		
	edge := edge first.
	window := view open.

	bounds := view root bounds.	
	form := Form extent: bounds extent depth: Display depth.
	canvas := FormCanvas on: form.
	view root acceptVisitor: (MOMorphPainterVisitor newCanvas: canvas).
	attachPoints := edge shape getAttachPointsFor: edge.
	self assert: attachPoints = {(15@12). (35@12)}.
	
	nodeA announce: (MOMouseDrag new step: 30@50).
	self assert: (edge shape getAttachPointsFor: edge) =  {(42@60). (37@15)}.

	pointToLookup := (((42@60) + (37@15)) / 2) asIntegerPoint.
	self assert: (edge shape lineSegmentsFor: edge) size = 1.
	self assert: (edge shape lineSegmentsFor: edge) first controlPoints = ( LineSegment from: 42@60 to: 37@15) controlPoints.	
	self assert: (edge shape containsPoint: pointToLookup for: edge).
	self assert: (view root elementAt: pointToLookup) == edge.
	

! !

!MOViewRendererTest methodsFor: 'edges' stamp: 'TudorGirba 5/26/2012 22:28'!
testNestedEdgesAreInMostSpecificParent

	| edge |
	view node: 1 forIt: [
		view shape label.
		view nodes: (10 to: 11).
		view treeLayout.
	].
	edge := view edge: 10 -> 11 from: #key to: #value.
	"the edge should be in the most specific parent of the
	from and to nodes"

	self assert: view root edges isEmpty.	
	self assert: view root nodes anyOne edges size = 1.
	self assert: view root nodes anyOne edges first = edge! !

!MOViewRendererTest methodsFor: 'children shape' stamp: ''!
testNoMoreThanOneChildrenInABuilder
	| builder v |
	builder := MOFormsBuilder new.
	builder column; row; row.
	builder x: 1 y: 1 add: (v := MOChildrenShape new).
	
	self assert: (builder x: 1 y: 1) == v.
	self shouldnt: [ builder x: 1 y: 1 add: MOChildrenShape new ] raise: Error.
	self should: [ builder x: 1 y: 2 add: MOChildrenShape new ] raise: Error.! !

!MOViewRendererTest methodsFor: 'event' stamp: ''!
testNoPopup

	self assert: (view interaction hasHandlerFor: MOMouseEnter).
	view interaction noPopup.
	self deny: (view interaction hasHandlerFor: MOMouseEnter).
	view node: 10.
! !

!MOViewRendererTest methodsFor: 'nodes' stamp: ''!
testNodeForDomainValues
	| node13 node2 nodes |
	
	node2 := OrderedCollection new.
	node13 := view nodes: #(1 10) forEach: [:each |
		node2 add: (view node: 2)].

	nodes := view nodeForDomainValues: #(1 2 10).
	nodes with: {node13 first . node2 first . node13 second} do: [:v1 :v2 |
		self assert: v1 == v2].
	
	self assert: (view nodeForDomainValue: #zork ifAbsent: [ true ])! !

!MOViewRendererTest methodsFor: 'nodes' stamp: ''!
testNodeForDomainValues2
	| node13 node2 nodes |
	
	nodes := view nodes: (1 to: 2).

	self assert: (view nodeForDomainValues: #(1 2)) asArray = nodes asArray.
	self assert: (view nodeForDomainValues: 1) asArray = {nodes first}.
	self assert: (view nodeForDomainValues: #(1 2 3)) asArray = nodes asArray.! !

!MOViewRendererTest methodsFor: 'nodes' stamp: ''!
testNodeForItWith

	| fig1 fig2 fig3 |
	
	view shape: (MORectangleShape fillColor: Color blue).
	fig1 := view node: #box
		using: (MORectangleShape fillColor: Color red)
		forIt: [ fig2 := view node: #bar].
	
	fig3 := view node: #zork.
	window := view open.
	self assert: (fig1 shape fillColor == Color red).
	self assert: (fig2 shape fillColor == Color white).
	
	"I find the next assert strange. I would have expected blue instead"
	self assert: (fig3 shape fillColor == Color white).! !

!MOViewRendererTest methodsFor: 'nodes' stamp: ''!
testNodes
	
	view nodes: #(1 2 3 4 5 ).
	self assert: view root nodes size = 5.
	self assert: (view root layout isKindOf: MOHorizontalLineLayout)! !

!MOViewRendererTest methodsFor: 'nodes' stamp: ''!
testNodes2

	view nodes: #(0 10 ) forEach: [:each |
		view nodes: (each + 1 to: each + 9)].
	
	self assert: view root nodes size = 2.
	self assert: view root allNodes size = 20.! !

!MOViewRendererTest methodsFor: 'nodes' stamp: ''!
testNodesForEach

	view 
		nodes: #(1 2 3 )
		forEach: [ :each | view nodes: (1 to: each) ].
	self assert: view root nodes first nodes size = 1.
	self assert: view root nodes last nodes size = 3.
	view root nodesDo: [ :each | self assert: each owner = view root ].
	view root nodes last nodesDo: [ :each | self assert: each owner = view root nodes last ]! !

!MOViewRendererTest methodsFor: 'event' stamp: ''!
testPopupView
	|  node viewBlock |
	
	
view shape rectangle size: 50.
view interaction popupView: (viewBlock := [:myNode :myView | 
	myView shape rectangle size: 50; fillColor: Color lightBlue. 
	myView node: 'World'
]).
view interaction action: #inspect.
node := view node: 'hello'.
	window := view open.
	
	self assert: (node popup isNil).
	"This is hard to programmatically generate the popupView:, since the mouse location is used"
	"node announce: (MOMouseEnter event: nil on: node)."
	
	"Instead, we generate it manually"
	node popup: (MOPopup showView: (viewBlock value: node value: (MOViewRenderer new))).
	self assert: (node popup notNil).
	
	"We simulate a click on the node"
	node announce: (MOMouseDown event: nil on: node).
	
	"The menu must have disapeared"
	self assert: (node popup isNil).	! !

!MOViewRendererTest methodsFor: 'edges' stamp: ''!
testPositionsEdges
	|  edge nodes v |
	
	nodes := view nodes: #(1 2).
	edge := view edge: 1 -> 2 from: #key to: #value.

	"We check that there is no default initialization"
	self assert: edge toPositions isNil.
	self assert: edge fromPositions isNil.

	window := view open.
	v := edge shape computeAttachPointsFor: nodes first and: nodes second.
	self assert: v first = (10@7).
	self assert: v second = (20@7).
	window delete.

	"Opening the window (and thus displaying the edge) does not initialize the positions"
	self assert: edge toPositions isEmpty.
	self assert: edge fromPositions isEmpty.

	"Set the positions"
	edge fromPositions: #(#bottomCenter).
	edge toPositions: #(#topCenter).
	self assert: edge fromPositions = #(#bottomCenter).
	self assert: edge toPositions = #(#topCenter).

	window := view open.
	v := edge shape computeAttachPointsFor: nodes first and: nodes second.
	self assert: v first = (7@10).
	self assert: v second = (22@5).
	window delete.

! !

!MOViewRendererTest methodsFor: 'edges' stamp: ''!
testPositionsEdgesInSubnodes
	| subclasses  edges  firstEdge firstEdgeBounds |
	subclasses := {MOShape . MOLineShape . MONodeShape . MOCurveFitterShape .   MOFilledShape}.
	
	view node: #foo forIt: [
		view nodes: (subclasses sortedAs: #name).
		view shape: (MOOrthoVerticalLineShape new).
		edges := view edgesFrom: #superclass.
		view treeLayout
	].
	window := view open.

	firstEdge := edges detect: [ :e | e model == MOFilledShape ].
	firstEdgeBounds := firstEdge bounds.
	"Could be on the left or right hand side"
	self assert:( {( 5.0@30 corner: 10.0@35) . (13.0@30) corner: (18.0@35)} includes: firstEdge source bounds).
				 
	
	self assert:( { (13.0@55) corner: (18.0@60) . ( 5.0@55 corner: 10.0@60)} includes: firstEdge target bounds).
	! !

!MOViewRendererTest methodsFor: 'shapes' stamp: ''!
testRectangleShapeAndInnerNodes
"
self debug: #testRectangleShapeAndInnerNodes
"
	|  b nodes outerNode realWidth |
	
	b := [ :each | view interaction forwarder. view nodes: (1 to: 10). view verticalLineLayout gapSize: 2 ].

	
	nodes := view nodes: (1 to: 5) forEach: [ :each |
		view interaction forwarder.
		view nodes: (10 to: 15) forEach: b.
		
		view horizontalLineLayout gapSize: 0 ; stretch.
	].

	outerNode := nodes first.
	"There hasn't been a layout so far"
	self assert: (outerNode bounds width
						~= (outerNode nodes inject: 0 into: [:sum :el | el bounds width + sum ])).

	window := view open.
	self assert: (nodes allSatisfy: [:n | n bounds extent = nodes anyOne bounds extent]).
	
	realWidth := outerNode nodes inject: 0 into: [:sum :el | el bounds width + sum ].
	
	"Dichotomy of message sending"
	self assert: (outerNode shape computeExtentHavingChildrenFor: outerNode) = (realWidth@90).	
	self assert: (outerNode shape computeWidthFor: outerNode) = realWidth.	

	self assert: (outerNode shape widthFor: outerNode) = realWidth.	
	self assert: (outerNode shape computeExtentFor: outerNode) x = realWidth.	
	self assert: (outerNode shape computeBoundsFor: outerNode) width = realWidth.	
	self assert: (outerNode bounds width = realWidth).! !

!MOViewRendererTest methodsFor: 'nodes' stamp: ''!
testRemoveAllNodes

	view node: 1 forIt: [ view nodes: (2 to: 5) ].
	view root nodes first removeAllNodes.
	
	view node: 1 forIt: [ view nodes: (1 to: 5) ].
	view root nodes first removeAllNodes.! !

!MOViewRendererTest methodsFor: 'nodes' stamp: ''!
testRemoveNodesAndEdges

	
	view nodes: (1 to: 3).
	view edges: {1 -> 2 . 2 -> 3} from: #key to: #value.
	
	window := view open.

	self assert: view root allNodes size = 3.
	self assert: view root allEdges size = 2.
	
	"remove 1 node"
	view root removeNodeWithValue:  1.
	self assert: view root allNodes size = 2.
	self assert: view root allEdges size = 1.

	"remove the remaining nodes"
	view root removeNodesWithValue: {2 . 3}.
	self assert: view root allNodes isEmpty.
	self assert: view root allEdges isEmpty
	! !

!MOViewRendererTest methodsFor: 'subviews' stamp: ''!
testResetCacheInEdgesAndSubviews
	|  subnodes edge |

	
	view shape rectangle width: 70; height: 60.
	view node: 10 forIt: [
		view shape rectangle size: 30.
		subnodes := view nodes: #('a' 'b').
		edge := view edge: 'a' -> 'b' from: 'a' to: 'b' ].
	window := view open.
	
	self assert: (subnodes collect: #bounds) =  {5@5 corner: 35@35. 45@5 corner: 75@35}.
	self assert: ((edge shape getAttachPointsFor: edge) =  {40@25. 50@25}).
	self assert: ((edge shape computeAttachPointsFor: edge source and: edge target) = {40@25. 50@25}).
	self assert: (subnodes noneSatisfy: #isSelected).


	"So the cache of the attach point much be set of nil"
	self assert: (edge cacheToPoint notNil).
	self assert: (edge cacheFromPoint notNil).

	subnodes second resetCacheInEdges.
	self assert: (edge cacheToPoint isNil).
	self assert: (edge cacheFromPoint isNil).

	! !

!MOViewRendererTest methodsFor: 'shapes' stamp: ''!
testRoundedRectangleShape
	|  ui node1 node2 node3 |
	

	view shape roundedRectangle width: 100; height: 40.
	node1 := view node: #foo.
	node2 := view node: #bar forIt: [
		view shape roundedRectangle width: 50; height: 20.
		node3 := view node: #zork.
	].
		
	window := view open.

	self assert: (node1 bounds width = 100).
	self assert: (node1 bounds height = 40).
! !

!MOViewRendererTest methodsFor: 'edges' stamp: ''!
testScope

	|  edge node1 node2 |
	
	view nodes: #(1 2) forEach: [:each |
		view nodes: #(1 2)].
	view edges: #(1 2)
		from: [:ea | ea]
		to: [:ea | ea + 1]
		fromGlobal: true
		toGlobal: true.
	window := view open.
		
	self assert: (view root edges size = 1).
	edge := view root edges first.
	
	self assert: (view root nodes size = 2).
	node1 := view root nodes first.
	node2 := view root nodes second.
	self assert: (edge source == node1).
	self assert: (edge target == node2).
! !

!MOViewRendererTest methodsFor: 'edges' stamp: ''!
testScope2

	|  edge node1 node2 |
	
	view nodes: #(1 2) forEach: [:each |
		view nodes: #(1 2)].
	view edges: #(1 2)
		from: [:ea | ea]
		to: [:ea | ea + 1]
		fromGlobal: false
		toGlobal: false.
	window := view open.
		
	self assert: (view root edges size = 1).
	edge := view root edges first.

	self assert: (view localLookupNode: 1) == view root nodes first. 
	self assert: (view localLookupNode: 2) == view root nodes second. 

	self assert: (view root nodes size = 2).
	node1 := view root nodes first.
	node2 := view root nodes second.
	self assert: (edge source == node1).
	self assert: (edge target == node2).
	
! !

!MOViewRendererTest methodsFor: 'edges' stamp: 'AlexandreBergel 6/4/2012 19:11'!
testScope3
	"Edges are defined in a forEach:"

	|  edge |

	
	view interaction popupText.
	view nodes: #(1 2) forEach: [:each |
		view nodes: #(1 2).
		view edges: {each}
			from: 1
			to: 2
			fromGlobal: false
			toGlobal: true ].
	window := view open.
		
	self assert: (view root edges size = 2).
	self assert: (view root nodes first edges isEmpty).
	self assert: (view root nodes second edges isEmpty).
	
	edge := view root edges first.
	self assert: (edge source == view root nodes first nodes first).
	self assert: (edge target == view root nodes second).
	
	edge := view root edges second.
	self assert: (edge source == view root nodes second nodes first).
	self assert: (edge target == view root nodes second).

	! !

!MOViewRendererTest methodsFor: 'edges' stamp: ''!
testScope4
	"Edges are defined in a forEach:"
	|  tab |

	tab := #(1 2 3 4 5).
	
	view nodes: (1 to: 2) forEach: [:each |
		view interaction popupText.
		view nodes: (1 to: 2) forEach: [:eac |
			view nodes: tab.
			view gridLayout.
		].
		view edges: view domain
			from: [:ea | 1] 
			to: [:ea | ea+1] 
			fromGlobal: false 
			toGlobal: false. 
	].

	window := view open.

		
	self assert: (view root edges isEmpty).
	self assert: (view root nodes first edges size = 2).
	self assert: (view root nodes first edges first source == view root nodes first nodes first).
	self assert: (view root nodes first edges first target == view root nodes first nodes second).
	
	self assert: (view root nodes first edges second source == view root nodes first nodes first).
	self assert: (view root nodes first edges second target == view root nodes first nodes first nodes third).! !

!MOViewRendererTest methodsFor: 'nodes' stamp: ''!
testSelectAndDeselect

	|  firstNode secondNode |
	
	view nodes: (1 to: 2) forEach: [ :v |
		view nodes: (1 to: 2).
	].

	view getRenderedForm.
	
	self assert: (view root nodes allSatisfy: #hasCachedForm).

	firstNode := view root nodes first.
	secondNode := view root nodes second.
	
	firstNode select.
	self deny: (firstNode hasCachedForm).
	self assert: (secondNode hasCachedForm).
	
	view getRenderedForm.
	self assert: (view root nodes allSatisfy: #hasCachedForm).
	firstNode deselect.
	self deny: (firstNode hasCachedForm).
	self assert: (secondNode hasCachedForm).
! !

!MOViewRendererTest methodsFor: 'nodes' stamp: ''!
testSelectedNodes

	|  nodes root |
	
	view shape rectangle size: 50.
	nodes := view nodes: #($a $b $c) forEach: [ :each | 
		view shape rectangle size: 20.
		view node: 3
	].
	window := view open.
	root := view root.
	self assert: (root allSelectedNodes isEmpty).

	nodes first select.
	self assert: (root allSelectedNodes size = 1).	
	
	nodes first deselect.
	self assert: (root allSelectedNodes isEmpty).	
	
	nodes second select.
	self assert: (root allSelectedNodes size = 1).	

	nodes first nodes first select.
	self assert: (root allSelectedNodes size = 2).	
	self assert: (root allSelectedNodes includesAllOf: {nodes first nodes first . nodes second})


! !

!MOViewRendererTest methodsFor: 'nodes' stamp: ''!
testSelectedNodesFromEvent

	|  nodes root bounds form canvas |
	
	view shape rectangle size: 50.
	nodes := view nodes: #($a $b $c) forEach: [ :each | 
		view shape rectangle size: 20.
		view node: 3
	].
	window := view open.
	
	bounds := view root bounds.	
	form := Form extent: bounds extent depth: Display depth.
	canvas := FormCanvas on: form.	
	nodes do: [ :n |  
		n cacheCanvas: canvas. 
		n updateOwner ]. 

	self assert: (nodes first hasCachedForm).
	self assert: (nodes second hasCachedForm).


	
	root := view root.
	self assert: (root allSelectedNodes isEmpty).

	nodes first announce: MOElementSelection new.
	self assert: (root allSelectedNodes size = 1).	
	
	nodes second announce: MOElementSelection new.
	self assert: (root allSelectedNodes size = 1).	

	nodes first nodes first announce: MOElementSelection new.
	self assert: (root allSelectedNodes size = 1).	
	self assert: (root allSelectedNodes includesAllOf: {nodes first nodes first}).

	nodes second nodes first announce: MOElementSelection new.
	self assert: (root allSelectedNodes size = 1).	
	self assert: (root allSelectedNodes includesAllOf: {nodes second nodes first}).

	"Cache of previously selected nodes must have their cache reseted"
	self deny: (nodes first hasCachedForm).
	self deny: (nodes second hasCachedForm).

! !

!MOViewRendererTest methodsFor: 'shapes' stamp: ''!
testShapeAndFraction
	| s n |
	s := MORectangleShape new height: 30.5; width: 10000/3.
	n := MONode on: 10.
	self assert: (s heightFor: n) isInteger.
	self assert: (s widthFor: n) isInteger.! !

!MOViewRendererTest methodsFor: 'subviews' stamp: ''!
testSize
	|  biggerNode1 biggerNode2 |
	
	view shape: (MORectangleShape new
		width: 180; height: 100).
	view node: 1 forIt: [
		view nodes: (1 to: 100).
		view layout: (MOGridLayout withGap: 2)].
	
	view shape: (MORectangleShape new).
	view node: 2 forIt: [
		view nodes: (1 to: 100).
		view layout: (MOGridLayout withGap: 2)].
	
	biggerNode1 := view root nodes first.
	biggerNode2 := view root nodes second.
		
	"The view hasn't been open yet. Therefore, all nodes are located in the same location"
	"This could change in the future, but this is the way it is now"
"	self assert: (theNode bounds = (0@0 extent: 5@5)).
	self assert: (theNode children allSatisfy: [:subNode | subNode bounds = (0@0 extent: 5@5)]).
"
	"We layout the all thing!!"
"	view root applyLayout."

	window := view open.
	self assert: biggerNode2 bounds extent = ( 117@72).
	
	self assert: biggerNode1 bounds origin =  (5@5).
	self assert: biggerNode1 bounds extent = (180@100).

! !

!MOViewRendererTest methodsFor: 'sorting' stamp: ''!
testSortedAs

	self assert: (#(5 4 3 2 1) sortedAs: #value) asArray = #(1 2 3 4 5).
	self assert: (#(5 4 3 2 1) sortedAs: #odd) asArray = #(5 1 3 4 2).	
	self assert: (#(5 4 3 2 1) sortedAs: #even) asArray = #(2 4 3 5 1).	
	self assert: (#(5 4 3 2 1) sortedAs: [:v1 :v2 | v1 < v2 ]) asArray = #(1 2 3 4 5).	

	self assert: (#(5 4 3 2 1) sortedAs: [:v | v * v ]) asArray = #(1 2 3 4 5).	
	self assert: (#(5 4 3 2 1) sortedAs: #squared) asArray = #(1 2 3 4 5).	
	self assert: (#(5 4 3 2 1) sortedAs: [:v | v negated ]) asArray = #(5 4 3 2 1).	! !

!MOViewRendererTest methodsFor: 'layout' stamp: ''!
testSpringLayout
	| nodes  |
	
	view shape rectangle
		width: 40;
		text: #yourself.
		
	nodes := view nodes: (1 to: 20).

	view shape arrowedLine.
	view edges: (2 to: 20) from: 1 to: #yourself.

	view springLayout! !

!MOViewRendererTest methodsFor: 'centered and stretch' stamp: ''!
testStetch
	|  nodes |
	
	view shape: (MORectangleShape new width: #yourself; height: #yourself).
	nodes := view nodes: #(10 100).
	view layout: (MOHorizontalLineLayout new stretchVertically ).
	window := view open.
	
	self assert: (nodes first bounds = ( 5@5 corner: 15@105)).
	self assert: (nodes second bounds = ( 25@5 corner: 125@105)).! !

!MOViewRendererTest methodsFor: 'centered and stretch' stamp: ''!
testStetch2
	|  nodes |
	
	view shape: (MORectangleShape new width: #yourself; height: #yourself).
	nodes := view nodes: #(10 100).
	view layout: (MOVerticalLineLayout new stretchHorizontally).
	window := view open.
	
	self assert: (nodes first bounds = ( 5@5 corner: 105@15)).
	self assert: (nodes second bounds = ( 5@25 corner: 105@125)).! !

!MOViewRendererTest methodsFor: 'subviews' stamp: ''!
testSubview1
	"It seems that once we asked for the bounds on elements, the layout is not properly done. I have to check why (alexandre bergel)"
	
	|  biggerNode |
	
	view interaction draggable.
	view nodes: #(a) forEach: [:each |
		view nodes: (1 to: 5).
	].

	self assert: (view root nodes size = 1).
	
	biggerNode := view root nodes first.
	self assert: (biggerNode model == #a).
	self assert: (biggerNode nodes size = 5).
	self assert: (biggerNode nodes collect: [:node | node model]) asArray = (1 to: 5) asArray.

	window := view open.

	self assert: biggerNode bounds extent = (biggerNode shape computeBoundsFor: biggerNode) extent.	
	self assert: biggerNode bounds = ( 5@5 corner: 80@20 ).
	self assert: (biggerNode shape computeBoundsFor: biggerNode) = ( 0@0 corner: 75@15).
 
	self assert: biggerNode nodes first bounds = ( 5@5 corner: 10@10 ).
	self assert: biggerNode nodes second bounds = ( 20@5 corner: 25@10 ).
	self assert: biggerNode nodes third bounds = ( 35@5 corner: 40@10 ).
	self assert: biggerNode nodes fourth bounds = ( 50@5 corner: 55@10 ).

	self assert: biggerNode bounds = ( 5@5 corner: 80@20 ).
	self assert: (biggerNode shape computeBoundsFor: biggerNode) = ( 0@0 corner: 75@15).
 
	self assert: biggerNode nodes first bounds = ( 5@5 corner: 10@10 ).
	self assert: biggerNode nodes second bounds = ( 20@5 corner: 25@10 ).
	self assert: biggerNode nodes third bounds = ( 35@5 corner: 40@10 ).
	self assert: biggerNode nodes fourth bounds = ( 50@5 corner: 55@10 ).
! !

!MOViewRendererTest methodsFor: 'subviews' stamp: ''!
testSubview2
	"Two subviews"
		
	|  theNode |
	
	view nodes: #(a) forEach: [:each |
		view nodes: (1 to: 5) forEach: [:e |
			view nodes: (1 to: 5).
		].
	].

	self assert: (view root nodes size = 1).
	
	theNode := view root nodes first.
	self assert: (theNode model == #a).
	self assert: (theNode nodes size = 5).
	self assert: (theNode nodes collect: [:node | node model]) asArray = (1 to: 5) asArray.

	window := view open.

	self assert: theNode bounds = (theNode shape boundsFor: theNode).	
	self assert: theNode bounds extent = (theNode shape boundsFor: theNode) extent.	
	self assert: theNode bounds = ( 5@5 corner: 430@30).
 
	self assert: theNode nodes first bounds = ( 5@5 corner: 80@20).
	self assert: theNode nodes second bounds = ( 90@5 corner: 165@20).
	self assert: theNode nodes third bounds = ( 175@5 corner: 250@20).
	self assert: theNode nodes fourth bounds = ( 260@5 corner: 335@20).


	self assert: theNode bounds = ( 5@5 corner: 430@30).
	self assert: (theNode shape computeBoundsFor: theNode) = ( 0@0 corner: 425@25).
 
	self assert: theNode nodes first bounds = ( 5@5 corner: 80@20).
	self assert: theNode nodes second bounds = ( 90@5 corner: 165@20).
	self assert: theNode nodes third bounds = ( 175@5 corner: 250@20).
	self assert: theNode nodes fourth bounds = ( 260@5 corner: 335@20).

! !

!MOViewRendererTest methodsFor: 'subviews' stamp: ''!
testSubview3
	"One subview"
		
	|  biggerNode |
	

	view nodes: #(a) forEach: [:each |
		view shape: (MORectangleShape new width: 10; height: 15).
		view nodes: (1 to: 1).
	].

	self assert: (view root nodes size = 1).
	
	biggerNode := view root nodes first.
	self assert: (biggerNode model == #a).
	self assert: (biggerNode nodes size = 1).
	self assert: (biggerNode nodes collect: [:node | node model]) asArray = {1}.
	
	"The view hasn't been open yet. Therefore, all nodes are located in the same location"
	"This could change in the future, but this is the way it is now"
	"self assert: (biggerNode bounds = ( 0@0 corner: 15@20))."
"	self assert: (theNode children allSatisfy: [:subNode | subNode bounds = (0@0 extent: 5@5)]).
"
	"We layout the all thing!!"
"	view root applyLayout."

	window := view open.

	"bounds returns the bounds using an absolute position"
	"bounds returns a relative bounds, so, the origin is always 0@0"
	self assert: biggerNode bounds ~= (biggerNode shape computeBoundsFor: biggerNode).	
	self assert: biggerNode bounds extent = (biggerNode shape computeBoundsFor: biggerNode) extent.	

	self assert: biggerNode bounds = (biggerNode shape boundsFor: biggerNode).
	self assert: biggerNode bounds = ( 5@5 corner: 25@30).
 
	self assert: biggerNode nodes first bounds = ( 5@5 corner: 15@20).
! !

!MOViewRendererTest methodsFor: 'subviews' stamp: ''!
testSubview4
	"One subview"
		
	|  biggerNode insideNode |
	

	view nodes: #(a) forEach: [:each |
		view shape: (MORectangleShape new width: 10; height: 15).
		view nodes: (1 to: 5).
	].

	self assert: (view root nodes size = 1).
	
	biggerNode := view root nodes first.
	self assert: (biggerNode model == #a).
	
	"The view hasn't been open yet. Therefore, all nodes are located in the same location"
	"This could change in the future, but this is the way it is now"
	self assert: (biggerNode bounds = ( 0@0 corner: 10@15)).
"	self assert: (theNode children allSatisfy: [:subNode | subNode bounds = (0@0 extent: 5@5)]).
"
	"We layout the all thing!!"
"	view root applyLayout."

	window := view open.

	self assert: biggerNode bounds = (biggerNode shape boundsFor: biggerNode).
	self assert: biggerNode bounds extent = (biggerNode shape boundsFor: biggerNode) extent.
	self assert: biggerNode bounds = ( 5@5 corner: 105@30).
 
	self assert: biggerNode nodes first bounds = ( 5@5 corner: 15@20).
	self assert: biggerNode nodes second bounds = ( 25@5 corner: 35@20).
	self assert: biggerNode nodes third bounds = ( 45@5 corner: 55@20).
	
	insideNode := biggerNode nodes first.
	self assert: (insideNode shape computeBoundsFor: insideNode) = ( 0@0 corner: 10@15).! !

!MOViewRendererTest methodsFor: 'subviews' stamp: ''!
testSubview5
	|  outterNode innerNode |
	
	outterNode := view node: 1 forIt: [
		view shape rectangle size: 30. 
		innerNode := view node: 2 ].
	
	window := view open.
	
	self assert: (outterNode bounds = ( 5@5 corner: 45@45)).
	self assert: (innerNode bounds = ( 5@5 corner: 35@35)).
	self assert: (innerNode absoluteBounds  = ( 10@10 corner: 40@40)).
	
	self assert: (view root elementAt: (10@10)) == innerNode.

! !

!MOViewRendererTest methodsFor: 'threading' stamp: ''!
testThreading1
	|  allProcessesBefore allProcessesAfter theProcess |
	allProcessesBefore := Process allInstances.
	
	
	view interaction 
		every: 100 do: [  ].
	view nodes: #(1 2).
	window := view open.

	allProcessesAfter := Process allInstances.	
	self assert: allProcessesAfter size = (allProcessesBefore size + 1).
	theProcess := (allProcessesAfter copyWithoutAll: allProcessesBefore) first.
	self deny: theProcess isTerminated.
	
	window delete.
	window := nil.
	
	self assert: theProcess isSuspended! !

!MOViewRendererTest methodsFor: 'nodes' stamp: ''!
testTranslation

	|  firstEdge |
	
	view nodes: #(1 2).
	view edge: 1 from: #yourself to: [:x | x + 1].
	
	"We open and delete the window to update the position of each of the graph elements. 
	I find this rather ugly. We should compute the position without having to open the window."
	window := view open.

	self assert: (view root bounds = ( 0@0 corner: 25@10)).
	self assert: (view root absoluteBounds = ( 0@0 corner: 25@10)).
	self assert: (view root absoluteBounds = ( 0@0 corner: 25@10)).
	self assert: (view root shapeBounds isDictionary).

	
	"shapeBounds contains the cache. it cannot be empty once the view has been rendered"
	self assert: (view root nodes allSatisfy: [:node | node shapeBounds notEmpty ]).
	
	firstEdge := view root edges first.	
"	self assert: (firstEdge origin = (7@10))."
	self assert: (firstEdge source origin = (5@5)).
	self assert: (firstEdge target origin = (20@5)).

	"-------------------------------"
	"-------------------------------"
	view root translateBy: 8@5. 
	"We go a bit deeper in the uglyness. We update the window to reflect the change in the position"
	window changed.	

	self assert: (view root bounds = ( 0@0 corner: 33@15)).
	self assert: (view root absoluteBounds = ( 0@0 corner: 33@15)).
	self assert: (view root origin = ( 0@0)).

	"The next breaks, is that normal?"
	"self assert: (view root edges first absoluteBounds = (...))."

	"self assert: (firstEdge origin = (0@0))."
	self assert: (firstEdge source origin = (13@10)).
	self assert: (firstEdge target origin = (28@10)).

	"-------------------------------"
	"-------------------------------"
	view root translateBy: 8@5. 
	window changed.

	self assert: (view root absoluteBounds = ( 0@0 corner: 41@20)).
	self assert: (view root origin = (0@0)).

	self assert: (view root edges first source origin = ( 21@15)).
	self assert: (view root edges first target origin = ( 36@15)).
! !

!MOViewRendererTest methodsFor: 'nodes' stamp: ''!
testTranslation2

	|  figure bounds |
	
	view node: #abc.
	figure := view root nodes first.

	window := view open.

	bounds := figure shape boundsFor: figure.
	self assert: (bounds extent = (5@5)).
	self assert: figure origin = (5@5).
	
	figure translateTo: 45@34.
	
	bounds := figure shape boundsFor: figure.
	self assert: (bounds extent = (5@5)).
	self assert: figure origin = (45@34).! !

!MOViewRendererTest methodsFor: 'nodes' stamp: ''!
testTranslationUsingEvent

	|  firstNode oldPosition edge |
	
	view noInteraction.
	view interaction subscribe: MOMouseDrag do: 
		[:ann |  ann element translateBy: ann step.].

	view nodes: #(1 2).
	edge := view edge: 1 from: #yourself to: [:x | x + 1].
	
	"We open and delete the window to update the position of each of the graph elements. 
	I find this rather ugly. We should compute the position without having to open the window."
	window := view open.

	self assert: (view root bounds = (0@0 corner: 25@10)).
	self assert: (view root absoluteBounds = (0@0 corner: 25@10)).
	self assert: (view root absoluteBounds = (0@0 corner: 25@10)).

	"We get a hand on the first node"
	firstNode := view root nodes first.
	self assert: (firstNode model = 1).
	self assert: (firstNode origin = (5@5)).
	oldPosition := firstNode origin.

	"We try to get the node from the left top corner"
	self assert: (view root elementAt: (5@5)) == firstNode.
	"... from the bottom right now"	
	self assert: (view root elementAt: ((5@5) + (firstNode width @ firstNode height))) == edge.
	self assert: (view root elementAt: ((4@4) + (firstNode width @ firstNode height))) == firstNode.

	view noInteraction.
	view interaction subscribe: MOMouseDrag do: [:ann | ann element translateBy: ann step].
 	view announce: ((MOMouseDrag event: nil on: firstNode) step: 1@2).
	
	self assert: ( 6@7 = firstNode origin).
	
	"Since the figure has moved, we cannot get it from its old location"
	self assert: (view root elementAt: (5@5)) == view root.
	self assert: (view root elementAt: (6@7)) == firstNode.
	self assert: (view root elementAt: ((6@7) + (firstNode width @ firstNode height))) == edge.
	self assert: (view root elementAt: ((5@6) + (firstNode width @ firstNode height))) == firstNode.

	"The general bounds must have grown"
	self assert: (view root bounds = ( 0@0 corner: 25@12)).! !

!MOViewRendererTest methodsFor: 'children shape' stamp: ''!
testUMLShape

	|  builder umlShape form form2 canvas2 shape1 shape2 nodes |
	
	builder := MOFormsBuilder new.
	builder column; fill.
	builder row; fill; row; fill.
	builder x: 1 y: 1 add: 
		(shape1 := MORectangleShape new width: 30; height: 15; fillColor: Color green).
	builder x: 1 y: 2 add: 
		(shape2 := MORectangleShape new width: 60; height: 80; fillColor: Color blue).
	umlShape := builder shape.
	
	view shape: umlShape.
	nodes := view nodes: {MOShape . MORectangleShape}.
	
	view root applyLayout.
	form := view getRenderedForm.

	"---"
	self assert: (nodes first absoluteBounds =  (5@5 corner: 65@100)).
	self assert: (nodes second absoluteBounds = (75@5 corner: 135@100)).

	self assert: (umlShape builder boundsOf: shape1 for: nodes first) = ( 0@0 corner: 60@15).
	self assert: (umlShape builder boundsOf: shape2 for: nodes second) = ( 0@15 corner: 60@95).
	! !

!MOViewRendererTest methodsFor: 'children shape' stamp: ''!
testUMLShape2

	|  builder umlShape form form2 canvas2 shape1 shape2 nodes |
	
	builder := MOFormsBuilder new.
	builder column; fill.
	builder row; fill; row; fill.
	builder x: 1 y: 1 add: 
		(shape1 := MORectangleShape new width: 30; height: 15; fillColor: Color green).
	builder x: 1 y: 2 add: 
		(shape2 := MORectangleShape new width: 60; height: 80; fillColor: Color blue).
	umlShape := builder shape.
	
	view shape: umlShape.
	nodes := view nodes: {MOShape}.
	
	view root applyLayout.
	form := view getRenderedForm.
	
	"We draw in a form"
	form2 := Form extent: view root bounds extent depth: Display depth.
	canvas2 := FormCanvas on: form2.
	canvas2 
			frameAndFillRectangle: (( 0@0 corner: 60@15) translateBy: 5@5)
			fillColor: Color green
			borderWidth: 1
			borderColor: Color black.
	canvas2 
			frameAndFillRectangle: (( 0@0 corner: 60@80) translateBy: 5@20)
			fillColor: Color blue
			borderWidth: 1
			borderColor: Color black.

	self assert: form bits = form2 bits.
! !

!MOViewRendererTest methodsFor: 'children shape' stamp: ''!
testUMLShape3

	|  builder umlShape form form2 canvas2 shape1 shape2 nodes |
	
	builder := MOFormsBuilder new.
	builder column; fill.
	builder row; fill; row; fill.
	builder x: 1 y: 1 add: 
		(shape1 := MORectangleShape new width: 30; height: #first; fillColor: Color green).
	builder x: 1 y: 2 add: 
		(shape2 := MORectangleShape new width: 60; height: #second; fillColor: Color blue).
	umlShape := builder shape.
	
	view shape: umlShape.
	nodes := view nodes: {{30 . 80} . {15 . 90} . {40 . 100}}.
	
	view root applyLayout.
	form := view getRenderedForm.
		
	self assert: (nodes first shape computeBoundsFor: nodes first) = ( 0@0 corner: 60@110).
	self assert: (nodes second shape computeBoundsFor: nodes second )= ( 0@0 corner: 60@105).
	self assert: (nodes third shape computeBoundsFor: nodes third) = ( 0@0 corner: 60@140).
	
	self assert: (nodes first shapeBoundsAt: shape1) = ( 0@0 corner: 60@30).
	self assert: (nodes first shapeBoundsAt: shape2) = ( 0@30 corner: 60@110).

	self assert: (nodes second shapeBoundsAt: shape1) = ( 0@0 corner: 60@15).
	self assert: (nodes second shapeBoundsAt: shape2) = ( 0@15 corner: 60@105).

	self assert: (nodes third shapeBoundsAt: shape1) = ( 0@0 corner: 60@40).
	self assert: (nodes third shapeBoundsAt: shape2) = ( 0@40 corner: 60@140).

		
	"We draw in a form"
	form2 := Form extent: view root bounds extent depth: Display depth.
	canvas2 := FormCanvas on: form2.
	canvas2 
			frameAndFillRectangle: (( 0@0 corner: 60@30) translateBy: 5@5)
			fillColor: Color green
			borderWidth: 1
			borderColor: Color black.
	canvas2 
			frameAndFillRectangle: (( 0@30 corner: 60@110) translateBy: 5@5)
			fillColor: Color blue
			borderWidth: 1
			borderColor: Color black.
			

	canvas2 
			frameAndFillRectangle: (( 0@0 corner: 60@15) translateBy: ( nodes second absoluteBounds origin))
			fillColor: Color green
			borderWidth: 1
			borderColor: Color black.
	canvas2 
			frameAndFillRectangle: (( 0@15 corner: 60@105) translateBy: ( nodes second absoluteBounds origin))
			fillColor: Color blue
			borderWidth: 1
			borderColor: Color black.


	canvas2 
			frameAndFillRectangle: (( 0@0 corner: 60@40) translateBy: ( nodes third absoluteBounds origin))
			fillColor: Color green
			borderWidth: 1
			borderColor: Color black.
	canvas2 
			frameAndFillRectangle: (( 0@40 corner: 60@140) translateBy: ( nodes third absoluteBounds origin)) 
			fillColor: Color blue
			borderWidth: 1
			borderColor: Color black.

	self assert: form bits = form2 bits.
	"
	form2 asMorph openInWindow
"! !

!MOViewRendererTest methodsFor: 'nodes' stamp: ''!
testUnExistingEdge


	"If there is no node"
	view edgesFrom: [:each | each].
	self assert: (view root edges isEmpty).
	
	view nodes: #(1).
	view edgesFrom: [:each | each].
	self assert: (view root edges isEmpty).
	
	view nodes: #(1).
	view edgesFrom: [:each | each * 2].
	self assert: (view root edges isEmpty).! !

!MOViewRendererTest methodsFor: 'interaction' stamp: ''!
testUpdateContent
	"We define a small visualization"
	
	view nodes: (1 to: 3).
	view
		edges:
			{(1 -> 2).
			(1 -> 3)}
		from: #key
		to: #value.
	view treeLayout.
	window := view open.
	
	self assert: (view root nodes includes: (view root edges first source)).
	self assert: (view root nodes includes: (view root edges first target)).
	self assert: (view root nodes includes: (view root edges second source)).
	self assert: (view root nodes includes: (view root edges second target)).

	
	"We clear and generate a new visualization"
	view root clear.
	view nodes: (1 to: 2).
	view
		edges:
			{(1 -> 2) }
		from: #key
		to: #value.
	view treeLayout.
	view root applyLayout.
	view updateWindow.
	
	self assert: view root nodes first == (view localLookupNode: 1).
	 
	self assert: (view lookupNode: 1 globally: false) == view root nodes first.
	self assert: (view root nodes includes: (view lookupNode: 1 globally: false)).
	self assert: (view lookupNode: 2 globally: true) == view root nodes second.
	self assert: (view root nodes includes: (view lookupNode: 2 globally: false)).
	
	self assert: (view root nodes includes: (view root edges first source)).
	self assert: (view root nodes includes: (view root edges first target)).
! !

!MOViewRendererTest methodsFor: 'shapes' stamp: ''!
testUpdatingShapeOnEdge

	| edge1 edge2 |
	
	view nodes: #(1 2 3).
	view edgesFromAssociations: {1 -> 2 . 2 -> 3}.
	edge1 := view root edges first.
	edge2 := view root edges second.
	
	self assert: (edge1 shape == edge2 shape).
	self assert: edge1 shape color == edge1 shape defaultColor.
	
	edge1 copyShapeAndDo: [ :s | s color: Color red ].

	self assert: edge1 shape color == Color red. 
	self assert: (edge1 shape ~~ edge2 shape).
	self assert: edge2 shape color == edge2 shape defaultColor.! !

!MOViewRendererTest methodsFor: 'shapes' stamp: ''!
testVerticalOrthogonalLine

	view nodes: (1 to: 10).
	view shape verticalOrthogonalLine.
	view edges: (1 to: 10) from:  1 to: # yourself.
	view treeLayout.
	window := view open.

	self assert: ((view root nodes collect: #bounds) asArray includesAllOf: {37.0@5 corner: 42.0@10. 5.0@30 corner: 10.0@35. 13.0@30 corner: 18.0@35. 21.0@30 corner: 26.0@35. 29.0@30 corner: 34.0@35. 37.0@30 corner: 42.0@35. 45.0@30 corner: 50.0@35. 53.0@30 corner: 58.0@35. 61.0@30 corner: 66.0@35. 69.0@30 corner: 74.0@35})! !

!MOViewRendererTest methodsFor: 'shapes' stamp: ''!
testWrongShape
	"
	self debug: #testWrongShape
	"
	|  nodes |
	
	view shape line.
	nodes := view nodes: (1 to: 10).
	self assert: (view shape notNil).
	self shouldnt: [window := view open] raise: Error.
	self assert: (nodes allSatisfy: [:n | n shape isKindOf: view defaultNodeShapeClass]).
	
	self assert: (view defaultNodeShapeClass new isNodeShape).
	self deny: (view defaultNodeShapeClass new isLineShape).
	self deny: (view defaultLineShapeClass new isNodeShape).
	self assert: (view defaultLineShapeClass new isLineShape).! !

!MOViewRendererTest methodsFor: 'zOrder' stamp: ''!
testZOrder
	|   x y ui zOrder |

	x := #(1 2).
	y := #(#(5 6) #(7 8)).

	
	view node: x using: MORectangleShape withBorder forIt: [
		view nodes: x using: MORectangleShape withBorder.
	].
	view node:  y using: MORectangleShape withBorder forIt: [
		view nodes: y using: MORectangleShape withBorder forEach: [ :b |
			view nodes: b.
		].
	].
	view edge: 1 -> 7 from: #key to: #value.
	window := view open.

	zOrder := view root elementsToDisplay collect: [:e | e zOrder].
	self assert: zOrder asArray = #(3 3 5 5 5 5 6 7 7 7 7).

! !

!MOViewRendererTest methodsFor: 'zooming' stamp: ''!
testZoom
	"No subviews"
	|  node |
	
	view node: #foobar.	

	node := view nodeForDomainValue: #foobar.
	self assert: node bounds = (0@0 extent: 5@5).
	
	window := view open.
	self assert: node bounds = (5@5 extent: 5@5).
	view root increaseZoom.
	self assert: node bounds ~= (5@5 extent: 5@5).
	self assert: node bounds = ((5@5 extent: 5@5) scaleBy: 1.1).

	view root decreaseZoom.
	self assert: node bounds = ( 4.95@4.95 corner: 9.9@9.9).

! !

!MOViewRendererTest methodsFor: 'zooming' stamp: ''!
testZoomWithSubview
	"No subviews"
	|  node |
	
	view node: #foobar forIt: [
		view nodes: (1 to: 100).
		view gridLayout
		"view layout: (MOGridLayout new gapSize: 6)"].	

	node := view nodeForDomainValue: #foobar.
	self assert: node bounds = ( 0@0 corner: 5@5).
	
	window := view open.
	self assert: node bounds = ( 5@5 corner: 200@125).
	self assert: node nodes first bounds = ( 5@5 corner: 10@10).
	
	view root increaseZoom.
	self assert: node bounds = (( 5@5 corner: 200@125) scaleBy: 1.1).
	self assert: node nodes first bounds = (( 5@5 corner: 10@10) scaleBy: 1.1).

	view root decreaseZoom.
	self assert: node bounds = ((( 5@5 corner: 200@125) scaleBy: 1.1) scaleBy: 0.9).
	self assert: node nodes first bounds = 
			((( 5@5 corner: 10@10) scaleBy: 1.1) scaleBy: 0.9).
! !

!MOViewRendererTest methodsFor: 'zooming' stamp: ''!
testZoomWithText
	"No subviews"
	|  node nodeHello nodeWorld boundsHello boundsWorld nodeHelloFontSize nodeWorldFontSize |
	
	view shape label.
	view node: 'hello'.
	view node: #foobar forIt: [
		view shape label.
		view node: 'world' ].

	window := view open.

	nodeHello := view nodeForDomainValue: 'hello'.
	nodeWorld := view nodeForDomainValue: 'world'.
	boundsHello := nodeHello bounds.
	boundsWorld := nodeWorld bounds.

	nodeHelloFontSize := nodeHello shape fontSizeFor: nodeHello.
	nodeWorldFontSize := nodeWorld shape fontSizeFor: nodeWorld.

	view root increaseZoom.
	
	self assert: (nodeHello bounds extent > boundsHello extent).
	self assert: (nodeWorld bounds extent > boundsWorld extent).
	
	self assert: (nodeHello shape fontSizeFor: nodeHello) > nodeHelloFontSize. 
	self assert: (nodeWorld shape fontSizeFor: nodeWorld) > nodeWorldFontSize. 	

	self assert: (nodeHello shape computeTextHeightFor: nodeHello) isInteger.
	self assert: (nodeWorld shape computeTextHeightFor: nodeWorld) isInteger.
	self deny: (nodeHello shape fontSizeFor: nodeHello) isInteger.
	self deny: (nodeWorld shape fontSizeFor: nodeWorld) isInteger.

	
	view root decreaseZoom.
	view root decreaseZoom.
	
	self assert: (boundsHello extent > nodeHello bounds extent).
	self assert: (boundsWorld extent > nodeWorld bounds extent).

	self assert: (nodeHello shape fontSizeFor: nodeHello) < nodeHelloFontSize. 
	self assert: (nodeWorld shape fontSizeFor: nodeWorld) < nodeWorldFontSize. 	

	self assert: (nodeHello shape computeTextHeightFor: nodeHello) isInteger.
	self assert: (nodeWorld shape computeTextHeightFor: nodeWorld) isInteger.
	self deny: (nodeHello shape fontSizeFor: nodeHello) isInteger.
	self deny: (nodeWorld shape fontSizeFor: nodeWorld) isInteger.


! !

!MOViewRendererTest methodsFor: 'type covering' stamp: ''!
typeOfEdgeFromToUsing
	^ #(MOArrowedLineShape) collect: [:s | Smalltalk at: s]! !

!MOViewRendererTest methodsFor: 'type covering' stamp: ''!
typeOfEdgesFromToAllUsing
	^ #(MOArrowedLineShape) collect: [:s | Smalltalk at: s]! !

!MOViewRendererTest methodsFor: 'type covering' stamp: ''!
typeOfEdgesFromToAllUsingFromGlobalToGlobal
	^ #(MOArrowedLineShape) collect: [:s | Smalltalk at: s]! !

!MOViewRendererTest methodsFor: 'type covering' stamp: ''!
typeOfEdgesFromToUsing
	^ #(MOArrowedLineShape) collect: [:s | Smalltalk at: s]! !

!MOViewRendererTest methodsFor: 'type covering' stamp: ''!
typeOfMOViewRendererAnnounce
	^ #(MOMouseLeave MOKeyPressed MOKeyDown MOKeyEvent MOZoomEvent MOContentChanged MOMouseEvent MOKeyUp MOMouseMove MOMouseUp) collect: [:s | Smalltalk at: s]! !

!MOViewRendererTest methodsFor: 'type covering' stamp: ''!
typeOfMOViewRendererNodeUsing
	^ #(MOEllipseShape MOFixedRectangleShape MORoundedRectangleShape) collect: [:s | Smalltalk at: s]! !

!MOViewRendererTest methodsFor: 'type covering' stamp: ''!
typeOfMOViewRendererNodesUsing
	^ #(MOFixedRectangleShape MORoundedRectangleShape) collect: [:s | Smalltalk at: s]! !

!MOViewRendererTest methodsFor: 'type covering' stamp: ''!
typeOfMOViewRendererNodesUsingForEach
	^ #(MOFormsShape) collect: [:s | Smalltalk at: s]! !

!MOViewRendererTest methodsFor: 'type covering' stamp: ''!
typeOfMOViewRendererlayout
	^ #(MOFlowLayout MOBulletTreeLayout MOSugiyamaLayout) collect: [:s | Smalltalk at: s]! !

!MOViewRendererTest methodsFor: 'type covering' stamp: ''!
typeOfNodeUsingForIt
	^ #(MOEllipseShape MOFixedRectangleShape MORoundedRectangleShape) collect: [:s | Smalltalk at: s]! !

!MOViewRendererTest methodsFor: 'type covering' stamp: ''!
typeOfNodesUsingForEach
	^ #(MOEllipseShape MOFixedRectangleShape MORoundedRectangleShape) collect: [:s | Smalltalk at: s]! !

!MondrianTest methodsFor: 'running' stamp: ''!
tearDown
	super tearDown.
	window ifNotNil: [window delete. window := nil]! !

TestCase subclass: #MondrianTutorialTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Tests'!

!MondrianTutorialTest methodsFor: 'tests' stamp: 'BenComan 5/11/2012 02:50'!
testImageGeneration
	| tutorial latexTheme |

	tutorial := MondrianTutorial new.
	latexTheme := StandAloneLatexTheme new.
	tutorial latexTheme: latexTheme.
	self assert: (tutorial translateLessonIntoLatex: '<code>view nodes: (1 to: 20)</code> foo bar') =  '\begin{verbatim}view nodes: (1 to: 20)\end{verbatim} foo bar'.

	tutorial resetPictureCounter.
	self assert: (tutorial translateLessonIntoLatex: 'foo bar zork<codeWithImage>view nodes: (1 to: 20)</codeWithImage>foo bar') =   
( 'foo bar zork\begin{verbatim}view nodes: (1 to: 20)\end{verbatim}
\begin{center}\includegraphics[scale=' , latexTheme figureScale printString , ']{figures/picture1.png}\end{center}
foo bar').

	tutorial resetPictureCounter.
	self assert: (tutorial translateLessonIntoLatex: '<codeWithImage>view nodes: (1 to: 20)</codeWithImage><codeWithImage>b := 2</codeWithImage>') =  
('\begin{verbatim}view nodes: (1 to: 20)\end{verbatim}
\begin{center}\includegraphics[scale=' , latexTheme figureScale printString , ']{figures/picture1.png}\end{center}
\begin{verbatim}b := 2\end{verbatim}
\begin{center}\includegraphics[scale=' , latexTheme figureScale printString , ']{figures/picture2.png}\end{center}
')! !

!MondrianTutorialTest methodsFor: 'tests' stamp: ''!
testOpenTutorial

	| prof mockView |
	prof := ProfStef new.
	prof player: (
		TutorialPlayer new 
			tutorial: MondrianTutorial new; 
			yourself).
	mockView := MockLessonView new.
	prof lessonView: mockView.
	prof open.

	prof player size timesRepeat: [ prof next ].  ! !

!MondrianTutorialTest methodsFor: 'tests' stamp: ''!
testTranslation
	| tutorial |
	tutorial := MondrianTutorial new.
	tutorial latexTheme: PharoByExampleLatexTheme new.
	self assert: (tutorial translateLessonIntoLatex: '<code>a := 1</code>') = '\begin{code}{}a := 1\end{code}'.
	self assert: (tutorial translateLessonIntoLatex: '<code>a := 1</code> <code>b := 2</code>') = '\begin{code}{}a := 1\end{code} \begin{code}{}b := 2\end{code}'.

	self assert: (tutorial translateLessonIntoLatex: '| a | a := 1') = '| a | a := 1'.
	
	tutorial latexTheme: StandAloneLatexTheme new.
	self assert: (tutorial translateLessonIntoLatex: '<code>a := 1</code>') =  '\begin{verbatim}a := 1\end{verbatim}'.
	self assert: (tutorial translateLessonIntoLatex: 'foo bar <sel>selector1</sel> <sel>selector2:</sel>') =  'foo bar \co{\#selector1} \co{\#selector2:}'! !

HealthReportProducer subclass: #MondrianHealth
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Tests'!

!MondrianHealth classSide methodsFor: 'public' stamp: ''!
produceReport
	^ self new produceReport! !

!MondrianHealth methodsFor: 'logging' stamp: ''!
logMethodCoverage
	^ #(#(58 'September 26, 2009'))
	,#(#(58.5 'MethodCoverage'))	,#(#(58.83 'MethodCoverage'))	,#(#(58.83 'MethodCoverage'))	,#(#(60.41 'MethodCoverage'))	,#(#(60.13 'MethodCoverage'))	,#(#(60.03 'MethodCoverage'))	,#(#(60.03 'MethodCoverage'))	,#(#(60.03 'MethodCoverage'))	,#(#(60.03 'MethodCoverage'))	,#(#(60.03 'MethodCoverage'))	,#(#(60.03 'MethodCoverage'))	,#(#(60.03 'MethodCoverage'))	,#(#(60.22 'MethodCoverage'))	,#(#(55.03 'MethodCoverage'))	,#(#(55.11 'MethodCoverage'))	,#(#(52.11 'MethodCoverage'))	,#(#(55.11 'MethodCoverage'))	,#(#(56.04 'MethodCoverage'))	,#(#(55.08 'MethodCoverage'))	,#(#(55.65 'MethodCoverage'))	,#(#(55.54 'MethodCoverage'))	,#(#(54.63 'MethodCoverage'))	,#(#(54.71 'MethodCoverage'))	,#(#(54.78 'MethodCoverage'))	,#(#(54.77 'MethodCoverage'))	,#(#(54.77 'MethodCoverage'))	,#(#(60.6 'MethodCoverage'))! !

!MondrianHealth methodsFor: 'public' stamp: ''!
produceReportOn: stream
	"
	self new produceReport
	"
	self writeGeneralInfoOn: stream.
	self writeBenchmarkResultsOn: stream.
	self writeBenchmark2ResultsOn: stream.
	
	self writeBenchmark3ResultsOn: stream.
	self writeBenchmark4ResultsOn: stream.
	self writeBenchmark5ResultsOn: stream.

	self writeBenchmark6ResultsOn: stream.

	self writeBenchmark7ResultsOn: stream.

	self writeBenchmark8ResultsOn: stream.	
		
	self writeBenchmark9ResultsOn: stream.			


"	self writeCoverageInfoOn: stream.
"! !

!MondrianHealth methodsFor: 'report producers' stamp: ''!
writeBenchmark2ResultsOn: stream
	| bench numberOfEdges results v |
	numberOfEdges := #(10 20 30 40 50 60 70 80 90 100 200 300).
	results := OrderedCollection new.
	stream
		nextPutAll: 'Benchmark ManyEdges (simple rendering of edges) :';
		cr.
	bench := MOBenchmarks new.
	self displayProgressBar:
			[ :bar | 
			numberOfEdges do:
					[ :number | 
					Transcript
						show: 'number of edges = ';
						show: number printString;
						cr.
					v := bench reportManyEdgesBenchFor: number.
					results add: v.
					stream
						nextPutAll: '  ';
						nextPutAll: number printString;
						nextPutAll: ' edges => ';
						nextPutAll: v printString;
						nextPutAll: ' ms';
						cr.
					self incrProgressBar: bar ] ]
		size: numberOfEdges size
		titled: 'ManyEdges benchmark'.
	"self openMondrianGraphOn: (numberOfEdges with: results collect: [ :v1 :v2 | v1 -> v2 ]) titled: 'ManyEdges'"! !

!MondrianHealth methodsFor: 'report producers' stamp: ''!
writeBenchmark3ResultsOn: stream
	| bench numberOfNodes results v |
	numberOfNodes := #(5 10 15).
	results := OrderedCollection new.
	
	stream nextPutAll: 'Benchmark ManyInnerNodes :'; cr.
	bench := MOBenchmarks new.
	self displayProgressBar: [:bar | 
			numberOfNodes
				do: [:number |
						v := bench reportManyInnerNodesBenchFor: number.
						results add: v.
						stream nextPutAll: '  ';
							nextPutAll: number printString;
							nextPutAll: ' nodes => ';
							nextPutAll: v printString;
							nextPutAll: ' ms';
							cr.
						self incrProgressBar: bar]]
		 size: numberOfNodes size  titled: 'ManyInnerNodes benchmark'.

	"self openMondrianGraphOn: (numberOfNodes with: results collect: [:v1 :v2 | v1 -> v2]) titled: 'ManyNodes'.
"! !

!MondrianHealth methodsFor: 'report producers' stamp: ''!
writeBenchmark4ResultsOn: stream
	| bench numberOfNodes results v |
	numberOfNodes := #(5 10 15).
	results := OrderedCollection new.
	
	stream nextPutAll: 'Benchmark Displaying ManyInnerNodes :'; cr.
	bench := MOBenchmarks new.
	self displayProgressBar: [:bar | 
			numberOfNodes
				do: [:number |
						v := bench reportManyDisplayingInnerNodesBenchFor: number.
						results add: v.
						stream nextPutAll: '  ';
							nextPutAll: number printString;
							nextPutAll: ' nodes => ';
							nextPutAll: v printString;
							nextPutAll: ' ms';
							cr.
						self incrProgressBar: bar]]
		 size: numberOfNodes size  titled: 'Displaying ManyInnerNodes benchmark'.

	"self openMondrianGraphOn: (numberOfNodes with: results collect: [:v1 :v2 | v1 -> v2]) titled: 'Displaying ManyNodes'.
"! !

!MondrianHealth methodsFor: 'report producers' stamp: ''!
writeBenchmark5ResultsOn: stream
	| bench numberOfNodes results v |
	numberOfNodes := #(1 2 3 4).
	results := OrderedCollection new.
	
	stream nextPutAll: 'Benchmark Displaying ManyInnerNodesAndEdges :'; cr.
	bench := MOBenchmarks new.
	self displayProgressBar: [:bar | 
			numberOfNodes
				do: [:number |
						v := bench reportManyDisplayingInnerNodesAndEdgesBenchFor: number.
						results add: v.
						stream nextPutAll: '  ';
							nextPutAll: number printString;
							nextPutAll: ' nodes => ';
							nextPutAll: v printString;
							nextPutAll: ' ms';
							cr.
						self incrProgressBar: bar]]
		 size: numberOfNodes size  titled: 'Displaying ManyInnerNodesAndEdges benchmark'.
! !

!MondrianHealth methodsFor: 'report producers' stamp: ''!
writeBenchmark6ResultsOn: stream
	| bench numberOfIterations results v |
	numberOfIterations := #(100 500 1000 1500 2000 2500).
	results := OrderedCollection new.
	
	stream nextPutAll: 'Benchmark Displaying elementAt :'; cr.
	bench := MOBenchmarks new.
	self displayProgressBar: [:bar | 
			numberOfIterations
				do: [:number |
						v := bench reportElementAtFor:  number.
						results add: v.
						stream nextPutAll: '  ';
							nextPutAll: number printString;
							nextPutAll: ' nodes => ';
							nextPutAll: v printString;
							nextPutAll: ' ms';
							cr.
						self incrProgressBar: bar]]
		 size: numberOfIterations size  titled: 'Benchmark Displaying elementAt :'.
! !

!MondrianHealth methodsFor: 'report producers' stamp: ''!
writeBenchmark7ResultsOn: stream
	| v numberOfNodes |

	numberOfNodes := 2000.
	stream nextPutAll: 'Benchmark many small nodes : '; cr.
	v := MOBenchmarks new reportManySmallNodesBenchFor: numberOfNodes.
	stream nextPutAll: '  ';
			nextPutAll: numberOfNodes printString;
			nextPutAll: ' nodes => ';
			nextPutAll: v printString;
			nextPutAll: ' ms';
			cr.! !

!MondrianHealth methodsFor: 'report producers' stamp: ''!
writeBenchmark8ResultsOn: stream
	| v numberOfNodes |
	numberOfNodes := 500.
	stream nextPutAll: 'Benchmark edges bounds : '; cr.
	v := MOBenchmarks new reportEdgeBounds: 500.
	stream nextPutAll: '  ';
			nextPutAll: numberOfNodes printString;
			nextPutAll: ' nodes => ';
			nextPutAll: v printString;
			nextPutAll: ' ms';
			cr.! !

!MondrianHealth methodsFor: 'report producers' stamp: ''!
writeBenchmark9ResultsOn: stream
	| v numberOfNodes |
	numberOfNodes := 20000.
	stream nextPutAll: 'Benchmark subnodes lookup : '; cr.
	v := MOBenchmarks new reportEdgeBounds: numberOfNodes.
	stream nextPutAll: '  ';
			nextPutAll: numberOfNodes printString;
			nextPutAll: ' nodes => ';
			nextPutAll: v printString;
			nextPutAll: ' ms';
			cr.! !

!MondrianHealth methodsFor: 'report producers' stamp: ''!
writeBenchmarkResultsOn: stream
	| bench numberOfNodes results v |
	numberOfNodes := #(100 200 300 400 500 600 700 800 900 1000 1600 3200 6400).
	results := OrderedCollection new.
	
	stream nextPutAll: 'Benchmark ManyNode (simple rendering of nodes) :'; cr.
	bench := MOBenchmarks new.
	self displayProgressBar: [:bar | 
			numberOfNodes
				do: [:number |
						v := bench reportManyNodesBenchFor: number.
						results add: v.
						stream nextPutAll: '  ';
							nextPutAll: number printString;
							nextPutAll: ' nodes => ';
							nextPutAll: v printString;
							nextPutAll: ' ms';
							cr.
						self incrProgressBar: bar]]
		 size: numberOfNodes size  titled: 'ManyNodes benchmark'.

	"self openMondrianGraphOn: (numberOfNodes with: results collect: [:v1 :v2 | v1 -> v2]) titled: 'ManyNodes'.
"! !

HealthReportProducer subclass: #MondrianHealth2
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Tests'!

!MondrianHealth2 commentStamp: 'AlexandreBergel 12/30/2010 12:25' prior: 0!
MondrianHealth2 check the health of Mondrian, including running the tests

| str |
str := WriteStream on: String new.
MondrianHealth2 new runAndWriteResultInStream:  str.
str!

!MondrianHealth2 methodsFor: 'structural metrics' stamp: ''!
metric001LinesOfCode
	<description: 'Number of lines of code in Mondrian'>
	^ self mondrianMethods inject: 0 into: 
		[ :sum :el | sum + el numberOfLinesOfCode ]
! !

!MondrianHealth2 methodsFor: 'structural metrics' stamp: ''!
metric002NumberOfClasses
	<description: 'Number of methods in Mondrian'>
	^ self mondrianClasses size! !

!MondrianHealth2 methodsFor: 'structural metrics' stamp: ''!
metric003NumberOfMethods
	<description: 'Number of methods in Mondrian'>
	^ self mondrianMethods size! !

!MondrianHealth2 methodsFor: 'benchmarks' stamp: ''!
metric004ManyEdges
	<description: 'Rendering of 300 edges'>
	
	| bench |
	bench := MOBenchmarks new.
	^ bench reportManyEdgesBenchFor: 300! !

!MondrianHealth2 methodsFor: 'benchmarks' stamp: ''!
metric005SubnodesLookup
	<description: 'Asking of 10000 edges bounds'>
	
	| bench |
	bench := MOBenchmarks new.
	^ bench reportEdgeBounds: 10000! !

!MondrianHealth2 methodsFor: 'benchmarks' stamp: ''!
metric006NonVisibleEdges
	<description: 'Displaying non visible edges'>
	
	"
	self new metric006NonVisibleEdges
	"

	| view edges firstNode bounds form canvas |
	view := MOViewRenderer new.
	view nodes: (1 to: 1000).
	edges := view edges: {1} from: #yourself toAll: (2 to: 1000).
	view root applyLayout.

	firstNode := view root nodes first.
	firstNode announce: (MOMouseDrag step: 0 @ 30).
	firstNode announce: (MOMouseDrag step: 0 @ -30).

	bounds := view canvas bounds.

	form := Form extent: bounds extent depth: Display depth.
	canvas := FormCanvas on: form.

	^ Time millisecondsToRun: [ 
		150 timesRepeat: [
			canvas clipBy: ( 0@20 corner: 14995@40) during:
				[ :aCanvas | view root acceptVisitor: (MOMorphPainterVisitor newCanvas: aCanvas). ]	] ] 
! !

!MondrianHealth2 methodsFor: 'benchmarks' stamp: ''!
metric007Rendering
	<description: 'Rendering simple view'>
	
	"
	self new metric007Rendering
	"

	| view edges firstNode bounds form canvas |
	view := MOViewRenderer new.
	view nodes: (1 to: 1000).
	edges := view edges: {1} from: #yourself toAll: (2 to: 1000).
	view root applyLayout.

	^ Time millisecondsToRun: [ 
		20 timesRepeat: [ view getRenderedForm ] ] 
! !

!MondrianHealth2 methodsFor: 'util' stamp: ''!
mondrianClasses
	| classes |
	classes := OrderedCollection new.
	(PackageInfo allPackages select: [:p| p packageName beginsWith: 'Mondrian-']) 
		do: [:p | classes addAll: p classes ].
	^ classes ! !

!MondrianHealth2 methodsFor: 'util' stamp: ''!
mondrianMethods
	| methods |
	methods := OrderedCollection new.
	self mondrianClasses do: [:c | methods addAll: c methods ].
	^ methods! !

!MondrianHealth2 methodsFor: 'accessing-computed' stamp: ''!
packages
	^ PackageInfo allPackages select: [ :p | 'Mondrian-*' match: p packageName ]! !

Object subclass: #MOBenchmarks
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Tests'!

!MOBenchmarks methodsFor: 'benchmarks' stamp: ''!
edges
	"
	Time millisecondsToRun: self new edges        
		=> 13922 (Alexandre.Bergel 9/25/2009 21:06 -- with the cache, but it is buggy)
		=> 26589 (Alexandre.Bergel 9/25/2009 21:04 - new and faster machine, but without the cache)
		
		=> 14198 (Alexandre.Bergel 8/18/2009 21:40)
		
		=> 13238 (Alexandre.Bergel 5/14/2009 01:03) Doru screwed everything now
		=> 13044 (Alexandre.Bergel 5/14/2009 00:58) Doru screwed up something
		=> 12982 (ab 5/12/2009 22:54 With Jannik pair programming) smarter cache
		=> 13033 (ab 5/12/2009 22:31) node are always cached
		=> 12773 April 15, 2009 (13:55) no aliasing
		=> 22602 April 15, 2009 (13:54) with aliasing
		=> 12961 April 15, 2009 (13:50) Minor addition (cache for edges)
		=> 12919 April 10, 2009 (23:51) Fixed a bug when using the cache
		=> 13234 April 10, 2009 (23:09) Added caches in edges
		=> 21914 April 10, 2009 (17:56) Added a cache for bounds in MORoot, see comments of MORoot>>absoluteBounds
		=> 28266 April 10, 2009 (17:37) Removed haltOnce
		=> 35737 April 10, 2009 (17:32) Aliasing removed, use of absolute bounds instead of relative bounds
		=> 27260 March 26, 2009 (11:18) 
		=> 28470 March 10, 2009 (13:55) Replaced calls to 
					drawPath:borderWidth:borderColor: by line:to:width:color: in BalloonCanvas
		=> 41319 March 10, 2009 (13:52) No idea how I gained improvement
		=> 43842 March 9, 2009 (13:17) No idea why this is slow
		=> 36441 March 6, 2009 (10:42) removed some indirection with nil
		=> 38067 March 6, 2009 (10:18) I replaced IdentityDictionary by RBIdentityDictionary
		=> 51333 March 6, 2009 (9:55)

	MessageTally spyOn: MOBenchmarks new edges
	"
	
	| view bounds form canvas size |
	view := MOViewRenderer new.
	size := 30.
	view nodes: (1 to: size).
	view edges: (1 to: size) from: #yourself toAll: (1 to: size).

	view root applyLayout.
	bounds := view root bounds.
	
	form := Form extent: bounds extent depth: Display depth.
	canvas := FormCanvas on: form.

	^ [1 to: 1000 do: [:i | view root acceptVisitor: (MOMorphPainterVisitor newCanvas: canvas).]]
	
"Time millisecondsToRun:[	| view bounds form canvas size |
	view := MOViewRenderer new.
	size := 30.
	view nodes: (1 to: size).
	view edges: (1 to: size) from: #yourself toAll: (1 to: size).

	view root applyLayout.
	bounds := view root bounds.
	
	form := Form extent: bounds extent.
	canvas := BalloonCanvas on: form.

	^ [1 to: 1000 do: [:i | view root displayOn: canvas]]]"
	! !

!MOBenchmarks methodsFor: 'benchmarks' stamp: ''!
edges2
	"
	self new edges2    
		=> 1325 (Alexandre.Bergel 9/25/2009 21:04 - new and faster machine)
		
		=>  1707 (Alexandre.Bergel 8/18/2009 21:40)
		=>  1543 (Alexandre.Bergel 4/10/2009 23:10)
		=>  1352 (Alexandre.Bergel 4/10/2009 17:47)
		=> 1291 (Alexandre.Bergel 4/2/2009 01:16)
		=> 2692 (March 26, 2009)  -- VW: 19
	"
	
	| view size|
	view := MOViewRenderer new.
	size := 5.
	view nodes: (1 to: size) forEach: [:each | view nodes: (1000 to: 1010). view gridLayout].
	view edges: (1 to: size) from: #yourself toAll: [:each | (1 to: size) copyWithout: each].

	^ Time millisecondsToRun: [
		1 to: 1000 do: [:i | view root applyLayout]]

"	^ Time millisecondsToRun: [view root applyLayout].
""	bounds := view root bounds.
	
	form := Form extent: bounds extent.
	canvas := BalloonCanvas on: form.
"
"	^ Time millisecondsToRun: [
		1 to: 1000 do: [:i | view root displayOn: canvas]]
	
"	

"VW CODE:

Time millisecondsToRun: [	
  | view size |
	view := ViewRenderer new.
	view := MOViewRenderer new.
	size := 5.
	view nodes: (1 to: size) forEach: [:each | view nodes: (1000 to: 1010). view gridLayout].
	view edges: (1 to: size) from: #yourself toAll: [:each | (1 to: size) copyWithout: each].

	1 to: 1000 do: [:i | view canvas]].
"! !

!MOBenchmarks methodsFor: 'forms' stamp: ''!
form1
	"
	self new form1
	Pair programming with Fabrizio Perin

	MessageTally spyOn: MOBenchmarks new form1
	Time millisecondsToRun: self new form1 
	
	=>  155 (ab 5/12/2009 22:29) 
	=>  145 Alexandre.Bergel 5/10/2009 23:52 Added the form cache
	=> 2476 Alexandre.Bergel 5/10/2009 22:28  No cache at all
	MessageTally spyOn: MOBenchmarks new form1
	"
	
	| view bounds form canvas size |
	view := MOViewRenderer new.

	size := 30.
	view nodes: (1 to: 5) forEach: [:each|
		view nodes: (1 to: size).
		view edges: (1 to: size) from: #yourself toAll: (1 to: size) using: (MOStraightLineShape new) fromGlobal: false toGlobal: false.
		view gridLayout].

	view root applyLayout.
	bounds := view root bounds.
	
	"The size of each node is 90@75"
	
	form := Form extent: bounds extent depth: Display depth.
	canvas := FormCanvas on: form.

	^ [1 to: 50 do: [:i | view root acceptVisitor: (MOMorphPainterVisitor newCanvas: canvas).]]
! !

!MOBenchmarks methodsFor: 'benchmarks' stamp: ''!
manyNodes
	"
	Time millisecondsToRun: self new manyNodes  

	=> 3200 nodes =>  13088 ms	
	=> 1600 nodes =>  3349 ms	
	=> 800 nodes =>   817 ms	
	=> 400 nodes =>   211 ms	
	=> 200 nodes =>   56 ms	
	=> 100 nodes =>  18 ms

	
	
	=> 800 nodes => 831 ms (Alexandre.Bergel 8/18/2009 22:10) Added caches for bounds and removed unnecessary call in origin. 
	=> 800 nodes => 1888 (Alexandre.Bergel 8/18/2009 21:45) added a cache for the bounds
	=> 800 nodes => 3811 (Alexandre.Bergel 8/18/2009 21:42) just added a temporary variable in MORoot>>updatesBounds to call bounds only once
	=> 800 nodes => 3962 (Alexandre.Bergel 8/18/2009 21:39)

	No optimization Alexandre.Bergel 7/22/2009 09:39
	=> 3200 nodes =>  62365 ms	
	=> 1600 nodes =>  17317 ms	
	=> 800 nodes =>  4587 ms	
	=> 400 nodes =>  1152 ms	
	=> 200 nodes =>  252 ms	
	=> 100 nodes =>  65 ms



	MessageTally spyOn: self new manyNodes
	"
	
	| view |
	view := MOViewRenderer new.
	view nodes: (1 to: 3200).
	^ [ view root applyLayout ] 
	
"Time millisecondsToRun:[	| view bounds form canvas size |
	view := MOViewRenderer new.
	size := 30.
	view nodes: (1 to: size).
	view edges: (1 to: size) from: #yourself toAll: (1 to: size).

	view root applyLayout.
	bounds := view root bounds.
	
	form := Form extent: bounds extent.
	canvas := BalloonCanvas on: form.

	^ [1 to: 1000 do: [:i | view root displayOn: canvas]]]"
	! !

!MOBenchmarks methodsFor: 'benchmarks' stamp: ''!
openSimpleGraph
	"
	self new openSimpleGraph 
	=> 2424 (Alexandre.Bergel 8/18/2009 21:40)
	=> 1231 (Alexandre.Bergel 4/10/2009 23:11)
	=> 1471 (Alexandre.Bergel 4/10/2009 17:58)
	=> 1178 (Alexandre.Bergel 4/2/2009 01:17)
	=> 3674 (March 27, 2009)  (VW: 10408)
	"
	^ Time millisecondsToRun: 
	[| view |
	view := MOViewRenderer new.
	view nodes: (1 to: 100) forEach: [:each|
		view nodes: (1 to: 500).
		view gridLayout
	].
	view gridLayout.
	view open.
]! !

!MOBenchmarks methodsFor: 'benchmarks' stamp: ''!
reportEdgeBounds: iteration
	"
	self new reportEdgeBounds: 500  
	=> 135 (AlexandreBergel 9/14/2010 10:21 & Henrik) with introducing the cache set in MOEdge>>bounds
	=> 1630 (AlexandreBergel 9/14/2010 10:20 & Henrik)

 	"

	| view edges |
	view := MOViewRenderer new.
	view nodes: (1 to: 1000).
	edges := view edges: {1} from: #yourself toAll: (2 to: 1000).

	^ Time millisecondsToRun: [ iteration timesRepeat: [ edges do: #bounds] ] 
! !

!MOBenchmarks methodsFor: 'benchmarks' stamp: ''!
reportElementAtFor: iteration
	"
	self new reportElementAtFor: 100

	KaiProfiler viewProfiling: [	self new reportElementAtFor: 2500 ]
	"
	| view node center |
	view := MOViewRenderer new.
	view node: #a forIt: [ view node: #a forIt: [ view node: #a forIt: [ view node: #a forIt: [ node := view node: #a ] ] ] ].
	"view open.
	"
	
	center := node bounds center.
	^ [ iteration timesRepeat: [(view root elementAt: center )] ] timeToRun! !

!MOBenchmarks methodsFor: 'benchmarks' stamp: ''!
reportManyDisplayingInnerNodesAndEdgesBenchFor: numberOfNodes


	| view bounds form canvas |
	view := MOViewRenderer new.
	view nodes: (1 to: numberOfNodes) forEach: 
		[:each1| view nodes: (1 to: numberOfNodes) forEach: 
			[:each2| view nodes: (1 to: numberOfNodes) forEach: 
				[:each3| view nodes: (1 to: numberOfNodes). view edges:  (1 to: numberOfNodes) from: #yourself toAll: (1 to: numberOfNodes)] ] ].
	 view root applyLayout.

	bounds := view root bounds.	
	form := Form extent: bounds extent depth: Display depth.
	canvas := FormCanvas on: form.
	^ Time millisecondsToRun: [ 100 timesRepeat: [ view root acceptVisitor: (MOMorphPainterVisitor newCanvas: canvas). ] ] 
! !

!MOBenchmarks methodsFor: 'benchmarks' stamp: ''!
reportManyDisplayingInnerNodesBenchFor: numberOfNodes


	| view bounds form canvas |
	view := MOViewRenderer new.
	view nodes: (1 to: numberOfNodes) forEach: 
		[:each1| view nodes: (1 to: numberOfNodes) forEach: 
			[:each2| view nodes: (1 to: numberOfNodes) forEach: 
				[:each3| view nodes: (1 to: numberOfNodes) ] ] ].
	 view root applyLayout.

	bounds := view root bounds.	
	form := Form extent: bounds extent depth: Display depth.
	canvas := FormCanvas on: form.
	^ Time millisecondsToRun: [ view root acceptVisitor: (MOMorphPainterVisitor newCanvas: canvas). ] 
! !

!MOBenchmarks methodsFor: 'benchmarks' stamp: ''!
reportManyEdgesBenchFor: numberOfEdges

	| v view |
	v := 1 to: numberOfEdges.
	view := MOViewRenderer new.
	view nodes: v.

	^ Time millisecondsToRun: [
		view edges: v from: 1 toAll: v.
		view root applyLayout ] ! !

!MOBenchmarks methodsFor: 'benchmarks' stamp: ''!
reportManyInnerNodesBenchFor: numberOfNodes

	| view |
	view := MOViewRenderer new.
	view nodes: (1 to: numberOfNodes) forEach: 
		[:each1| view nodes: (1 to: numberOfNodes) forEach: 
			[:each2| view nodes: (1 to: numberOfNodes) forEach: 
				[:each3| view nodes: (1 to: numberOfNodes) ] ] ].
	^ Time millisecondsToRun: [ view root applyLayout ] 
! !

!MOBenchmarks methodsFor: 'benchmarks' stamp: ''!
reportManyNodesBenchFor: numberOfNodes


	| view |
	view := MOViewRenderer new.
	view nodes: (1 to: numberOfNodes).
	^ Time millisecondsToRun: [ view root applyLayout ] 
! !

!MOBenchmarks methodsFor: 'benchmarks' stamp: ''!
reportManySmallNodesBenchFor: numberOfNodes

	"
	self new reportManySmallNodesBenchFor: 2000
	"
	| view bounds form canvas node |
	view := MOViewRenderer new.
	view nodes: {$a . $b . $c} forEach: [:each|
		view node: #foo forIt: [
			view nodes: (1 to: numberOfNodes).
			view gridLayout gapSize: 1]
	].
	view root applyLayout.
	
	bounds := view root bounds.	
	form := Form extent: bounds extent depth: Display depth.
	canvas := FormCanvas on: form.
	
"	^ Time millisecondsToRun: [ view root displayOn: canvas ]"
	node := view root nodes first.
	
	^ Time millisecondsToRun: [ 10 timesRepeat: [ node announce: (MOMouseDrag step: 1@2). view root acceptVisitor: (MOMorphPainterVisitor newCanvas: canvas). ] ]

"	KaiProfiler viewProfiling: [ 5 timesRepeat: [ node announce: (MOMouseDrag step: 1@2). view root displayOn: canvas ] ]
"! !

!MOBenchmarks methodsFor: 'benchmarks' stamp: ''!
reportPackageBlueprintLikeProfiling

	"
	self new reportPackageBlueprintLikeProfiling 
	=>7204 (Alexandre Bergel 7/15/2010 13:30)
	"
	| view bounds form canvas node |
	view := MOViewRenderer new.
	view nodes: #(1 2) forEach: [:each |
		view nodes: (1 to: 26) forEach: [:each2|
			view node: #foo forIt: [
				view node: #foo forIt: [
					view nodes: (1 to: 114).
					view gridLayout gapSize: 1
				]
			]
		].
		view gridLayout gapSize: 1
	].
	view root applyLayout.
	
	node := view root nodes first.
	bounds := view root bounds.	
	form := Form extent: bounds extent depth: Display depth.
	canvas := FormCanvas on: form.

	^ Time millisecondsToRun: [ 5 timesRepeat: [ node announce: (MOMouseDrag step: 1@2). view root acceptVisitor: (MOMorphPainterVisitor newCanvas: canvas). ] ]

! !

!MOBenchmarks methodsFor: 'benchmarks' stamp: ''!
reportSubnodesLookup: iteration
	"
	self new reportSubnodesLookup: 20000  
	=> 1810 (AlexandreBergel and Henrik 9/12/2010 17:50)
 	"

	| view |
	view := MOViewRenderer new.
	view nodes: (1 to: 2) forEach: [ :each | 
		view nodes: (2 to: 4) forEach: [ :each2 | 
			view nodes: (4 to: 8) forEach: [ :each3 | 
				view node: each * each2 * each3.
			] 
		] 
	].


	^ Time millisecondsToRun: [ iteration timesRepeat: [ view localLookupNode: 8 * 4 * 2 ] ] 
! !

!MOBenchmarks methodsFor: 'benchmarks' stamp: ''!
subviews
	"
	MessageTally spyOn: self new subviews
	Time millisecondsToRun: self new subviews
		=>  179 (Alexandre.Bergel 8/18/2009 21:40)
		=>  153  (ab 5/12/2009 22:55 with Jannik) smarter cache
		=>   159  (Alexandre.Bergel 5/11/2009 00:03) Added cache
		=>  986 (Alexandre.Bergel 4/30/2009 18:58) Added a test foir the border
		=> 868 (Alexandre.Bergel 4/10/2009 23:13)
		=> 772 (Alexandre.Bergel 4/10/2009 17:59)
		 1872 Feb 21 (with cache shapeBounds, but it seems that it does not recurs)
		2697 Feb 21
	"
	
	| view bounds form canvas |
	view := MOViewRenderer new.
	view nodes: #(a) forEach: [:each |
		view nodes: (1 to: 5) forEach: [:e |
			view nodes: (1 to: 5).
		].
	].

"	^ Time millisecondsToRun: [
		1 to: 1000 do: [:i | view root applyLayout]]"

	view root applyLayout.
	bounds := view root bounds.
	
	form := Form extent: bounds extent depth: Display depth.
	canvas := FormCanvas on: form.

	^[ 1 to: 1000 do: [:i | view root acceptVisitor: (MOMorphPainterVisitor newCanvas: canvas).] ]
	
	! !

!MOBenchmarks methodsFor: 'benchmarks' stamp: ''!
subviews2
	"
	MessageTally spyOn: self new subviews2   
	Time millisecondsToRun: self new subviews2   
		=> 1145 (Alexandre.Bergel 8/18/2009 21:41)
		
		=> 833  	(Alexandre.Bergel 4/30/2009 18:59) Added a test for the border
		=> 788 	(Alexandre.Bergel 4/10/2009 23:13)
		=> 868 	(Alexandre.Bergel 4/10/2009 17:59) 
		=> 774  (Alexandre.Bergel 4/2/2009 01:15)
		=> 993  (Alexandre.Bergel 4/2/2009 01:14)
		=> 1055 (Alexandre.Bergel 4/2/2009 01:13)
		=> 1176 (Alexandre.Bergel 4/2/2009 01:12)
		=> 1273 (Alexandre.Bergel 4/2/2009 01:04)
		=> 1900 (April 2, 2009, 01:03)
		=> 1445 (March 26, 2009)  -- VW: 11
	"
	
	| view size|
	view := MOViewRenderer new.
	size := 5.
	view nodes: (1 to: size) forEach: [:each | view nodes: (1 to: size). view gridLayout].
	
"	^ MessageTally spyOn: [
		1 to: 1000 do: [:i | view root applyLayout]]
"	
	^ [ 1 to: 1000 do: [:i | view root applyLayout] ]


"	^ Time millisecondsToRun: [view root applyLayout].
"

"	bounds := view root bounds.
	
	form := Form extent: bounds extent.
	canvas := BalloonCanvas on: form.
"
"	^ Time millisecondsToRun: [
		1 to: 1000 do: [:i | view root displayOn: canvas]]
	
"	

"VW CODE:

Time millisecondsToRun: [	
  | view size |
	view := ViewRenderer new.
	size := 5.
	view nodes: (1 to: size) forEach: [:each | view nodes: (1 to: size). view gridLayout].
	1 to: 1000 do: [:i | view canvas]].
"! !

Object subclass: #MOTestModel
	instanceVariableNames: 'a1 b1 b2 c1 c2 c3 c4 d1 d2 d3 nodes edges'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Tests'!

!MOTestModel methodsFor: 'accessing' stamp: ''!
a1
	
	^a1! !

!MOTestModel methodsFor: 'accessing' stamp: ''!
allEdges
	
	^edges! !

!MOTestModel methodsFor: 'accessing' stamp: ''!
allNodes
	
	^nodes! !

!MOTestModel methodsFor: 'accessing' stamp: ''!
b1
	
	^b1! !

!MOTestModel methodsFor: 'accessing' stamp: ''!
b2
	
	^b2! !

!MOTestModel methodsFor: 'accessing' stamp: ''!
c1
	
	^c1! !

!MOTestModel methodsFor: 'accessing' stamp: ''!
d1
	
	^d1! !

!MOTestModel methodsFor: 'accessing' stamp: ''!
d2
	
	^d2! !

!MOTestModel methodsFor: 'accessing' stamp: ''!
d3
	
	^d3! !

!MOTestModel methodsFor: 'initialize-release' stamp: ''!
initialize
	
	self initializeNodes.
	self initializeEdges! !

!MOTestModel methodsFor: 'initialize-release' stamp: ''!
initializeEdges
	
	edges := OrderedCollection new.
	edges add: (Array with: a1 with: b1).
	edges add: (Array with: a1 with: b2).
	edges add: (Array with: b1 with: c1).
	edges add: (Array with: b1 with: c2).
	edges add: (Array with: b2 with: c3).
	edges add: (Array with: b2 with: c4).
	edges add: (Array with: c1 with: d1).
	edges add: (Array with: c1 with: d2).
	edges add: (Array with: c1 with: d3)! !

!MOTestModel methodsFor: 'initialize-release' stamp: ''!
initializeNodes
	
	a1 := (OrderedCollection new)
		add: #a1;
		add: 3;
		add: 6;
		add: 3;
		add: self;
		yourself.
	b1 := (OrderedCollection new)
		add: #b1;
		add: 1;
		add: 5;
		add: 0;
		add: self;
		yourself.
	b2 := (OrderedCollection new)
		add: #b2;
		add: 0;
		add: 2;
		add: 9;
		add: self;
		yourself.
	c1 := (OrderedCollection new)
		add: #c1;
		add: 5;
		add: 0;
		add: 15;
		add: self;
		yourself.
	c2 := (OrderedCollection new)
		add: #c2;
		add: 6;
		add: 0;
		add: 18;
		add: self;
		yourself.
	c3 := (OrderedCollection new)
		add: #c3;
		add: 9;
		add: 9;
		add: 22;
		add: self;
		yourself.
	c4 := (OrderedCollection new)
		add: #c4;
		add: 8;
		add: 7;
		add: 0;
		add: self;
		yourself.
	d1 := (OrderedCollection new)
		add: #d1;
		add: 12;
		add: 4;
		add: 4;
		add: self;
		yourself.
	d2 := (OrderedCollection new)
		add: #d2;
		add: 2;
		add: 13;
		add: 19;
		add: self;
		yourself.
	d3 := (OrderedCollection new)
		add: #d3;
		add: 15;
		add: 8;
		add: 12;
		add: self;
		yourself.
	nodes := (OrderedCollection new)
		add: a1;
		add: b1;
		add: b2;
		add: c1;
		add: c2;
		add: c3;
		add: c4;
		add: d1;
		add: d2;
		add: d3;
		yourself! !

!MOTestModel methodsFor: 'shapes' stamp: ''!
shapeForEdge
	
	^ MOStraightLineShape color: [:entity | Color lightGray]! !

!MOTestModel methodsFor: 'shapes' stamp: ''!
shapeForNode
	
	^ (MORectangleShape
						width: [:anEntity | 5 + (anEntity at: 2)]
						height: [:anEntity | 5 + (anEntity at: 3)]
						fillColor:
							(MONColorLinearNormalizer
								inContext: self allNodes
								withCommand: [:entity | entity at: 4]))
	! !

Object subclass: #MondrianArchitecture
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Tests'!

!MondrianArchitecture classSide methodsFor: 'defaults' stamp: ''!
default
	^ self new ! !

!MondrianArchitecture methodsFor: 'rules' stamp: ''!
architecturalRules
	"Each package depends on a group of packages"

	^{
		#'Mondrian-ComplexShape' -> {#'Mondrian-Shapes' . #'Mondrian-Core'}  .
		#'Mondrian-Core' -> {#'Mondrian-Core-Cache'} .
		#'Mondrian-Easel' -> {#'Mondrian-Core' . #'Mondrian-Pharo-Morphic'} .
		#'Mondrian-Events' ->  {#'Mondrian-Core'} .
		#'Mondrian-Example' -> {#'Mondrian-Core' . #'Mondrian-Easel' . #'Mondrian-ComplexShape'}.
		#'Mondrian-Example-Tests' -> {#'Mondrian-Example' . #'Mondrian-Core' }.
		#'Mondrian-FADELayout' -> {#'Mondrian-Layouts' .#'Mondrian-Core' }.
		#'Mondrian-FADELayout-Tests'  -> {#'Mondrian-FADELayout' . #'Mondrian-Core'} .
		#'Mondrian-Help' -> {#'Mondrian-Core' . #'Mondrian-Easel'} .
		#'Mondrian-Help-Tests'  -> {#'Mondrian-Help'} .
		#'Mondrian-Layouts' -> {#'Mondrian-Core'} .
		#'Mondrian-Normalizers' -> {#'Mondrian-Core'}.
		#'Mondrian-Pharo-Morphic' -> {#'Mondrian-Core'} .
		#'Mondrian-Pharo-Tests'  -> {#'Mondrian-Pharo-Morphic' . #'Mondrian-Core' . #'Mondrian-Easel'} .
		#'Mondrian-Shapes' -> {#'Mondrian-Core'} .
		#'Mondrian-Tests'  -> {#'Mondrian-Core' . #'Mondrian-Easel' . #'Mondrian-ComplexShape'} .
		#'Mondrian-Util' -> {#'Mondrian-Core'} .
		#'Mondrian-Util-Tests' -> {#'Mondrian-Util' } .
		#'Mondrian-Visitor' -> {#'Mondrian-Core'} .
		#'Mondrian-Visitor-Tests' -> {#'Mondrian-Visitor' . #'Mondrian-Core'} 
	} asDictionary.! !

!MondrianArchitecture methodsFor: 'metadata' stamp: ''!
canditateOfClassNamed: aClassname andMethodNamed: aMethodName forSelector: asecondMethodName

	| d |
	d := self methodsDependencies. 
	^ ((d at:  aClassname -> aMethodName ifAbsent: [ ^ false ])
		detect: [ :ass | ass value == asecondMethodName ]) 
! !

!MondrianArchitecture methodsFor: 'metadata' stamp: ''!
classNamed: aClassname andMethodNamed: aMethodName reference: anotherClassName andMethod: asecondMethodName

	| d v |
	d := self methodsDependencies. 
	v := d at:  aClassname -> aMethodName ifAbsentPut: #().
	d at:  aClassname -> aMethodName put: 
		(v asOrderedCollection add: anotherClassName -> asecondMethodName; asArray).
	self class compile: 'methodsDependencies', String cr, '	^ ', d storeString.! !

!MondrianArchitecture methodsFor: 'metadata' stamp: ''!
hasClassNamed: aClassname andMethodNamed: aMethodName hasUniqueCandidateForSelector: asecondMethodName

	| d |
	d := self methodsDependencies. 
	^ (d at:  aClassname -> aMethodName ifAbsent: [ ^ false ])
		anySatisfy: [ :ass | ass value == asecondMethodName ] 
! !

!MondrianArchitecture methodsFor: 'metadata' stamp: ''!
isClassNamed: aClassname andMethodNamed: aMethodName reference: anotherClassName andMethod: asecondMethodName

	"
	 MondrianArchitecture new isClassNamed: #MOFADELayout andMethodNamed: #treeCode:in:insideOf: reference: #MOGraphElement  andMethod: #center
	
	 MondrianArchitecture new isClassNamed: #MOFADE2Layout andMethodNamed: #doExecute: reference: #MORoot  andMethod: #translateBy:

	 MondrianArchitecture new isClassNamed: #MOFADELayout andMethodNamed: #treeCode:in:insideOf: reference: #Rectangle  andMethod: #center

	"
	| d |
	d := self methodsDependencies. 
	^ (d at:  aClassname -> aMethodName ifAbsent: [ ^ false ])
		includes: anotherClassName -> asecondMethodName! !

!MondrianArchitecture methodsFor: 'metadata' stamp: ''!
methodsDependencies
	^ ((Dictionary new) add: ((#Color->#strongColors)->((Array new: 1) at: 1 put: (#OrderedCollection->#add:); yourself)); add: ((#MOGraphElement->#extent)->((Array new: 1) at: 1 put: (#MOBoundedShape->#extentFor:); yourself)); add: ((#MOGraphElement->#computeParentShape)->((Array new: 1) at: 1 put: (#MOGraphElement->#shape); yourself)); add: ((#MOGraphElement->#bounds)->((Array new: 1) at: 1 put: (#MOShape->#computeBoundsFor:); yourself)); add: ((#MOGraphElement->#cacheNamed:of:ifAbsentInitializeWith:)->((Array new: 1) at: 1 put: (#MOShape->#isCached); yourself)); add: ((#Color->#lightColors)->((Array new: 1) at: 1 put: (#OrderedCollection->#add:); yourself)); add: ((#MOEasel->#menuCommandOn:)->((Array new: 2) at: 1 put: (#ToolBuilder->#parent:); at: 2 put: (#ToolBuilder->#action:); yourself)); add: ((#Collection->#maxValue:)->((Array new: 1) at: 1 put: (#BlockClosure->#value:); yourself)); add: ((#MOGraphElement->#hasParentShape)->((Array new: 1) at: 1 put: (#MOShape->#isFormsShape); yourself)); add: ((#MOPopup->#showView:zoomedInBy:)->((Array new: 1) at: 1 put: (#Rectangle->#translateBy:); yourself)); add: ((#MOLayout->#withEdges:)->((Array new: 1) at: 1 put: (#MOLayout->#initialize); yourself)); add: ((#MOLayout->#executeOnNode:)->((Array new: 1) at: 1 put: (#MOLayout->#doExecute:); yourself)); add: ((#MOLayout->#toPositions)->((Array new: 1) at: 1 put: (#MOLayout->#defaultToPositions); yourself)); add: ((#MONode->#acceptVisitor:)->((Array new: 1) at: 1 put: (#MOVisitor->#visitNode:); yourself)); add: ((#MONodeShape->#absoluteBoundsFor:)->((Array new: 1) at: 1 put: (#MONodeShape->#positionOfChildrenForOwner:); yourself)); "add: ((#MONode->#displayWithoutCacheOn:)->((Array new: 1) at: 1 put: (#MOShape->#display:on:); yourself));" add: ((#MONode->#applyLayout)->((Array new: 1) at: 1 put: (#MOShape->#computeBoundsFor:); yourself)); add: ((#MOLayout->#forNodes:)->((Array new: 1) at: 1 put: (#Behavior->#new); yourself)); add: ((#MOLayout->#fromPositions)->((Array new: 1) at: 1 put: (#MOLayout->#defaultFromPositions); yourself)); add: ((#MOShape->#boundsFor:)->((Array new: 1) at: 1 put: (#MOShape->#computeBoundsFor:); yourself)); add: ((#MOShape->#updateBounds:for:)->((Array new: 1) at: 1 put: (#MONodeShape->#computeExtentFor:); yourself)); add: ((#MOShape->#defaultNodeShapeInstance)->((Array new: 1) at: 1 put: (#Behavior->#new); yourself)); add: ((#MOShape->#defaultLineShapeInstance)->((Array new: 1) at: 1 put: (#Behavior->#new); yourself)); "add: ((#MONode->#displayOn:)->((Array new: 1) at: 1 put: (#Canvas->#paintImage:at:); yourself)); "add: ((#MOBoundedShape->#computeExtentHavingChildrenFor:)->((Array new: 1) at: 1 put: (#MOBoundedShape->#heightFor:); yourself)); add: ((#MOBoundedShape->#width:height:fillColor:)->((Array new: 1) at: 1 put: (#MOBoundedShape->#height:); yourself)); add: ((#Collection->#sortedAs:)->((Array new: 1) at: 1 put: (#BlockClosure->#value:); yourself)); add: ((#MOGraphElement->#hasBuilderAChildShape)->((Array new: 1) at: 1 put: (#MOFormsAdaptor->#shape); yourself)); yourself)! !

!MondrianArchitecture methodsFor: 'metadata' stamp: ''!
methodsDependenciesOLD
	^ ((Dictionary new) add: ((#MOCanvasTest->#setUp)->((Array new: 1) at: 1 put: (#MORoot->#bounds); yourself)); add: ((#MOQuadTree->#addElement:)->((Array new: 1) at: 1 put: (#MOGraphElement->#bounds); yourself)); add: ((#MOQuadTree->#findElementsAt:)->((Array new: 1) at: 1 put: (#MOGraphElement->#bounds); yourself)); add: ((#MOQuadTree->#findElementsIntersecting:)->((Array new: 1) at: 1 put: (#MOGraphElement->#bounds); yourself)); add: ((#MOFormsBuilder->#checkIfOK:)->((Array new: 1) at: 1 put: (#Rectangle->#height); yourself)); add: ((#MOPopup->#showView:zoomedInBy:)->((Array new: 1) at: 1 put: (#Rectangle->#translateBy:); yourself)); add: ((#UMLEnhClassDiagram->#umlShape)->((Array new: 1) at: 1 put: (#MOFormsBuilder->#shape); yourself)); add: ((#MOLayout->#defaultAdjustWidthFor:and:)->((Array new: 1) at: 1 put: (#Rectangle->#width); yourself)); add: ((#MOLayout->#defaultAdjustHeightFor:and:)->((Array new: 1) at: 1 put: (#Rectangle->#height); yourself)); add: ((#MOLayout->#withEdges:)->((Array new: 1) at: 1 put: (#MOLayout->#initialize); yourself)); add: ((#MOGraphElement->#translateAbsoluteCacheBy:)->((Array new: 1) at: 1 put: (#Rectangle->#translateBy:); yourself)); add: ((#MOGraphElement->#extent)->((Array new: 1) at: 1 put: (#MOBoundedShape->#extentFor:); yourself)); add: ((#MOGraphElement->#getAssociatedBuilderFor:)->((Array new: 1) at: 1 put: (#MOGraphElement->#shape); yourself)); add: ((#MOGraphElement->#hasParentShape)->((Array new: 1) at: 1 put: (#MOGraphElement->#shape); yourself)); add: ((#MOGraphElement->#cacheNamed:of:ifAbsentInitializeWith:)->((Array new: 1) at: 1 put: (#MOShape->#isCached); yourself)); add: ((#MOGraphElement->#hasCacheNamed:of:)->((Array new: 1) at: 1 put: (#MOGraphElement->#shape); yourself)); add: ((#MOGraphElement->#bounds)->((Array new: 1) at: 1 put: (#MOShape->#computeBoundsFor:); yourself)); add: ((#MOGraphElement->#getAssociatedBuilder)->((Array new: 1) at: 1 put: (#MOGraphElement->#shape); yourself)); add: ((#MOGraphElement->#center)->((Array new: 1) at: 1 put: (#Rectangle->#center); yourself)); add: ((#MOGraphElement->#cacheNamed:of:)->((Array new: 1) at: 1 put: (#MOGraphElement->#shape); yourself)); add: ((#MOGraphElement->#computeChildrenOrigin)->((Array new: 1) at: 1 put: (#MOGraphElement->#shape); yourself)); add: ((#MOGraphElement->#computeParentShape)->((Array new: 1) at: 1 put: (#MOGraphElement->#shape); yourself)); add: ((#UMLClassDiagram->#umlShape)->((Array new: 1) at: 1 put: (#MOFormsBuilder->#shape); yourself)); add: ((#MORoot->#expandToIncludePoint:)->((Array new: 1) at: 1 put: (#Rectangle->#height); yourself)); add: ((#MORoot->#translateBy:)->((Array new: 1) at: 1 put: (#MONode->#translateBy:); yourself)); add: ((#MORoot->#initialize)->((Array new: 1) at: 1 put: (#Behavior->#new); yourself)); add: ((#MORoot->#updateBounds)->((Array new: 1) at: 1 put: (#MONode->#height:); yourself)); add: ((#MondrianHelpTest->#testOpeningBrowser)->((Array new: 1) at: 1 put: (#Behavior->#new); yourself)); add: ((#MOAnnouncer->#highlightWhenOver:)->((Array new: 2) at: 1 put: (#MOGraphElement->#shape); at: 2 put: (#MOGraphElement->#shape); yourself)); add: ((#MOAnnouncer->#highlightWhenOver)->((Array new: 2) at: 1 put: (#MOGraphElement->#shape); at: 2 put: (#MOGraphElement->#shape); yourself)); add: ((#MOFADESpringBenchmarks->#timeBenchmarkFor:and:)->((Array new: 1) at: 1 put: (#MOViewRenderer->#shape); yourself)); add: ((#MOFADESpringBenchmarks->#benchmack)->((Array new: 1) at: 1 put: (#MOViewRenderer->#shape); yourself)); add: ((#MOFADESpringBenchmarks->#benchmack2)->((Array new: 1) at: 1 put: (#MOViewRenderer->#shape); yourself)); add: ((#MOEaselCommandAbout->#execute)->((Array new: 1) at: 1 put: (#DisplayScreen->#width); yourself)); add: ((#MOEasel->#menuCommandOn:)->((Array new: 1) at: 1 put: (#ToolBuilder->#parent:); yourself)); add: ((#MOEasel->#generate)->((Array new: 1) at: 1 put: (#Behavior->#new); yourself)); add: ((#MOCanvas->#exportAsSVG)->((Array new: 1) at: 1 put: (#MORoot->#bounds); yourself)); add: ((#MOCanvas->#updateBounds)->((Array new: 1) at: 1 put: (#MORoot->#bounds); yourself)); add: ((#MOEaselExamples->#umlShape)->((Array new: 1) at: 1 put: (#MOFormsBuilder->#shape); yourself)); add: ((#MOViewRenderer->#getRenderedForm)->((Array new: 1) at: 1 put: (#MORoot->#bounds); yourself)); add: ((#MOViewRenderer->#edgeShape)->((Array new: 1) at: 1 put: (#Behavior->#new); yourself)); add: ((#MOViewRenderer->#nodeShape)->((Array new: 1) at: 1 put: (#Behavior->#new); yourself)); add: ((#MOViewRenderer->#open)->((Array new: 1) at: 1 put: (#Behavior->#new); yourself)); add: ((#MOViewRenderer->#forNode:do:)->((Array new: 1) at: 1 put: (#MOGraphElement->#shape); yourself)); add: ((#MOMatrixRendererTest->#testRendering)->((Array new: 1) at: 1 put: (#MOMatrixRenderer->#rows:); yourself)); add: ((#DistributionMapExample->#render:)->((Array new: 1) at: 1 put: (#MOBoundedShape->#size:); yourself)); add: ((#MOFADE2Layout->#doExecute:)->((Array new: 1) at: 1 put: (#MONode->#translateTo:); yourself)); add: ((#MOFADELayout->#doIncrementalStep:)->((Array new: 2) at: 1 put: (#MONode->#translateBy:); at: 2 put: (#MONode->#translateBy:); yourself)); add: ((#MOFADELayout->#treeCodeFor:)->((Array new: 1) at: 1 put: (#Rectangle->#center); yourself)); add: ((#MOFADELayout->#treeCode:in:insideOf:)->((Array new: 1) at: 1 put: (#Rectangle->#center); yourself)); add: ((#MOFADELayout->#coulombRepulsion:and:)->((Array new: 1) at: 1 put: (#Rectangle->#width); yourself)); add: ((#MOMatrixRenderer->#cellShape)->((Array new: 1) at: 1 put: (#MOBoundedShape->#size:); yourself)); add: ((#MOMatrixRenderer->#enclosingShape)->((Array new: 1) at: 1 put: (#MOBoundedShape->#size:); yourself)); add: ((#MOFADELayout->#treeCode:in:)->((Array new: 1) at: 1 put: (#Rectangle->#center); yourself)); add: ((#MOXMLVisitor->#visitNode:)->((Array new: 1) at: 1 put: (#MOGraphElement->#bounds); yourself)); add: ((#MOTikzVisitor->#visitNode:)->((Array new: 1) at: 1 put: (#MOGraphElement->#shape); yourself)); add: ((#MOTikzVisitor->#visitEdge:)->((Array new: 1) at: 1 put: (#MOGraphElement->#shape); yourself)); add: ((#MOFADEForceBasedBenchmarks->#timeBenchmarkFor:and:)->((Array new: 1) at: 1 put: (#MOViewRenderer->#shape); yourself)); add: ((#MOSVGVisitor->#visitNode:)->((Array new: 1) at: 1 put: (#MOGraphElement->#shape); yourself)); add: ((#MOSVGVisitor->#visitEdge:)->((Array new: 1) at: 1 put: (#MOGraphElement->#shape); yourself)); add: ((#MOShape->#defaultNodeShapeInstance)->((Array new: 1) at: 1 put: (#Behavior->#new); yourself)); add: ((#MOShape->#defaultLineShapeInstance)->((Array new: 1) at: 1 put: (#Behavior->#new); yourself)); add: ((#MOShape->#updateBounds:for:)->((Array new: 1) at: 1 put: (#MOShape->#computeExtentFor:); yourself)); add: ((#MOShape->#computeBoundsFor:)->((Array new: 1) at: 1 put: (#MOShape->#computeExtentFor:); yourself)); add: ((#MOShape->#computeExtentFor:)->((Array new: 1) at: 1 put: (#MOShape->#heightFor:); yourself)); add: ((#MOShape->#boundsFor:)->((Array new: 1) at: 1 put: (#MOShape->#computeBoundsFor:); yourself)); add: ((#MOFADELayoutTest->#setUp)->((Array new: 1) at: 1 put: (#MOViewRenderer->#shape); yourself)); add: ((#MOFADELayoutTest->#testNodeSizeEffect)->((Array new: 1) at: 1 put: (#MOViewRenderer->#shape); yourself)); add: ((#MOFADELayoutTest->#testTreeCode)->((Array new: 1) at: 1 put: (#Rectangle->#center); yourself)); "add: ((#MONode->#displayWithoutCacheOn:)->((Array new: 1) at: 1 put: (#MOShape->#display:on:); yourself));" add: ((#MONode->#elementAt:ifAbsent:)->((Array new: 1) at: 1 put: (#MOGraphElement->#shape); yourself)); add: ((#MONode->#translateBy:bounded:)->((Array new: 1) at: 1 put: (#Rectangle->#translateBy:); yourself)); add: ((#MONode->#getBoundedTranslationStep:)->((Array new: 1) at: 1 put: (#MOGraphElement->#bounds); yourself)); add: ((#MONode->#position)->((Array new: 1) at: 1 put: (#Rectangle->#center); yourself)); add: ((#MONode->#setBoundsTo:)->((Array new: 1) at: 1 put: (#Rectangle->#height); yourself)); add: ((#MONode->#height)->((Array new: 1) at: 1 put: (#Rectangle->#height); yourself)); add: ((#MONode->#applyLayout)->((Array new: 1) at: 1 put: (#MOShape->#computeBoundsFor:); yourself)); add: ((#MONode->#width)->((Array new: 1) at: 1 put: (#Rectangle->#width); yourself)); add: ((#MOFADELayoutTest->#testBasic)->((Array new: 1) at: 1 put: (#MOViewRenderer->#shape); yourself)); "add: ((#MOEdge->#displayWithoutCacheOn:)->((Array new: 1) at: 1 put: (#MOShape->#display:on:); yourself));" add: ((#MOCanvasTest->#testCanvasSize2)->((Array new: 1) at: 1 put: (#MOGraphElement->#bounds); yourself)); add: ((#MOCanvasTest->#testCanvasSize3)->((Array new: 1) at: 1 put: (#Morph->#bounds); yourself)); add: ((#MOCanvasTest->#testCanvasStretch)->((Array new: 1) at: 1 put: (#MORoot->#bounds); yourself)); add: ((#MOEdge->#bounds)->((Array new: 1) at: 1 put: (#MOLineShape->#computeBoundsFor:); yourself)); "add: ((#MOEdge->#displayOn:)->((Array new: 1) at: 1 put: (#MOShape->#display:on:); yourself));" add: ((#MOEaselCommandDebugView->#execute)->((Array new: 1) at: 1 put: (#MOViewRenderer->#shape); yourself)); add: ((#MOCanvasTest->#testCanvasSize)->((Array new: 1) at: 1 put: (#MOGraphElement->#bounds); yourself)); add: ((#MOFadeGroupTest->#testBasic)->((Array new: 1) at: 1 put: (#Rectangle->#center); yourself)); add: ((#MOQuadTree->#removeElement:)->((Array new: 1) at: 1 put: (#MOGraphElement->#bounds); yourself)); add: ((#MOQuadTree->#splitAdd:)->((Array new: 1) at: 1 put: (#MOGraphElement->#bounds); yourself)); add: ((#MOCanvasTest->#testBasic)->((Array new: 1) at: 1 put: (#Morph->#bounds); yourself)); yourself)! !

NullCanvas subclass: #CountingNullCanvas
	instanceVariableNames: 'numberOfRectangles numberOfLines numberOfImages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Tests'!

!CountingNullCanvas classSide methodsFor: 'instance creation' stamp: ''!
on: aForm
	"we do nothing"
	^ self new! !

!CountingNullCanvas methodsFor: 'accessing' stamp: ''!
clipRect
	^1@1 extent: 100000@100000! !

!CountingNullCanvas methodsFor: 'drawing-rectangles' stamp: ''!
frameAndFillRectangle: bounds fillColor: c borderWidth: width borderColor: cc
	numberOfRectangles := numberOfRectangles + 1.
! !

!CountingNullCanvas methodsFor: 'initialization' stamp: ''!
initialize 
	super initialize .
	numberOfRectangles := 0.
	numberOfLines := 0.
	numberOfImages := 0! !

!CountingNullCanvas methodsFor: 'drawing' stamp: ''!
line: fromPoint to: toPoint width: width color: color
	numberOfLines := numberOfLines + 1.! !

!CountingNullCanvas methodsFor: 'accessing' stamp: ''!
numberOfImages
	^ numberOfImages! !

!CountingNullCanvas methodsFor: 'accessing' stamp: ''!
numberOfLines
	^ numberOfLines! !

!CountingNullCanvas methodsFor: 'accessing' stamp: ''!
numberOfRectangles
	^ numberOfRectangles! !

!CountingNullCanvas methodsFor: 'drawing-images' stamp: ''!
paintImage: cacheForm at: position
	numberOfImages := numberOfImages + 1.
! !

!CountingNullCanvas methodsFor: 'hook' stamp: ''!
warpImage: aForm transform: aTransform at: extraOffset sourceRect: sourceRect cellSize: cellSize
	"We cancel this method. It is defined as abstract in the Canvas class"! !
